@@ -322,9 +322,11 @@ namespace Commons.Xml.Relaxng
 			RelaxngGrammar g = null;
 			try {
 				xtr = new XmlTextReader (uri.AbsoluteUri, (Stream) grammar.Resolver.GetEntity (uri, null, typeof (Stream)));
-				RelaxngReader r = new RelaxngReader (xtr, ns);
+				RelaxngReader r = new RelaxngReader (xtr, ns, grammar.Resolver);
 				r.MoveToContent ();
 				g = r.ReadPattern () as RelaxngGrammar;
+			} catch (Exception ex) { // umm, bad catch though :-(
+				throw new RelaxngException (this, String.Format("Could not include grammar {0}: {1}", uri.AbsoluteUri, ex.Message), ex);
 			} finally {
 				if (xtr != null)
 					xtr.Close ();
@@ -455,12 +457,18 @@ namespace Commons.Xml.Relaxng
 
 		public static RelaxngPattern Read (XmlReader xmlReader, RelaxngDatatypeProvider provider)
 		{
-			RelaxngReader r = new RelaxngReader (xmlReader, null);
+			return Read (xmlReader, provider, new XmlUrlResolver ());
+		}
+		
+		public static RelaxngPattern Read (XmlReader xmlReader, RelaxngDatatypeProvider provider, XmlResolver xmlResolver)
+		{
+			RelaxngReader r = new RelaxngReader (xmlReader, null, xmlResolver);
 			if (r.ReadState == ReadState.Initial)
 				r.Read ();
 			r.MoveToContent ();
 			RelaxngPattern p = r.ReadPattern ();
 			p.DataProvider = provider;
+			p.XmlResolver = xmlResolver;
 			return p;
 		}
 
@@ -1224,7 +1232,7 @@ namespace Commons.Xml.Relaxng
 			XmlTextReader xtr = null;
 			try {
 				xtr = new XmlTextReader (uri.AbsoluteUri, (Stream) grammar.Resolver.GetEntity (uri, null, typeof (Stream)));
-				RelaxngReader r = new RelaxngReader (xtr, ns);
+				RelaxngReader r = new RelaxngReader (xtr, ns, grammar.Resolver);
 				r.MoveToContent ();
 				RelaxngPattern p = r.ReadPattern ();
 				p.DataProvider = grammar.Provider;
