@@ -167,7 +167,7 @@ namespace System.Windows.Forms.X11Internal {
 		} 
 
 		internal override Size FrameBorderSize { 
-			get { throw new NotImplementedException(); }
+			get { return new Size (4, 4); }
 		}
 
 		internal override Size IconSize {
@@ -231,7 +231,7 @@ namespace System.Windows.Forms.X11Internal {
 		} 
 
 		internal override Rectangle VirtualScreen {
-			get { return WorkingArea; }
+			get { return display.VirtualScreen; }
 		} 
 
 		internal override Rectangle WorkingArea {
@@ -290,21 +290,9 @@ namespace System.Windows.Forms.X11Internal {
 		}
 
 		// XXX this implementation should probably be shared between all non-win32 backends
-		internal override bool CalculateWindowRect (ref Rectangle ClientRect, int Style, int ExStyle, Menu menu, out Rectangle WindowRect)
+		internal override bool CalculateWindowRect (ref Rectangle ClientRect, CreateParams cp, Menu menu, out Rectangle WindowRect)
 		{
-			FormBorderStyle	border_style;
-			TitleStyle	title_style;
-			bool border_static;
-			int caption_height;
-			int tool_caption_height;
-
-			// XXX this method should be static on Hwnd, not X11Hwnd
-			X11Hwnd.DeriveStyles (Style, ExStyle, out border_style, out border_static, out title_style,
-					      out caption_height, out tool_caption_height);
-
-			WindowRect = Hwnd.GetWindowRectangle(border_style, border_static, menu, title_style,
-							     caption_height, tool_caption_height,
-							     ClientRect);
+			WindowRect = Hwnd.GetWindowRectangle (cp, menu, ClientRect);
 			return true;
 		}
 
@@ -536,7 +524,10 @@ namespace System.Windows.Forms.X11Internal {
 		// XXX this should be shared amongst non-win32 backends
 		internal override bool GetFontMetrics (Graphics g, Font font, out int ascent, out int descent)
 		{
-			return Xlib.GetFontMetrics(g.GetHdc(), font.ToHfont(), out ascent, out descent);
+			FontFamily ff = font.FontFamily;
+			ascent = ff.GetCellAscent (font.Style);
+			descent = ff.GetCellDescent (font.Style);
+			return true;
 		}
 
 
@@ -701,8 +692,8 @@ namespace System.Windows.Forms.X11Internal {
 
 		internal override void PostQuitMessage(int exitCode)
 		{
+			display.PostMessage (display.FosterParent.Handle, Msg.WM_QUIT, IntPtr.Zero, IntPtr.Zero);
 			display.Flush ();
-			ThreadQueue(Thread.CurrentThread).PostQuitState = true;
 		}
 
 		[MonoTODO]
@@ -885,7 +876,17 @@ namespace System.Windows.Forms.X11Internal {
 			queue.SetTimer (timer);
 		}
 
-		internal override bool SetTopmost(IntPtr handle, IntPtr handle_owner, bool enabled)
+		internal override bool SetTopmost(IntPtr handle, bool enabled)
+		{
+			X11Hwnd hwnd = (X11Hwnd) Hwnd.ObjectFromHandle (handle);
+
+			if (hwnd == null)
+				return false;
+
+			return hwnd.SetTopmost (enabled);
+		}
+
+		internal override bool SetOwner(IntPtr handle, IntPtr handle_owner)
 		{
 			X11Hwnd hwnd;
 
@@ -896,7 +897,7 @@ namespace System.Windows.Forms.X11Internal {
 
 			X11Hwnd hwnd_owner = (X11Hwnd)Hwnd.ObjectFromHandle(handle_owner);
 
-			return hwnd.SetTopmost (hwnd_owner, enabled);
+			return hwnd.SetOwner (hwnd_owner);
 		}
 
 		internal override bool SetVisible (IntPtr handle, bool visible, bool activate)
@@ -1002,6 +1003,31 @@ namespace System.Windows.Forms.X11Internal {
 			display.SystrayRemove (handle, ref tt);
 		}
 
+#if NET_2_0
+		NotifyIcon.BalloonWindow balloon_window;
+
+		internal override void SystrayBalloon(IntPtr handle, int timeout, string title, string text, ToolTipIcon icon)
+		{
+			Control control = Control.FromHandle(handle);
+			
+			if (control == null)
+				return;
+
+			if (balloon_window != null) {
+				balloon_window.Close ();
+				balloon_window.Dispose ();
+			}
+
+			balloon_window = new NotifyIcon.BalloonWindow (handle);
+			balloon_window.Title = title;
+			balloon_window.Text = text;
+			balloon_window.Timeout = timeout;
+			balloon_window.Show ();
+			
+			SendMessage(handle, Msg.WM_USER, IntPtr.Zero, (IntPtr) Msg.NIN_BALLOONSHOW);	
+		}
+#endif
+
 		internal override bool Text (IntPtr handle, string text)
 		{
 			X11Hwnd	hwnd = (X11Hwnd) Hwnd.ObjectFromHandle(handle);
