@@ -195,23 +195,23 @@ namespace System.Windows.Forms
 		static readonly Color	def_background_color = ThemeEngine.Current.DataGridBackgroundColor;
 		static readonly Color	def_caption_backcolor = ThemeEngine.Current.DataGridCaptionBackColor;
 		static readonly Color	def_caption_forecolor = ThemeEngine.Current.DataGridCaptionForeColor;
-		static readonly Color	def_parentrowsback_color = ThemeEngine.Current.DataGridParentRowsBackColor;
-		static readonly Color	def_parentrowsfore_color = ThemeEngine.Current.DataGridParentRowsForeColor;
+		static readonly Color	def_parent_rows_backcolor = ThemeEngine.Current.DataGridParentRowsBackColor;
+		static readonly Color	def_parent_rows_forecolor = ThemeEngine.Current.DataGridParentRowsForeColor;
 
 		/* colors */
 		// XXX this needs addressing. Control.background_color should not be internal.
 		new Color background_color;
 		Color caption_backcolor;
 		Color caption_forecolor;
-		Color parentrowsback_color;
-		Color parentrowsfore_color;
+		Color parent_rows_backcolor;
+		Color parent_rows_forecolor;
 
 		/* flags to determine which areas of the datagrid are shown */
 		bool caption_visible;
-		bool parentrows_visible;
+		bool parent_rows_visible;
 
 		GridTableStylesCollection styles_collection;
-		DataGridParentRowsLabelStyle parentrowslabel_style;
+		DataGridParentRowsLabelStyle parent_rows_label_style;
 		DataGridTableStyle default_style;
 		DataGridTableStyle grid_style;
 		DataGridTableStyle current_style;
@@ -223,10 +223,10 @@ namespace System.Windows.Forms
 
 		/* layout/rendering */
 		bool allow_navigation;
-		int first_visiblerow;
-		int first_visiblecolumn;
-		int visiblerow_count;
-		int visiblecolumn_count;
+		int first_visible_row;
+		int first_visible_column;
+		int visible_row_count;
+		int visible_column_count;
 		Font caption_font;
 		string caption_text;
 		bool flatmode;
@@ -272,7 +272,7 @@ namespace System.Windows.Forms
 		bool is_editing;		// Current cell is edit mode
 		bool is_changing;
 
-		internal Stack dataSourceStack;
+		internal Stack data_source_stack;
 
 		#endregion // Local Variables
 
@@ -287,11 +287,11 @@ namespace System.Windows.Forms
 			caption_text = string.Empty;
 			caption_visible = true;
 			datamember = string.Empty;
-			parentrowsback_color = def_parentrowsback_color;
-			parentrowsfore_color = def_parentrowsfore_color;
-			parentrows_visible = true;
+			parent_rows_backcolor = def_parent_rows_backcolor;
+			parent_rows_forecolor = def_parent_rows_forecolor;
+			parent_rows_visible = true;
 			current_cell = new DataGridCell ();
-			parentrowslabel_style = DataGridParentRowsLabelStyle.Both;
+			parent_rows_label_style = DataGridParentRowsLabelStyle.Both;
 			selected_rows = new Hashtable ();
 			selection_start = -1;
 			rows = new DataGridRelationshipRow [0];
@@ -311,7 +311,7 @@ namespace System.Windows.Forms
 
 			SetStyle (ControlStyles.UserMouse, true);
 
-			dataSourceStack = new Stack ();
+			data_source_stack = new Stack ();
 
 			back_button_image = ResourceImageLoader.Get ("go-previous.png");
 			//back_button_image.MakeTransparent (Color.Transparent);
@@ -340,7 +340,7 @@ namespace System.Windows.Forms
 			set { grid_style.AllowSorting = value; }
 		}
 
-		public Color AlternatingBackColor  {
+		public Color AlternatingBackColor {
 			get { return grid_style.AlternatingBackColor; }
 			set { grid_style.AlternatingBackColor = value; }
 		}
@@ -369,7 +369,7 @@ namespace System.Windows.Forms
 				if (base.BackgroundImage == value)
 					return;
 
-    				base.BackgroundImage = value;
+				base.BackgroundImage = value;
 				Invalidate ();
 			}
 		}
@@ -616,7 +616,7 @@ namespace System.Windows.Forms
 
 		[Browsable(false)]
 		public int FirstVisibleColumn {
-			get { return first_visiblecolumn; }
+			get { return first_visible_column; }
 		}
 
 		[DefaultValue(false)]
@@ -733,11 +733,11 @@ namespace System.Windows.Forms
 		}
 
 		public Color ParentRowsBackColor {
-			get { return parentrowsback_color; }
+			get { return parent_rows_backcolor; }
 			set {
-				if (parentrowsback_color != value) {
-					parentrowsback_color = value;
-					if (parentrows_visible) {
+				if (parent_rows_backcolor != value) {
+					parent_rows_backcolor = value;
+					if (parent_rows_visible) {
 						Refresh ();
 					}
 				}
@@ -745,11 +745,11 @@ namespace System.Windows.Forms
 		}
 
 		public Color ParentRowsForeColor {
-			get { return parentrowsfore_color; }
+			get { return parent_rows_forecolor; }
 			set {
-				if (parentrowsfore_color != value) {
-					parentrowsfore_color = value;
-					if (parentrows_visible) {
+				if (parent_rows_forecolor != value) {
+					parent_rows_forecolor = value;
+					if (parent_rows_visible) {
 						Refresh ();
 					}
 				}
@@ -759,11 +759,11 @@ namespace System.Windows.Forms
 		[DefaultValue(DataGridParentRowsLabelStyle.Both)]
 		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
 		public DataGridParentRowsLabelStyle ParentRowsLabelStyle {
-			get { return parentrowslabel_style; }
+			get { return parent_rows_label_style; }
 			set {
-				if (parentrowslabel_style != value) {
-					parentrowslabel_style = value;
-					if (parentrows_visible) {
+				if (parent_rows_label_style != value) {
+					parent_rows_label_style = value;
+					if (parent_rows_visible) {
 						Refresh ();
 					}
 
@@ -774,10 +774,10 @@ namespace System.Windows.Forms
 
 		[DefaultValue(true)]
 		public bool ParentRowsVisible {
-			get { return parentrows_visible; }
+			get { return parent_rows_visible; }
 			set {
-				if (parentrows_visible != value) {
-					parentrows_visible = value;
+				if (parent_rows_visible != value) {
+					parent_rows_visible = value;
 					CalcAreasAndInvalidate ();
 					OnParentRowsVisibleChanged (EventArgs.Empty);
 				}
@@ -831,7 +831,7 @@ namespace System.Windows.Forms
 			set { grid_style.SelectionBackColor = value; }
 		}
 
-		public Color SelectionForeColor  {
+		public Color SelectionForeColor {
 			get { return grid_style.SelectionForeColor; }
 			set { grid_style.SelectionForeColor = value; }
 		}
@@ -867,12 +867,12 @@ namespace System.Windows.Forms
 
 		[Browsable(false)]
 		public int VisibleColumnCount {
-			get { return visiblecolumn_count; }
+			get { return visible_column_count; }
 		}
 
 		[Browsable(false)]
 		public int VisibleRowCount {
-			get { return visiblerow_count; }
+			get { return visible_row_count; }
 		}
 
 		#endregion	// Public Instance Properties
@@ -897,7 +897,7 @@ namespace System.Windows.Forms
 		}
 
 		internal int FirstVisibleRow {
-			get { return first_visiblerow; }
+			get { return first_visible_row; }
 		}
 		
 		internal int RowsCount {
@@ -923,7 +923,7 @@ namespace System.Windows.Forms
 		}
 		
 		internal bool ShowParentRows {
-			get { return ParentRowsVisible && dataSourceStack.Count > 0; }
+			get { return ParentRowsVisible && data_source_stack.Count > 0; }
 		}
 		
 		#endregion Private Instance Properties
@@ -998,7 +998,7 @@ namespace System.Windows.Forms
 			ResumeLayout (false);
 
 			/* XX need to redraw from @row down */
-			CalcAreasAndInvalidate ();			
+			CalcAreasAndInvalidate ();
 		}
 
 		protected internal virtual void ColumnStartedEditing (Control editingControl)
@@ -1137,18 +1137,18 @@ namespace System.Windows.Forms
 
 		protected virtual void GridVScrolled (object sender, ScrollEventArgs se)
 		{
-			int old_first_visiblerow = first_visiblerow;
-			first_visiblerow = se.NewValue;
+			int old_first_visible_row = first_visible_row;
+			first_visible_row = se.NewValue;
 
-			if (first_visiblerow == old_first_visiblerow)
+			if (first_visible_row == old_first_visible_row)
 				return;
 
 			UpdateVisibleRowCount ();
 
-			if (first_visiblerow == old_first_visiblerow)
+			if (first_visible_row == old_first_visible_row)
 				return;
 			
-			ScrollToRow (old_first_visiblerow, first_visiblerow);
+			ScrollToRow (old_first_visible_row, first_visible_row);
 		}
 
 		public HitTestInfo HitTest (Point position)
@@ -1162,7 +1162,7 @@ namespace System.Windows.Forms
 		// From Point to Cell
 		public HitTestInfo HitTest (int x, int y)
 		{
-			if (columnhdrs_area.Contains (x, y)) {
+			if (column_headers_area.Contains (x, y)) {
 				int offset_x = x + horiz_pixeloffset;
 				int column_x;
 				int column_under_mouse = FromPixelToColumn (offset_x, out column_x);
@@ -1177,7 +1177,7 @@ namespace System.Windows.Forms
 				}
 			}
 
-			if (rowhdrs_area.Contains (x, y)) {
+			if (row_headers_area.Contains (x, y)) {
 				int posy;
 				int rcnt = FirstVisibleRow + VisibleRowCount;
 				for (int r = FirstVisibleRow; r < rcnt; r++) {
@@ -1208,9 +1208,9 @@ namespace System.Windows.Forms
 				pos_y = cells_area.Y + rows[row].VerticalOffset - rows[FirstVisibleRow].VerticalOffset;
 				if (y <= pos_y + rows[row].Height) {
 					int col_pixel;
-					int column_cnt = first_visiblecolumn + visiblecolumn_count;
+					int column_cnt = first_visible_column + visible_column_count;
 					if (column_cnt > 0) {
-						for (int column = first_visiblecolumn; column < column_cnt; column++) {
+						for (int column = first_visible_column; column < column_cnt; column++) {
 							if (CurrentTableStyle.GridColumnStyles[column].bound == false)
 								continue;
 							col_pixel = GetColumnStartingPixel (column);
@@ -1248,10 +1248,10 @@ namespace System.Windows.Forms
 		[MonoTODO]
 		public void NavigateBack ()
 		{
-			if (dataSourceStack.Count == 0)
+			if (data_source_stack.Count == 0)
 				return;
 
-			DataGridDataSource source = (DataGridDataSource)dataSourceStack.Pop ();
+			DataGridDataSource source = (DataGridDataSource)data_source_stack.Pop ();
 			list_manager = source.list_manager;
 			rows = source.Rows;
 			selected_rows = source.SelectedRows;
@@ -1272,7 +1272,7 @@ namespace System.Windows.Forms
 			previous_source.SelectedRows = selected_rows;
 			previous_source.SelectionStart = selection_start;
 
-			dataSourceStack.Push (previous_source);
+			data_source_stack.Push (previous_source);
 
 			rows = null;
 			selected_rows = new Hashtable ();
@@ -1402,24 +1402,16 @@ namespace System.Windows.Forms
 		protected override void OnLayout (LayoutEventArgs levent)
 		{
 			base.OnLayout (levent);
-			CalcAreasAndInvalidate ();			
+			CalcAreasAndInvalidate ();
 		}
 
 		protected override void OnLeave (EventArgs e)
 		{
 			base.OnLeave (e);
 
-#if false
-			/* we get an OnLeave call when the
-			 * DataGridTextBox control is focused, so we
-			 * need to ignore that.  If we get an OnLeave
-			 * call when a child control is not receiving
-			 * focus, we need to cancel the current
-			 * edit. */
 			if (cursor_in_add_row) {
 				ListManager.CancelCurrentEdit ();
 			}
-#endif
 		}
 
 		protected override void OnMouseDown (MouseEventArgs e)
@@ -1462,7 +1454,7 @@ namespace System.Windows.Forms
 			case HitTestType.RowHeader:
 				bool expansion_click = false;
 				if (CurrentTableStyle.HasRelations) {
-					if (e.X > rowhdrs_area.X + rowhdrs_area.Width / 2) {
+					if (e.X > row_headers_area.X + row_headers_area.Width / 2) {
 						/* it's in the +/- space */
 						if (IsExpanded (testinfo.Row))
 							Collapse (testinfo.Row);
@@ -1473,23 +1465,20 @@ namespace System.Windows.Forms
 					}
 				}
 
-				if (!ctrl_pressed &&
-				    !shift_pressed &&
-				    !expansion_click) {
+				CancelEditing ();
+				CurrentRow = testinfo.Row;
+
+				if (!ctrl_pressed && !shift_pressed && !expansion_click) {
 					ResetSelection (); // Invalidates selected rows
 				}
 
-				if ((shift_pressed ||
-				     expansion_click)
-				    && selection_start != -1) {
+				if ((shift_pressed || expansion_click) && selection_start != -1) {
 					ShiftSelection (testinfo.Row);
 				} else { // ctrl_pressed or single item
 					selection_start = testinfo.Row;
 					Select (testinfo.Row);
 				}
 
-				CancelEditing ();
-				CurrentRow = testinfo.Row;
 				OnRowHeaderClick (EventArgs.Empty);
 
 				break;
@@ -1527,9 +1516,9 @@ namespace System.Windows.Forms
 
 			case HitTestType.ColumnResize:
 				if (e.Clicks == 2) {
-					// double click column resize goes here
-				}
-				else {
+					EndEdit ();
+					ColumnResize (testinfo.Column);
+				} else {
 					resize_column = testinfo.Column;
 					column_resize_active = true;
 					resize_column_x = e.X;
@@ -1541,9 +1530,9 @@ namespace System.Windows.Forms
 
 			case HitTestType.RowResize:
 				if (e.Clicks == 2) {
-					// double click row resize goes here
-				}
-				else {
+					EndEdit ();
+					RowResize (testinfo.Row);
+				} else {
 					resize_row = testinfo.Row;
 					row_resize_active = true;
 					resize_row_y = e.Y;
@@ -1616,8 +1605,7 @@ namespace System.Windows.Forms
 						if (!back_button_mouseover)
 							Invalidate (back_button_rect);
 						back_button_mouseover = true;
-					}
-					else if (back_button_mouseover) {
+					} else if (back_button_mouseover) {
 						Invalidate (back_button_rect);
 						back_button_mouseover = false;
 					}
@@ -1626,8 +1614,7 @@ namespace System.Windows.Forms
 						if (parent_rows_button_mouseover)
 							Invalidate (parent_rows_button_rect);
 						parent_rows_button_mouseover = true;
-					}
-					else if (parent_rows_button_mouseover) {
+					} else if (parent_rows_button_mouseover) {
 						Invalidate (parent_rows_button_rect);
 						parent_rows_button_mouseover = false;
 					}
@@ -1663,8 +1650,7 @@ namespace System.Windows.Forms
 				width_of_all_columns += resize_column_width_delta;
 				Edit ();
 				Invalidate ();
-			}
-			else if (row_resize_active) {
+			} else if (row_resize_active) {
 				row_resize_active = false;
 
 				if (resize_row_height_delta + rows[resize_row].Height < 0)
@@ -1676,16 +1662,14 @@ namespace System.Windows.Forms
 
 				Edit ();
 				CalcAreasAndInvalidate ();
-			}
-			else if (back_button_active) {
+			} else if (back_button_active) {
 				if (back_button_rect.Contains (e.X, e.Y)) {
 					Invalidate (back_button_rect);
 					NavigateBack ();
 					OnBackButtonClicked (this, EventArgs.Empty);
 				}
 				back_button_active = false;
-			}
-			else if (parent_rows_button_active) {
+			} else if (parent_rows_button_active) {
 				if (parent_rows_button_rect.Contains (e.X, e.Y)) {
 					Invalidate (parent_rows_button_rect);
 					ParentRowsVisible = !ParentRowsVisible;
@@ -1710,8 +1694,7 @@ namespace System.Windows.Forms
 					/* left */
 					pixels = Math.Max (horiz_scrollbar.Minimum,
 							   horiz_scrollbar.Value - horiz_scrollbar.LargeChange);
-				}
-				else {
+				} else {
 					/* right */
 					pixels = Math.Min (horiz_scrollbar.Maximum - horiz_scrollbar.LargeChange + 1,
 							   horiz_scrollbar.Value + horiz_scrollbar.LargeChange);
@@ -1727,8 +1710,7 @@ namespace System.Windows.Forms
 					/* up */
 					pixels = Math.Max (vert_scrollbar.Minimum,
 							   vert_scrollbar.Value - vert_scrollbar.LargeChange);
-				}
-				else {
+				} else {
 					/* down */
 					pixels = Math.Min (vert_scrollbar.Maximum - vert_scrollbar.LargeChange + 1,
 							   vert_scrollbar.Value + vert_scrollbar.LargeChange);
@@ -1812,9 +1794,9 @@ namespace System.Windows.Forms
 			if (extend_selection) {
 				CancelEditing ();
 				ShiftSelection (CurrentRow);
-			}
-			else {
+			} else {
 				ResetSelection ();
+				selection_start = CurrentRow;
 			}
 		}
 
@@ -1857,8 +1839,7 @@ namespace System.Windows.Forms
 						CurrentColumn --;
 					else if ((CurrentRow > 0) && (CurrentColumn == 0))
 						CurrentCell = new DataGridCell (CurrentRow - 1, CurrentTableStyle.GridColumnStyles.Count - 1);
-				}
-				else {
+				} else {
 					if (CurrentColumn < CurrentTableStyle.GridColumnStyles.Count - 1)
 						CurrentColumn ++;
 					else if ((CurrentRow <= RowsCount) && (CurrentColumn == CurrentTableStyle.GridColumnStyles.Count - 1))
@@ -1872,8 +1853,7 @@ namespace System.Windows.Forms
 			case Keys.Right:
 				if (ctrl_pressed) {
 					CurrentColumn = CurrentTableStyle.GridColumnStyles.Count - 1;
-				}
-				else {
+				} else {
 					if (CurrentColumn < CurrentTableStyle.GridColumnStyles.Count - 1) {
 						CurrentColumn ++;
 					} else if (CurrentRow < RowsCount - 1
@@ -1890,8 +1870,7 @@ namespace System.Windows.Forms
 			case Keys.Left:
 				if (ctrl_pressed) {
 					CurrentColumn = 0;
-				}
-				else {
+				} else {
 					if (current_cell.ColumnNumber > 0)
 						CurrentColumn --;
 					else if (CurrentRow > 0)
@@ -1970,9 +1949,8 @@ namespace System.Windows.Forms
 				if (is_editing)
 					return false;
 				else if (selected_rows.Keys.Count > 0) {
-					foreach (int row in selected_rows.Keys) {
-						ListManager.RemoveAt (row);						
-					}
+					foreach (int row in selected_rows.Keys)
+						ListManager.RemoveAt (row);
 					selected_rows.Clear ();
 					CalcAreasAndInvalidate ();
 				}
@@ -1985,12 +1963,11 @@ namespace System.Windows.Forms
 
 		protected override bool ProcessKeyPreview (ref Message m)
 		{
-			if ((Msg)m.Msg == Msg.WM_KEYDOWN) {
+			if ((Msg) m.Msg == Msg.WM_KEYDOWN) {
 				Keys key = (Keys) m.WParam.ToInt32 ();
 				KeyEventArgs ke = new KeyEventArgs (key);
-				if (ProcessGridKey (ke) == true) {
+				if (ProcessGridKey (ke))
 					return true;
-				}
 			}
 
 			return base.ProcessKeyPreview (ref m);
@@ -2073,6 +2050,8 @@ namespace System.Windows.Forms
 
 		public void Select (int row)
 		{
+			EndEdit();
+
 			if (selected_rows.Count == 0)
 				selection_start = row;
 
@@ -2134,12 +2113,12 @@ namespace System.Windows.Forms
 
 		protected virtual bool ShouldSerializeParentRowsBackColor ()
 		{
-			return parentrowsback_color != def_parentrowsback_color;
+			return parent_rows_backcolor != def_parent_rows_backcolor;
 		}
 
 		protected virtual bool ShouldSerializeParentRowsForeColor ()
 		{
-			return parentrowsback_color != def_parentrowsback_color;
+			return parent_rows_backcolor != def_parent_rows_backcolor;
 		}
 
 		protected bool ShouldSerializePreferredRowHeight ()
@@ -2248,32 +2227,32 @@ namespace System.Windows.Forms
 
 		private void EnsureCellVisibility (DataGridCell cell)
 		{
-			if (cell.ColumnNumber <= first_visiblecolumn ||
-				cell.ColumnNumber + 1 >= first_visiblecolumn + visiblecolumn_count) {			
+			if (cell.ColumnNumber <= first_visible_column ||
+				cell.ColumnNumber + 1 >= first_visible_column + visible_column_count) {
 
-				first_visiblecolumn = GetFirstColumnForColumnVisibility (first_visiblecolumn, cell.ColumnNumber);
-                                int pixel = GetColumnStartingPixel (first_visiblecolumn);
+				first_visible_column = GetFirstColumnForColumnVisibility (first_visible_column, cell.ColumnNumber);
+				int pixel = GetColumnStartingPixel (first_visible_column);
 				ScrollToColumnInPixels (pixel);
 				horiz_scrollbar.Value = pixel;
 				Update();
 			}
 
-			if (cell.RowNumber < first_visiblerow ||
-			    cell.RowNumber + 1 >= first_visiblerow + visiblerow_count) {
+			if (cell.RowNumber < first_visible_row ||
+			    cell.RowNumber + 1 >= first_visible_row + visible_row_count) {
 
-                                if (cell.RowNumber + 1 >= first_visiblerow + visiblerow_count) {
-					int old_first_visiblerow = first_visiblerow;
-					first_visiblerow = 1 + cell.RowNumber - visiblerow_count;
+				if (cell.RowNumber + 1 >= first_visible_row + visible_row_count) {
+					int old_first_visible_row = first_visible_row;
+					first_visible_row = 1 + cell.RowNumber - visible_row_count;
 					UpdateVisibleRowCount ();
-					ScrollToRow (old_first_visiblerow, first_visiblerow);
+					ScrollToRow (old_first_visible_row, first_visible_row);
 				} else {
-					int old_first_visiblerow = first_visiblerow;
-					first_visiblerow = cell.RowNumber;
+					int old_first_visible_row = first_visible_row;
+					first_visible_row = cell.RowNumber;
 					UpdateVisibleRowCount ();
-					ScrollToRow (old_first_visiblerow, first_visiblerow);
+					ScrollToRow (old_first_visible_row, first_visible_row);
 				}
 
-				vert_scrollbar.Value = first_visiblerow;
+				vert_scrollbar.Value = first_visible_row;
 			}
 		}
 
@@ -2326,12 +2305,12 @@ namespace System.Windows.Forms
 			if (old_lm != list_manager) {
 				BindColumns ();
 
-				/* reset first_visiblerow to 0 here before
+				/* reset first_visible_row to 0 here before
 				 * doing anything that'll requires us to
 				 * figure out if we need a scrollbar. */
 				vert_scrollbar.Value = 0;
 				horiz_scrollbar.Value = 0;
-				first_visiblerow = 0;
+				first_visible_row = 0;
 
 				if (recreate_rows)
 					RecreateDataGridRows (false);
@@ -2372,9 +2351,8 @@ namespace System.Windows.Forms
 			if (start_index == -1)
 				return;
 
-			for (int i = start_index + 1; i < rows.Length; i ++) {
+			for (int i = start_index + 1; i < rows.Length; i ++)
 				rows[i].VerticalOffset = rows[i-1].VerticalOffset + rows[i-1].Height;
-			}
 
 			CalcAreasAndInvalidate ();
 		}
@@ -2385,20 +2363,17 @@ namespace System.Windows.Forms
 				string list_name = list_manager.GetListName (null);
 				if (TableStyles[list_name] == null) {
 					// no style exists by the supplied name
-					current_style.GridColumnStyles.Clear ();			
+					current_style.GridColumnStyles.Clear ();
 					current_style.CreateColumnsForTable (false);
-				}
-				else if (CurrentTableStyle == grid_style ||
+				} else if (CurrentTableStyle == grid_style ||
 					 CurrentTableStyle.MappingName != list_name) {
 					// If the style has been defined by the user, use it
 					CurrentTableStyle = styles_collection[list_name];
 					current_style.CreateColumnsForTable (true);
-				}
-				else {
+				} else {
 					current_style.CreateColumnsForTable (true);
 				}
-			}
-			else
+			} else
 				current_style.CreateColumnsForTable (false);
 		}
 
@@ -2425,8 +2400,7 @@ namespace System.Windows.Forms
 				ResetSelection ();
 				if (rows == null || RowsCount != rows.Length - (ShowEditRow ? 1 : 0))
 					RecreateDataGridRows (true);
-			}
-			else {
+			} else {
 				InvalidateRow (e.Index);
 			}
 		}
@@ -2437,44 +2411,35 @@ namespace System.Windows.Forms
 				return;
 			
 			string list_name = ListManager.GetListName (null);
-			switch (e.Action){
-				case CollectionChangeAction.Add: {
-					if (e.Element != null && String.Compare (list_name, ((DataGridTableStyle)e.Element).MappingName, true) == 0) {
-						CurrentTableStyle = (DataGridTableStyle)e.Element;
-						((DataGridTableStyle) e.Element).CreateColumnsForTable (false);
-					}
-					break;
+			switch (e.Action) {
+			case CollectionChangeAction.Add:
+				if (e.Element != null && String.Compare (list_name, ((DataGridTableStyle)e.Element).MappingName, true) == 0) {
+					CurrentTableStyle = (DataGridTableStyle)e.Element;
+					((DataGridTableStyle) e.Element).CreateColumnsForTable (false);
 				}
-
-				case CollectionChangeAction.Remove: {
-					if (e.Element != null && String.Compare (list_name, ((DataGridTableStyle)e.Element).MappingName, true) == 0) {
-						CurrentTableStyle = default_style;						
+				break;
+			case CollectionChangeAction.Remove:
+				if (e.Element != null && String.Compare (list_name, ((DataGridTableStyle)e.Element).MappingName, true) == 0) {
+					CurrentTableStyle = default_style;
+					current_style.GridColumnStyles.Clear ();
+					current_style.CreateColumnsForTable (false);
+				}
+				break;
+			case CollectionChangeAction.Refresh:
+				if (CurrentTableStyle == default_style
+					|| String.Compare (list_name, CurrentTableStyle.MappingName, true) != 0) {
+					DataGridTableStyle style = styles_collection [list_name];
+					if (style != null) {
+						CurrentTableStyle = style;
+						current_style.CreateColumnsForTable (false);
+					} else {
+						CurrentTableStyle = default_style;
 						current_style.GridColumnStyles.Clear ();
 						current_style.CreateColumnsForTable (false);
 					}
-					break;
-				}	
-
-				
-				case CollectionChangeAction.Refresh: {
-					if (CurrentTableStyle == default_style
-					    || String.Compare (list_name, CurrentTableStyle.MappingName, true) != 0) {
-
-						DataGridTableStyle style = styles_collection [list_name];
-						if (style != null) {
-							CurrentTableStyle = style;
-							current_style.CreateColumnsForTable (false);
-						}
-						else {
-							CurrentTableStyle = default_style;
-							current_style.GridColumnStyles.Clear ();
-							current_style.CreateColumnsForTable (false);
-						}
-					}
-
-					break;
 				}
-			}						
+				break;
+			}
 			CalcAreasAndInvalidate ();
 		}
 
@@ -2489,7 +2454,7 @@ namespace System.Windows.Forms
 			if (CurrentTableStyle.GridColumnStyles.Count == 0)
 				return;
 
-			if (CurrentTableStyle.GridColumnStyles[CurrentColumn].bound == false)
+			if (!CurrentTableStyle.GridColumnStyles[CurrentColumn].bound)
 				return;
 
 			is_editing = true;
@@ -2497,7 +2462,7 @@ namespace System.Windows.Forms
 
 			CurrentTableStyle.GridColumnStyles[CurrentColumn].Edit (ListManager,
 				CurrentRow, GetCellBounds (CurrentRow, CurrentColumn),
-				_readonly, "", true);
+				_readonly, null, true);
 		}
 
 		private void EndEdit ()
@@ -2505,12 +2470,11 @@ namespace System.Windows.Forms
 			if (CurrentTableStyle.GridColumnStyles.Count == 0)
 				return;
 
-			if (CurrentTableStyle.GridColumnStyles[current_cell.ColumnNumber].bound == false)
+			if (!CurrentTableStyle.GridColumnStyles[current_cell.ColumnNumber].bound)
 				return;
 
 			EndEdit (CurrentTableStyle.GridColumnStyles[current_cell.ColumnNumber],
-				 current_cell.RowNumber,
-				 false);
+				current_cell.RowNumber, false);
 		}
 
 		private void ShiftSelection (int index)
@@ -2526,17 +2490,15 @@ namespace System.Windows.Forms
 			if (index >= selection_start) {
 				start = selection_start;
 				end = index;
-			}
-			else {
+			} else {
 				start = index;
 				end = selection_start;
 			}
 
 			if (start == -1) start = 0;
 
-			for (int idx = start; idx <= end; idx ++) {
+			for (int idx = start; idx <= end; idx ++)
 				Select (idx);
-			}
 		}
 
 		private void ScrollToColumnInPixels (int pixel)
@@ -2550,7 +2512,7 @@ namespace System.Windows.Forms
 
 			Rectangle area = cells_area;
 
-			if (ColumnHeadersVisible == true) {
+			if (ColumnHeadersVisible) {
 				area.Y -= ColumnHeadersArea.Height;
 				area.Height += ColumnHeadersArea.Height;
 			}
@@ -2578,8 +2540,7 @@ namespace System.Windows.Forms
 			if (new_row > old_row) { // Scrolldown
 				for (i = old_row; i < new_row; i ++)
 					pixels -= rows[i].Height;
-			}
-			else {
+			} else {
 				for (i = new_row; i < old_row; i ++)
 					pixels += rows[i].Height;
 			}
@@ -2600,12 +2561,47 @@ namespace System.Windows.Forms
 			XplatUI.ScrollWindow (Handle, rows_area, 0, pixels, false);
 
 			/* if the row is still */
-			if (CurrentRow >= first_visiblerow && CurrentRow < first_visiblerow + visiblerow_count)
+			if (CurrentRow >= first_visible_row && CurrentRow < first_visible_row + visible_row_count)
 				Edit ();
 		}
 
-		#endregion Private Instance Methods
 
+		private void ColumnResize (int column) 
+		{
+			CurrencyManager source = this.ListManager;
+			DataGridColumnStyle style = CurrentTableStyle.GridColumnStyles[column];
+			string headerText = style.HeaderText;
+			using (Graphics g = base.CreateGraphics ()) {
+				int rows = source.Count;
+				int width = (int)g.MeasureString (headerText, CurrentTableStyle.HeaderFont).Width + 4;
+
+				for (int i = 0; i < rows; i++) {
+					int rowColWidth = (int)style.GetPreferredSize (g, style.GetColumnValueAtRow (source, i)).Width;
+					if (rowColWidth > width)
+						width = rowColWidth;
+				}
+				if (style.Width != width)
+					style.Width = width;
+			}
+		}
+
+		private void RowResize (int row)
+		{
+			CurrencyManager source = this.ListManager;
+			using (Graphics g = base.CreateGraphics ()) {
+				GridColumnStylesCollection columns = CurrentTableStyle.GridColumnStyles;
+				int colCount = columns.Count;
+				int rowCount = source.Count;
+				int height = 0;
+				for (int i = 0; i < colCount; i++) {
+					object val = columns[i].GetColumnValueAtRow (source, row);
+					height = Math.Max (columns[i].GetPreferredHeight (g, val), height);
+				}
+				if (this.DataGridRows[row].Height != height)
+					this.DataGridRows[row].Height = height;
+			}
+		}
+		#endregion Private Instance Methods
 
 		#region Events
 		static object AllowNavigationChangedEvent = new object ();
@@ -2733,9 +2729,9 @@ namespace System.Windows.Forms
 		int width_of_all_columns;
 
 		internal Rectangle caption_area;
-		internal Rectangle columnhdrs_area;	// Used columns header area
-		internal int columnhdrs_maxwidth; 	// Total width (max width) for columns headrs
-		internal Rectangle rowhdrs_area;	// Used Headers rows area
+		internal Rectangle column_headers_area;	// Used columns header area
+		internal int column_headers_max_width; 	// Total width (max width) for columns headrs
+		internal Rectangle row_headers_area;	// Used Headers rows area
 		internal Rectangle cells_area;
 		#endregion // Local Variables
 
@@ -2768,8 +2764,8 @@ namespace System.Windows.Forms
 				return 0;
 				
 			if (CurrentTableStyle.CurrentRowHeadersVisible) {
-				width += rowhdrs_area.X + rowhdrs_area.Width;
-				column_x += rowhdrs_area.X + rowhdrs_area.Width;
+				width += row_headers_area.X + row_headers_area.Width;
+				column_x += row_headers_area.X + row_headers_area.Width;
 			}
 
 			for (int col = 0; col < cnt; col++) {
@@ -2807,14 +2803,14 @@ namespace System.Windows.Forms
 		}
 		
 		// Which column has to be the first visible column to ensure a column visibility
-		int GetFirstColumnForColumnVisibility (int current_first_visiblecolumn, int column)
+		int GetFirstColumnForColumnVisibility (int current_first_visible_column, int column)
 		{
 			int new_col = column;
 			int width = 0;
 			
-			if (column > current_first_visiblecolumn) { // Going forward
-				for (new_col = column; new_col >= 0; new_col--){
-					if (CurrentTableStyle.GridColumnStyles[new_col].bound == false)
+			if (column > current_first_visible_column) { // Going forward
+				for (new_col = column; new_col >= 0; new_col--) {
+					if (!CurrentTableStyle.GridColumnStyles[new_col].bound)
 						continue;
 					width += CurrentTableStyle.GridColumnStyles[new_col].Width;
 					
@@ -2831,7 +2827,7 @@ namespace System.Windows.Forms
 		bool in_calc_grid_areas;
 		void CalcGridAreas ()
 		{
-			if (IsHandleCreated == false) // Delay calculations until the handle is created
+			if (!IsHandleCreated) // Delay calculations until the handle is created
 				return;
 
 			/* make sure we don't happen to end up in this method again */
@@ -2864,7 +2860,7 @@ namespace System.Windows.Forms
 
 			/* use a loop to iteratively calculate whether
 			 * we need horiz/vert scrollbars. */
-			for (int i = 0; i < 3; i ++) {
+			for (int i = 0; i < 3; i++) {
 				if (needVert)
 					visible_cells_width = cells_area.Width - vert_scrollbar.Width;
 				if (needHoriz)
@@ -2873,7 +2869,7 @@ namespace System.Windows.Forms
 				UpdateVisibleRowCount ();
 
 				needHoriz = (width_of_all_columns > visible_cells_width);
-				needVert = (allrows > visiblerow_count);
+				needVert = (allrows > visible_row_count);
 			}
 
 			int horiz_scrollbar_width = ClientRectangle.Width;
@@ -2895,8 +2891,8 @@ namespace System.Windows.Forms
 					parent_rows.Width -= vert_scrollbar.Width;
 
 				if (!ShowingColumnHeaders) {
-					if (columnhdrs_area.X + columnhdrs_area.Width > vert_scrollbar.Location.X) {
-						columnhdrs_area.Width -= vert_scrollbar.Width;
+					if (column_headers_area.X + column_headers_area.Width > vert_scrollbar.Location.X) {
+						column_headers_area.Width -= vert_scrollbar.Width;
 					}
 				}
 
@@ -2905,8 +2901,8 @@ namespace System.Windows.Forms
 			}
 
 			if (needVert) {
-				if (rowhdrs_area.Y + rowhdrs_area.Height > ClientRectangle.Y + ClientRectangle.Height) {
-					rowhdrs_area.Height -= horiz_scrollbar.Height;
+				if (row_headers_area.Y + row_headers_area.Height > ClientRectangle.Y + ClientRectangle.Height) {
+					row_headers_area.Height -= horiz_scrollbar.Height;
 				}
 
 				vert_scrollbar.Size = new Size (vert_scrollbar.Width,
@@ -2915,21 +2911,19 @@ namespace System.Windows.Forms
 				vert_scrollbar.Maximum = vert_scrollbar_maximum;
 				Controls.Add (vert_scrollbar);
 				vert_scrollbar.Visible = true;
-			}
-			else {
+			} else {
 				Controls.Remove (vert_scrollbar);
 				vert_scrollbar.Visible = false;
 			}
 
 			if (needHoriz) {
 				horiz_scrollbar.Size = new Size (horiz_scrollbar_width,
-								 horiz_scrollbar.Height);
+					horiz_scrollbar.Height);
 
 				horiz_scrollbar.Maximum = horiz_scrollbar_maximum;
 				Controls.Add (horiz_scrollbar);
 				horiz_scrollbar.Visible = true;
-			}
-			else {
+			} else {
 				Controls.Remove (horiz_scrollbar);
 				horiz_scrollbar.Visible = false;
 			}
@@ -2950,15 +2944,14 @@ namespace System.Windows.Forms
 				if (caption_area.Height < back_button_image.Height)
 					caption_area.Height = back_button_image.Height;
 				caption_area.Height += 2;
-			}
-			else
+			} else
 				caption_area.Height = 0;
 		}
 
 		void CalcCellsArea ()
 		{
-			cells_area.X = ClientRectangle.X + rowhdrs_area.Width;
-			cells_area.Y = columnhdrs_area.Y + columnhdrs_area.Height;
+			cells_area.X = ClientRectangle.X + row_headers_area.Width;
+			cells_area.Y = column_headers_area.Y + column_headers_area.Height;
 			cells_area.Width = ClientRectangle.X + ClientRectangle.Width - cells_area.X;
 			if (cells_area.Width < 0)
 				cells_area.Width = 0;
@@ -2971,29 +2964,29 @@ namespace System.Windows.Forms
 		{
 			int max_width_cols;
 
-			columnhdrs_area.X = ClientRectangle.X;
-			columnhdrs_area.Y = parent_rows.Y + parent_rows.Height;
+			column_headers_area.X = ClientRectangle.X;
+			column_headers_area.Y = parent_rows.Y + parent_rows.Height;
 
 			// TODO: take into account Scrollbars
-			columnhdrs_maxwidth = ClientRectangle.X + ClientRectangle.Width - columnhdrs_area.X;
-			max_width_cols = columnhdrs_maxwidth;
+			column_headers_max_width = ClientRectangle.X + ClientRectangle.Width - column_headers_area.X;
+			max_width_cols = column_headers_max_width;
 
 			if (CurrentTableStyle.CurrentRowHeadersVisible)
 				max_width_cols -= RowHeaderWidth;
 
 			if (width_of_all_columns > max_width_cols) {
-				columnhdrs_area.Width = columnhdrs_maxwidth;
+				column_headers_area.Width = column_headers_max_width;
 			} else {
-				columnhdrs_area.Width = width_of_all_columns;
+				column_headers_area.Width = width_of_all_columns;
 
 				if (CurrentTableStyle.CurrentRowHeadersVisible)
-					columnhdrs_area.Width += RowHeaderWidth;
+					column_headers_area.Width += RowHeaderWidth;
 			}
 
 			if (ShowingColumnHeaders)
-				columnhdrs_area.Height = CurrentTableStyle.HeaderFont.Height + 6;
+				column_headers_area.Height = CurrentTableStyle.HeaderFont.Height + 6;
 			else
-				columnhdrs_area.Height = 0;
+				column_headers_area.Height = 0;
 		}
 
 		void CalcParentRows ()
@@ -3002,36 +2995,35 @@ namespace System.Windows.Forms
 			parent_rows.Y = caption_area.Y + caption_area.Height;
 			parent_rows.Width = ClientRectangle.Width;
 			if (ShowParentRows)
-				parent_rows.Height = (CaptionFont.Height + 3) * dataSourceStack.Count;
+				parent_rows.Height = (CaptionFont.Height + 3) * data_source_stack.Count;
 			else
 				parent_rows.Height = 0;
 		}
 
 		void CalcParentButtons ()
 		{
-			if (dataSourceStack.Count > 0 && CaptionVisible) {
+			if (data_source_stack.Count > 0 && CaptionVisible) {
 				back_button_rect = new Rectangle (ClientRectangle.X + ClientRectangle.Width - 2 * (caption_area.Height - 2) - 8,
 								  caption_area.Height / 2 - back_button_image.Height / 2,
 								  back_button_image.Width, back_button_image.Height);
 				parent_rows_button_rect = new Rectangle (ClientRectangle.X + ClientRectangle.Width - (caption_area.Height - 2) - 4,
 									 caption_area.Height / 2 - parent_rows_button_image.Height / 2,
 									 parent_rows_button_image.Width, parent_rows_button_image.Height);
-			}
-			else {
+			} else {
 				back_button_rect = parent_rows_button_rect = Rectangle.Empty;
 			}
 		}
 
 		void CalcRowHeaders ()
 		{
-			rowhdrs_area.X = ClientRectangle.X;
-			rowhdrs_area.Y = columnhdrs_area.Y + columnhdrs_area.Height;
-			rowhdrs_area.Height = ClientRectangle.Height + ClientRectangle.Y - rowhdrs_area.Y;
+			row_headers_area.X = ClientRectangle.X;
+			row_headers_area.Y = column_headers_area.Y + column_headers_area.Height;
+			row_headers_area.Height = ClientRectangle.Height + ClientRectangle.Y - row_headers_area.Y;
 
 			if (CurrentTableStyle.CurrentRowHeadersVisible)
-				rowhdrs_area.Width = RowHeaderWidth;
+				row_headers_area.Width = RowHeaderWidth;
 			else
-				rowhdrs_area.Width = 0;
+				row_headers_area.Width = 0;
 		}
 
 		int GetVisibleRowCount (int visibleHeight)
@@ -3044,7 +3036,7 @@ namespace System.Windows.Forms
 				rows_height += rows[r].Height;
 			}
 
-			if (r < rows.Length - 1)
+			if (r <= rows.Length - 1)
 				r ++;
 
 			return r - FirstVisibleRow;
@@ -3053,34 +3045,34 @@ namespace System.Windows.Forms
 		void UpdateVisibleColumn ()
 		{
 			if (CurrentTableStyle.GridColumnStyles.Count == 0) {
-				visiblecolumn_count = 0;
-				return;	
+				visible_column_count = 0;
+				return;
 			}
 			
 			int col;
 			int max_pixel = horiz_pixeloffset + cells_area.Width;
 			int unused;
 
-			first_visiblecolumn = FromPixelToColumn (horiz_pixeloffset, out unused);
+			first_visible_column = FromPixelToColumn (horiz_pixeloffset, out unused);
 
 			col = FromPixelToColumn (max_pixel, out unused);
 			
-			visiblecolumn_count = 1 + col - first_visiblecolumn;
+			visible_column_count = 1 + col - first_visible_column;
 
-			visiblecolumn_count = 0;
-			for (int i = first_visiblecolumn; i <= col; i ++) {
+			visible_column_count = 0;
+			for (int i = first_visible_column; i <= col; i ++) {
 				if (CurrentTableStyle.GridColumnStyles[i].bound)
-					visiblecolumn_count++;
+					visible_column_count++;
 			}
 
-			if (first_visiblecolumn + visiblecolumn_count < CurrentTableStyle.GridColumnStyles.Count) { 
-				visiblecolumn_count++; // Partially visible column
+			if (first_visible_column + visible_column_count < CurrentTableStyle.GridColumnStyles.Count) { 
+				visible_column_count++; // Partially visible column
 			}
 		}
 
 		void UpdateVisibleRowCount ()
 		{
-			visiblerow_count = GetVisibleRowCount (cells_area.Height);
+			visible_row_count = GetVisibleRowCount (cells_area.Height);
 
 			CalcRowHeaders (); // Height depends on num of visible rows
 		}
@@ -3113,10 +3105,10 @@ namespace System.Windows.Forms
 		void InvalidateRowHeader (int row)
 		{
 			Rectangle rect_rowhdr = new Rectangle ();
-			rect_rowhdr.X = rowhdrs_area.X;
-			rect_rowhdr.Width = rowhdrs_area.Width;
+			rect_rowhdr.X = row_headers_area.X;
+			rect_rowhdr.Width = row_headers_area.Width;
 			rect_rowhdr.Height = rows[row].Height;
-			rect_rowhdr.Y = rowhdrs_area.Y + rows[row].VerticalOffset - rows[FirstVisibleRow].VerticalOffset;
+			rect_rowhdr.Y = row_headers_area.Y + rows[row].VerticalOffset - rows[FirstVisibleRow].VerticalOffset;
 			Invalidate (rect_rowhdr);
 		}
 
@@ -3177,7 +3169,7 @@ namespace System.Windows.Forms
 			maximum = RowsCount;
 			
 			if (ShowEditRow && RowsCount > 0) {
-				maximum++;	
+				maximum++;
 			}
 			
 			vert_scrollbar.LargeChange = VLargeChange;
@@ -3189,7 +3181,7 @@ namespace System.Windows.Forms
 		// Returns the ColumnHeaders area excluding the rectangle shared with RowHeaders
 		internal Rectangle ColumnHeadersArea {
 			get {
-				Rectangle columns_area = columnhdrs_area;
+				Rectangle columns_area = column_headers_area;
 
 				if (CurrentTableStyle.CurrentRowHeadersVisible) {
 					columns_area.X += RowHeaderWidth;
@@ -3204,7 +3196,7 @@ namespace System.Windows.Forms
 		}
 
 		internal Rectangle RowHeadersArea {
-			get { return rowhdrs_area; }
+			get { return row_headers_area; }
 		}
 
 		internal Rectangle ParentRowsArea {
