@@ -87,11 +87,11 @@ namespace System.Windows.Forms {
 		private bool			is_loaded;
 		internal bool			is_changing_visible_state;
 		internal bool			has_been_visible;
+		private bool			shown_raised;  // The shown event is only raised once
 
 #if NET_2_0
 		private MenuStrip		main_menu_strip;
 		private bool			show_icon = true;
-		private bool			shown_raised;  // The shown event is only raised once
 #endif
 		#endregion	// Local Variables
 
@@ -151,7 +151,7 @@ namespace System.Windows.Forms {
 			}
 		}
 		
-		private void UpdateSizeGripVisible ()
+		private new void UpdateSizeGripVisible ()
 		{
 			// Following link explains when to show size grip:
 			// http://forums.microsoft.com/MSDN/ShowPost.aspx?PostID=138687&SiteID=1
@@ -177,16 +177,50 @@ namespace System.Windows.Forms {
 					size_grip.Visible = false;
 			} else {
 				if (size_grip == null) {
-					size_grip = new SizeGrip ();
-					this.Controls.AddImplicit (size_grip);
+					size_grip = new SizeGrip (this);
+					size_grip.Virtual = true;
+					size_grip.FillBackground = false;
 				}
-				size_grip.Width = SystemInformation.VerticalScrollBarWidth;
-				size_grip.Height = SystemInformation.HorizontalScrollBarHeight;
-				size_grip.Location = new Point (ClientSize.Width - size_grip.Width, ClientSize.Height - size_grip.Height);
 				size_grip.Visible = true;
 			}
 		}
 		
+		internal void ChangingParent (Control new_parent)
+		{
+			if (IsMdiChild) {
+				return;
+			}
+			
+			bool recreate_necessary = false;
+			
+			if (new_parent == null) {
+				recreate_necessary = window_manager is FormWindowManager;
+				window_manager = null;
+			} else if (new_parent is MdiClient) {
+				window_manager = new MdiWindowManager (this, (MdiClient) new_parent);
+			} else {
+				window_manager = new FormWindowManager (this);
+				recreate_necessary = true;
+			}
+			
+			if (recreate_necessary && IsHandleCreated) {
+				RecreateHandle ();
+			}
+		
+			if (window_manager != null) {
+				window_manager.UpdateWindowState (window_state, window_state, true);
+			}
+		}
+
+		internal override bool FocusInternal (bool skip_check)
+		{
+			if (IsMdiChild) {
+				// MS always creates handles when Focus () is called for mdi clients.
+				if (!IsHandleCreated)
+					CreateHandle ();
+			} 
+			return base.FocusInternal (skip_check);
+		}
 		#endregion	// Private & Internal Methods
 
 		#region Public Classes
@@ -247,10 +281,7 @@ namespace System.Windows.Forms {
 			default_maximized_bounds = Rectangle.Empty;
 			owned_forms = new Form.ControlCollection(this);
 			transparency_key = Color.Empty;
-
-			// FIXME: this should disappear just as soon as the handle creation is done in the right place (here is too soon()
-			UpdateBounds();
-
+			InternalClientSize = new Size (this.Width - (SystemInformation.FrameBorderSize.Width * 2), this.Height - (SystemInformation.FrameBorderSize.Height * 2) - SystemInformation.CaptionHeight);
 		}
 		#endregion	// Public Constructor & Destructor
 
@@ -480,12 +511,13 @@ namespace System.Windows.Forms {
 					window_manager.UpdateBorderStyle (value);
 				}
 
+				Size current_client_size = ClientSize;
 				UpdateStyles();
 				
-				if (this.Visible) 
-					this.Size = SizeFromClientSize (this.ClientSize);
-				else
+				if (this.IsHandleCreated) {
+					this.Size = InternalSizeFromClientSize (current_client_size);
 					XplatUI.InvalidateNC (this.Handle);
+				}
 			}
 		}
 
@@ -611,9 +643,6 @@ namespace System.Windows.Forms {
 			set {
 				if (maximize_box != value) {
 					maximize_box = value;
-					if (IsHandleCreated) {
-						RecreateHandle();
-					}
 					UpdateStyles();
 				}
 			}
@@ -672,13 +701,15 @@ namespace System.Windows.Forms {
 
 				if (value != null) {
 					mdi_parent = value;
-					window_manager = new MdiWindowManager (this,
-							mdi_parent.MdiContainer);
+					if (window_manager == null) {
+						window_manager = new MdiWindowManager (this, mdi_parent.MdiContainer);
+					}
+					
 					mdi_parent.MdiContainer.Controls.Add (this);
 					mdi_parent.MdiContainer.Controls.SetChildIndex (this, 0);
-
-					RecreateHandle ();
-
+					
+					if (IsHandleCreated)
+						RecreateHandle ();
 				} else if (mdi_parent != null) {
 					mdi_parent = null;
 
@@ -686,8 +717,10 @@ namespace System.Windows.Forms {
 					window_manager = null;
 					FormBorderStyle = form_border_style;
 
-					RecreateHandle ();
+					if (IsHandleCreated)
+						RecreateHandle ();
 				}
+				is_toplevel = mdi_parent == null;
 			}
 		}
 
@@ -795,9 +828,6 @@ namespace System.Windows.Forms {
 			set {
 				if (minimize_box != value) {
 					minimize_box = value;
-					if (IsHandleCreated) {
-						RecreateHandle();
-					}
 					UpdateStyles();
 				}
 			}
@@ -903,9 +933,9 @@ namespace System.Windows.Forms {
 						owner.AddOwnedForm(this);
 					if (IsHandleCreated) {
 						if (owner != null && owner.IsHandleCreated) {
-							XplatUI.SetTopmost(this.window.Handle, owner.window.Handle, true);
+							XplatUI.SetOwner(this.window.Handle, owner.window.Handle);
 						} else {
-							XplatUI.SetTopmost(this.window.Handle, IntPtr.Zero, false);
+							XplatUI.SetOwner(this.window.Handle, IntPtr.Zero);
 						}
 					}
 				}
@@ -921,8 +951,10 @@ namespace System.Windows.Forms {
 					this.show_icon = value;
 					UpdateStyles ();
 					
-					XplatUI.SetIcon (this.Handle, value == true ? this.Icon : null);
-					XplatUI.InvalidateNC (this.Handle);
+					if (IsHandleCreated) {
+						XplatUI.SetIcon (this.Handle, value == true ? this.Icon : null);
+						XplatUI.InvalidateNC (this.Handle);
+					}
 				}
 			}
 		}			
@@ -975,32 +1007,7 @@ namespace System.Windows.Forms {
 			}
 
 			set {
-				if (start_position == FormStartPosition.WindowsDefaultLocation) {		// Only do this if it's not set yet
-					start_position = value;
-					if (IsHandleCreated) {
-						switch(start_position) {
-							case FormStartPosition.CenterParent: {
-								CenterToParent();
-								break;
-							}
-
-							case FormStartPosition.CenterScreen: {
-								CenterToScreen();
-								break;
-							}
-
-							case FormStartPosition.Manual: {
-								Left = CreateParams.X;
-								Top = CreateParams.Y;
-								break;
-							}
-
-							default: {
-								break;
-							}
-						}
-					}
-				}
+				start_position = value;
 			}
 		}
 
@@ -1050,7 +1057,7 @@ namespace System.Windows.Forms {
 				if (topmost != value) {
 					topmost = value;
 					if (IsHandleCreated)
-						XplatUI.SetTopmost(window.Handle, owner != null ? owner.window.Handle : IntPtr.Zero, value);
+						XplatUI.SetTopmost(window.Handle, value);
 				}
 			}
 		}
@@ -1066,7 +1073,7 @@ namespace System.Windows.Forms {
 
 				AllowTransparency = true;
 				UpdateStyles();
-				if ((XplatUI.SupportsTransparency () & TransparencySupport.Set) != 0)
+				if (IsHandleCreated && (XplatUI.SupportsTransparency () & TransparencySupport.Set) != 0)
 					XplatUI.SetWindowTransparency(Handle, Opacity, transparency_key);
 			}
 		}
@@ -1075,7 +1082,8 @@ namespace System.Windows.Forms {
 		[MWFCategory("Layout")]
 		public FormWindowState WindowState {
 			get {
-				if (IsHandleCreated) {
+				// Don't actually rely on the WM until we've been shown
+				if (IsHandleCreated && shown_raised) {
 
 					if (window_manager != null)
 						return window_manager.GetWindowState ();
@@ -1091,7 +1099,7 @@ namespace System.Windows.Forms {
 			set {
 				FormWindowState old_state = window_state;
 				window_state = value;
-				if (IsHandleCreated) {
+				if (IsHandleCreated && shown_raised) {
 
 					if (window_manager != null) {
 						window_manager.SetWindowState (old_state, value);
@@ -1118,19 +1126,45 @@ namespace System.Windows.Forms {
 				cp.Param = 0;
 				cp.Parent = IntPtr.Zero;
 				cp.menu = ActiveMenu;
+				cp.control = this;
 
-				if (start_position == FormStartPosition.WindowsDefaultLocation && !IsMdiChild) {
-					cp.X = unchecked((int)0x80000000);
-					cp.Y = unchecked((int)0x80000000);
-				} else {
+				if (((Parent != null || !TopLevel) && !IsMdiChild)) {
+					// Parented forms and non-toplevel forms always gets the specified location, no matter what
 					cp.X = Left;
 					cp.Y = Top;
+				} else {
+					switch (start_position) {
+					case FormStartPosition.Manual:
+						cp.X = Left;
+						cp.Y = Top;
+						break;
+					case FormStartPosition.CenterScreen:
+						if (IsMdiChild) {
+							cp.X = Math.Max ((MdiParent.mdi_container.ClientSize.Width - Width) / 2, 0);
+							cp.Y = Math.Max ((MdiParent.mdi_container.ClientSize.Height - Height) / 2, 0);
+						} else {
+							cp.X = Math.Max ((Screen.PrimaryScreen.WorkingArea.Width - Width) / 2, 0);
+							cp.Y = Math.Max ((Screen.PrimaryScreen.WorkingArea.Height - Height) / 2, 0);
+						}
+						break;
+					case FormStartPosition.CenterParent:
+					case FormStartPosition.WindowsDefaultBounds:
+					case FormStartPosition.WindowsDefaultLocation:
+						cp.X = int.MinValue;
+						cp.Y = int.MinValue;
+						break;
+					}
 				}
 				cp.Width = Width;
 				cp.Height = Height;
 
 				cp.Style = (int)(WindowStyles.WS_CLIPCHILDREN | WindowStyles.WS_CLIPSIBLINGS);
 
+				if (Parent != null) {
+					cp.Parent = Parent.Handle;
+					cp.Style |= (int) WindowStyles.WS_CHILD;
+				}
+
 				if (IsMdiChild) {
 					cp.Style |= (int)(WindowStyles.WS_CHILD | WindowStyles.WS_CAPTION);
 					if (Parent != null) {
@@ -1234,8 +1268,8 @@ namespace System.Windows.Forms {
 				if (HelpButton && !MaximizeBox && !MinimizeBox) {
 					cp.ExStyle |= (int)WindowExStyles.WS_EX_CONTEXTHELP;
 				}
-				
-				if (Visible)
+
+				if (VisibleInternal || this.IsRecreating)
 					cp.Style |= (int)WindowStyles.WS_VISIBLE;
 
 				if (opacity < 1.0 || TransparencyKey != Color.Empty) {
@@ -1246,6 +1280,19 @@ namespace System.Windows.Forms {
 					cp.Style |= (int)(WindowStyles.WS_DISABLED);
 				}
 
+				if (!ControlBox && Text == string.Empty) {
+					cp.WindowStyle &= ~WindowStyles.WS_DLGFRAME;
+				}
+				
+				// Fake the window styles for mdi, toolwindows and parented forms
+				if (cp.HasWindowManager) {
+					// Remove all styles but WS_VISIBLE.
+					cp.WindowStyle &= WindowStyles.WS_VISIBLE;
+					// Set styles that enables us to use the window manager.
+					cp.WindowStyle |= WindowStyles.WS_CHILD | WindowStyles.WS_CLIPCHILDREN | WindowStyles.WS_CLIPSIBLINGS;
+					cp.ExStyle = 0;
+				}
+				
 				return cp;
 			}
 		}
@@ -1278,6 +1325,16 @@ namespace System.Windows.Forms {
 				}
 			}
 		}
+		
+#if !NET_2_0
+		internal
+#else
+		[MonoTODO ("Implemented for Win32, needs X11 implementation")]
+		protected 
+#endif
+		virtual bool ShowWithoutActivation {
+			get { return false; }
+		}
 		#endregion	// Protected Instance Properties
 
 		#region Public Static Methods
@@ -1378,6 +1435,12 @@ namespace System.Windows.Forms {
 			if (owner == this)
 				throw new InvalidOperationException ("The 'owner' cannot be the form being shown.");
 
+			if (TopLevelControl != this) {
+				throw new InvalidOperationException ("Forms that are not top level"
+					+ " forms cannot be displayed as a modal dialog. Remove the"
+					+ " form from any parent form before calling Show.");
+			}
+
 			base.Show ();
 		}
 #endif
@@ -1391,34 +1454,48 @@ namespace System.Windows.Forms {
 			bool		confined;
 			IntPtr		capture_window;
 
-			owner = null;
+			Form owner_to_be = null;
+
+			if ((ownerWin32 == null) && (Application.MWFThread.Current.Context != null)) {
+				ownerWin32 = Application.MWFThread.Current.Context.MainForm;
+			}
 
 			if (ownerWin32 != null) {
 				Control c = Control.FromHandle (ownerWin32.Handle);
 				if (c != null)
-					owner = c.TopLevelControl as Form;
+					owner_to_be = c.TopLevelControl as Form;
 			}
 
-			if (owner == this) {
-				throw new InvalidOperationException("The 'ownerWin32' cannot be the form being shown.");
+			if (owner_to_be == this) {
+				throw new ArgumentException ("Forms cannot own themselves or their owners.", "owner");
 			}
 
 			if (is_modal) {
-				throw new InvalidOperationException("The form is already displayed as a modal dialog.");
+				throw new InvalidOperationException ("The form is already displayed as a modal dialog.");
 			}
 
 			if (Visible) {
-				throw new InvalidOperationException("Already visible forms cannot be displayed as a modal dialog. Set the Visible property to 'false' prior to calling Form.ShowDialog.");
+				throw new InvalidOperationException ("Forms that are already "
+					+ " visible cannot be displayed as a modal dialog. Set the"
+					+ " form's visible property to false before calling"
+					+ " ShowDialog.");
 			}
 
 			if (!Enabled) {
-				throw new InvalidOperationException("Cannot display a disabled form as modal dialog.");
+				throw new InvalidOperationException ("Forms that are not enabled"
+					+ " cannot be displayed as a modal dialog. Set the form's"
+					+ " enabled property to true before calling ShowDialog.");
 			}
 
 			if (TopLevelControl != this) {
-				throw new InvalidOperationException("Can only display TopLevel forms as modal dialog.");
+				throw new InvalidOperationException ("Forms that are not top level"
+					+ " forms cannot be displayed as a modal dialog. Remove the"
+					+ " form from any parent form before calling ShowDialog.");
 			}
 
+			if (owner_to_be != null)
+				owner = owner_to_be;
+				
 			#if broken
 			// Can't do this, will screw us in the modal loop
 			form_parent_window.Parent = this.owner;
@@ -1520,6 +1597,12 @@ namespace System.Windows.Forms {
 			int	w;
 			int	h;
 
+			// MS creates the handle here.
+			if (TopLevel) {
+				if (!IsHandleCreated)
+					CreateHandle ();
+			}
+			
 			if (Width > 0) {
 				w = Width;
 			} else {
@@ -1549,6 +1632,12 @@ namespace System.Windows.Forms {
 			int	w;
 			int	h;
 
+			// MS creates the handle here.
+			if (TopLevel) {
+				if (!IsHandleCreated)
+					CreateHandle ();
+			}
+			
 			if (Width > 0) {
 				w = Width;
 			} else {
@@ -1574,6 +1663,10 @@ namespace System.Windows.Forms {
 		protected override void CreateHandle() {
 			base.CreateHandle ();
 
+			if (!IsHandleCreated) {
+				return;
+			}
+			
 			Application.AddForm (this);
 			
 			UpdateBounds();
@@ -1590,17 +1683,21 @@ namespace System.Windows.Forms {
 			}
 
 			if ((owner != null) && (owner.IsHandleCreated)) {
-				XplatUI.SetTopmost(window.Handle, owner.window.Handle, true);
+				XplatUI.SetOwner(window.Handle, owner.window.Handle);
+			}
+
+			if (topmost) {
+				XplatUI.SetTopmost(window.Handle, topmost);
 			}
 
 			for (int i = 0; i < owned_forms.Count; i++) {
 				if (owned_forms[i].IsHandleCreated)
-					XplatUI.SetTopmost(owned_forms[i].window.Handle, window.Handle, true);
+					XplatUI.SetOwner(owned_forms[i].window.Handle, window.Handle);
 			}
 			
 			if (window_manager != null) {
 				if (window_state != FormWindowState.Normal) {
-					window_manager.SetWindowState (FormWindowState.Normal, window_state);
+					window_manager.SetWindowState ((FormWindowState) int.MaxValue, window_state);
 				}
 				XplatUI.RequestNCRecalc (window.Handle);
 			}
@@ -1659,10 +1756,6 @@ namespace System.Windows.Forms {
 			
 			// Send initial location
 			OnLocationChanged(EventArgs.Empty);
-
-			if (IsMdiContainer) {
-				mdi_container.LayoutMdi (MdiLayout.Cascade);
-			}
 		}
 
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
@@ -1775,16 +1868,16 @@ namespace System.Windows.Forms {
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
 		protected override void OnPaint (PaintEventArgs pevent) {
 			base.OnPaint (pevent);
+
+			if (size_grip != null) {
+				size_grip.HandlePaint (this, pevent);
+			}
 		}
 
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
 		protected override void OnResize(EventArgs e) {
 			base.OnResize(e);
 
-			
-			if (size_grip != null) {
-				size_grip.Location = new Point (ClientSize.Width - size_grip.Width, ClientSize.Height - size_grip.Height);
-			}
 		}
 
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
@@ -1923,6 +2016,12 @@ namespace System.Windows.Forms {
 		protected override void Select(bool directed, bool forward) {
 			Form	parent;
 
+
+			// MS causes the handle to be created here.
+			if (!IsHandleCreated)
+				if (!IsHandleCreated)
+					CreateHandle ();
+			
 			if (directed) {
 				base.SelectNextControl(null, forward, true, true, true);
 			}
@@ -1960,25 +2059,56 @@ namespace System.Windows.Forms {
 
 			clientsize_set = new Size(x, y);
 
-			if (XplatUI.CalculateWindowRect(ref ClientRect, cp.Style, cp.ExStyle, cp.menu, out WindowRect)) {
+			if (XplatUI.CalculateWindowRect(ref ClientRect, cp, cp.menu, out WindowRect)) {
 				SetBounds(bounds.X, bounds.Y, WindowRect.Width, WindowRect.Height, BoundsSpecified.Size);
 			}
 		}
 
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
-		protected override void SetVisibleCore(bool value) {
-			is_changing_visible_state = true;
-			has_been_visible = value || has_been_visible;
-			base.SetVisibleCore (value);
-			is_changing_visible_state = false;
+		protected override void SetVisibleCore(bool value)
+		{
+			if (IsMdiChild && !MdiParent.Visible) {
+				if (value != Visible) {
+					MdiWindowManager wm = (MdiWindowManager) window_manager;
+					wm.IsVisiblePending = value;
+					OnVisibleChanged (EventArgs.Empty);
+					return;
+				}
+			} else {
+				is_changing_visible_state = true;
+				has_been_visible = value || has_been_visible;
+				base.SetVisibleCore (value);
+				
+				if (value && WindowState != FormWindowState.Normal)
+					XplatUI.SendMessage (Handle, Msg.WM_SHOWWINDOW, (IntPtr)1, IntPtr.Zero);
+					
+				is_changing_visible_state = false;
+			}
+			
+			if (value && IsMdiContainer) {
+				Form [] children = MdiChildren;
+				for (int i = 0; i < children.Length; i++) {
+					Form child = children [i];
+					MdiWindowManager wm = (MdiWindowManager) child.window_manager;
+					if (!child.IsHandleCreated && wm.IsVisiblePending) {
+						wm.IsVisiblePending = false;
+						child.Visible = true;
+					}
+				}
+			}
+			
+			if (value && IsMdiChild){
+				PerformLayout ();
+				ThemeEngine.Current.ManagedWindowSetButtonLocations (window_manager);
+			}
 			
-#if NET_2_0
 			// Shown event is only called once, the first time the form is made visible
 			if (value && !shown_raised) {
+#if NET_2_0
 				this.OnShown (EventArgs.Empty);
+#endif
 				shown_raised = true;
 			}
-#endif
 		}
 
 		protected override void UpdateDefaultButton() {
@@ -1991,28 +2121,32 @@ namespace System.Windows.Forms {
 			Console.WriteLine(DateTime.Now.ToLongTimeString () + " Form {0} ({2}) received message {1}", window.Handle == IntPtr.Zero ? this.Text : XplatUI.Window(window.Handle), m.ToString (), Text);
 #endif
 
-			if (window_manager != null && window_manager.HandleMessage (ref m)) {
+			if (window_manager != null && window_manager.WndProc (ref m)) {
 				return;
 			}
 
 			switch((Msg)m.Msg) {
 				case Msg.WM_DESTROY: {
-					base.WndProc(ref m);
-					if (!RecreatingHandle) {
+					if (!RecreatingHandle)
 						this.closing = true;
-					}
-					return;
-				}
 
-				case Msg.WM_CLOSE_INTERNAL: {
-					DestroyHandle();
-					break;
+					base.WndProc(ref m);
+					return;
 				}
 
 				case Msg.WM_CLOSE: {
 					Form act = Form.ActiveForm;
+					// Don't close this form if there's another modal form visible.
 					if (act != null && act != this && act.Modal == true) {
-						return;
+						// Check if any of the parents up the tree is the modal form, 
+						// in which case we can still close this form.
+						Control current = this;
+						while (current != null && current.Parent != act) {
+							current = current.Parent;
+						}
+						if (current == null || current.Parent != act) {
+							return;
+						}
 					}
 
 					if (mdi_container != null) {
@@ -2048,6 +2182,9 @@ namespace System.Windows.Forms {
 						}
 					}
 
+					
+					mdi_parent = null;
+
 					return;
 				}
 
@@ -2104,7 +2241,25 @@ namespace System.Windows.Forms {
 				}
 
 				// Menu drawing
-				case Msg.WM_NCLBUTTONDOWN: {
+                case Msg.WM_NCHITTEST: {
+					if (XplatUI.IsEnabled (Handle) && ActiveMenu != null) {
+						int x = LowOrder ((int)m.LParam.ToInt32 ());
+						int y = HighOrder ((int)m.LParam.ToInt32 ());
+
+						XplatUI.ScreenToMenu (ActiveMenu.Wnd.window.Handle, ref x, ref y);
+
+						// If point is under menu return HTMENU, it prevents Win32 to return HTMOVE.
+						if ((x > 0) && (y > 0) && (x < ActiveMenu.Rect.Width) && (y < ActiveMenu.Rect.Height)) {
+							m.Result = new IntPtr ((int)HitTest.HTMENU);
+							return;
+						}
+					}
+
+					base.WndProc (ref m);
+					return;
+				}
+
+                case Msg.WM_NCLBUTTONDOWN: {
 					if (XplatUI.IsEnabled (Handle) && ActiveMenu != null) {
 						ActiveMenu.OnMouseDown(this, new MouseEventArgs (FromParamToMouseButtons ((int) m.WParam.ToInt32()), mouse_clicks, Control.MousePosition.X, Control.MousePosition.Y, 0));
 					}
@@ -2121,7 +2276,8 @@ namespace System.Windows.Forms {
 					base.WndProc(ref m);
 					return;
 				}
-				case Msg.WM_NCLBUTTONUP: {
+
+                case Msg.WM_NCLBUTTONUP: {
 					if (ActiveMaximizedMdiChild != null) {
 						ActiveMaximizedMdiChild.HandleMenuMouseUp (ActiveMenu,
 								LowOrder ((int)m.LParam.ToInt32 ()),
@@ -2201,11 +2357,14 @@ namespace System.Windows.Forms {
 
 				case Msg.WM_MOUSEMOVE: {
 					if (XplatUI.IsEnabled (Handle) && active_tracker != null) {
-						MouseEventArgs args;
+						MouseEventArgs args = new MouseEventArgs (
+													FromParamToMouseButtons ((int) m.WParam.ToInt32()),
+													mouse_clicks, 
+													Control.MousePosition.X, 
+													Control.MousePosition.Y, 
+													0);
 
-						args = new MouseEventArgs (FromParamToMouseButtons ((int) m.WParam.ToInt32()), 
-							mouse_clicks,  LowOrder ((int) m.LParam.ToInt32 ()), HighOrder ((int) m.LParam.ToInt32 ()),  0);
-						active_tracker.OnMotion(new MouseEventArgs (args.Button, args.Clicks, Control.MousePosition.X, Control.MousePosition.Y, args.Delta));
+						active_tracker.OnMotion (args);
 						break;
 					}
 					base.WndProc(ref m);
@@ -2216,14 +2375,27 @@ namespace System.Windows.Forms {
 				case Msg.WM_MBUTTONDOWN:
 				case Msg.WM_RBUTTONDOWN: {					
 					if (XplatUI.IsEnabled (Handle) && active_tracker != null) {
-						MouseEventArgs args;
+						MouseEventArgs args = new MouseEventArgs (
+													FromParamToMouseButtons ((int) m.WParam.ToInt32()),
+													mouse_clicks, 
+													Control.MousePosition.X, 
+													Control.MousePosition.Y, 
+													0);
 
-						args = new MouseEventArgs (FromParamToMouseButtons ((int) m.WParam.ToInt32()), 
-							mouse_clicks, LowOrder ((int) m.LParam.ToInt32 ()), HighOrder ((int) m.LParam.ToInt32 ()), 0);
-						active_tracker.OnMouseDown(new MouseEventArgs (args.Button, args.Clicks, Control.MousePosition.X, Control.MousePosition.Y, args.Delta));
+						if (!active_tracker.OnMouseDown (args)) {
+							Control control = GetRealChildAtPoint (Cursor.Position);
+							if (control != null) {
+								Point pt = control.PointToClient (Cursor.Position);
+								XplatUI.SendMessage (control.Handle, (Msg) m.Msg, m.WParam, MakeParam (pt.X, pt.Y));
+							}
+						}
+						
 						return;
 					}
-					base.WndProc(ref m);
+#if NET_2_0
+					ToolStripManager.FireAppClicked ();
+#endif
+					base.WndProc (ref m);
 					return;
 				}
 
@@ -2231,7 +2403,6 @@ namespace System.Windows.Forms {
 				case Msg.WM_MBUTTONUP:
 				case Msg.WM_RBUTTONUP: {
 					if (XplatUI.IsEnabled (Handle) && active_tracker != null) {
-						MouseEventArgs args;
 						MouseButtons mb = FromParamToMouseButtons ((int) m.WParam.ToInt32());
 						
 						// We add in the button that was released (not sent in WParam)
@@ -2246,9 +2417,15 @@ namespace System.Windows.Forms {
 								mb |= MouseButtons.Right;
 								break;
 						}
+
+						MouseEventArgs args = new MouseEventArgs (
+													mb,
+													mouse_clicks, 
+													Control.MousePosition.X, 
+													Control.MousePosition.Y, 
+													0);
 						
-						args = new MouseEventArgs (mb, mouse_clicks, LowOrder ((int) m.LParam.ToInt32 ()), HighOrder ((int) m.LParam.ToInt32 ()), 0);
-						active_tracker.OnMouseUp(new MouseEventArgs (args.Button, args.Clicks, Control.MousePosition.X, Control.MousePosition.Y, args.Delta));
+						active_tracker.OnMouseUp (args);
 						mouse_clicks = 1;
 						return;
 					}
@@ -2346,6 +2523,8 @@ namespace System.Windows.Forms {
 				}
 			}
 		}
+
+		internal override bool ActivateOnShow { get { return !this.ShowWithoutActivation; } }
 		
 		#region Events
 		static object ActivatedEvent = new object ();
@@ -2448,7 +2627,6 @@ namespace System.Windows.Forms {
 			add { base.TabIndexChanged += value; }
 			remove { base.TabIndexChanged -= value; }
 		}
-		#endregion	// Events
 
 #if NET_2_0
 		[SettingsBindable (true)]
@@ -2622,5 +2800,6 @@ namespace System.Windows.Forms {
 				eh (this, e);
 		}
 #endif
+		#endregion	// Events
 	}
 }
