@@ -189,7 +189,7 @@ namespace System.Windows.Forms {
 		{
 			base.OnResize (e);
 
-			if (Parent != null)
+			if (Parent != null && Parent.IsHandleCreated)
 				XplatUI.InvalidateNC (Parent.Handle);
 			// Should probably make this into one loop
 			SizeScrollBars ();
@@ -355,6 +355,9 @@ namespace System.Windows.Forms {
 		{
 			if (lock_sizing)
 				return;
+			
+			if (!IsHandleCreated)
+				return;
 
 			if (Controls.Count == 0 || ((Form) Controls [0]).WindowState == FormWindowState.Maximized) {
 				if (hbar != null)
@@ -429,13 +432,10 @@ namespace System.Windows.Forms {
 
 			if (need_hbar && need_vbar) {
 				if (sizegrip == null) {
-					sizegrip = new SizeGrip ();
-					sizegrip.CapturedControl = this.ParentForm;
+					sizegrip = new SizeGrip (this.ParentForm);
 					Controls.AddImplicit (sizegrip);
 				}
 				sizegrip.Location = new Point (hbar.Right, vbar.Bottom);
-				sizegrip.Width = vbar.Width;
-				sizegrip.Height = hbar.Height;
 				sizegrip.Visible = true;
 				XplatUI.SetZOrder (sizegrip.Handle, vbar.Handle, false, false);
 			} else if (sizegrip != null) {
@@ -525,6 +525,9 @@ namespace System.Windows.Forms {
 
 		private void ArrangeWindows ()
 		{
+			if (!IsHandleCreated)
+				return;
+				
 			int change = 0;
 			if (prev_bottom != -1)
 				change = Bottom - prev_bottom;
@@ -550,10 +553,7 @@ namespace System.Windows.Forms {
 
 		internal void ArrangeIconicWindows (bool rearrange_all)
 		{
-			int xspacing = 160;
-			int yspacing = 27;
-
-			Rectangle rect = new Rectangle (0, 0, xspacing, yspacing);
+			Rectangle rect = Rectangle.Empty;
 
 			lock_sizing = true;
 			foreach (Form form in Controls) {
@@ -568,24 +568,19 @@ namespace System.Windows.Forms {
 					continue;
 				}
 				
-				// Need to get the width in the loop cause some themes might have
-				// different widths for different styles
-				int bw = ThemeEngine.Current.ManagedWindowBorderWidth (wm);
-				
-				int height = wm.TitleBarHeight + (bw * 2);
-				
 				bool success = true;
 				int startx, starty, currentx, currenty;
+
+				rect.Size = wm.IconicSize;
 				
 				startx = 0;
-				starty = ClientSize.Height - yspacing;
+				starty = ClientSize.Height - rect.Height;
 				currentx = startx;
 				currenty = starty;
 				
 				do {
 					rect.X = currentx;
 					rect.Y = currenty;
-					rect.Height = height;
 					success = true;
 					foreach (Form form2 in Controls) {
 						if (form2 == form || form2.window_state != FormWindowState.Minimized)
@@ -597,10 +592,10 @@ namespace System.Windows.Forms {
 						}
 					}
 					if (!success) {	
-						currentx += xspacing;
-						if (currentx + xspacing > Right) {
+						currentx += rect.Width;
+						if (currentx + rect.Width > Right) {
 							currentx = startx;
-							currenty -= Math.Max(yspacing, height);
+							currenty -= rect.Height;
 						} 
 					}
 				} while (!success);
@@ -610,7 +605,7 @@ namespace System.Windows.Forms {
 			lock_sizing = false;
 		}
 
-		internal void CloseChildForm (Form form)
+		internal void ChildFormClosed (Form form)
 		{
 			if (Controls.Count > 1) {
 				Form next = (Form) Controls [1];
@@ -620,7 +615,6 @@ namespace System.Windows.Forms {
 			}
 
 			Controls.Remove (form);
-			form.Close ();
 
 			if (Controls.Count == 0) {
 				XplatUI.RequestNCRecalc (Parent.Handle);
@@ -684,8 +678,10 @@ namespace System.Windows.Forms {
 			SetWindowStates (wm);
 			if (current != form) {
 				form.has_focus = false;
-				XplatUI.InvalidateNC (current.Handle);
-				XplatUI.InvalidateNC (form.Handle);
+				if (current.IsHandleCreated)
+					XplatUI.InvalidateNC (current.Handle);
+				if (form.IsHandleCreated)
+					XplatUI.InvalidateNC (form.Handle);
 			}
 			active_child = (Form) Controls [0];
 			
@@ -729,7 +725,7 @@ namespace System.Windows.Forms {
 			if (!form.Visible)
 				return false;
 			
-			bool is_active = wm.IsActive();
+			bool is_active = wm.IsActive;
 			bool maximize_this = false;
 			
 			if (!is_active){
@@ -780,6 +776,27 @@ namespace System.Windows.Forms {
 #if NET_2_0
 			if (form.MdiParent.MainMenuStrip != null)
 				form.MdiParent.MainMenuStrip.RefreshMdiItems ();
+
+			// Implicit menu strip merging
+			// - When child is activated
+			// - Parent form must have a MainMenuStrip
+			// - Find the first menustrip on the child
+			// - Merge
+			MenuStrip parent_menu = form.MdiParent.MainMenuStrip;
+
+			if (parent_menu != null) {
+				if (parent_menu.IsCurrentlyMerged)
+					ToolStripManager.RevertMerge (parent_menu);
+					
+				MenuStrip child_menu = null;
+
+				foreach (Control c in form.Controls)
+					if (c is MenuStrip)
+						child_menu = (MenuStrip)c;
+
+				if (child_menu != null)
+					ToolStripManager.Merge (child_menu, parent_menu);
+			}
 #endif
 
 			return maximize_this;
@@ -791,7 +808,7 @@ namespace System.Windows.Forms {
 
 			MdiWindowManager wm = (MdiWindowManager) form.window_manager;
 
-			if (!is_activating_child && new_window_state == FormWindowState.Maximized && !wm.IsActive ()) {
+			if (!is_activating_child && new_window_state == FormWindowState.Maximized && !wm.IsActive) {
 				ActivateChild (form);
 				return;
 			}
@@ -826,8 +843,8 @@ namespace System.Windows.Forms {
 			if (mdiclient_layout)
 				Parent.PerformLayout ();
 
-			//XplatUI.RequestNCRecalc (Parent.Handle);
-			//XplatUI.RequestNCRecalc (form.Handle);
+			XplatUI.RequestNCRecalc (Parent.Handle);
+			XplatUI.RequestNCRecalc (form.Handle);
 			if (!setting_windowstates)
 				SizeScrollBars ();
 		}
