@@ -25,7 +25,6 @@
 //
 //
 
-
 using System.Drawing;
 using System.ComponentModel;
 using System.Drawing.Imaging;
@@ -51,19 +50,28 @@ namespace System.Windows.Forms
 		private int minimum;
 		internal int step;
 		internal int val;
+		internal DateTime start = DateTime.Now;
 		internal Rectangle client_area = new Rectangle ();
 #if NET_2_0
 		internal ProgressBarStyle style;
-#endif		
+		Timer marquee_timer;
+#endif
+
+#if NET_2_0
+		private static readonly Color defaultForeColor = SystemColors.Highlight;
+#endif
 		#endregion	// Local Variables
 
 		#region events
+
+#if ONLY_1_1
 		[Browsable (false)]
-		[EditorBrowsable (EditorBrowsableState.Never)]		
+		[EditorBrowsable (EditorBrowsableState.Never)]
 		public new event EventHandler BackColorChanged {
 			add { base.BackColorChanged += value; }
 			remove { base.BackColorChanged -= value; }
 		}
+#endif
 		
 		[Browsable (false)]
 		[EditorBrowsable (EditorBrowsableState.Never)]
@@ -100,12 +108,14 @@ namespace System.Windows.Forms
 			remove { base.FontChanged -= value; }
 		}
 		
+#if ONLY_1_1
 		[Browsable (false)]
 		[EditorBrowsable (EditorBrowsableState.Never)]
 		public new event EventHandler ForeColorChanged {
 			add { base.ForeColorChanged += value; }
 			remove { base.ForeColorChanged -= value; }
 		}
+#endif
 		
 		[Browsable (false)]
 		[EditorBrowsable (EditorBrowsableState.Never)]
@@ -149,12 +159,14 @@ namespace System.Windows.Forms
 			remove { base.Paint -= value; }
 		}
 		
+#if ONLY_1_1
 		[Browsable (false)]
 		[EditorBrowsable (EditorBrowsableState.Never)]
 		public new event EventHandler RightToLeftChanged {
 			add { base.RightToLeftChanged += value; }
 			remove { base.RightToLeftChanged -= value; }
 		}
+#endif
 		
 		[Browsable (false)]
 		[EditorBrowsable (EditorBrowsableState.Never)]
@@ -189,6 +201,10 @@ namespace System.Windows.Forms
 				| ControlStyles.UseTextForAccessibility
 #endif
 				, false);
+
+#if NET_2_0
+			ForeColor = defaultForeColor;
+#endif
 		}
 		#endregion	// Public Constructors
 
@@ -253,10 +269,11 @@ namespace System.Windows.Forms
 		[EditorBrowsable (EditorBrowsableState.Never)]
 		public override Font Font
 		{
-			get { return base.Font;	}
+			get { return base.Font; }
 			set { base.Font = value; }
 		}
 
+#if ONLY_1_1
 		// Setting this property in MS .Net 1.1 does not have any visual effect and it
 		// does not fire a FontChanged event
 		[Browsable (false)]
@@ -266,6 +283,7 @@ namespace System.Windows.Forms
 			get { return base.ForeColor; }
 			set { base.ForeColor = value; }
 		}
+#endif
 
 		[Browsable (false)]
 		[EditorBrowsable (EditorBrowsableState.Never)]
@@ -308,6 +326,7 @@ namespace System.Windows.Forms
 			}
 		}
 
+#if ONLY_1_1
 		[Browsable (false)]
 		[EditorBrowsable (EditorBrowsableState.Never)]
 		public override RightToLeft RightToLeft
@@ -315,6 +334,7 @@ namespace System.Windows.Forms
 			get { return base.RightToLeft; }
 			set { base.RightToLeft = value; }
 		}
+#endif
 
 		[DefaultValue (10)]
 		public int Step
@@ -327,11 +347,6 @@ namespace System.Windows.Forms
 		}
 
 #if NET_2_0
-		[MonoTODO("Currently only implements the 1.1 behavior")]
-		//
-		// This means that we must implement Blocks, Continuous and Marquee
-		// rendering modes
-		//
 		[Browsable (true)]
 		[DefaultValue (ProgressBarStyle.Blocks)]
 		[EditorBrowsable (EditorBrowsableState.Always)]
@@ -342,11 +357,28 @@ namespace System.Windows.Forms
 
 			set {
 				style = value;
+				
+				if (style == ProgressBarStyle.Marquee) {
+					if (marquee_timer == null) {
+						marquee_timer = new Timer ();
+						marquee_timer.Interval = 10;
+						marquee_timer.Tick += new EventHandler (marquee_timer_Tick);
+					}
+					marquee_timer.Start ();
+				} else {
+					if (marquee_timer != null) {
+						marquee_timer.Stop ();
+					}
+				}
 			}
 		}
+
+		void marquee_timer_Tick (object sender, EventArgs e)
+		{
+			Invalidate ();
+		}
 		
-		int marquee_animation_speed = 350;
-		[MonoTODO("Currently does nothing")]
+		int marquee_animation_speed = 100;
 		[DefaultValue (100)]
 		public int MarqueeAnimationSpeed {
 			get {
@@ -385,9 +417,11 @@ namespace System.Windows.Forms
 			}
 			set {
 				if (value < Minimum || value > Maximum)
-					throw new ArgumentException(
-						string.Format("'{0}' is not a valid value for 'Value'. 'Value' should be between 'Minimum' and 'Maximum'", value));
-
+#if NET_2_0
+					throw new ArgumentOutOfRangeException ("Value", string.Format("'{0}' is not a valid value for 'Value'. 'Value' should be between 'Minimum' and 'Maximum'", value));
+#else
+					throw new ArgumentException(string.Format("'{0}' is not a valid value for 'Value'. 'Value' should be between 'Minimum' and 'Maximum'", value));
+#endif
 				val = value;
 				Refresh ();
 			}
@@ -433,6 +467,13 @@ namespace System.Windows.Forms
 			Refresh ();	// FIXME - calculate delta and only expose that
 		}
 
+#if NET_2_0
+		public override void ResetForeColor ()
+		{
+			ForeColor = defaultForeColor;
+		}
+#endif
+
 		public override string ToString()
 		{
 			return string.Format ("{0}, Minimum: {1}, Maximum: {2}, Value: {3}",
@@ -445,6 +486,7 @@ namespace System.Windows.Forms
 		#endregion	// Public Instance Methods
 
 		#region Private Instance Methods
+
 		private void UpdateAreas ()
 		{
 			client_area.X = client_area.Y = 2;
@@ -453,18 +495,18 @@ namespace System.Windows.Forms
 		}
 
 		private void OnResizeTB (Object o, EventArgs e)
-    		{
-    			if (Width <= 0 || Height <= 0)
-    				return;
+		{
+			if (Width <= 0 || Height <= 0)
+				return;
 
 			UpdateAreas ();
 			Invalidate();	// Invalidate the full surface, blocks will not match
-    		}
+		}
 
 		internal override void OnPaintInternal (PaintEventArgs pevent)
 		{
-                        ThemeEngine.Current.DrawProgressBar (pevent.Graphics, pevent.ClipRectangle, this);
-		}		
+			ThemeEngine.Current.DrawProgressBar (pevent.Graphics, pevent.ClipRectangle, this);
+		}
 
 		#endregion
 	}
