@@ -27,6 +27,8 @@
 
 namespace System.Windows.Forms 
 {
+#if !NET_2_0
 	[Serializable]
+#endif
 	public delegate void SelectedGridItemChangedEventHandler(object sender, SelectedGridItemChangedEventArgs e);
 }
