@@ -48,6 +48,7 @@ namespace System.Windows.Forms {
 		private MenuItem	paste;
 		private MenuItem	delete;
 		private MenuItem	select_all;
+
 #if NET_2_0
 		private bool use_system_password_char = false;
 		private AutoCompleteStringCollection auto_complete_custom_source = null;
@@ -62,8 +63,10 @@ namespace System.Windows.Forms {
 			scrollbars = RichTextBoxScrollBars.None;
 			alignment = HorizontalAlignment.Left;
 			this.LostFocus +=new EventHandler(TextBox_LostFocus);
-			this.BackColor = ThemeEngine.Current.ColorWindow;
-			this.ForeColor = ThemeEngine.Current.ColorWindowText;
+
+			BackColor = SystemColors.Window;
+			ForeColor = SystemColors.WindowText;
+			backcolor_set = false;
 
 			SetStyle (ControlStyles.StandardClick | ControlStyles.StandardDoubleClick, false);
 			SetStyle (ControlStyles.FixedHeight, true);
@@ -85,14 +88,31 @@ namespace System.Windows.Forms {
 			paste.Click += new EventHandler(paste_Click);
 			delete.Click += new EventHandler(delete_Click);
 			select_all.Click += new EventHandler(select_all_Click);
+
+			document.multiline = false;
 		}
 		#endregion	// Public Constructors
 
-
 		#region Private & Internal Methods
 		private void TextBox_LostFocus(object sender, EventArgs e) {
-			Invalidate();
+			if (hide_selection)
+				document.InvalidateSelectionArea ();
 		}
+
+		internal override Color ChangeBackColor (Color backColor)
+		{
+			if (backColor == Color.Empty) {
+#if NET_2_0
+				if (!ReadOnly)
+					backColor = SystemColors.Window;
+#else
+				backColor = SystemColors.Window;
+#endif
+				backcolor_set = false;
+			}
+			return backColor;
+		}
+
 #if NET_2_0
 		void OnAutoCompleteCustomSourceChanged(object sender, CollectionChangeEventArgs e) {
 			if(auto_complete_source == AutoCompleteSource.CustomSource) {
@@ -221,7 +241,7 @@ namespace System.Windows.Forms {
 			set {
 				if (value != password_char) {
 					password_char = value;
-					if (!multiline) {
+					if (!Multiline) {
 						document.PasswordChar = value.ToString();
 					} else {
 						document.PasswordChar = "";
@@ -240,6 +260,10 @@ namespace System.Windows.Forms {
 			}
 
 			set {
+				if (!Enum.IsDefined (typeof (ScrollBars), value))
+					throw new InvalidEnumArgumentException ("value", (int) value,
+						typeof (ScrollBars));
+
 				if (value != (ScrollBars)scrollbars) {
 					scrollbars = (RichTextBoxScrollBars)value;
 					base.CalculateScrollBars();
@@ -281,8 +305,10 @@ namespace System.Windows.Forms {
 				if (value != alignment) {
 					alignment = value;
 
+					document.alignment = value;
+
 					// MS word-wraps if alignment isn't left
-					if (multiline) {
+					if (Multiline) {
 						if (alignment != HorizontalAlignment.Left) {
 							document.Wrap = true;
 						} else {
@@ -328,11 +354,13 @@ namespace System.Windows.Forms {
 
 		protected override void OnGotFocus(EventArgs e) {
 			base.OnGotFocus (e);
+			if (selection_length == -1 && !has_been_focused)
+				SelectAllNoScroll ();
+			has_been_focused = true;
 		}
 
 		protected override void OnHandleCreated(EventArgs e) {
 			base.OnHandleCreated (e);
-			SelectAll ();
 		}
 
 		protected override void OnMouseUp(MouseEventArgs e) {
@@ -348,7 +376,10 @@ namespace System.Windows.Forms {
 		protected override void WndProc(ref Message m) {
 			switch ((Msg)m.Msg) {
 				case Msg.WM_LBUTTONDOWN:
-					FocusInternal ();
+					// When the textbox gets focus by LBUTTON (but not by middle or right)
+					// it does not do the select all / scroll thing.
+					has_been_focused = true;
+					FocusInternal (true);
 					break;
 			}
 
