@@ -50,7 +50,6 @@ namespace System.Windows.Forms {
 		internal bool			hide_selection;
 		internal int			max_length;
 		internal bool			modified;
-		internal bool			multiline;
 		internal char			password_char;
 		internal bool			read_only;
 		internal bool			word_wrap;
@@ -63,6 +62,9 @@ namespace System.Windows.Forms {
 		internal Timer			scroll_timer;
 		internal bool			richtext;
 		internal bool			show_selection;		// set to true to always show selection, even if no focus is set
+		
+		internal bool has_been_focused;
+
 		internal int			selection_length = -1;	// set to the user-specified selection length, or -1 if none
 		internal bool show_caret_w_selection;  // TextBox shows the caret when the selection is visible
 		internal int			requested_height;
@@ -75,6 +77,8 @@ namespace System.Windows.Forms {
 		internal int 			click_point_y;
 		internal CaretSelection		click_mode;
 		internal Bitmap			bmp;
+		internal BorderStyle actual_border_style;
+		internal bool shortcuts_enabled = true;
 		#if Debug
 		internal static bool	draw_lines = false;
 		#endif
@@ -89,11 +93,11 @@ namespace System.Windows.Forms {
 			accepts_tab = false;
 			auto_size = true;
 			border_style = BorderStyle.Fixed3D;
+			actual_border_style = BorderStyle.Fixed3D;
 			character_casing = CharacterCasing.Normal;
 			hide_selection = true;
 			max_length = 32767;
 			modified = true;
-			multiline = false;
 			password_char = '\0';
 			read_only = false;
 			word_wrap = true;
@@ -236,12 +240,8 @@ namespace System.Windows.Forms {
 				return base.BackColor;
 			}
 			set {
-				if (value != ThemeEngine.Current.ColorWindow) {
-					backcolor_set = true;
-				} else {
-					backcolor_set = false;
-				}
-				base.BackColor = value;
+				backcolor_set = true;
+				base.BackColor = ChangeBackColor (value);
 			}
 		}
 
@@ -260,8 +260,20 @@ namespace System.Windows.Forms {
 		[DispId(-504)]
 		[MWFCategory("Appearance")]
 		public BorderStyle BorderStyle {
-			get { return InternalBorderStyle; }
-			set { 
+			get { return actual_border_style; }
+			set {
+				if (value == actual_border_style)
+					return;
+
+				if (actual_border_style != BorderStyle.Fixed3D || value != BorderStyle.Fixed3D)
+					Invalidate ();
+
+				actual_border_style = value;
+				document.UpdateMargins ();
+
+				if (value != BorderStyle.Fixed3D)
+					value = BorderStyle.None;
+
 				InternalBorderStyle = value; 
 				OnBorderStyleChanged(EventArgs.Empty);
 			}
@@ -332,8 +344,8 @@ namespace System.Windows.Forms {
 
 					do {
 						line = document.GetLine (i++);
-						lt.Append (line.text.ToString ());
-					} while (line.soft_break && i <= count);
+						lt.Append (line.TextWithoutEnding ());
+					} while (line.ending == LineEnding.Wrap && i <= count);
 
 					lines.Add (lt.ToString ());	
 				}
@@ -355,21 +367,15 @@ namespace System.Windows.Forms {
 				for (i = 0; i < l; i++) {
 
 					// Don't add the last line if it is just an empty line feed
-					// the line feed is reflected in the previous line's soft_break = false
+					// the line feed is reflected in the previous line's ending 
 					if (i == l - 1 && value [i].Length == 0)
 						break;
 
-					bool carriage_return = false;
-					if (value [i].EndsWith ("\r")) {
-						value [i] = value [i].Substring (0, value [i].Length - 1);
-						carriage_return = true;
-					}
+					LineEnding ending = LineEnding.Rich;
+					if (value [i].EndsWith ("\r"))
+						ending = LineEnding.Hard;
 
-					document.Add(i+1, CaseAdjust(value[i]), alignment, Font, brush);
-					if (carriage_return) {
-						Line line = document.GetLine (i + 1);
-						line.carriage_return = true;
-					}
+					document.Add (i + 1, CaseAdjust (value [i]), alignment, Font, brush, ending);
 				}
 
 				document.ResumeRecalc (true);
@@ -418,14 +424,14 @@ namespace System.Windows.Forms {
 		[MWFCategory("Behavior")]
 		public virtual bool Multiline {
 			get {
-				return multiline;
+				return document.multiline;
 			}
 
 			set {
-				if (value != multiline) {
-					multiline = value;
+				if (value != document.multiline) {
+					document.multiline = value;
 					// Make sure we update our size; the user may have already set the size before going to multiline
-					if (multiline && requested_height != -1) {
+					if (document.multiline && requested_height != -1) {
 						Height = requested_height;
 						requested_height = -1;
 					}
@@ -436,9 +442,7 @@ namespace System.Windows.Forms {
 					OnMultilineChanged(EventArgs.Empty);
 				}
 
-				document.multiline = multiline;
-
-				if (multiline) {
+				if (document.multiline) {
 					document.Wrap = word_wrap;
 					document.PasswordChar = "";
 
@@ -451,7 +455,8 @@ namespace System.Windows.Forms {
 					}
 				}
 
-				CalculateDocument ();
+				if (IsHandleCreated)
+					CalculateDocument ();
 			}
 		}
 
@@ -460,7 +465,7 @@ namespace System.Windows.Forms {
 		[EditorBrowsable(EditorBrowsableState.Advanced)]
 		public int PreferredHeight {
 			get {
-				return Font.Height + (BorderStyle == BorderStyle.None ? 0 : 7);
+				return Font.Height + (BorderStyle != BorderStyle.Fixed3D ? 0 : 7);
 			}
 		}
 
@@ -474,7 +479,16 @@ namespace System.Windows.Forms {
 			set {
 				if (value != read_only) {
 					read_only = value;
+#if NET_2_0
+					if (!backcolor_set) {
+						if (read_only)
+							background_color = SystemColors.Control;
+						else
+							background_color = SystemColors.Window;
+					}
+#endif
 					OnReadOnlyChanged(EventArgs.Empty);
+					Invalidate ();
 				}
 			}
 		}
@@ -520,12 +534,12 @@ namespace System.Windows.Forms {
 					start = document.LineTagToCharIndex(document.selection_start.line, document.selection_start.pos);
 
 					document.CharIndexToLineTag(start + value, out line, out tag, out pos);
-					document.SetSelectionEnd(line, pos);
+					document.SetSelectionEnd(line, pos, true);
 					document.PositionCaret(line, pos);
 				} else {
 					selection_length = -1;
 
-					document.SetSelectionEnd(document.selection_start.line, document.selection_start.pos);
+					document.SetSelectionEnd(document.selection_start.line, document.selection_start.pos, true);
 					document.PositionCaret(document.selection_start.line, document.selection_start.pos);
 				}
 			}
@@ -543,17 +557,24 @@ namespace System.Windows.Forms {
 			}
 
 			set {
-				document.SetSelectionStart(value);
+				document.SetSelectionStart(value, false);
 				if (selection_length > -1 ) {
-					document.SetSelectionEnd(value + selection_length);
+					document.SetSelectionEnd(value + selection_length, true);
 				} else {
-					document.SetSelectionEnd(value);
+					document.SetSelectionEnd(value, true);
 				}
 				document.PositionCaret(document.selection_start.line, document.selection_start.pos);
 				ScrollToCaret();
 			}
 		}
 
+#if NET_2_0
+		public virtual bool ShortcutsEnabled {
+			get { return shortcuts_enabled; }
+			set { shortcuts_enabled = value; }
+		}
+#endif
+
 		[Localizable(true)]
 		public override string Text {
 			get {
@@ -565,18 +586,17 @@ namespace System.Windows.Forms {
 
 				Line line = null;
 				for (int i = 1; i <= document.Lines; i++) {
-					if (line != null && line.carriage_return)
-						sb.Append ("\r");
-					if (line != null && !line.soft_break)
-						sb.Append (Environment.NewLine);
 					line = document.GetLine (i);
-					sb.Append(line.text.ToString());
+					sb.Append(line.text.ToString ());
 				}
 
 				return sb.ToString();
 			}
 
 			set {
+				// reset to force a select all next time the box gets focus
+				has_been_focused = false;
+
 				if (value == Text)
 					return;
 
@@ -592,7 +612,8 @@ namespace System.Windows.Forms {
 					ScrollToCaret ();
 				} else {
 					document.Empty();
-					CalculateDocument();
+					if (IsHandleCreated)
+						CalculateDocument ();
 				}
 
 				// set the var so OnModifiedChanged is not raised
@@ -607,21 +628,7 @@ namespace System.Windows.Forms {
 				if (document == null || document.Root == null || document.Root.text == null) {
 					return 0;
 				}
-
-				if (!multiline) {
-					return document.Root.text.Length;
-				} else {
-					int	total;
-					int	i;
-
-					total = 0;
-					for (i = 1; i < document.Lines; i++) {
-						total += document.GetLine(i).text.Length + Environment.NewLine.Length;
-					}
-					total += document.GetLine(i).text.Length;
-
-					return total;
-				}
+				return Text.Length;
 			}
 		}
 
@@ -635,7 +642,7 @@ namespace System.Windows.Forms {
 
 			set {
 				if (value != word_wrap) {
-					if (multiline) {
+					if (document.multiline) {
 						word_wrap = value;
 						document.Wrap = value;
 					}
@@ -659,22 +666,24 @@ namespace System.Windows.Forms {
 		#endregion	// Protected Instance Properties
 
 		#region Public Instance Methods
-		public void AppendText(string text) {
-			//if (multiline) {
+		public void AppendText(string text)
+		{
+			// Save some cycles and only check the Text if we are one line
+			bool is_empty = document.Lines == 1 && Text == String.Empty; 
 
-				document.MoveCaret(CaretDirection.CtrlEnd);				
-				document.Insert (document.caret.line, /* document.caret.tag, */ document.caret.pos, false, text);
-				//	CalculateDocument();
-				//} else {
-				//	document.MoveCaret(CaretDirection.CtrlEnd);
-				//	document.InsertStringAtCaret(text, true);
-
-				//	Invalidate();
-				// }
-
-			document.MoveCaret(CaretDirection.CtrlEnd);
+			document.MoveCaret (CaretDirection.CtrlEnd);
+			document.Insert (document.caret.line, document.caret.pos, false, text);
+			document.MoveCaret (CaretDirection.CtrlEnd);
 			document.SetSelectionToCaret (true);
 
+			if (!is_empty)
+				ScrollToCaret ();
+
+			//
+			// Avoid the initial focus selecting all when append text is used
+			//
+			has_been_focused = true;
+
 			OnTextChanged(EventArgs.Empty);
 		}
 
@@ -704,7 +713,10 @@ namespace System.Windows.Forms {
 				o.SetData(DataFormats.Rtf, ((RichTextBox)this).SelectedRtf);
 			}
 			Clipboard.SetDataObject (o);
-			document.ReplaceSelection ("", false);
+
+			document.undo.BeginUserAction (Locale.GetText ("Cut"));
+			document.ReplaceSelection (String.Empty, false);
+			document.undo.EndUserAction ();
 		}
 
 		public void Paste() {
@@ -727,10 +739,27 @@ namespace System.Windows.Forms {
 			Line	last;
 
 			last = document.GetLine(document.Lines);
-			document.SetSelectionStart(document.GetLine(1), 0);
-			document.SetSelectionEnd(last, last.text.Length);
+			document.SetSelectionStart(document.GetLine(1), 0, false);
+			document.SetSelectionEnd(last, last.text.Length, true);
 			document.PositionCaret (document.selection_end.line, document.selection_end.pos);
 			selection_length = -1;
+
+			CaretMoved (this, null);
+
+			document.DisplayCaret ();
+		}
+
+		internal void SelectAllNoScroll ()
+		{
+			Line last;
+
+			last = document.GetLine(document.Lines);
+			document.SetSelectionStart(document.GetLine(1), 0, false);
+			document.SetSelectionEnd(last, last.text.Length, false);
+			document.PositionCaret (document.selection_end.line, document.selection_end.pos);
+			selection_length = -1;
+
+			document.DisplayCaret ();
 		}
 
 		public override string ToString() {
@@ -744,6 +773,7 @@ namespace System.Windows.Forms {
 
 		#region Protected Instance Methods
 		protected override void CreateHandle() {
+			CalculateDocument ();
 			base.CreateHandle ();
 			document.AlignCaret();
 			ScrollToCaret();
@@ -756,14 +786,14 @@ namespace System.Windows.Forms {
 
 			switch (keyData & Keys.KeyCode) {
 				case Keys.Enter: {
-					if (multiline && accepts_return) {
+					if (accepts_return && document.multiline) {
 						return true;
 					}
 					return false;
 				}
 
 				case Keys.Tab: {
-					if (accepts_tab && multiline) {
+					if (accepts_tab && document.multiline) {
 						if ((keyData & Keys.Control) == 0) {
 							return true;
 						}
@@ -807,7 +837,7 @@ namespace System.Windows.Forms {
 		protected override void OnFontChanged(EventArgs e) {
 			base.OnFontChanged (e);
 
-			if (auto_size && !multiline) {
+			if (auto_size && !document.multiline) {
 				if (PreferredHeight != ClientSize.Height) {
 					Height = PreferredHeight;
 				}
@@ -857,46 +887,113 @@ namespace System.Windows.Forms {
 			control = (Control.ModifierKeys & Keys.Control) != 0;
 			shift = (Control.ModifierKeys & Keys.Shift) != 0;
 
-			switch (keyData & Keys.KeyCode) {
-				case Keys.X: {	// Cut (Ctrl-X)
+			if (shortcuts_enabled) {
+				switch (keyData & Keys.KeyCode) {
+				case Keys.X:
 					if (control) {
 						Cut();
 						return true;
 					}
 					return false;
-				}
 
-				case Keys.C: {	// Copy (Ctrl-C)
+				case Keys.C:
 					if (control) {
 						Copy();
 						return true;
 					}
 					return false;
-				}
 
-				case Keys.V: {	// Paste (Ctrl-V)
+				case Keys.V:
 					if (control) {
 						return Paste(Clipboard.GetDataObject(), null, true);
 					}
 					return false;
-				}
 
-				case Keys.Z: {	// Undo (Ctrl-Z)
+				case Keys.Z:
 					if (control) {
 						Undo();
 						return true;
 					}
 					return false;
-				}
 
-				case Keys.A: {	// Select All (Ctrl-A)
+				case Keys.A:
 					if (control) {
 						SelectAll();
 						return true;
 					}
 					return false;
-				}
 
+				case Keys.Insert:
+					if (shift) {
+						Paste(Clipboard.GetDataObject(), null, true);
+						return true;
+					}
+
+					if (control) {
+						Copy();
+						return true;
+					}
+
+					return false;
+
+				case Keys.Delete:
+					if (shift) {
+						Cut();
+						return true;
+					}
+
+					if (read_only)
+						break;
+
+					if (document.selection_visible) {
+						document.ReplaceSelection("", false);
+					} else {
+						// DeleteChar only deletes on the line, doesn't do the combine
+						if (document.CaretPosition >= document.CaretLine.TextLengthWithoutEnding ()) {
+							if (document.CaretLine.LineNo < document.Lines) {
+								Line	line;
+
+								line = document.GetLine(document.CaretLine.LineNo + 1);
+
+								// this line needs to be invalidated before it is combined
+								// because once it is combined, all it's coordinates will
+								// have changed
+								document.Invalidate (line, 0, line, line.text.Length);
+								document.Combine(document.CaretLine, line);
+
+								document.UpdateView(document.CaretLine,
+										document.Lines, 0);
+
+							}
+						} else {
+							if (!control) {
+								document.DeleteChar(document.CaretTag, document.CaretPosition, true);
+							} else {
+								int end_pos;
+
+								end_pos = document.CaretPosition;
+
+								while ((end_pos < document.CaretLine.Text.Length) && !Document.IsWordSeparator(document.CaretLine.Text[end_pos])) {
+									end_pos++;
+								}
+
+								if (end_pos < document.CaretLine.Text.Length) {
+									end_pos++;
+								}
+								document.DeleteChars(document.CaretTag, document.CaretPosition, end_pos - document.CaretPosition);
+							}
+						}
+					}
+
+					OnTextChanged(EventArgs.Empty);
+					document.AlignCaret();
+					document.UpdateCaret();
+					CaretMoved(this, null);
+					return true;
+				}
+			}
+
+			switch (keyData & Keys.KeyCode) {
 				case Keys.Left: {
 					if (control) {
 						document.MoveCaret(CaretDirection.WordBack);
@@ -907,7 +1004,7 @@ namespace System.Windows.Forms {
 							document.MoveCaret(CaretDirection.SelectionStart);
 						}
 					}
-
+					
 					if (!shift) {
 						document.SetSelectionToCaret(true);
 					} else {
@@ -1016,19 +1113,22 @@ namespace System.Windows.Forms {
 
 				case Keys.Enter: {
 					// ignoring accepts_return, fixes bug #76355
-					if (!read_only && multiline && (accepts_return || (FindForm() != null && FindForm().AcceptButton == null) || ((Control.ModifierKeys & Keys.Control) != 0))) {
+					if (!read_only && document.multiline && (accepts_return || (FindForm() != null && FindForm().AcceptButton == null) || ((Control.ModifierKeys & Keys.Control) != 0))) {
 						Line	line;
 
 						if (document.selection_visible) {
-							document.ReplaceSelection("", false);
+							document.ReplaceSelection("\n", false);
 						}
 
 						line = document.CaretLine;
 
-						document.Split (document.CaretLine, document.CaretTag, document.CaretPosition, false);
+						document.Split (document.CaretLine, document.CaretTag, document.CaretPosition);
+						line.ending = LineEnding.Rich;
+						document.InsertString (line, line.text.Length,
+										document.LineEndingToString (line.ending));
 						OnTextChanged(EventArgs.Empty);
-						document.UpdateView(line, 2, 0);
 
+						document.UpdateView (line, document.Lines - line.line_no, 0);
 						CaretMoved(this, null);
 						return true;
 					}
@@ -1036,7 +1136,7 @@ namespace System.Windows.Forms {
 				}
 
 				case Keys.Tab: {
-					if (!read_only && accepts_tab && multiline) {
+					if (!read_only && accepts_tab && document.multiline) {
 						document.InsertChar(document.CaretLine, document.CaretPosition, '\t');
 						if (document.selection_visible) {
 							document.ReplaceSelection("", false);
@@ -1050,27 +1150,13 @@ namespace System.Windows.Forms {
 					break;
 				}
 
-				case Keys.Insert: {
-					if (shift) {
-						Paste(Clipboard.GetDataObject(), null, true);
-						return true;
-					}
-
-					if (control) {
-						Copy();
-						return true;
-					}
-
-					// FIXME - need overwrite/insert toggle?
-					return false;
-				}
-
 				case Keys.PageUp: {
 					if ((Control.ModifierKeys & Keys.Control) != 0) {
 						document.MoveCaret(CaretDirection.CtrlPgUp);
 					} else {
 						document.MoveCaret(CaretDirection.PgUp);
 					}
+					document.DisplayCaret ();
 					return true;
 				}
 
@@ -1080,70 +1166,11 @@ namespace System.Windows.Forms {
 					} else {
 						document.MoveCaret(CaretDirection.PgDn);
 					}
-					return true;
-				}
-
-				case Keys.Delete: {
-					if (shift) {
-						Cut();
-						return true;
-					}
-
-					if (read_only) {
-						break;
-					}
-
-					if (document.selection_visible) {
-						document.ReplaceSelection("", false);
-					} else {
-						// DeleteChar only deletes on the line, doesn't do the combine
-						if (document.CaretPosition == document.CaretLine.Text.Length) {
-							if (document.CaretLine.LineNo < document.Lines) {
-								Line	line;
-
-								line = document.GetLine(document.CaretLine.LineNo + 1);
-								document.Combine(document.CaretLine, line);
-								document.UpdateView(document.CaretLine, 2, 0);
-
-								#if not_Debug
-								Line	check_first;
-								Line	check_second;
-
-								check_first = document.GetLine(document.CaretLine.LineNo);
-								check_second = document.GetLine(check_first.line_no + 1);
-
-								Console.WriteLine("Post-UpdateView: Y of first line: {0}, second line: {1}", check_first.Y, check_second.Y);
-								#endif
-
-								// Caret doesn't move
-							}
-						} else {
-							if (!control) {
-								document.DeleteChar(document.CaretTag, document.CaretPosition, true);
-							} else {
-								int end_pos;
-
-								end_pos = document.CaretPosition;
-
-								while ((end_pos < document.CaretLine.Text.Length) && !Document.IsWordSeparator(document.CaretLine.Text[end_pos])) {
-									end_pos++;
-								}
-
-								if (end_pos < document.CaretLine.Text.Length) {
-									end_pos++;
-								}
-								document.DeleteChars(document.CaretTag, document.CaretPosition, end_pos - document.CaretPosition);
-							}
-						}
-					}
-
-					OnTextChanged(EventArgs.Empty);
-					document.AlignCaret();
-					document.UpdateCaret();
-					CaretMoved(this, null);
+					document.DisplayCaret ();
 					return true;
 				}
 			}
+
 			return false;
 		}
 
@@ -1154,7 +1181,9 @@ namespace System.Windows.Forms {
 
 			// delete only deletes on the line, doesn't do the combine
 			if (document.selection_visible) {
+				document.undo.BeginUserAction (Locale.GetText ("Delete"));
 				document.ReplaceSelection("", false);
+				document.undo.EndUserAction ();
 				fire_changed = true;
 			}
 			document.SetSelectionToCaret(true);
@@ -1165,10 +1194,13 @@ namespace System.Windows.Forms {
 					int	new_caret_pos;
 
 					line = document.GetLine(document.CaretLine.LineNo - 1);
-					new_caret_pos = line.text.Length;
+					new_caret_pos = line.TextLengthWithoutEnding ();
 
+					// Invalidate the old line position before we do the combine
+					document.Invalidate (line, 0, line, line.text.Length);
 					document.Combine(line, document.CaretLine);
-					document.UpdateView(line, 1, 0);
+
+					document.UpdateView(line, document.Lines - line.LineNo, 0);
 					document.PositionCaret(line, new_caret_pos);
 					document.SetSelectionToCaret (true);
 					document.UpdateCaret();
@@ -1187,18 +1219,22 @@ namespace System.Windows.Forms {
 				} else {
 					int start_pos;
 
+					
 					start_pos = document.CaretPosition - 1;
-
 					while ((start_pos > 0) && !Document.IsWordSeparator(document.CaretLine.Text[start_pos - 1])) {
 						start_pos--;
 					}
+
+					document.undo.BeginUserAction (Locale.GetText ("Delete"));
 					document.DeleteChars(document.CaretTag, start_pos, document.CaretPosition - start_pos);
+					document.undo.EndUserAction ();
 					document.PositionCaret(document.CaretLine, start_pos);
 					document.SetSelectionToCaret (true);
 				}
 				document.UpdateCaret();
 				fire_changed = true;
 			}
+
 			if (fire_changed) {
 				OnTextChanged(EventArgs.Empty);
 			}
@@ -1208,7 +1244,7 @@ namespace System.Windows.Forms {
 		protected override void SetBoundsCore(int x, int y, int width, int height, BoundsSpecified specified) {
 			// Make sure we don't get sized bigger than we want to be
 			if (!richtext) {
-				if (!multiline) {
+				if (!document.multiline) {
 					if (height != PreferredHeight) {
 						requested_height = height;
 						height = PreferredHeight;
@@ -1266,6 +1302,9 @@ namespace System.Windows.Forms {
 
 					if (document.Length < max_length) {
 						document.InsertCharAtCaret(c, true);
+#if NET_2_0
+						OnTextUpdate ();
+#endif
 						OnTextChanged(EventArgs.Empty);
 						CaretMoved(this, null);
 					} else {
@@ -1280,13 +1319,13 @@ namespace System.Windows.Forms {
 			}
 
 			case Msg.WM_SETFOCUS:
-				document.CaretHasFocus ();
 				base.WndProc(ref m);
+				document.CaretHasFocus ();
 				break;
 
 			case Msg.WM_KILLFOCUS:
-				document.CaretLostFocus ();
 				base.WndProc(ref m);
+				document.CaretLostFocus ();
 				break;
 
 			default:
@@ -1421,45 +1460,14 @@ namespace System.Windows.Forms {
 			} else {
 				pevent.Graphics.FillRectangle(ThemeEngine.Current.ResPool.GetSolidBrush(ThemeEngine.Current.ColorControl), pevent.ClipRectangle);
 			}
-			pevent.Graphics.TextRenderingHint=TextRenderingHint.AntiAlias;
-
+			
 			// Draw the viewable document
 			document.Draw(pevent.Graphics, pevent.ClipRectangle);
 
-			Rectangle	rect = ClientRectangle;
-			rect.Width--;
-			rect.Height--;
-			//pevent.Graphics.DrawRectangle(ThemeEngine.Current.ResPool.GetPen(ThemeEngine.Current.ColorControlDark), rect);
-
-			#if Debug
-				int		start;
-				int		end;
-				Line		line;
-				int		line_no;
-				Pen		p;
-
-				p = new Pen(Color.Red, 1);
-
-				// First, figure out from what line to what line we need to draw
-				start = document.GetLineByPixel(pevent.ClipRectangle.Top - document.ViewPortY, false).line_no;
-				end = document.GetLineByPixel(pevent.ClipRectangle.Bottom - document.ViewPortY, false).line_no;
-
-				//Console.WriteLine("Starting drawing on line '{0}'", document.GetLine(start));
-				//Console.WriteLine("Ending drawing on line '{0}'", document.GetLine(end));
-
-				line_no = start;
-				while (line_no <= end) {
-					line = document.GetLine(line_no);
-
-					if (draw_lines) {
-						for (int i = 0; i < line.text.Length; i++) {
-							pevent.Graphics.DrawLine(p, (int)line.widths[i] - document.ViewPortX, line.Y - document.ViewPortY, (int)line.widths[i] - document.ViewPortX, line.Y + line.height  - document.ViewPortY);
-						}
-					}
-
-					line_no++;
-				}
-			#endif
+			//
+			// OnPaint does not get raised on MS (see bug #80639)
+			// 
+			pevent.Handled = true;
 		}
 
 		private bool IsDoubleClick (MouseEventArgs e)
@@ -1581,7 +1589,8 @@ namespace System.Windows.Forms {
 		}
 
 		private void TextBoxBase_SizeChanged(object sender, EventArgs e) {
-			CalculateDocument();
+			if (IsHandleCreated)
+				CalculateDocument ();
 		}
 
 		private void TextBoxBase_MouseWheel(object sender, MouseEventArgs e) {
@@ -1591,10 +1600,10 @@ namespace System.Windows.Forms {
 			}
 
 			if (e.Delta < 0)
-				vscroll.Value = Math.Min (vscroll.Value + SystemInformation.MouseWheelScrollLines,
+				vscroll.Value = Math.Min (vscroll.Value + SystemInformation.MouseWheelScrollLines * 5,
 						Math.Max (0, vscroll.Maximum - document.ViewPortHeight + 1));
 			else
-				vscroll.Value = Math.Max (0, vscroll.Value - SystemInformation.MouseWheelScrollLines);
+				vscroll.Value = Math.Max (0, vscroll.Value - SystemInformation.MouseWheelScrollLines * 5);
 		}
 
 		internal virtual void SelectWord ()
@@ -1638,14 +1647,14 @@ namespace System.Windows.Forms {
 
 			document.SetSelection (document.caret.line, start, document.caret.line, end);
 			document.PositionCaret (document.selection_end.line, document.selection_end.pos);
+			document.DisplayCaret();
 		}
 
-		internal void CalculateDocument() {
-			if (!IsHandleCreated) {
-				return;
-			}
-			document.RecalculateDocument(CreateGraphicsInternal());
-			CalculateScrollBars();
+		internal void CalculateDocument()
+		{
+			CalculateScrollBars ();
+			document.RecalculateDocument (CreateGraphicsInternal ());
+
 
 			 if (document.caret.line != null && document.caret.line.Y < document.ViewPortHeight) {
 				// The window has probably been resized, making the entire thing visible, so
@@ -1659,15 +1668,13 @@ namespace System.Windows.Forms {
 		internal void CalculateScrollBars () {
 			// FIXME - need separate calculations for center and right alignment
 
-			if (!multiline) {
-				PositionControls ();
-				return;
-			}
+			//	
 
 			if (document.Width >= document.ViewPortWidth) {
 				hscroll.SetValues (0, Math.Max (1, document.Width), -1,
 						document.ViewPortWidth < 0 ? 0 : document.ViewPortWidth);
-				hscroll.Enabled = true;
+				if (document.multiline)
+					hscroll.Enabled = true;
 			} else {
 				hscroll.Enabled = false;
 				hscroll.Maximum = document.ViewPortWidth;
@@ -1676,13 +1683,13 @@ namespace System.Windows.Forms {
 			if (document.Height >= document.ViewPortHeight) {
 				vscroll.SetValues (0, Math.Max (1, document.Height), -1,
 						document.ViewPortHeight < 0 ? 0 : document.ViewPortHeight);
-				vscroll.Enabled = true;
+				if (document.multiline)
+					vscroll.Enabled = true;
 			} else {
 				vscroll.Enabled = false;
 				vscroll.Maximum = document.ViewPortHeight;
 			}
 
-
 			if (!WordWrap) {
 				if ((scrollbars & RichTextBoxScrollBars.Horizontal) != 0) {
 					if (((scrollbars & RichTextBoxScrollBars.ForcedHorizontal) != 0) || hscroll.Enabled) {
@@ -1788,8 +1795,6 @@ namespace System.Windows.Forms {
 						// Force the first tick
 						ScrollTimerTickHandler (null, EventArgs.Empty);
 					}
-
-					return;
 				}
 
 				document.PositionCaret(e.X + document.ViewPortX, e.Y + document.ViewPortY);
@@ -1812,8 +1817,7 @@ namespace System.Windows.Forms {
 							ThemeEngine.Current.ResPool.GetSolidBrush(ForeColor),
 							null, FormatSpecified.Font | FormatSpecified.Color);
 				}
-				document.UpdateView (document.GetLine (1), 0);
-				document.ResumeRecalc (true);
+				document.ResumeRecalc (false);
 
 				// Make sure the caret height is matching the new font height
 				document.AlignCaret();
@@ -1854,9 +1858,10 @@ namespace System.Windows.Forms {
 			Point	pos;
 			int	height;
 
-			if (canvas_width < 1 || canvas_height < 1)
+			if (!IsHandleCreated || canvas_width < 1 || canvas_height < 1)
 				return;
-			
+
+  			document.MoveCaretToTextTag ();
 			pos = document.Caret;
 
 			//Console.WriteLine("Caret now at {0} (Thumb: {1}x{2}, Canvas: {3}x{4}, Document {5}x{6})", pos, hscroll.Value, vscroll.Value, canvas_width, canvas_height, document.Width, document.Height);
@@ -1902,7 +1907,7 @@ namespace System.Windows.Forms {
 				// FIXME - implement center cursor alignment
 			}
 
-			if (!multiline) {
+			if (!document.multiline) {
 				return;
 			}
 
@@ -1953,7 +1958,7 @@ namespace System.Windows.Forms {
 				return true;
 			} else if (format.Name == DataFormats.Bitmap) {
 				document.undo.BeginUserAction (Locale.GetText ("Paste"));
-				document.InsertImage (document.caret.tag, document.caret.pos, (Image) clip.GetData (DataFormats.Bitmap));
+				//	document.InsertImage (document.caret.line, document.caret.pos, (Image) clip.GetData (DataFormats.Bitmap));
 				document.MoveCaret (CaretDirection.CharForward);
 				document.undo.EndUserAction ();
 				return true;
@@ -1983,9 +1988,17 @@ namespace System.Windows.Forms {
 
 			return true;
 		}
+
+		internal abstract Color ChangeBackColor (Color backColor);
+
 		#endregion	// Private Methods
 
 #if NET_2_0
+		// This is called just before OnTextChanged is called.
+		internal virtual void OnTextUpdate ()
+		{
+		}
+		
 		protected override void OnTextChanged (EventArgs e)
 		{
 			base.OnTextChanged (e);
