@@ -26,16 +26,17 @@
 //	Jonathan Pobst (monkey@jpobst.com)
 //
 
-#if NET_2_0
 using System.Drawing;
 using System.Runtime.InteropServices;
 using System.Text;
 using System.Drawing.Text;
-using System.Windows.Forms.VisualStyles;
 
 namespace System.Windows.Forms
 {
-	public sealed class TextRenderer
+#if NET_2_0
+	public sealed 
+#endif
+	class TextRenderer
 	{
 		private static Bitmap measure_bitmap = new Bitmap (1, 1);
 
@@ -44,6 +45,7 @@ namespace System.Windows.Forms
 		}
 
 		#region Public Methods
+#if NET_2_0
 		public static void DrawText (IDeviceContext dc, string text, Font font, Point pt, Color foreColor)
 		{
 			DrawTextInternal (dc, text, font, pt, foreColor, Color.Transparent, TextFormatFlags.Default, false);
@@ -113,11 +115,17 @@ namespace System.Windows.Forms
 		{
 			return MeasureTextInternal (dc, text, font, proposedSize, flags, false);
 		}
+#endif
 		#endregion
 
 		#region Internal Methods That Do Stuff
+#if NET_2_0
 		internal static void DrawTextInternal (IDeviceContext dc, string text, Font font, Rectangle bounds, Color foreColor, Color backColor, TextFormatFlags flags, bool useDrawString)
+#else
+		internal static void DrawTextInternal (Graphics dc, string text, Font font, Rectangle bounds, Color foreColor, Color backColor, TextFormatFlags flags, bool useDrawString)
+#endif
 		{
+#if NET_2_0
 			if (dc == null)
 				throw new ArgumentNullException ("dc");
 
@@ -133,8 +141,26 @@ namespace System.Windows.Forms
 				Rectangle new_bounds = PadRectangle (bounds, flags);
 				new_bounds.Offset ((int)(dc as Graphics).Transform.OffsetX, (int)(dc as Graphics).Transform.OffsetY);
 
-				IntPtr hdc = dc.GetHdc ();
-
+				IntPtr hdc = IntPtr.Zero;
+				bool clear_clip_region = false;
+				
+				// If we need to use the graphics clipping region, add it to our hdc
+				if ((flags & TextFormatFlags.PreserveGraphicsClipping) == TextFormatFlags.PreserveGraphicsClipping) {
+					Graphics graphics = (Graphics)dc;
+					Region clip_region = graphics.Clip;
+					
+					if (!clip_region.IsInfinite (graphics)) {
+						IntPtr hrgn = clip_region.GetHrgn (graphics);
+						hdc = dc.GetHdc ();
+						SelectClipRgn (hdc, hrgn);
+						clip_region.ReleaseHrgn (hrgn);
+						clear_clip_region = true;
+					}
+				}
+				
+				if (hdc == IntPtr.Zero)
+					hdc = dc.GetHdc ();
+					
 				// Set the fore color
 				if (foreColor != Color.Empty)
 					SetTextColor (hdc, ColorTranslator.ToWin32 (foreColor));
@@ -148,7 +174,7 @@ namespace System.Windows.Forms
 					SetBkMode (hdc, 1);	//1-Transparent, 2-Opaque
 				}
 
-				UXTheme.RECT r = UXTheme.RECT.FromRectangle (new_bounds);
+				XplatUIWin32.RECT r = XplatUIWin32.RECT.FromRectangle (new_bounds);
 
 				IntPtr prevobj;
 
@@ -162,40 +188,55 @@ namespace System.Windows.Forms
 					Win32DrawText (hdc, text, text.Length, ref r, (int)flags);
 				}
 
+				if (clear_clip_region)
+					SelectClipRgn (hdc, IntPtr.Zero);
+
 				dc.ReleaseHdc ();
 			}
 			// Use Graphics.DrawString as a fallback method
 			else {
+#endif
 				Graphics g;
 
+#if NET_2_0
 				if (dc is Graphics)
 					g = (Graphics)dc;
 				else
 					g = Graphics.FromHdc (dc.GetHdc ());
+#else
+					g = (Graphics)dc;
+
+#endif
 
 				StringFormat sf = FlagsToStringFormat (flags);
 
 				Rectangle new_bounds = PadDrawStringRectangle (bounds, flags);
 
-				using (Brush b = new SolidBrush (foreColor))
-					g.DrawString (text, font, b, new_bounds, sf);
+				g.DrawString (text, font, ThemeEngine.Current.ResPool.GetSolidBrush (foreColor), new_bounds, sf);
 
+#if NET_2_0
 				if (!(dc is Graphics)) {
 					g.Dispose ();
 					dc.ReleaseHdc ();
 				}
 			}
+#endif
 		}
 
+#if NET_2_0
 		internal static Size MeasureTextInternal (IDeviceContext dc, string text, Font font, Size proposedSize, TextFormatFlags flags, bool useMeasureString)
+#else
+		internal static Size MeasureTextInternal (Graphics dc, string text, Font font, Size proposedSize, TextFormatFlags flags, bool useMeasureString)
+#endif
 		{
+#if NET_2_0
 			if (!useMeasureString && (Environment.OSVersion.Platform == PlatformID.Win32NT || Environment.OSVersion.Platform == PlatformID.Win32Windows)) {
 				// Tell DrawText to calculate size instead of draw
 				flags |= (TextFormatFlags)1024;		// DT_CALCRECT
 
 				IntPtr hdc = dc.GetHdc ();
 
-				UXTheme.RECT r = UXTheme.RECT.FromRectangle (new Rectangle (Point.Empty, proposedSize));
+				XplatUIWin32.RECT r = XplatUIWin32.RECT.FromRectangle (new Rectangle (Point.Empty, proposedSize));
 
 				IntPtr prevobj;
 
@@ -215,7 +256,7 @@ namespace System.Windows.Forms
 				// just makes something up as well.  This will require lots of tweaking to match MS.  :(
 				Size retval = r.ToRectangle ().Size;
 
-				if (retval.Width > 0) {
+				if (retval.Width > 0 && (flags & TextFormatFlags.NoPadding) == 0) {
 					retval.Width += 6;
 					retval.Width += (int)retval.Height / 8;
 				}
@@ -223,19 +264,23 @@ namespace System.Windows.Forms
 				return retval;
 			}
 			else {
+#endif
 				Graphics g = Graphics.FromImage (measure_bitmap);
 
 				Size retval = g.MeasureString (text, font).ToSize ();
 
-				if (retval.Width > 0)
-					retval.Width += 6;
+				if (retval.Width > 0 && (flags & TextFormatFlags.NoPadding) == 0)
+					retval.Width += 9;
 
 				return retval;
 			}
+#if NET_2_0
 		}
+#endif
 		#endregion
 
-		#region Internal Methods That Are Just Overloads
+#region Internal Methods That Are Just Overloads
+#if NET_2_0
 		internal static void DrawTextInternal (IDeviceContext dc, string text, Font font, Point pt, Color foreColor, bool useDrawString)
 		{
 			DrawTextInternal (dc, text, font, pt, foreColor, Color.Transparent, TextFormatFlags.Default, useDrawString);
@@ -262,10 +307,14 @@ namespace System.Windows.Forms
 		}
 
 		internal static void DrawTextInternal (IDeviceContext dc, string text, Font font, Rectangle bounds, Color foreColor, TextFormatFlags flags, bool useDrawString)
+#else
+		internal static void DrawTextInternal (Graphics dc, string text, Font font, Rectangle bounds, Color foreColor, TextFormatFlags flags, bool useDrawString)
+#endif
 		{
 			DrawTextInternal (dc, text, font, bounds, foreColor, Color.Transparent, flags, useDrawString);
 		}
 
+#if NET_2_0
 		internal static void DrawTextInternal (IDeviceContext dc, string text, Font font, Point pt, Color foreColor, Color backColor, TextFormatFlags flags, bool useDrawString)
 		{
 			Size sz = MeasureTextInternal (dc, text, font, useDrawString);
@@ -292,13 +341,14 @@ namespace System.Windows.Forms
 			return MeasureTextInternal (dc, text, font, proposedSize, TextFormatFlags.Default, useMeasureString);
 		}
 
+#endif
 		internal static Size MeasureTextInternal (string text, Font font, Size proposedSize, TextFormatFlags flags, bool useMeasureString)
 		{
 			return MeasureTextInternal (Graphics.FromImage (measure_bitmap), text, font, proposedSize, flags, useMeasureString);
 		}
-		#endregion
+#endregion
 
-		#region Private Methods
+#region Private Methods
 		private static StringFormat FlagsToStringFormat (TextFormatFlags flags)
 		{
 			StringFormat sf = new StringFormat ();
@@ -413,11 +463,12 @@ namespace System.Windows.Forms
 
 			return r;
 		}
-		#endregion
+#endregion
 
-		#region DllImports (Windows)
+#region DllImports (Windows)
+#if NET_2_0
 		[DllImport ("user32", CharSet = CharSet.Unicode, EntryPoint = "DrawText")]
-		static extern int Win32DrawText (IntPtr hdc, string lpStr, int nCount, ref UXTheme.RECT lpRect, int wFormat);
+		static extern int Win32DrawText (IntPtr hdc, string lpStr, int nCount, ref XplatUIWin32.RECT lpRect, int wFormat);
 
 		[DllImport ("gdi32")]
 		static extern int SetTextColor (IntPtr hdc, int crColor);
@@ -433,7 +484,10 @@ namespace System.Windows.Forms
 
 		[DllImport ("gdi32")]
 		static extern bool DeleteObject (IntPtr objectHandle);
-		#endregion
-	}
-}
+
+		[DllImport("gdi32")]
+		static extern bool SelectClipRgn(IntPtr hdc, IntPtr hrgn);
 #endif
+#endregion
+	}
+}
\ No newline at end of file
