@@ -151,7 +151,7 @@ namespace System.Windows.Forms
 		[Browsable (false)]
 		[EditorBrowsable (EditorBrowsableState.Never)]
 		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
-		public override ContextMenuStrip ContextMenuStrip {
+		public new ContextMenuStrip ContextMenuStrip {
 			get { return base.ContextMenuStrip; }
 			set { base.ContextMenuStrip = value; }
 		}
@@ -225,7 +225,7 @@ namespace System.Windows.Forms
 		[EditorBrowsable (EditorBrowsableState.Advanced)]
 		protected override ControlCollection CreateControlsInstance ()
 		{
-			return base.CreateControlsInstance ();
+			return new ToolStripContainerTypedControlCollection (this);
 		}
 
 		protected override void OnSizeChanged (EventArgs e)
@@ -289,6 +289,15 @@ namespace System.Windows.Forms
 			remove { base.ForeColorChanged -= value; }
 		}
 		#endregion
+
+		#region Private Class : ToolStripContainerTypedControlCollection
+		private class ToolStripContainerTypedControlCollection : ControlCollection
+		{
+			public ToolStripContainerTypedControlCollection (Control owner) : base (owner)
+			{
+			}
+		}
+		#endregion
 	}
 }
 #endif
