@@ -38,7 +38,7 @@ namespace System.Windows.Forms
 	[ClassInterface(ClassInterfaceType.AutoDispatch)]
 	[DefaultEvent ("Load")]
 	[ToolboxItem (false)]
-	[Docking (DockingBehavior.AutoDock)]
+	[Docking (DockingBehavior.Never)]
 	[InitializationEvent ("Load")]
 	public class ToolStripContentPanel : Panel
 	{
@@ -159,9 +159,12 @@ namespace System.Windows.Forms
 
 				return this.renderer;
 			}
-			set { 
-				this.renderer = value;
-				this.OnRendererChanged (EventArgs.Empty);
+			set {
+				if (this.renderer != value) {
+					this.renderer = value;
+					this.render_mode = ToolStripRenderMode.Custom;
+					this.OnRendererChanged (EventArgs.Empty);
+				}
 			}
 		}
 
@@ -203,7 +206,7 @@ namespace System.Windows.Forms
 			base.OnHandleCreated (e);
 		}
 
-		[EditorBrowsable (EditorBrowsableState.Always)]
+		[EditorBrowsable (EditorBrowsableState.Advanced)]
 		protected virtual void OnLoad (EventArgs e)
 		{
 			EventHandler eh = (EventHandler)(Events [LoadEvent]);
@@ -211,7 +214,7 @@ namespace System.Windows.Forms
 				eh (this, e);
 		}
 
-		[EditorBrowsable (EditorBrowsableState.Always)]
+		[EditorBrowsable (EditorBrowsableState.Advanced)]
 		protected override void OnPaintBackground (PaintEventArgs pevent)
 		{
 			base.OnPaintBackground (pevent);
