@@ -189,8 +189,6 @@ namespace System.Windows.Forms {
 			TreeView tree_view = null;
 			if (owner != null) {
 				tree_view = owner.TreeView;
-				if (owner.IsRoot)
-					tree_view.top_node = null;
 				if (tree_view != null) {
 					tree_view.UpdateBelow (owner);
 					tree_view.RecalculateVisibleOrder (owner);
@@ -284,26 +282,8 @@ namespace System.Windows.Forms {
 
 			if (tree_view != null) {
 				tree_view.RecalculateVisibleOrder (prev);
-				if (removed == tree_view.top_node) {
 
-					if (removed.IsRoot) {
-						tree_view.top_node = null;
-					} else {
-						OpenTreeNodeEnumerator oe = new OpenTreeNodeEnumerator (removed);
-						if (oe.MovePrevious () && oe.MovePrevious ()) {
-							tree_view.top_node = oe.CurrentNode;
-						} else {
-							removed.is_expanded = false;
-							oe = new OpenTreeNodeEnumerator (removed);
-							if (oe.MoveNext () && oe.MoveNext ()) {
-								tree_view.top_node = oe.CurrentNode;
-							} else {
-								tree_view.top_node = null;
-							}
-						}
-					}
-				}
-				if (removed == tree_view.selected_node) {
+				if (removed == tree_view.SelectedNode) {
 					OpenTreeNodeEnumerator oe = new OpenTreeNodeEnumerator (removed);
 					if (oe.MoveNext () && oe.MoveNext ()) {
 						new_selected = oe.CurrentNode;
@@ -357,9 +337,6 @@ namespace System.Windows.Forms {
 			if (tree_view != null) {
 				TreeNode prev = GetPrevNode (node);
 
-				if (tree_view.top_node == null)
-					tree_view.top_node = node;
-
 				if (node.IsVisible)
 					tree_view.RecalculateVisibleOrder (prev);
 				if (owner == tree_view.root_node || node.Parent.IsVisible && node.Parent.IsExpanded)
@@ -432,11 +409,11 @@ namespace System.Windows.Forms {
 		}
 
 		// Would be nice to do this without running through the collection twice
-		internal void Sort () {
-			Array.Sort (nodes, 0, count, new TreeNodeComparer (Application.CurrentCulture.CompareInfo));
+		internal void Sort (IComparer sorter) {
+			Array.Sort (nodes, 0, count, sorter == null ? new TreeNodeComparer (Application.CurrentCulture.CompareInfo) : sorter);
 
 			for (int i = 0; i < count; i++) {
-				nodes [i].Nodes.Sort ();
+				nodes [i].Nodes.Sort (sorter);
 			}
 		}
 
