@@ -40,22 +40,26 @@ namespace System.Windows.Forms {
 		private string path_separator = "\\";
 		private int item_height = -1;
 		private bool sorted;
-		internal TreeNode top_node;
 		internal TreeNode root_node;
 		internal bool nodes_added;
 		private TreeNodeCollection nodes;
 
-		private TreeViewAction selection_action = TreeViewAction.Unknown;
-		internal TreeNode selected_node = null;
-		private TreeNode focused_node = null;
+		private TreeViewAction selection_action;
+		private TreeNode selected_node;
+		private TreeNode pre_selected_node;
+		private TreeNode focused_node;
 		private TreeNode highlighted_node;
-		private Rectangle mouse_rect = Rectangle.Empty;
-		private bool select_mmove = false;
+		private Rectangle mouse_rect;
+		private bool select_mmove;
 
 		private ImageList image_list;
 		private int image_index = -1;
 		private int selected_image_index = -1;
 
+#if NET_2_0
+		private string image_key;
+		private string selected_image_key;
+#endif
 		private bool full_row_select;
 		private bool hot_tracking;
 		private int indent = 19;
@@ -84,12 +88,18 @@ namespace System.Windows.Forms {
 		private bool update_needed;
 		
 		private Pen dash;
+		private Color line_color;
 		private StringFormat string_format;
 
 		private int drag_begin_x = 0;
 		private int drag_begin_y = 0;
 		private long handle_count = 1;
 
+		private TreeViewDrawMode draw_mode;
+
+#if NET_2_0
+		IComparer tree_view_node_sorter;
+#endif
 		#endregion	// Fields
 
 		#region Public Constructors	
@@ -143,7 +153,12 @@ namespace System.Windows.Forms {
 		#region Public Instance Properties
 		public override Color BackColor {
 			get { return base.BackColor;}
-			set { base.BackColor = value; }
+			set {
+				base.BackColor = value;
+
+				CreateDashPen ();
+				Invalidate ();
+			}
 		}
 
 
@@ -328,8 +343,17 @@ namespace System.Windows.Forms {
 		[Browsable(false)]
 		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
 		public TreeNode SelectedNode {
-			get { return selected_node; }
+			get {
+				if (!IsHandleCreated)
+					return pre_selected_node;
+				return selected_node;
+			}
 			set {
+				if (!IsHandleCreated) {
+					pre_selected_node = value;
+					return;
+				}
+
 				if (selected_node == value) {
 					selection_action = TreeViewAction.Unknown;
 					return;
@@ -358,7 +382,7 @@ namespace System.Windows.Forms {
 				selected_node = value;
 				focused_node = value;
 
-				if (full_row_select) {
+				if (full_row_select || draw_mode != TreeViewDrawMode.Normal) {
 					invalid.X = 0;
 					invalid.Width = ViewportRectangle.Width;
 				}
@@ -422,15 +446,22 @@ namespace System.Windows.Forms {
 		public bool Sorted {
 			get { return sorted; }
 			set {
+#if NET_2_0
+				if (sorted == value)
+					return;
+				sorted = value;
+				//LAMESPEC: The documentation says that setting this to true should sort alphabetically if TreeViewNodeSorter is set.
+				// There seems to be a bug in the Microsoft implementation.
+				if (sorted && tree_view_node_sorter == null) {
+					Sort (null);
+				}
+#else
 				if (sorted != value)
 					sorted = value;
 				if (sorted) {
-					Nodes.Sort ();
-					top_node = root_node;
-					RecalculateVisibleOrder (root_node);
-					UpdateScrollBars ();
-					Invalidate ();
+					Sort (null);
 				}
+#endif
 			}
 		}
 
@@ -445,9 +476,39 @@ namespace System.Windows.Forms {
 		[Browsable(false)]
 		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
 		public TreeNode TopNode {
-			get { return top_node; }
+			get {
+				if (root_node.FirstNode == null)
+					return null;
+				OpenTreeNodeEnumerator one = new OpenTreeNodeEnumerator (root_node.FirstNode);
+				one.MoveNext ();
+				for (int i = 0; i < skipped_nodes; i++)
+					one.MoveNext ();
+				return one.CurrentNode;
+			}
+#if NET_2_0
+			set {
+				SetTop (value);
+			}
+#endif
 		}
 
+#if NET_2_0
+		public IComparer TreeViewNodeSorter {
+			get {
+				return tree_view_node_sorter;
+			}
+			set {
+				tree_view_node_sorter = value;
+				if (tree_view_node_sorter != null) {
+					Sort();
+					//LAMESPEC: The documentation says that setting this should set Sorted to false.
+					// There seems to be a bug in the Microsoft implementation.
+					sorted = true;
+				}
+			}
+		}
+#endif
+
 		[Browsable(false)]
 		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
 		public int VisibleCount {
@@ -456,6 +517,75 @@ namespace System.Windows.Forms {
 			}
 		}
 
+#if NET_2_0
+		/// According to MSDN this property has no effect on the treeview
+		protected override bool DoubleBuffered {
+			get { return true; }
+			set { /* whatever */ }
+		}
+#endif
+
+#if NET_2_0
+		public
+#else
+		private
+#endif
+		Color LineColor {
+			get {
+				if (line_color == Color.Empty) {
+					Color res = ControlPaint.Dark (BackColor);
+					if (res == BackColor)
+						res = ControlPaint.Light (BackColor);
+					return res;
+				}
+				return line_color;
+			}
+			set {
+				line_color = value;
+				if (show_lines) {
+					CreateDashPen ();
+					Invalidate ();
+				}
+			}
+		}
+#if NET_2_0
+		[Localizable (true)]
+		public string ImageKey {
+			get { return image_key; }
+			set {
+				if (image_key == value)
+					return;
+				image_index = -1;
+				image_key = value;
+				Invalidate ();
+			}
+		}
+
+		public string SelectedImageKey {
+			get { return selected_image_key; }
+			set {
+				if (selected_image_key == value)
+					return;
+				selected_image_index = -1;
+				selected_image_key = value;
+				UpdateNode (SelectedNode);
+			}
+		}
+#endif
+
+#if NET_2_0
+		public override ImageLayout BackgroundImageLayout {
+			get { return base.BackgroundImageLayout; }
+			set { base.BackgroundImageLayout = value; }
+		}
+#endif
+
+#if NET_2_0
+		public TreeViewDrawMode DrawMode {
+			get { return draw_mode; }
+			set { draw_mode = value; }
+		}
+#endif
 		#endregion	// Public Instance Properties
 
 		#region Protected Instance Properties
@@ -487,14 +617,36 @@ namespace System.Windows.Forms {
 				if (update_needed) {
 					RecalculateVisibleOrder (root_node);
 					UpdateScrollBars ();
-					if (SelectedNode != null)
-						SelectedNode.EnsureVisible ();
+					//	if (SelectedNode != null)
+					//		SelectedNode.EnsureVisible ();
 					Invalidate (ViewportRectangle);
 					update_needed = false;
 				}
 			}
 		}
 
+#if NET_2_0
+		public
+#else
+		private
+#endif
+		void Sort ()
+		{
+#if NET_2_0
+			Sort (Nodes.Count >= 2 ? tree_view_node_sorter : null);
+#else
+			Sort (null);
+#endif
+		}
+
+		void Sort (IComparer sorter) 
+		{
+			Nodes.Sort (sorter);
+			RecalculateVisibleOrder (root_node);
+			UpdateScrollBars ();
+			Invalidate ();
+		}
+
 		public void ExpandAll ()
 		{
 			BeginUpdate ();
@@ -502,22 +654,37 @@ namespace System.Windows.Forms {
 
 			EndUpdate ();
 
+			///
+			/// Everything below this is basically an emulation of a strange bug on MS
+			/// where they don't always scroll to the last node on ExpandAll
+			///
+#if NET_2_0
+			if (!IsHandleCreated)
+				return;
+#endif
+			
+			bool found = false;
+			foreach (TreeNode child in Nodes) {
+				if (child.Nodes.Count > 0)
+					found = true;
+			}
+			
+			if (!found)
+				return;
+
 			// Walk all the way to the end, then walk back visible count
 			//to find the new top node
 			OpenTreeNodeEnumerator walk = new OpenTreeNodeEnumerator (root_node);
 			while (walk.MoveNext ())
 			{ }
 
-			for (int i = 0; i < VisibleCount - 1; i++)
-				walk.MovePrevious ();
-
 			SetTop (walk.CurrentNode);
 		}
 
 		
 		public void CollapseAll ()
 		{
-			TreeNode walk = top_node;
+			TreeNode walk = TopNode;
 			
 			while (walk.parent != root_node)
 				walk = walk.parent;
@@ -563,8 +730,11 @@ namespace System.Windows.Forms {
 		#region Protected Instance Methods
 		protected override void CreateHandle () {
 			base.CreateHandle ();
-
+			RecalculateVisibleOrder (root_node);
 			UpdateScrollBars ();
+
+			if (pre_selected_node != null)
+				SelectedNode = pre_selected_node;
 		}
 
 		protected override void Dispose (bool disposing) {
@@ -729,6 +899,14 @@ namespace System.Windows.Forms {
 				eh (this, e);
 		}
 
+#if NET_2_0
+		protected virtual void OnDrawNode(DrawTreeNodeEventArgs e) {
+			DrawTreeNodeEventHandler eh = (DrawTreeNodeEventHandler)(Events[DrawNodeEvent]);
+			if (eh != null)
+				eh(this, e);
+		}
+#endif
+
 		protected internal virtual void OnAfterCheck (TreeViewEventArgs e) {
 			TreeViewEventHandler eh = (TreeViewEventHandler)(Events [AfterCheckEvent]);
 			if (eh != null)
@@ -855,9 +1033,6 @@ namespace System.Windows.Forms {
 			if (nodes.Count <= 0)
 				return null;
 
-			if (top_node == null)
-				top_node = nodes [0];
-
 			OpenTreeNodeEnumerator o = new OpenTreeNodeEnumerator (TopNode);
 			int move = y / ActualItemHeight;
 			for (int i = -1; i < move; i++) {
@@ -930,6 +1105,8 @@ namespace System.Windows.Forms {
 			} else
 				order = start.visible_order;
 
+			
+			
 			OpenTreeNodeEnumerator walk = new OpenTreeNodeEnumerator (start);
 			while (walk.MoveNext ()) {
 				walk.CurrentNode.visible_order = order;
@@ -941,21 +1118,12 @@ namespace System.Windows.Forms {
 
 		internal void SetTop (TreeNode node)
 		{
-			if (!vbar.is_visible)
+			if (!vbar.is_visible) {
+				skipped_nodes = node.visible_order;
 				return;
+			}
 
-			TreeNode first = root_node.FirstNode;
-
-			if (first == null)
-				return;  // I don't think its possible for this to happen
-
-			OpenTreeNodeEnumerator walk = new OpenTreeNodeEnumerator (first);
-			int offset = -1;
-			int vc = VisibleCount;
-			while (walk.CurrentNode != node && walk.MoveNext () && offset < vbar.Maximum - vc)
-				offset++;
-
-			SetVScrollTop (walk.CurrentNode);
+			vbar.Value = Math.Min (node.visible_order, vbar.Maximum - VisibleCount);
 		}
 
 		internal void SetBottom (TreeNode node)
@@ -1048,15 +1216,18 @@ namespace System.Windows.Forms {
 			Draw (pe.ClipRectangle, pe.Graphics);
 		}
 
+		internal void CreateDashPen ()
+		{
+			dash = new Pen (LineColor, 1);
+			dash.DashStyle = DashStyle.Dot;
+		}
+
 		private void Draw (Rectangle clip, Graphics dc)
 		{
 			dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (BackColor), clip);
 
-			Color dash_color = ControlPaint.Dark (BackColor);
-			if (dash_color == BackColor)
-				dash_color = ControlPaint.Light (BackColor);
-			dash = new Pen (dash_color, 1);
-			dash.DashStyle = DashStyle.Dot;
+			if (dash == null)
+				CreateDashPen ();
 
 			Rectangle viewport = ViewportRectangle;
 			Rectangle original_clip = clip;
@@ -1075,7 +1246,7 @@ namespace System.Windows.Forms {
 				if (current.GetY () > clip.Bottom)
 					break;
 
-				DrawNode (current, dc, clip);
+				DrawTreeNode (current, dc, clip);
 			}
 
 			if (hbar.Visible && vbar.Visible) {
@@ -1088,7 +1259,7 @@ namespace System.Windows.Forms {
 
 		private void DrawNodePlusMinus (TreeNode node, Graphics dc, int x, int middle)
 		{
-			dc.DrawRectangle (SystemPens.ControlDark, x, middle - 4, 8, 8);
+			dc.DrawRectangle (SystemPens.ControlDarkDark, x, middle - 4, 8, 8);
 
 			if (node.IsExpanded) {
 				dc.DrawLine (SystemPens.ControlDarkDark, x + 2, middle, x + 6, middle); 
@@ -1160,25 +1331,55 @@ namespace System.Windows.Forms {
 		{
 			// Rectangle r = new Rectangle (x, y + 2, ImageList.ImageSize.Width, ImageList.ImageSize.Height);
 
-			if (!RectsIntersect (clip, x, y + 2, ImageList.ImageSize.Width, ImageList.ImageSize.Height))
+			if (!RectsIntersect (clip, x, y, ImageList.ImageSize.Width, ImageList.ImageSize.Height))
 				return;
 
 			if (ImageList == null)
 				return;
 
 			int use_index = -1;
-			if (node.ImageIndex > -1 && node.ImageIndex < ImageList.Images.Count) {
-				use_index = node.ImageIndex;
-			} else if (ImageIndex > -1 && ImageIndex < ImageList.Images.Count) {
-				use_index = ImageIndex;
+
+			if (!node.IsSelected) {
+				if (node.ImageIndex > -1 && node.ImageIndex < ImageList.Images.Count) {
+					use_index = node.ImageIndex;
+				} else if (ImageIndex > -1 && ImageIndex < ImageList.Images.Count) {
+					use_index = ImageIndex;
+				}
+			} else {
+				if (node.SelectedImageIndex > -1 && node.SelectedImageIndex < ImageList.Images.Count) {
+					use_index = node.SelectedImageIndex;
+				} else if (SelectedImageIndex > -1 && SelectedImageIndex < ImageList.Images.Count) {
+					use_index = SelectedImageIndex;
+				}
 			}
 
+#if NET_2_0
+
+			if (!node.IsSelected) {
+				if (use_index == -1 && !String.IsNullOrEmpty (node.ImageKey)) {
+					use_index = image_list.Images.IndexOfKey (node.ImageKey);
+				}
+
+				if (use_index == -1 && !String.IsNullOrEmpty (ImageKey)) {
+					use_index = image_list.Images.IndexOfKey (ImageKey);
+				}
+			} else {
+				if (use_index == -1 && !String.IsNullOrEmpty (node.SelectedImageKey)) {
+					use_index = image_list.Images.IndexOfKey (node.SelectedImageKey);
+				}
+
+				if (use_index == -1 && !String.IsNullOrEmpty (SelectedImageKey)) {
+					use_index = image_list.Images.IndexOfKey (SelectedImageKey);
+				}
+			}
+#endif
+
 			if (use_index == -1 && ImageList.Images.Count > 0) {
 				use_index = 0;
 			}
 
 			if (use_index != -1) {
-				ImageList.Draw (dc, x, y + 2, ImageList.ImageSize.Width, 
+				ImageList.Draw (dc, x, y, ImageList.ImageSize.Width, 
 						ImageList.ImageSize.Height, use_index);
 			}
 		}
@@ -1253,7 +1454,10 @@ namespace System.Windows.Forms {
 			if (Focused && focused_node == node) {
 				ControlPaint.DrawFocusRectangle (dc, r, ForeColor, BackColor);
 			}
-			r.Inflate(-1, -1);
+			if (draw_mode != TreeViewDrawMode.Normal)
+				return;
+
+			r.Inflate (-1, -1);
 			if (Focused && node == highlighted_node) {
 				dc.FillRectangle (ThemeEngine.Current.ResPool.GetSolidBrush (ThemeEngine.Current.ColorHighlight), r);
 			} else if (!hide_selection && node == highlighted_node) {
@@ -1268,17 +1472,20 @@ namespace System.Windows.Forms {
 			if (!full_row_select)
 				DrawSelectionAndFocus(node, dc, node.Bounds);
 
+			
 			Font font = node.NodeFont;
 			if (node.NodeFont == null)
 				font = Font;
 			Color text_color = (Focused && node == highlighted_node ?
 					ThemeEngine.Current.ColorHighlightText : node.ForeColor);
+			if (text_color.IsEmpty)
+				text_color = ForeColor;
 			dc.DrawString (node.Text, font,
 					ThemeEngine.Current.ResPool.GetSolidBrush (text_color),
 					node.Bounds, string_format);
 		}
 
-		private void DrawNode (TreeNode node, Graphics dc, Rectangle clip)
+		private void DrawTreeNode (TreeNode node, Graphics dc, Rectangle clip)
 		{
 			int child_count = node.nodes.Count;
 			int y = node.GetY ();
@@ -1289,17 +1496,47 @@ namespace System.Windows.Forms {
 				DrawSelectionAndFocus (node, dc, r);
 			}
 
-			if ((show_root_lines || node.Parent != null) && show_plus_minus && child_count > 0)
-				DrawNodePlusMinus (node, dc, node.GetLinesX () - Indent + 5, middle);
+			if (draw_mode == TreeViewDrawMode.Normal || draw_mode == TreeViewDrawMode.OwnerDrawText) {
+				if ((show_root_lines || node.Parent != null) && show_plus_minus && child_count > 0)
+					DrawNodePlusMinus (node, dc, node.GetLinesX () - Indent + 5, middle);
 
-			if (checkboxes)
-				DrawNodeCheckBox (node, dc, CheckBoxLeft (node) - 3, middle);
+				if (checkboxes)
+					DrawNodeCheckBox (node, dc, CheckBoxLeft (node) - 3, middle);
 
-			if (show_lines)
-				DrawNodeLines (node, dc, clip, dash, node.GetLinesX (), y, middle);
+				if (show_lines)
+					DrawNodeLines (node, dc, clip, dash, node.GetLinesX (), y, middle);
 
-			if (ImageList != null)
-                                DrawNodeImage (node, dc, clip, node.GetImageX (), y);
+				if (ImageList != null)
+					DrawNodeImage (node, dc, clip, node.GetImageX (), y);
+			}
+
+			
+#if NET_2_0
+			if (draw_mode != TreeViewDrawMode.Normal) {
+				dc.FillRectangle (Brushes.White, node.Bounds);
+				TreeNodeStates tree_node_state = TreeNodeStates.Default;;
+				if (node.IsSelected)
+					tree_node_state = TreeNodeStates.Selected;
+				if (node.Checked)
+					tree_node_state |= TreeNodeStates.Checked;
+				if (node == focused_node)
+					tree_node_state |= TreeNodeStates.Focused;
+				Rectangle node_bounds = node.Bounds;
+				if (draw_mode == TreeViewDrawMode.OwnerDrawText) {
+					node_bounds.X += 3;
+					node_bounds.Y += 1;
+				} else {
+					node_bounds.X = 0;
+					node_bounds.Width = Width;
+				}
+
+				DrawTreeNodeEventArgs e = new DrawTreeNodeEventArgs (dc, node, node_bounds, tree_node_state);
+
+				OnDrawNode (e);				
+				if (!e.DrawDefault)
+					return;
+			}
+#endif
 
 			if (!node.IsEditing)
 				DrawStaticNode (node, dc);
@@ -1364,10 +1601,16 @@ namespace System.Windows.Forms {
 					hbar_bounds_set = false;
 				}
 
+				
 				vbar.Visible = true;
+				if (skipped_nodes > 0) {
+					int skip = Math.Min (skipped_nodes, vbar.Maximum - VisibleCount);
+					skipped_nodes = 0;
+					vbar.Value = skip;
+					skipped_nodes = skip;
+				}
 			} else {
 				skipped_nodes = 0;
-				top_node = root_node.FirstNode;
 				RecalculateVisibleOrder (root_node);
 				vbar.Visible = false;
 				vbar_bounds_set = false;
@@ -1428,42 +1671,16 @@ namespace System.Windows.Forms {
 			if (skipped_nodes == pos)
 				return;
 
-			int old_skip = skipped_nodes;
+			int diff = skipped_nodes - pos;
 			skipped_nodes = pos;
-			int diff = old_skip - skipped_nodes;
 
-			// Determine the new top node if we have to
-			if (new_top == null) {
-				if (top_node == null)
-					top_node = nodes [0];
-
-				OpenTreeNodeEnumerator walk = new OpenTreeNodeEnumerator (TopNode);
-				if (diff < 0) {
-					for (int i = diff; i <= 0; i++)
-						walk.MoveNext ();
-					new_top = walk.CurrentNode;
-				} else {
-					for (int i = 0; i <= diff; i++)
-						walk.MovePrevious ();
-					new_top = walk.CurrentNode;
-				}
-			}
-
-			top_node = new_top;
 			int y_move = diff * ActualItemHeight;
 			XplatUI.ScrollWindow (Handle, ViewportRectangle, 0, y_move, false);
 		}
 
 		private void SetVScrollTop (TreeNode new_top)
 		{
-			OpenTreeNodeEnumerator walk = new OpenTreeNodeEnumerator (root_node);
-
-			skipped_nodes = -1;
-			while (walk.MoveNext () && walk.CurrentNode != new_top)
-				skipped_nodes++;
-
-			top_node = new_top;
-			vbar.Value = skipped_nodes;
+			vbar.Value = new_top.visible_order - VisibleCount;
 		}
 
 		private void HScrollBarValueChanged(object sender, EventArgs e)
@@ -1487,7 +1704,7 @@ namespace System.Windows.Forms {
 				return;
 			}
 
-			Rectangle below = new Rectangle (0, node.Bounds.Bottom + 2, ViewportRectangle.Width,
+			Rectangle below = new Rectangle (0, node.Bounds.Bottom, ViewportRectangle.Width,
 					ViewportRectangle.Height - node.Bounds.Bottom);
 
 			int amount = count_to_next * ActualItemHeight;
@@ -1496,7 +1713,6 @@ namespace System.Windows.Forms {
 				XplatUI.ScrollWindow (Handle, below, 0, amount, false);
 
 			if (show_plus_minus) {
-				//int linesx = node.GetLinesX ();
 				Invalidate (new Rectangle (0, node.GetY (), Width, ActualItemHeight));
 			}
 		}
@@ -1508,8 +1724,8 @@ namespace System.Windows.Forms {
 				return;
 			}
 
-			Rectangle below = new Rectangle (0, node.Bounds.Bottom + 2, ViewportRectangle.Width,
-					ViewportRectangle.Height - node.Bounds.Bottom + 2);
+			Rectangle below = new Rectangle (0, node.Bounds.Bottom, ViewportRectangle.Width,
+					ViewportRectangle.Height - node.Bounds.Bottom);
 
 			int amount = count_to_next * ActualItemHeight;
 
@@ -1517,7 +1733,6 @@ namespace System.Windows.Forms {
 				XplatUI.ScrollWindow (Handle, below, 0, -amount, false);
 
 			if (show_plus_minus) {
-				//int linesx = node.GetLinesX ();
 				Invalidate (new Rectangle (0, node.GetY (), Width, ActualItemHeight));
 			}
 		}
@@ -1544,7 +1759,12 @@ namespace System.Windows.Forms {
 
 		private void FontChangedHandler (object sender, EventArgs e)
 		{
-			InvalidateNodeWidthRecursive (root_node);
+			if (IsHandleCreated) {
+				TreeNode top = TopNode;
+				InvalidateNodeWidthRecursive (root_node);
+
+				SetTop (top);
+			}
 		}
 
 		private void InvalidateNodeWidthRecursive (TreeNode node)
@@ -1557,9 +1777,9 @@ namespace System.Windows.Forms {
 
 		private void GotFocusHandler (object sender, EventArgs e)
 		{
-			if (selected_node == null)
-				SelectedNode = top_node;
-			else
+			if (selected_node == null && pre_selected_node != null)
+				SelectedNode = pre_selected_node;
+			else if (selected_node != null)
 				UpdateNode (selected_node);
 		}
 
@@ -1626,16 +1846,20 @@ namespace System.Windows.Forms {
 
 			Rectangle invalid;
 			if (!ce.Cancel) {
-				if (focused_node != null) {
-					invalid = Rectangle.Union (Bloat (focused_node.Bounds),
-							Bloat (highlighted_node.Bounds));
-				} else {
-					invalid = Bloat (highlighted_node.Bounds);
-				}
+				TreeNode prev_focused_node = focused_node;
+				TreeNode prev_highlighted_node = highlighted_node;
+				
 				selected_node = highlighted_node;
 				focused_node = highlighted_node;
 				OnAfterSelect (new TreeViewEventArgs (selected_node, TreeViewAction.ByMouse));
 
+				if (prev_focused_node != null) {
+					invalid = Rectangle.Union (Bloat (prev_focused_node.Bounds),
+							Bloat (prev_highlighted_node.Bounds));
+				} else {
+					invalid = Bloat (prev_highlighted_node.Bounds);
+				}
+
 				Invalidate (invalid);
 			} else {
 				highlighted_node = focused_node;
@@ -1709,6 +1933,11 @@ namespace System.Windows.Forms {
 		static object BeforeExpandEvent = new object ();
 		static object BeforeLabelEditEvent = new object ();
 		static object BeforeSelectEvent = new object ();
+#if NET_2_0
+		static object DrawNodeEvent = new object ();
+		static object NodeMouseClickEvent = new object ();
+		static object NodeMouseDoubleClickEvent = new object();
+#endif
 
 		public event ItemDragEventHandler ItemDrag {
 			add { Events.AddHandler (ItemDragEvent, value); }
@@ -1765,6 +1994,24 @@ namespace System.Windows.Forms {
 			remove { Events.RemoveHandler (BeforeSelectEvent, value); }
 		}
 
+#if NET_2_0
+		public event DrawTreeNodeEventHandler DrawNode {
+			add { Events.AddHandler (DrawNodeEvent, value); }
+			remove { Events.RemoveHandler (DrawNodeEvent, value); }
+		}
+
+		public event TreeNodeMouseClickEventHandler NodeMouseClick {
+			add { Events.AddHandler (NodeMouseClickEvent, value); }
+			remove { Events.RemoveHandler (NodeMouseClickEvent, value); }
+		}
+
+
+		public event TreeNodeMouseClickEventHandler NodeMouseDoubleClick {
+			add { Events.AddHandler (NodeMouseDoubleClickEvent, value); }
+			remove { Events.RemoveHandler (NodeMouseDoubleClickEvent, value); }
+		}
+#endif
+
 		[Browsable (false)]
 		[EditorBrowsable (EditorBrowsableState.Never)]	
 		public new event EventHandler BackgroundImageChanged {
