@@ -80,6 +80,12 @@ namespace System.Windows.Forms {
 			}
 		}
 
+		internal virtual int MouseWheelScrollDelta {
+			get {
+				return 120;
+			}
+		}
+		
 		internal virtual Point MousePosition {
 			get {
 				return Point.Empty;
@@ -207,9 +213,10 @@ namespace System.Windows.Forms {
 		internal abstract IntPtr DispatchMessage(ref MSG msg);
 
 		internal abstract bool SetZOrder(IntPtr hWnd, IntPtr AfterhWnd, bool Top, bool Bottom);
-		internal abstract bool SetTopmost(IntPtr hWnd, IntPtr hWndOwner, bool Enabled);
+		internal abstract bool SetTopmost(IntPtr hWnd, bool Enabled);
+		internal abstract bool SetOwner(IntPtr hWnd, IntPtr hWndOwner);
 
-		internal abstract bool CalculateWindowRect(ref Rectangle ClientRect, int Style, int ExStyle, Menu menu, out Rectangle WindowRect);
+		internal abstract bool CalculateWindowRect(ref Rectangle ClientRect, CreateParams cp, Menu menu, out Rectangle WindowRect);
 
 		internal abstract Region GetClipRegion(IntPtr hwnd);
 		internal abstract void SetClipRegion(IntPtr hwnd, Region region);
@@ -253,6 +260,9 @@ namespace System.Windows.Forms {
 		internal abstract bool SystrayAdd(IntPtr hwnd, string tip, Icon icon, out ToolTip tt);
 		internal abstract bool SystrayChange(IntPtr hwnd, string tip, Icon icon, ref ToolTip tt);
 		internal abstract void SystrayRemove(IntPtr hwnd, ref ToolTip tt);
+#if NET_2_0		
+		internal abstract void SystrayBalloon(IntPtr hwnd, int timeout, string title, string text, ToolTipIcon icon);
+#endif
 
 		internal abstract Point GetMenuOrigin(IntPtr hwnd);
 		internal abstract void MenuToScreen(IntPtr hwnd, ref int x, ref int y);
