@@ -8,6 +8,7 @@
 //
 
 using System;
+using System.ComponentModel;
 using System.Drawing;
 using System.Reflection;
 using System.Windows.Forms;
@@ -17,7 +18,7 @@ using NUnit.Framework;
 namespace MonoTests.System.Windows.Forms
 {
 	[TestFixture]
-	public class RichTextBoxBaseTest
+	public class RichTextBoxTest
 	{
 #if not
 		[Test]
@@ -71,8 +72,6 @@ namespace MonoTests.System.Windows.Forms
 
 			// S
 	
-			// [MonoTODO ("Assert.AreEqual (ScrollBars.Both, rTBox.ScrollBars, "#S1");")]
-			Assert.AreEqual (RichTextBoxScrollBars.Both, rTBox.ScrollBars, "#S1");
 			Assert.AreEqual ("", rTBox.SelectedText, "#S3");
 			rTBox.Text = "sample TextBox";
 			Assert.AreEqual (HorizontalAlignment.Left, rTBox.SelectionAlignment, "#S5");
@@ -111,7 +110,18 @@ namespace MonoTests.System.Windows.Forms
 			Assert.AreEqual (true, rTextBox.CanPaste (myFormat), "#Mtd1");
 		}
 #endif
-		
+
+		[Test]
+		public void BackColor ()
+		{
+			RichTextBox rtb = new RichTextBox ();
+			Assert.AreEqual (SystemColors.Window, rtb.BackColor, "#1");
+			rtb.BackColor = Color.Red;
+			Assert.AreEqual (Color.Red, rtb.BackColor, "#2");
+			rtb.BackColor = Color.Empty;
+			Assert.AreEqual (SystemColors.Window, rtb.BackColor, "#3");
+		}
+
 		[Test]
 		public void FindCharTest ()
 		{
@@ -140,7 +150,6 @@ namespace MonoTests.System.Windows.Forms
 		}
 
 		[Test]
-		[Category ("NotWorking")] // Find21 is failing with -1 instead of 72
 		public void FindTest() {
 			RichTextBox t = new RichTextBox();
 
@@ -194,5 +203,167 @@ namespace MonoTests.System.Windows.Forms
 			rtb.Font = new Font ("Arial", 16);
 			Assert.AreEqual (25, rtb.PreferredHeight, "#B3");
 		}
+
+		[Test]
+		public void ReadOnly_BackColor_NotSet ()
+		{
+			RichTextBox rtb = new RichTextBox ();
+			rtb.ReadOnly = true;
+			Assert.IsTrue (rtb.ReadOnly, "#A1");
+#if NET_2_0
+			Assert.AreEqual (SystemColors.Control, rtb.BackColor, "#A2");
+#else
+			Assert.AreEqual (SystemColors.Window, rtb.BackColor, "#A2");
+#endif
+
+			Form form = new Form ();
+			form.ShowInTaskbar = false;
+			form.Controls.Add (rtb);
+			form.Show ();
+
+			Assert.IsTrue (rtb.ReadOnly, "#B1");
+#if NET_2_0
+			Assert.AreEqual (SystemColors.Control, rtb.BackColor, "#B2");
+#else
+			Assert.AreEqual (SystemColors.Window, rtb.BackColor, "#B2");
+#endif
+
+			rtb.ResetBackColor ();
+			Assert.IsTrue (rtb.ReadOnly, "#C1");
+#if NET_2_0
+			Assert.AreEqual (SystemColors.Control, rtb.BackColor, "#C2");
+#else
+			Assert.AreEqual (SystemColors.Window, rtb.BackColor, "#C2");
+#endif
+
+			rtb.ReadOnly = false;
+			Assert.IsFalse (rtb.ReadOnly, "#D1");
+			Assert.AreEqual (SystemColors.Window, rtb.BackColor, "#D2");
+
+			rtb.ReadOnly = true;
+			Assert.IsTrue (rtb.ReadOnly, "#E1");
+#if NET_2_0
+			Assert.AreEqual (SystemColors.Control, rtb.BackColor, "#E2");
+#else
+			Assert.AreEqual (SystemColors.Window, rtb.BackColor, "#E2");
+#endif
+
+			rtb.BackColor = Color.Red;
+			Assert.IsTrue (rtb.ReadOnly, "#F1");
+			Assert.AreEqual (Color.Red, rtb.BackColor, "#F2");
+
+			rtb.ReadOnly = false;
+			Assert.IsFalse (rtb.ReadOnly, "#G1");
+			Assert.AreEqual (Color.Red, rtb.BackColor, "#G2");
+
+			rtb.ReadOnly = true;
+			Assert.IsTrue (rtb.ReadOnly, "#H1");
+			Assert.AreEqual (Color.Red, rtb.BackColor, "#H2");
+
+			rtb.ResetBackColor ();
+			Assert.IsTrue (rtb.ReadOnly, "#I1");
+#if NET_2_0
+			Assert.AreEqual (SystemColors.Control, rtb.BackColor, "#I2");
+#else
+			Assert.AreEqual (SystemColors.Window, rtb.BackColor, "#I2");
+#endif
+		}
+
+		[Test]
+		public void ReadOnly_BackColor_Set ()
+		{
+			RichTextBox rtb = new RichTextBox ();
+			rtb.BackColor = Color.Blue;
+			rtb.ReadOnly = true;
+			Assert.IsTrue (rtb.ReadOnly, "#A1");
+			Assert.AreEqual (Color.Blue, rtb.BackColor, "#A2");
+
+			Form form = new Form ();
+			form.ShowInTaskbar = false;
+			form.Controls.Add (rtb);
+			form.Show ();
+
+			Assert.IsTrue (rtb.ReadOnly, "#B1");
+			Assert.AreEqual (Color.Blue, rtb.BackColor, "#B2");
+
+			rtb.ReadOnly = false;
+			Assert.IsFalse (rtb.ReadOnly, "#C1");
+			Assert.AreEqual (Color.Blue, rtb.BackColor, "#C2");
+
+			rtb.ReadOnly = true;
+			Assert.IsTrue (rtb.ReadOnly, "#D1");
+			Assert.AreEqual (Color.Blue, rtb.BackColor, "#D2");
+
+			rtb.BackColor = Color.Red;
+			Assert.IsTrue (rtb.ReadOnly, "#E1");
+			Assert.AreEqual (Color.Red, rtb.BackColor, "#E2");
+
+			rtb.ReadOnly = false;
+			Assert.IsFalse (rtb.ReadOnly, "#F1");
+			Assert.AreEqual (Color.Red, rtb.BackColor, "#F2");
+
+			rtb.ReadOnly = true;
+			rtb.ResetBackColor ();
+			Assert.IsTrue (rtb.ReadOnly, "#G1");
+#if NET_2_0
+			Assert.AreEqual (SystemColors.Control, rtb.BackColor, "#G2");
+#else
+			Assert.AreEqual (SystemColors.Window, rtb.BackColor, "#G2");
+#endif
+
+			form.Dispose ();
+
+			rtb = new RichTextBox ();
+			rtb.ReadOnly = true;
+			rtb.BackColor = Color.Blue;
+			Assert.IsTrue (rtb.ReadOnly, "#H1");
+			Assert.AreEqual (Color.Blue, rtb.BackColor, "#H2");
+
+			form = new Form ();
+			form.ShowInTaskbar = false;
+			form.Controls.Add (rtb);
+			form.Show ();
+
+			Assert.IsTrue (rtb.ReadOnly, "#I1");
+			Assert.AreEqual (Color.Blue, rtb.BackColor, "#I2");
+
+			rtb.ReadOnly = false;
+			Assert.IsFalse (rtb.ReadOnly, "#J1");
+			Assert.AreEqual (Color.Blue, rtb.BackColor, "#J2");
+
+			rtb.ResetBackColor ();
+			Assert.IsFalse (rtb.ReadOnly, "#K1");
+			Assert.AreEqual (SystemColors.Window, rtb.BackColor, "#K2");
+		}
+
+		[Test]
+		public void ScrollBarsTest ()
+		{
+			RichTextBox rtb = new RichTextBox ();
+			Assert.AreEqual (RichTextBoxScrollBars.Both, rtb.ScrollBars, "#1");
+			rtb.ScrollBars = RichTextBoxScrollBars.Vertical;
+			Assert.AreEqual (RichTextBoxScrollBars.Vertical, rtb.ScrollBars, "#2");
+		}
+
+		[Test]
+		[ExpectedException (typeof (InvalidEnumArgumentException))]
+		public void ScrollBars_Invalid ()
+		{
+			RichTextBox rtb = new RichTextBox ();
+			rtb.ScrollBars = (RichTextBoxScrollBars) 666;
+		}
+
+		[Test]
+		public void SelectionFontTest ()
+		{
+			RichTextBox t = new RichTextBox();
+			t.Text = "123";
+			t.SelectionStart = 1;
+			t.SelectionLength = 1;
+			Font f = new Font(FontFamily.GenericMonospace, 120);
+			t.SelectionFont = f;
+
+			Assert.AreEqual (t.SelectionFont.Size, f.Size, "A1");
+		}
 	}
 }
