@@ -5,6 +5,7 @@
 //   Duncan Mak (duncan@ximian.com)
 //   Miguel de Icaza (miguel@novell.com)
 //   Hisham Mardam Bey (hisham.mardambey@gmail.com)
+//   Alp Toker (alp@atoker.com)
 //
 // (C) Ximian Inc, 2003.
 // (C) Novell Inc, 2003.
@@ -147,6 +148,36 @@ namespace Cairo {
         public class Context : IDisposable 
         {
                 internal IntPtr state = IntPtr.Zero;
+
+		static int native_glyph_size, c_compiler_long_size;
+		
+		static Context ()
+		{
+			//
+			// This is used to determine what kind of structure
+			// we should use to marshal Glyphs, as the public
+			// definition in Cairo uses `long', which can be
+			// 32 bits or 64 bits depending on the platform.
+			//
+			// We assume that sizeof(long) == sizeof(void*)
+			// except in the case of Win64 where sizeof(long)
+			// is 32 bits
+			//
+			int ptr_size = Marshal.SizeOf (typeof (IntPtr));
+			
+			PlatformID platform = Environment.OSVersion.Platform;
+			if (platform == PlatformID.Win32NT ||
+			    platform == PlatformID.Win32S ||
+			    platform == PlatformID.Win32Windows ||
+			    platform == PlatformID.WinCE ||
+			    ptr_size == 4){
+				c_compiler_long_size = 4;
+				native_glyph_size = Marshal.SizeOf (typeof (NativeGlyph_4byte_longs));
+			} else {
+				c_compiler_long_size = 8;
+				native_glyph_size = Marshal.SizeOf (typeof (Glyph));
+			}
+		}
 		
                 public Context (Surface surface)
                 {
@@ -221,6 +252,7 @@ namespace Cairo {
                         }
                 }
                 
+                //FIXME: obsolete this property
                 public Cairo.Color Color {
 			set { 
 				CairoAPI.cairo_set_source_rgba (state, value.R, 
@@ -349,10 +381,16 @@ namespace Cairo {
 			Color = new Color (r, g, b, a);
 		}
 
-				public void SetSourceSurface (Surface source, int x, int y)
-				{
-					CairoAPI.cairo_set_source_surface (state, source.Handle, x, y);
-				}
+		//[Obsolete ("Use SetSource method (with double parameters)")]
+		public void SetSourceSurface (Surface source, int x, int y)
+		{
+			CairoAPI.cairo_set_source_surface (state, source.Handle, x, y);
+		}
+
+		public void SetSource (Surface source, double x, double y)
+		{
+			CairoAPI.cairo_set_source_surface (state, source.Handle, x, y);
+		}
 
 #region Path methods
                 
@@ -578,25 +616,7 @@ namespace Cairo {
 		public Surface GroupTarget {
 			get {
 				IntPtr surface = CairoAPI.cairo_get_group_target (state);
-				SurfaceType st = CairoAPI.cairo_surface_get_type (surface);
-				switch (st) {
-					case SurfaceType.Image:
-						return new ImageSurface (surface, true);
-					case SurfaceType.XLib:
-						return new XlibSurface (surface, true);
-					case SurfaceType.Win32:
-						return new Win32Surface (surface, true);
-#if CAIRO_1_2
-					case SurfaceType.Pdf:
-						return new PdfSurface (surface, true);
-					case SurfaceType.PS:
-						return new PSSurface (surface, true);
-					case SurfaceType.Svg:
-						return new SvgSurface (surface, true);
-#endif
-					default:
-						return Surface.LookupExternalSurface (surface);
-				}
+				return Surface.LookupSurface (surface);
 			}
 		}
 #endif
@@ -622,6 +642,7 @@ namespace Cairo {
 		}
                 	
 
+		//FIXME: obsolete these methods
 		public void TransformPoint (ref double x, ref double y)
 		{
                 	CairoAPI.cairo_user_to_device (state, ref x, ref y);
@@ -684,22 +705,44 @@ namespace Cairo {
 			set { CairoAPI.cairo_set_font_options (state, value.Handle); }
 		}
 
+		[StructLayout(LayoutKind.Sequential)]
+		internal struct NativeGlyph_4byte_longs {
+			public int index;
+			public double x;
+			public double y;
+
+			public NativeGlyph_4byte_longs (Glyph source)
+			{
+				index = (int) source.index;
+				x = source.x;
+				y = source.y;
+			}
+		}
+
 		static internal IntPtr FromGlyphToUnManagedMemory(Glyph [] glyphs)
 		{
-			int size =  Marshal.SizeOf (glyphs[0]);
-			IntPtr dest = Marshal.AllocHGlobal (size * glyphs.Length);
+			IntPtr dest = Marshal.AllocHGlobal (native_glyph_size * glyphs.Length);
 			int pos = dest.ToInt32();
 
-			for (int i=0; i < glyphs.Length; i++, pos += size)
-				Marshal.StructureToPtr (glyphs[i], (IntPtr)pos, false);
+			if (c_compiler_long_size == 8){
+				foreach (Glyph g in glyphs){
+					Marshal.StructureToPtr (g, (IntPtr)pos, false);
+					pos += native_glyph_size;
+				}
+			} else {
+				foreach (Glyph g in glyphs){
+					NativeGlyph_4byte_longs n = new NativeGlyph_4byte_longs (g);
+					
+					Marshal.StructureToPtr (n, (IntPtr)pos, false);
+					pos += native_glyph_size;
+				}
+			}
 
 			return dest;
 		}
 
-
-                public void ShowGlyphs (Matrix matrix, Glyph[] glyphs)
-                {
-
+                public void ShowGlyphs (Glyph[] glyphs)
+		{
                         IntPtr ptr;
 
                         ptr = FromGlyphToUnManagedMemory (glyphs);
@@ -707,12 +750,22 @@ namespace Cairo {
                         CairoAPI.cairo_show_glyphs (state, ptr, glyphs.Length);
 
                         Marshal.FreeHGlobal (ptr);		
-                     
+		}
+
+		[Obsolete("The matrix argument was never used, use ShowGlyphs(Glyphs []) instead")]
+                public void ShowGlyphs (Matrix matrix, Glyph[] glyphs)
+                {
+			ShowGlyphs (glyphs);
                 }
 
+		[Obsolete("The matrix argument was never used, use GlyphPath(Glyphs []) instead")]
                 public void GlyphPath (Matrix matrix, Glyph[] glyphs)
                 {
+			GlyphPath (glyphs);
+		}
 
+		public void GlyphPath (Glyph[] glyphs)
+		{
                         IntPtr ptr;
 
                         ptr = FromGlyphToUnManagedMemory (glyphs);
@@ -765,8 +818,21 @@ namespace Cairo {
 
 		public TextExtents TextExtents (string utf8)
 		{
-			TextExtents extents = new TextExtents ();
-			CairoAPI.cairo_text_extents (state, utf8, ref extents);
+			TextExtents extents;
+			CairoAPI.cairo_text_extents (state, utf8, out extents);
+			return extents;
+		}
+
+		public TextExtents GlyphExtents (Glyph[] glyphs)
+		{
+			IntPtr ptr = FromGlyphToUnManagedMemory (glyphs);
+
+			TextExtents extents;
+
+			CairoAPI.cairo_glyph_extents (state, ptr, glyphs.Length, out extents);
+
+			Marshal.FreeHGlobal (ptr);
+
 			return extents;
 		}
         }
