@@ -46,6 +46,7 @@ namespace Mono.Data.Tds {
 		bool isSizeSet = false;
 		bool isNullable;
 		object value;
+		bool isVariableSizeType;
 
 		#endregion // Fields
 
@@ -126,6 +127,12 @@ namespace Mono.Data.Tds {
 			}
 		}
 
+		public bool IsVariableSizeType
+		{
+			get { return isVariableSizeType; }
+			set { isVariableSizeType = value; }
+		}
+
 		#endregion // Properties
 
 		#region Methods
@@ -145,7 +152,10 @@ namespace Mono.Data.Tds {
 				}
 			}
 			
-			StringBuilder result = new StringBuilder (String.Format ("{0} {1}", ParameterName, typeName));
+			string includeAt = "@";
+			if (ParameterName [0] == '@')
+				includeAt = "";
+			StringBuilder result = new StringBuilder (String.Format ("{0}{1} {2}", includeAt, ParameterName, typeName));
 			switch (typeName) {
 			case "decimal":
 			case "numeric":
@@ -270,6 +280,19 @@ namespace Mono.Data.Tds {
 			}
 		}
 
+		public void Validate (int index)
+		{
+			if ((this.direction == TdsParameterDirection.InputOutput || this.direction == TdsParameterDirection.Output) &&
+				 this.isVariableSizeType && (Value == DBNull.Value || Value == null) && Size == 0
+				) 
+			{
+				throw new InvalidOperationException (String.Format ("{0}[{1}]: the Size property should " +
+												"not be of size 0",
+												this.typeName,
+												index));
+			}
+		}
+
 		#endregion // Methods
 	}
 }
