@@ -32,11 +32,14 @@ using System.Text;
 using System.Collections;
 using System.Collections.Generic;
 
+
 using System.Data;
 using System.Data.Common;
+using System.ComponentModel;
 
 namespace System.Data.SqlClient
 {
+	[DefaultPropertyAttribute ("DataSource")]
 	public sealed class SqlConnectionStringBuilder : DbConnectionStringBuilder
 	{
 
@@ -64,6 +67,10 @@ namespace System.Data.SqlClient
 		private const bool 	DEF_REPLICATION 		= false;
 		private const string 	DEF_USERID 			= "";
 		private const string 	DEF_WORKSTATIONID 		= "";
+		private const string 	DEF_TYPESYSTEMVERSION 		= "Latest";
+		private const bool	DEF_TRUSTSERVERCERTIFICATE	= false;
+		private const bool	DEF_USERINSTANCE		= false;  
+		private const bool	DEF_CONTEXTCONNECTION		= false;	
 
 
 		#region // Fields
@@ -91,6 +98,10 @@ namespace System.Data.SqlClient
 		private bool 	_replication;
 		private string 	_userID;
 		private string 	_workstationID;
+		private bool	_trustServerCertificate;
+		private string	_typeSystemVersion;
+		private bool	_userInstance;
+		private bool	_contextConnection;
 
 		private static Dictionary <string, string> _keywords; // for mapping duplicate keywords
 		#endregion // Fields
@@ -149,12 +160,15 @@ namespace System.Data.SqlClient
 			_keywords ["USER"] 			= "User Id";
 			_keywords ["USER ID"] 			= "User Id";
 			_keywords ["WSID"] 			= "Workstation Id";
-			_keywords ["WORKSTATION ID"] 		= "Workstation Id";
-
+			_keywords ["WORKSTATION ID"]		= "Workstation Id";
+			_keywords ["USER INSTANCE"]		= "User Instance";
+			_keywords ["CONTEXT CONNECTION"]	= "Context Connection";
 		}
 		#endregion // Constructors
 
 		#region Properties
+		[DisplayNameAttribute ("Application Name")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public string ApplicationName { 
 			get { return _applicationName; }
 			set { 
@@ -163,6 +177,8 @@ namespace System.Data.SqlClient
 			}
 		}
 
+		[DisplayNameAttribute ("Asynchronous Processing")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public bool AsynchronousProcessing { 
 			get { return _asynchronousProcessing; }
 			set { 
@@ -171,6 +187,8 @@ namespace System.Data.SqlClient
 			}
 		}
 
+		[DisplayNameAttribute ("AttachDbFilename")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public string AttachDBFilename { 
 			get { return _attachDBFilename; }
 			set { 
@@ -178,7 +196,9 @@ namespace System.Data.SqlClient
 				_attachDBFilename = value; 
 			}
 		}
-
+		
+		[DisplayNameAttribute ("Connection Reset")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public bool ConnectionReset { 
 			get { return _connectionReset; }
 			set { 
@@ -186,7 +206,9 @@ namespace System.Data.SqlClient
 				_connectionReset = value; 
 			}
 		}
-
+		
+		[DisplayNameAttribute ("Connect Timeout")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public int ConnectTimeout { 
 			get { return _connectTimeout; }
 			set { 
@@ -195,6 +217,8 @@ namespace System.Data.SqlClient
 			}
 		}
 
+		[DisplayNameAttribute ("Current Language")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public string CurrentLanguage { 
 			get { return _currentLanguage; }
 			set { 
@@ -203,6 +227,8 @@ namespace System.Data.SqlClient
 			}
 		}
 
+		[DisplayNameAttribute ("Data Source")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public string DataSource { 
 			get { return _dataSource; }
 			set { 
@@ -211,6 +237,8 @@ namespace System.Data.SqlClient
 			}
 		}
 
+		[DisplayNameAttribute ("Encrypt")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public bool Encrypt { 
 			get { return _encrypt; }
 			set { 
@@ -219,6 +247,8 @@ namespace System.Data.SqlClient
 			}
 		}
 
+		[DisplayNameAttribute ("Enlist")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public bool Enlist { 
 			get { return _enlist; }
 			set { 
@@ -227,6 +257,8 @@ namespace System.Data.SqlClient
 			}
 		}
 
+		[DisplayNameAttribute ("Failover Partner")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public string FailoverPartner { 
 			get { return _failoverPartner; }
 			set { 
@@ -235,6 +267,8 @@ namespace System.Data.SqlClient
 			}
 		}
 
+		[DisplayNameAttribute ("Initial Catalog")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public string InitialCatalog { 
 			get { return _initialCatalog; }
 			set { 
@@ -243,6 +277,8 @@ namespace System.Data.SqlClient
 			}
 		}
 
+		[DisplayNameAttribute ("Integrated Security")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public bool IntegratedSecurity { 
 			get { return _integratedSecurity; }
 			set { 
@@ -267,6 +303,8 @@ namespace System.Data.SqlClient
 			get { return base.Keys; }
 		}
 
+		[DisplayNameAttribute ("Load Balance Timeout")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public int LoadBalanceTimeout { 
 			get { return _loadBalanceTimeout; }
 			set { 
@@ -275,6 +313,8 @@ namespace System.Data.SqlClient
 			}
 		}
 
+		[DisplayNameAttribute ("Max Pool Size")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public int MaxPoolSize { 
 			get { return _maxPoolSize; }
 			set { 
@@ -283,6 +323,8 @@ namespace System.Data.SqlClient
 			}
 		}
 
+		[DisplayNameAttribute ("Min Pool Size")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public int MinPoolSize { 
 			get { return _minPoolSize; }
 			set {
@@ -291,6 +333,8 @@ namespace System.Data.SqlClient
 			}
 		}
 
+		[DisplayNameAttribute ("MultipleActiveResultSets")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public bool MultipleActiveResultSets { 
 			get { return _multipleActiveResultSets; }
 			set { 
@@ -299,6 +343,8 @@ namespace System.Data.SqlClient
 			}
 		}
 
+		[DisplayNameAttribute ("Network Library")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public string NetworkLibrary { 
 			get { return _networkLibrary; }
 			set { 
@@ -307,6 +353,8 @@ namespace System.Data.SqlClient
 			}
 		}
 
+		[DisplayNameAttribute ("Packet Size")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public int PacketSize { 
 			get { return _packetSize; }
 			set { 
@@ -315,6 +363,9 @@ namespace System.Data.SqlClient
 			}
 		}
 
+		[DisplayNameAttribute ("Password")]
+		[PasswordPropertyTextAttribute (true)]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public string Password { 
 			get { return _password; }
 			set { 
@@ -323,6 +374,8 @@ namespace System.Data.SqlClient
 			}
 		}
 
+		[DisplayNameAttribute ("Persist Security Info")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public bool PersistSecurityInfo { 
 			get { return _persistSecurityInfo; }
 			set {
@@ -330,7 +383,9 @@ namespace System.Data.SqlClient
 				_persistSecurityInfo = value; 
 			}
 		}
-
+		
+		[DisplayNameAttribute ("Pooling")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public bool Pooling { 
 			get { return _pooling; }
 			set {
@@ -339,6 +394,8 @@ namespace System.Data.SqlClient
 			}
 		}
 
+		[DisplayNameAttribute ("Replication")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public bool Replication { 
 			get { return _replication; }
 			set { 
@@ -347,6 +404,8 @@ namespace System.Data.SqlClient
 			}
 		}
 
+		[DisplayNameAttribute ("User ID")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public string UserID { 
 			get { return _userID; }
 			set { 
@@ -359,6 +418,8 @@ namespace System.Data.SqlClient
 			get { return base.Values; }
 		}
 
+		[DisplayNameAttribute ("Workstation ID")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
 		public string WorkstationID { 
 			get { return _workstationID; }
 			set { 
@@ -366,6 +427,38 @@ namespace System.Data.SqlClient
 				_workstationID = value; 
 			}
 		}
+
+		public bool TrustServerCertificate { 
+			get { return _trustServerCertificate; } 
+			set {
+				base ["Trust Server Certificate"] = value;
+				 _trustServerCertificate = value;
+			}
+		}
+
+		public string TypeSystemVersion { 
+			get { return _typeSystemVersion; } 
+			set {
+				base ["Type System Version"] = value;				
+				_typeSystemVersion = value; 
+			}
+		}
+
+		public bool UserInstance { 
+			get { return _userInstance; }
+			set { 
+				base ["User Instance"] = value;
+				_userInstance = value;
+			}
+		}
+
+		public bool ContextConnection { 
+			get { return _contextConnection; }
+			set { 
+				base ["Context Connection"] = value;
+				_contextConnection = value;
+			}
+		}
 		#endregion // Properties
 
 		#region Methods
@@ -395,6 +488,10 @@ namespace System.Data.SqlClient
 			_replication		= DEF_REPLICATION;
 			_userID			= DEF_USERID;
 			_workstationID		= DEF_WORKSTATIONID;
+			_trustServerCertificate	= DEF_TRUSTSERVERCERTIFICATE;
+			_typeSystemVersion	= DEF_TYPESYSTEMVERSION;
+			_userInstance		= DEF_USERINSTANCE;
+			_contextConnection	= DEF_CONTEXTCONNECTION;
 		}
 
 		public override void Clear ()
@@ -491,6 +588,13 @@ namespace System.Data.SqlClient
 				} else 
 					this.CurrentLanguage = value.ToString ();
 				break;
+			case "CONTEXT CONNECTION" :
+				if (value == null) {
+					_contextConnection = DEF_CONTEXTCONNECTION;
+					base.Remove (mappedKey);
+				} else
+					this.ContextConnection = DbConnectionStringBuilderHelper.ConvertToBoolean (value);
+				break;
 			case "DATA SOURCE" :
 				if (value == null) {
 					_dataSource = DEF_DATASOURCE;
@@ -604,6 +708,13 @@ namespace System.Data.SqlClient
 				} else 
 					this.UserID = value.ToString ();
 				break;
+			case "USER INSTANCE" :
+				if (value == null) {
+					_userInstance = DEF_USERINSTANCE;
+					base.Remove (mappedKey);
+				} else
+					this.UserInstance = DbConnectionStringBuilderHelper.ConvertToBoolean (value);
+				break;
 			case "WORKSTATION ID" :
 				if (value == null) {
 					_workstationID = DEF_WORKSTATIONID;
