@@ -61,12 +61,15 @@ namespace System.Data.SqlClient {
 		DbType dbType;
 		ParameterDirection direction = ParameterDirection.Input;
 		bool isNullable;
-		bool isVariableSizeType = false;
 		bool isTypeSet = false;
 		int offset;
 		SqlDbType sqlDbType;
 		string sourceColumn;
 		DataRowVersion sourceVersion;
+		SqlCompareOptions compareInfo;
+		int localeId;
+		Object sqlValue;
+		string udtTypeName;
 
 		#endregion // Fields
 
@@ -170,14 +173,14 @@ namespace System.Data.SqlClient {
 			if (!isTypeSet)
 				throw new Exception ("all parameters to have an explicity set type");
 
-			if (isVariableSizeType) {
+			if (MetaParameter.IsVariableSizeType) {
 				if (SqlDbType == SqlDbType.Decimal && Precision == 0)
 					throw new Exception ("Parameter of type 'Decimal' have an explicitly set Precision and Scale");
 				else if (Size == 0)
 					throw new Exception ("all variable length parameters to have an explicitly set non-zero Size");
 			}
 		}
-
+	
 #if ONLY_1_0 || ONLY_1_1	
 		[Browsable (false)]
 		[DataSysDescription ("The parameter generic type.")]
@@ -275,25 +278,21 @@ namespace System.Data.SqlClient {
 		}
 
 		[DataCategory ("Data")]
+		[DefaultValue (0)]
 #if ONLY_1_0 || ONLY_1_1
 		[DataSysDescription ("For decimal, numeric, varnumeric DBTypes.")]
-		[DefaultValue (0)]
-#endif
-#if NET_2_0
 		[Browsable (false)]
 		[EditorBrowsable (EditorBrowsableState.Never)]
-#endif		
+#endif
 		public byte Precision {
 			get { return metaParameter.Precision; }
 			set { metaParameter.Precision = value; }
 		}
 
 		[DataCategory ("Data")]
+		[DefaultValue (0)]
 #if ONLY_1_0 || ONLY_1_1
 		[DataSysDescription ("For decimal, numeric, varnumeric DBTypes.")]
-		[DefaultValue (0)]
-#endif
-#if NET_2_0
 		[Browsable (false)]
                 [EditorBrowsable (EditorBrowsableState.Never)]
 #endif
@@ -383,11 +382,23 @@ namespace System.Data.SqlClient {
 			}
 		}
 
-//#if NET_2_0
-//		public SqlCompareOptions CompareInfo{
-
-//#endif
 #if NET_2_0
+		[Browsable (false)]
+		public SqlCompareOptions CompareInfo{
+		 	get{ return compareInfo; } 
+			set{ compareInfo = value; }
+		}
+
+		public int LocaleId { 
+			get { return localeId; }
+			set { localeId = value; }
+		}
+
+		public Object SqlValue { 
+			get { return sqlValue; }
+			set { sqlValue = value; }
+		}
+	
 		public override bool SourceColumnNullMapping {
 			get { return false ; }
 			set { }
@@ -487,17 +498,17 @@ namespace System.Data.SqlClient {
 			case DbType.AnsiString:
 				MetaParameter.TypeName = "varchar";
 				sqlDbType = SqlDbType.VarChar;
-				isVariableSizeType = true;
+				MetaParameter.IsVariableSizeType = true;
 				break;
 			case DbType.AnsiStringFixedLength:
 				MetaParameter.TypeName = "char";
 				sqlDbType = SqlDbType.Char;
-				isVariableSizeType = true;
+				MetaParameter.IsVariableSizeType = true;
 				break;
 			case DbType.Binary:
 				MetaParameter.TypeName = "varbinary";
 				sqlDbType = SqlDbType.VarBinary;
-				isVariableSizeType = true;
+				MetaParameter.IsVariableSizeType = true;
 				break;
 			case DbType.Boolean:
 				MetaParameter.TypeName = "bit";
@@ -551,12 +562,12 @@ namespace System.Data.SqlClient {
 			case DbType.String:
 				MetaParameter.TypeName = "nvarchar";
 				sqlDbType = SqlDbType.NVarChar;
-				isVariableSizeType = true;
+				MetaParameter.IsVariableSizeType = true;
 				break;
 			case DbType.StringFixedLength:
 				MetaParameter.TypeName = "nchar";
 				sqlDbType = SqlDbType.NChar;
-				isVariableSizeType = true;
+				MetaParameter.IsVariableSizeType = true;
 				break;
 			case DbType.Time:
 				MetaParameter.TypeName = "datetime";
@@ -662,7 +673,7 @@ namespace System.Data.SqlClient {
 			case SqlDbType.Binary:
 				MetaParameter.TypeName = "binary";
 				dbType = DbType.Binary;
-				isVariableSizeType = true;
+				MetaParameter.IsVariableSizeType = true;
 				break;
 			case SqlDbType.Timestamp:
 				MetaParameter.TypeName = "timestamp";
@@ -671,7 +682,7 @@ namespace System.Data.SqlClient {
 			case SqlDbType.VarBinary:
 				MetaParameter.TypeName = "varbinary";
 				dbType = DbType.Binary;
-				isVariableSizeType = true;
+				MetaParameter.IsVariableSizeType = true;
 				break;
 			case SqlDbType.Bit:
 				MetaParameter.TypeName = "bit";
@@ -680,7 +691,7 @@ namespace System.Data.SqlClient {
 			case SqlDbType.Char:
 				MetaParameter.TypeName = "char";
 				dbType = DbType.AnsiStringFixedLength;
-				isVariableSizeType = true;
+				MetaParameter.IsVariableSizeType = true;
 				break;
 			case SqlDbType.DateTime:
 				MetaParameter.TypeName = "datetime";
@@ -701,7 +712,7 @@ namespace System.Data.SqlClient {
 			case SqlDbType.Image:
 				MetaParameter.TypeName = "image";
 				dbType = DbType.Binary;
-				isVariableSizeType = true;
+				MetaParameter.IsVariableSizeType = true;
 				break;
 			case SqlDbType.Int:
 				MetaParameter.TypeName = "int";
@@ -718,17 +729,17 @@ namespace System.Data.SqlClient {
 			case SqlDbType.NChar:
 				MetaParameter.TypeName = "nchar";
 				dbType = DbType.StringFixedLength;
-				isVariableSizeType = true;
+				MetaParameter.IsVariableSizeType = true;
 				break;
 			case SqlDbType.NText:
 				MetaParameter.TypeName = "ntext";
 				dbType = DbType.String;
-				isVariableSizeType = true;
+				MetaParameter.IsVariableSizeType = true;
 				break;
 			case SqlDbType.NVarChar:
 				MetaParameter.TypeName = "nvarchar";
 				dbType = DbType.String;
-				isVariableSizeType = true;
+				MetaParameter.IsVariableSizeType = true;
 				break;
 			case SqlDbType.Real:
 				MetaParameter.TypeName = "real";
@@ -741,12 +752,12 @@ namespace System.Data.SqlClient {
 			case SqlDbType.Text:
 				MetaParameter.TypeName = "text";
 				dbType = DbType.AnsiString;
-				isVariableSizeType = true;
+				MetaParameter.IsVariableSizeType = true;
 				break;
 			case SqlDbType.VarChar:
 				MetaParameter.TypeName = "varchar";
 				dbType = DbType.AnsiString;
-				isVariableSizeType = true;
+				MetaParameter.IsVariableSizeType = true;
 				break;
 			case SqlDbType.TinyInt:
 				MetaParameter.TypeName = "tinyint";
