@@ -688,6 +688,8 @@ namespace System.Data
 			dataTable.RowDeleted     += new DataRowChangeEventHandler(OnRowDeleted);
 			dataTable.Columns.CollectionChanged += new CollectionChangeEventHandler(ColumnCollectionChanged);
 			dataTable.Constraints.CollectionChanged += new CollectionChangeEventHandler(OnConstraintCollectionChanged);
+			dataTable.ChildRelations.CollectionChanged += new CollectionChangeEventHandler(OnRelationCollectionChanged);
+			dataTable.ParentRelations.CollectionChanged += new CollectionChangeEventHandler(OnRelationCollectionChanged);
 
 			dataTable.Rows.ListChanged += new ListChangedEventHandler (OnRowCollectionChanged);
 		}
@@ -710,6 +712,8 @@ namespace System.Data
 			dataTable.RowDeleted     -= new DataRowChangeEventHandler(OnRowDeleted);
 			dataTable.Columns.CollectionChanged -= new CollectionChangeEventHandler(ColumnCollectionChanged);
 			dataTable.Constraints.CollectionChanged -= new CollectionChangeEventHandler(OnConstraintCollectionChanged);
+			dataTable.ChildRelations.CollectionChanged -= new CollectionChangeEventHandler(OnRelationCollectionChanged);
+			dataTable.ParentRelations.CollectionChanged -= new CollectionChangeEventHandler(OnRelationCollectionChanged);
 
 			dataTable.Rows.ListChanged -= new ListChangedEventHandler (OnRowCollectionChanged);
 		}
@@ -783,6 +787,7 @@ namespace System.Data
 			if (args.Action == CollectionChangeAction.Refresh)
 				OnListChanged (new ListChangedEventArgs (ListChangedType.PropertyDescriptorChanged,0,0));
 		}
+
 		private void OnConstraintCollectionChanged(object sender, CollectionChangeEventArgs args)
 		{
 			//	The Sort variable is set to the UniqueConstraint column.
@@ -799,6 +804,19 @@ namespace System.Data
 			// UpdateIndex() is not invoked here.
 		}
 
+		private void OnRelationCollectionChanged(object sender, CollectionChangeEventArgs args)
+		{
+			/* PropertyDescriptor Add */
+			if (args.Action == CollectionChangeAction.Add)
+				OnListChanged (new ListChangedEventArgs (ListChangedType.PropertyDescriptorAdded,0,0));
+			/* PropertyDescriptor Removed */
+			if (args.Action == CollectionChangeAction.Remove)
+				OnListChanged (new ListChangedEventArgs (ListChangedType.PropertyDescriptorDeleted,0,0));
+			/* FIXME: PropertyDescriptor Changed ???*/
+			if (args.Action == CollectionChangeAction.Refresh)
+				OnListChanged (new ListChangedEventArgs (ListChangedType.PropertyDescriptorChanged,0,0));
+		}
+
 		// internal use by Mono
 		protected void Reset() 
 		{
@@ -956,6 +974,9 @@ namespace System.Data
 
 		PropertyDescriptorCollection ITypedList.GetItemProperties (PropertyDescriptor[] listAccessors) 
 		{
+			if (dataTable == null)
+				return new PropertyDescriptorCollection (new PropertyDescriptor[0]);
+
 			// FIXME: use listAccessors somehow
 			PropertyDescriptor [] descriptors = 
 				new PropertyDescriptor [dataTable.Columns.Count + dataTable.ChildRelations.Count];
