@@ -32,6 +32,7 @@
 
 using NUnit.Framework;
 using System;
+using System.Xml;
 using System.Data.SqlTypes;
 using System.Threading;
 using System.Globalization;
@@ -963,6 +964,102 @@ namespace MonoTests.System.Data.SqlTypes
 			Assert.AreEqual ((byte)0, SqlBoolean.Zero.ByteValue, "Zero field does not work correctly");
 
 		}
+#if NET_2_0
+		[Test]
+		public void GetXsdTypeTest ()
+		{
+			XmlQualifiedName qualifiedName = SqlBoolean.GetXsdType (null);
+			NUnit.Framework.Assert.AreEqual ("boolean", qualifiedName.Name, "#A01");
+		}
+
+		[Test]
+		public void GreaterThanTest ()
+		{
+			SqlBoolean x = new SqlBoolean (-1);
+			SqlBoolean y = new SqlBoolean (true);
+			SqlBoolean z = new SqlBoolean ();
+			SqlBoolean z1 = new SqlBoolean (0);
+
+			NUnit.Framework.Assert.IsFalse ((x > y).Value, "#A01");
+			NUnit.Framework.Assert.AreEqual (x > z, SqlBoolean.Null, "#A02");
+			NUnit.Framework.Assert.IsTrue ((x > z1).Value, "#A03");
+			NUnit.Framework.Assert.AreEqual (y > z, SqlBoolean.Null, "#A04");
+			NUnit.Framework.Assert.IsFalse ((y > x).Value, "#A05");
+			NUnit.Framework.Assert.IsTrue ((y > z1).Value, "#A06");
+			NUnit.Framework.Assert.AreEqual (z > z1, SqlBoolean.Null, "#A07");
+			NUnit.Framework.Assert.AreEqual (z > x, SqlBoolean.Null, "#A08");
+			NUnit.Framework.Assert.AreEqual (z > y, SqlBoolean.Null, "#A09");
+			NUnit.Framework.Assert.AreEqual (z1 > z, SqlBoolean.Null, "#A10");
+			NUnit.Framework.Assert.IsFalse ((z1 > x).Value, "#A11");
+			NUnit.Framework.Assert.IsFalse ((z1 > y).Value, "#A12");
+		}
+
+		[Test]
+		public void GreaterThanOrEqualTest ()
+		{
+			SqlBoolean x = new SqlBoolean (-1);
+			SqlBoolean y = new SqlBoolean (true);
+			SqlBoolean z = new SqlBoolean ();
+			SqlBoolean z1 = new SqlBoolean (0);
+
+			NUnit.Framework.Assert.IsTrue ((x >= y).Value, "#A01");
+			NUnit.Framework.Assert.AreEqual (x >= z, SqlBoolean.Null, "#A02");
+			NUnit.Framework.Assert.IsTrue ((x >= z1).Value, "#A03");
+			NUnit.Framework.Assert.AreEqual (y >= z, SqlBoolean.Null, "#A04");
+			NUnit.Framework.Assert.IsTrue ((y >= x).Value, "#A05");
+			NUnit.Framework.Assert.IsTrue ((y >= z1).Value, "#A06");
+			NUnit.Framework.Assert.AreEqual (z >= z1, SqlBoolean.Null, "#A07");
+			NUnit.Framework.Assert.AreEqual (z >= x, SqlBoolean.Null, "#A08");
+			NUnit.Framework.Assert.AreEqual (z >= y, SqlBoolean.Null, "#A09");
+			NUnit.Framework.Assert.AreEqual (z1 >= z, SqlBoolean.Null, "#A10");
+			NUnit.Framework.Assert.IsFalse ((z1 >= x).Value, "#A11");
+			NUnit.Framework.Assert.IsFalse ((z1 >= y).Value, "#A12");
+		}
+
+		[Test]
+		public void LessThanTest ()
+		{
+			SqlBoolean x = new SqlBoolean (-1);
+			SqlBoolean y = new SqlBoolean (true);
+			SqlBoolean z = new SqlBoolean ();
+			SqlBoolean z1 = new SqlBoolean (0);
+
+			NUnit.Framework.Assert.IsFalse ((x < y).Value, "#A01");
+			NUnit.Framework.Assert.AreEqual (x < z, SqlBoolean.Null, "#A02");
+			NUnit.Framework.Assert.IsFalse ((x < z1).Value, "#A03");
+			NUnit.Framework.Assert.AreEqual (y < z, SqlBoolean.Null, "#A04");
+			NUnit.Framework.Assert.IsFalse ((y < x).Value, "#A05");
+			NUnit.Framework.Assert.IsFalse ((y < z1).Value, "#A06");
+			NUnit.Framework.Assert.AreEqual (z < z1, SqlBoolean.Null, "#A07");
+			NUnit.Framework.Assert.AreEqual (z < x, SqlBoolean.Null, "#A08");
+			NUnit.Framework.Assert.AreEqual (z < y, SqlBoolean.Null, "#A09");
+			NUnit.Framework.Assert.AreEqual (z1 < z, SqlBoolean.Null, "#A10");
+			NUnit.Framework.Assert.IsTrue ((z1 < x).Value, "#A11");
+			NUnit.Framework.Assert.IsTrue ((z1 < y).Value, "#A12");
+		}
+
+		[Test]
+		public void LessThanOrEqualTest ()
+		{
+			SqlBoolean x = new SqlBoolean (-1);
+			SqlBoolean y = new SqlBoolean (true);
+			SqlBoolean z = new SqlBoolean ();
+			SqlBoolean z1 = new SqlBoolean (0);
+
+			NUnit.Framework.Assert.IsTrue ((x <= y).Value, "#A01");
+			NUnit.Framework.Assert.AreEqual (x <= z, SqlBoolean.Null, "#A02");
+			NUnit.Framework.Assert.IsFalse ((x <= z1).Value, "#A03");
+			NUnit.Framework.Assert.AreEqual (y <= z, SqlBoolean.Null, "#A04");
+			NUnit.Framework.Assert.IsTrue ((y <= x).Value, "#A05");
+			NUnit.Framework.Assert.IsFalse ((y <= z1).Value, "#A06");
+			NUnit.Framework.Assert.AreEqual (z <= z1, SqlBoolean.Null, "#A07");
+			NUnit.Framework.Assert.AreEqual (z <= x, SqlBoolean.Null, "#A08");
+			NUnit.Framework.Assert.AreEqual (z <= y, SqlBoolean.Null, "#A09");
+			NUnit.Framework.Assert.AreEqual (z1 <= z, SqlBoolean.Null, "#A10");
+			NUnit.Framework.Assert.IsTrue ((z1 <= x).Value, "#A11");
+			NUnit.Framework.Assert.IsTrue ((z1 <= y).Value, "#A12");
+		}
+#endif
 	}
 }
 
