@@ -33,7 +33,11 @@
 
 using NUnit.Framework;
 using System;
+using System.Xml;
 using System.Data.SqlTypes;
+#if TARGET_JVM
+using DivideByZeroException = System.ArithmeticException;
+#endif
 
 namespace MonoTests.System.Data.SqlTypes
 {
@@ -656,6 +660,14 @@ namespace MonoTests.System.Data.SqlTypes
                         short TestShort = 14;
                         Assert.AreEqual ((long)14, ((SqlInt64)TestShort).Value, "#G01");
                 }
+#if NET_2_0
+		[Test]
+		public void GetXsdTypeTest ()
+		{
+			XmlQualifiedName qualifiedName = SqlInt64.GetXsdType (null);
+			NUnit.Framework.Assert.AreEqual ("long", qualifiedName.Name, "#A01");
+		}
+#endif
         }
 }
 
