@@ -33,6 +33,7 @@
 
 using NUnit.Framework;
 using System;
+using System.Xml;
 using System.Data.SqlTypes;
 using System.Globalization;
 using System.Threading;
@@ -181,6 +182,9 @@ namespace MonoTests.System.Data.SqlTypes
 		}
 
 		[Test]
+#if TARGET_JVM
+		[Ignore ("The option CompareOptions.IgnoreWidth is not supported")]
+#endif
                 public void CompareTo()
                 {
                         SqlByte Test = new SqlByte (1);
@@ -259,6 +263,9 @@ namespace MonoTests.System.Data.SqlTypes
                 }
 
 		[Test]
+#if TARGET_JVM
+		[Ignore ("The option CompareOptions.IgnoreWidth is not supported")]
+#endif
                 public void Greaters()
                 {
 
@@ -274,6 +281,9 @@ namespace MonoTests.System.Data.SqlTypes
                 }
 
 		[Test]
+#if TARGET_JVM
+		[Ignore ("The option CompareOptions.IgnoreWidth is not supported")]
+#endif
                 public void Lessers()
                 {
                         // LessThan()
@@ -362,7 +372,11 @@ namespace MonoTests.System.Data.SqlTypes
                                 byte test = Test1.GetUnicodeBytes () [105];
                                 Assert.Fail ("#N05");
                         } catch (Exception e) {
+#if TARGET_JVM
+								Assert.IsTrue (typeof (IndexOutOfRangeException).IsAssignableFrom(e.GetType()), "#N06");
+#else
                                 Assert.AreEqual (typeof (IndexOutOfRangeException), e.GetType(), "#N06");                                
+#endif
                         }
                 }
                       
@@ -528,6 +542,9 @@ namespace MonoTests.System.Data.SqlTypes
                 }
 
 		[Test]
+#if TARGET_JVM
+		[Ignore ("The option CompareOptions.IgnoreWidth is not supported")]
+#endif
                 public void ThanOrEqualOperators()
                 {
                         // == -operator
@@ -704,10 +721,14 @@ namespace MonoTests.System.Data.SqlTypes
                         Assert.AreEqual ("First TestStringPlainString", (String)(SqlString.Add (Test1, "PlainString")), "#AE02");
                         Assert.IsTrue (SqlString.Add (Test1, null).IsNull, "#AE03");
                 }
+
+		[Test]
+		public void GetXsdTypeTest ()
+		{
+			XmlQualifiedName qualifiedName = SqlString.GetXsdType (null);
+			NUnit.Framework.Assert.AreEqual ("string", qualifiedName.Name, "#A01");
+		}
 #endif
-
-
-
         }
 }
 
