@@ -5,7 +5,7 @@
 // 	Jordi Mas i Hernàndez (jordi@ximian.com)
 //	Sebastien Pouliot  <sebastien@ximian.com>
 //
-// Copyright (C) 2006 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2006, 2007 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -178,8 +178,7 @@ namespace MonoTests.System.Drawing.Imaging {
 		}
 
 		[Test]
-		[Category ("NotWorking")]
-		public void Bitmat1bitData ()
+		public void Bitmap1bitData ()
 		{
 			string sInFile = getInFile ("bitmaps/1bit.png");
 			using (Bitmap bmp = new Bitmap (sInFile)) {
@@ -188,6 +187,7 @@ namespace MonoTests.System.Drawing.Imaging {
 					Assert.AreEqual (bmp.Height, data.Height, "Height");
 					Assert.AreEqual (bmp.Width, data.Width, "Width");
 					Assert.AreEqual (PixelFormat.Format24bppRgb, data.PixelFormat, "PixelFormat");
+					Assert.AreEqual (864, data.Stride, "Stride");
 					int size = data.Height * data.Stride;
 					unsafe {
 						byte* scan = (byte*) data.Scan0;
@@ -388,8 +388,7 @@ namespace MonoTests.System.Drawing.Imaging {
 		}
 
 		[Test]
-		[Category ("NotWorking")]
-		public void Bitmat4bitData ()
+		public void Bitmap4bitData ()
 		{
 			string sInFile = getInFile ("bitmaps/4bit.png");
 			using (Bitmap bmp = new Bitmap (sInFile)) {
@@ -398,6 +397,7 @@ namespace MonoTests.System.Drawing.Imaging {
 					Assert.AreEqual (bmp.Height, data.Height, "Height");
 					Assert.AreEqual (bmp.Width, data.Width, "Width");
 					Assert.AreEqual (PixelFormat.Format24bppRgb, data.PixelFormat, "PixelFormat");
+					Assert.AreEqual (864, data.Stride, "Stride");
 					int size = data.Height * data.Stride;
 					unsafe {
 						byte* scan = (byte*) data.Scan0;
