@@ -46,39 +46,39 @@ namespace System.Web.Services.Protocols {
 	//
 	internal class SoapMethodStubInfo : MethodStubInfo
 	{
-		internal string Action;
-		internal string Binding;
+		internal readonly string Action;
+		internal readonly string Binding;
 
 		// The name/namespace of the request 
-		internal string RequestName;
-		internal string RequestNamespace;
+		internal readonly string RequestName;
+		internal readonly string RequestNamespace;
 
 		// The name/namespace of the response.
-		internal string ResponseName;
-		internal string ResponseNamespace;
-		
-		internal bool OneWay;
-		internal SoapParameterStyle ParameterStyle;
-		internal SoapBindingStyle SoapBindingStyle;
-		internal SoapBindingUse Use;
+		internal readonly string ResponseName;
+		internal readonly string ResponseNamespace;
 
-		internal HeaderInfo[] Headers;
-		internal HeaderInfo[] InHeaders;
-		internal HeaderInfo[] OutHeaders;
-		internal HeaderInfo[] FaultHeaders;
-		internal SoapExtensionRuntimeConfig[] SoapExtensions;
-		
-		internal XmlMembersMapping InputMembersMapping;
-		internal XmlMembersMapping OutputMembersMapping;
-		internal XmlMembersMapping InputHeaderMembersMapping;
-		internal XmlMembersMapping OutputHeaderMembersMapping;
-		internal XmlMembersMapping FaultHeaderMembersMapping;
-		
-		private int requestSerializerId;
-		private int responseSerializerId;
-		private int requestHeadersSerializerId = -1;
-		private int responseHeadersSerializerId = -1;
-		private int faultHeadersSerializerId = -1;
+		internal readonly bool OneWay;
+		internal readonly SoapParameterStyle ParameterStyle;
+		internal readonly SoapBindingStyle SoapBindingStyle;
+		internal readonly SoapBindingUse Use;
+
+		internal readonly HeaderInfo [] Headers;
+		internal readonly HeaderInfo [] InHeaders;
+		internal readonly HeaderInfo [] OutHeaders;
+		internal readonly HeaderInfo [] FaultHeaders;
+		internal readonly SoapExtensionRuntimeConfig [] SoapExtensions;
+
+		internal readonly XmlMembersMapping InputMembersMapping;
+		internal readonly XmlMembersMapping OutputMembersMapping;
+		internal readonly XmlMembersMapping InputHeaderMembersMapping;
+		internal readonly XmlMembersMapping OutputHeaderMembersMapping;
+		internal readonly XmlMembersMapping FaultHeaderMembersMapping;
+
+		private readonly int requestSerializerId;
+		private readonly int responseSerializerId;
+		private readonly int requestHeadersSerializerId = -1;
+		private readonly int responseHeadersSerializerId = -1;
+		private readonly int faultHeadersSerializerId = -1;
 		
 		internal XmlSerializer RequestSerializer
 		{
@@ -124,6 +124,10 @@ namespace System.Web.Services.Protocols {
 				ParameterStyle = parent.ParameterStyle;
 				SoapBindingStyle = parent.SoapBindingStyle;
 				OneWay = false;
+#if NET_2_0
+				if (parent.Type != source.DeclaringType)
+					Binding = source.DeclaringType.Name + parent.ProtocolName;
+#endif
 			}
 			else if (kind is SoapDocumentMethodAttribute){
 				SoapDocumentMethodAttribute dma = (SoapDocumentMethodAttribute) kind;
@@ -438,9 +442,7 @@ namespace System.Web.Services.Protocols {
 		Hashtable[] header_serializers = new Hashtable [3];
 		Hashtable[] header_serializers_byname = new Hashtable [3];
 		Hashtable methods_byaction = new Hashtable (); 
-#if NET_2_0
-		WsiProfiles wsi_claims = WsiProfiles.None;
-#endif
+
 		// Precomputed
 		internal SoapParameterStyle      ParameterStyle;
 		internal SoapServiceRoutingStyle RoutingStyle;
@@ -455,32 +457,16 @@ namespace System.Web.Services.Protocols {
 		{
 			xmlImporter = new XmlReflectionImporter ();
 			soapImporter = new SoapReflectionImporter ();
-			
-			object [] o;
-
-			o = Type.GetCustomAttributes (typeof (WebServiceBindingAttribute), false);
-			
+				
 			if (typeof (SoapHttpClientProtocol).IsAssignableFrom (Type))
 			{
-				if (o.Length == 0)
+				if (Bindings.Count == 0 || ((BindingInfo)Bindings[0]).WebServiceBindingAttribute == null)
 					throw new InvalidOperationException ("WebServiceBindingAttribute is required on proxy class '" + Type + "'.");
-				if (o.Length > 1)
+				if (Bindings.Count > 1)
 					throw new InvalidOperationException ("Only one WebServiceBinding attribute may be specified on type '" + Type + "'.");
-					
-				// Remove the default binding, it is not needed since there is always
-				// a binding attribute.
-				Bindings.Clear ();
-#if NET_2_0
-				WebServiceBindingAttribute wsba = (WebServiceBindingAttribute) o [0];
-				if (wsba.EmitConformanceClaims)
-					wsi_claims = wsba.ConformsTo;
-#endif
 			}
-				
-			foreach (WebServiceBindingAttribute at in o)
-				AddBinding (new BindingInfo (at, LogicalType.WebServiceNamespace));
 
-			o = Type.GetCustomAttributes (typeof (SoapDocumentServiceAttribute), false);
+			object [] o = Type.GetCustomAttributes (typeof (SoapDocumentServiceAttribute), false);
 			if (o.Length == 1){
 				SoapDocumentServiceAttribute a = (SoapDocumentServiceAttribute) o [0];
 
@@ -514,12 +500,6 @@ namespace System.Web.Services.Protocols {
 			SoapExtensions = SoapExtension.GetTypeExtensions (Type);
 		}
 
-#if NET_2_0
-		public override WsiProfiles WsiClaims {
-			get { return wsi_claims; }
-		}
-#endif
-
 		public override XmlReflectionImporter XmlImporter 
 		{
 			get { return xmlImporter; }
