@@ -32,6 +32,7 @@ using System;
 using System.CodeDom;
 using System.CodeDom.Compiler;
 using System.Collections;
+using System.Collections.Specialized;
 using System.Reflection;
 using System.Text;
 using System.Web.UI;
@@ -56,6 +57,8 @@ namespace System.Web.Compilation
 #if NET_2_0
 		bool isRebuilding = false;
 		protected Hashtable partialNameOverride = new Hashtable();
+		protected CodeTypeDeclaration partialClass;
+		protected CodeTypeReferenceExpression partialClassExpr;
 #endif
 		protected CodeTypeDeclaration mainClass;
 		protected CodeTypeReferenceExpression mainClassExpr;
@@ -72,33 +75,56 @@ namespace System.Web.Compilation
 			unit = new CodeCompileUnit ();
 #if NET_2_0
 			if (parser.IsPartial) {
-				string ns = null;
-				string classtype = parser.PartialClassName;
+				string partialns = null;
+				string partialclasstype = parser.PartialClassName;
 
-				if (classtype.Contains (".")) {
-					int dot = classtype.LastIndexOf (".");
-					ns = classtype.Substring (0, dot);
-					classtype = classtype.Substring (dot + 1);
+				int partialdot = partialclasstype.LastIndexOf ('.');
+				if (partialdot != -1) {
+					partialns = partialclasstype.Substring (0, partialdot);
+					partialclasstype = partialclasstype.Substring (partialdot + 1);
 				}
 				
-				mainNS = new CodeNamespace (ns);
-				mainClass = new CodeTypeDeclaration (classtype);
-				mainClass.IsPartial = true;	
-				mainClassExpr = new CodeTypeReferenceExpression (parser.PartialClassName);
-			} else {
-#endif
-			mainNS = new CodeNamespace ("ASP");
-			mainClass = new CodeTypeDeclaration (parser.ClassName);
-			CodeTypeReference baseTypeRef = new CodeTypeReference (parser.BaseType.FullName);
-#if NET_2_0
-			if (parser.BaseTypeIsGlobal)
-				baseTypeRef.Options |= CodeTypeReferenceOptions.GlobalReference;
-#endif
-			mainClass.BaseTypes.Add (baseTypeRef);
-			mainClassExpr = new CodeTypeReferenceExpression ("ASP." + parser.ClassName);
-#if NET_2_0
+				CodeNamespace partialNS = new CodeNamespace (partialns);
+				partialClass = new CodeTypeDeclaration (partialclasstype);
+				partialClass.IsPartial = true;
+				partialClassExpr = new CodeTypeReferenceExpression (parser.PartialClassName);
+				
+				unit.Namespaces.Add (partialNS);
+				partialClass.TypeAttributes = TypeAttributes.Public;
+				partialNS.Types.Add (partialClass);
 			}
 #endif
+
+			string mainclasstype = parser.ClassName;
+			string mainns = "ASP";
+
+#if NET_2_0
+			int maindot = mainclasstype.LastIndexOf ('.');
+			if (maindot != -1) {
+				mainns = mainclasstype.Substring (0, maindot);
+				mainclasstype = mainclasstype.Substring (maindot + 1);
+			}
+#endif
+
+			mainNS = new CodeNamespace (mainns);
+			mainClass = new CodeTypeDeclaration (mainclasstype);
+			CodeTypeReference baseTypeRef;
+#if NET_2_0
+			if (partialClass != null) {
+				baseTypeRef = new CodeTypeReference (parser.PartialClassName);
+				baseTypeRef.Options |= CodeTypeReferenceOptions.GlobalReference;
+			} else {
+				baseTypeRef = new CodeTypeReference (parser.BaseType.FullName);
+				if (parser.BaseTypeIsGlobal)
+					baseTypeRef.Options |= CodeTypeReferenceOptions.GlobalReference;
+			}
+#else
+			baseTypeRef = new CodeTypeReference (parser.BaseType.FullName);
+#endif
+			mainClass.BaseTypes.Add (baseTypeRef);
+
+			mainClassExpr = new CodeTypeReferenceExpression (mainns + "." + mainclasstype);
+
 			unit.Namespaces.Add (mainNS);
 			mainClass.TypeAttributes = TypeAttributes.Public;
 			mainNS.Types.Add (mainClass);
@@ -108,26 +134,41 @@ namespace System.Web.Compilation
 					mainNS.Imports.Add (new CodeNamespaceImport ((string) o));
 			}
 
+			// StringCollection.Contains has O(n) complexity, but
+			// considering the number of comparisons we make on
+			// average and the fact that using an intermediate array
+			// would be even more costly, this is fine here.
+			StringCollection refAsm = unit.ReferencedAssemblies;
+			string asmName;
 			if (parser.Assemblies != null) {
 				foreach (object o in parser.Assemblies) {
-					if (o is string)
-						unit.ReferencedAssemblies.Add ((string) o);
+					asmName = o as string;
+					if (asmName != null && !refAsm.Contains (asmName))
+						refAsm.Add (asmName);
 				}
 			}
 
 #if NET_2_0
 			ArrayList al = WebConfigurationManager.ExtraAssemblies;
 			if (al != null && al.Count > 0) {
-				foreach (object o in al)
-					if (o is string)
-						unit.ReferencedAssemblies.Add ((string) o);
+				foreach (object o in al) {
+					asmName = o as string;
+					if (asmName != null && !refAsm.Contains (asmName))
+						refAsm.Add (asmName);
+				}
 			}
 
 			IList list = BuildManager.CodeAssemblies;
 			if (list != null && list.Count > 0) {
-				foreach (object o in list)
-					if (o is string)
-						unit.ReferencedAssemblies.Add ((string) o);
+				Assembly asm;
+				foreach (object o in list) {
+					asm = o as Assembly;
+					if (o == null)
+						continue;
+					asmName = asm.Location;
+					if (asmName != null && !refAsm.Contains (asmName))
+						refAsm.Add (asmName);
+				}
 			}
 #endif
 			// Late-bound generators specifics (as for MonoBASIC/VB.NET)
@@ -160,6 +201,31 @@ namespace System.Web.Compilation
 			mainClass.Members.Add (fld);
 		}
 
+#if NET_2_0
+		void AssignAppRelativeVirtualPath (CodeConstructor ctor)
+		{
+			Type baseType = parser.CodeFileBaseClassType;
+
+			if (baseType == null)
+				baseType = parser.BaseType;
+			if (baseType == null)
+				return;
+			if (!baseType.IsSubclassOf (typeof (System.Web.UI.TemplateControl)))
+				return;
+			
+			string arvp = Path.Combine (parser.BaseVirtualDir, Path.GetFileName (parser.InputFile));
+			if (VirtualPathUtility.IsAbsolute (arvp))
+				arvp = "~" + arvp;
+			
+			CodeExpression cast = new CodeCastExpression (baseType, new CodeThisReferenceExpression ());
+			CodePropertyReferenceExpression arvpProp = new CodePropertyReferenceExpression (cast, "AppRelativeVirtualPath");
+			CodeAssignStatement arvpAssign = new CodeAssignStatement ();
+			arvpAssign.Left = arvpProp;
+			arvpAssign.Right = new CodePrimitiveExpression (arvp);
+			ctor.Statements.Add (arvpAssign);
+		}
+#endif
+		
 		protected virtual void CreateConstructor (CodeStatementCollection localVars,
 							  CodeStatementCollection trueStmt)
 		{
@@ -167,6 +233,9 @@ namespace System.Web.Compilation
 			ctor.Attributes = MemberAttributes.Public;
 			mainClass.Members.Add (ctor);
 
+#if NET_2_0
+			AssignAppRelativeVirtualPath (ctor);
+#endif
 			if (localVars != null)
 				ctor.Statements.AddRange (localVars);
 
@@ -210,6 +279,43 @@ namespace System.Web.Compilation
 		{
 		}
 
+#if NET_2_0
+		void InternalCreatePageProperty (string retType, string name, string contextProperty)
+		{
+			CodeMemberProperty property = new CodeMemberProperty ();
+			property.Name = name;
+			property.Type = new CodeTypeReference (retType);
+			property.Attributes = MemberAttributes.Family | MemberAttributes.Final;
+
+			CodeMethodReturnStatement ret = new CodeMethodReturnStatement ();
+			CodeCastExpression cast = new CodeCastExpression ();
+			ret.Expression = cast;
+			
+			CodePropertyReferenceExpression refexp = new CodePropertyReferenceExpression ();
+			refexp.TargetObject = new CodePropertyReferenceExpression (new CodeThisReferenceExpression (), "Context");
+			refexp.PropertyName = contextProperty;
+			
+			cast.TargetType = new CodeTypeReference (retType);
+			cast.Expression = refexp;
+			
+			property.GetStatements.Add (ret);
+			if (partialClass == null)
+				mainClass.Members.Add (property);
+			else
+				partialClass.Members.Add (property);
+		}
+		
+		protected void CreateProfileProperty ()
+		{
+			string retType;
+			if (AppCodeCompiler.HaveCustomProfile (WebConfigurationManager.GetSection ("system.web/profile") as ProfileSection))
+				retType = "ProfileCommon";
+			else
+				retType = "System.Web.Profile.DefaultProfile";
+			InternalCreatePageProperty (retType, "Profile", "Profile");
+		}
+#endif
+		
 		protected virtual void AddInterfaces ()
 		{
 			if (parser.Interfaces == null)
@@ -345,9 +451,21 @@ namespace System.Web.Compilation
 			if (results.NativeCompilerReturnValue == 0)
 				return;
 
-			StringWriter writer = new StringWriter();
-			provider.CreateGenerator().GenerateCodeFromCompileUnit (unit, writer, null);
-			throw new CompilationException (parser.InputFile, results.Errors, writer.ToString ());
+			string fileText = null;
+			CompilerErrorCollection errors = results.Errors;
+			CompilerError ce = (errors != null && errors.Count > 0) ? errors [0] : null;
+			string inFile = (ce != null) ? ce.FileName : null;
+			
+			if (inFile != null && File.Exists (inFile)) {
+				using (StreamReader sr = File.OpenText (inFile)) {
+					fileText = sr.ReadToEnd ();
+				}
+			} else {
+				StringWriter writer = new StringWriter();
+				provider.CreateGenerator().GenerateCodeFromCompileUnit (unit, writer, null);
+				fileText = writer.ToString ();
+			}
+			throw new CompilationException (parser.InputFile, errors, fileText);
 		}
 
 		protected string DynamicDir ()
