@@ -42,14 +42,21 @@ namespace System.Web.Compilation {
 	[ExpressionPrefix("ConnectionStrings")]
 	public class ConnectionStringsExpressionBuilder : ExpressionBuilder {
 
-		public override object EvaluateExpression (object target, BoundPropertyEntry entry, object parsedData, ExpressionBuilderContext context)
+		public override object EvaluateExpression (object target, BoundPropertyEntry entry,
+							   object parsedData, ExpressionBuilderContext context)
 		{
 			return GetConnectionString (entry.Expression.Trim());
 		}
 
-		public override CodeExpression GetCodeExpression (BoundPropertyEntry entry, object parsedData, ExpressionBuilderContext context)
-		{
-			throw new NotImplementedException ();
+		public override CodeExpression GetCodeExpression (BoundPropertyEntry entry, object parsedData,
+								  ExpressionBuilderContext context)
+		{			
+			Pair connString = parsedData as Pair;
+			return new CodeMethodInvokeExpression (
+				new CodeTypeReferenceExpression (typeof (ConnectionStringsExpressionBuilder)),
+				"GetConnectionString",
+				new CodeExpression [] {new CodePrimitiveExpression (connString.First)}
+			);
 		}
 
 		public static string GetConnectionString (string connectionStringName)
@@ -70,10 +77,9 @@ namespace System.Web.Compilation {
 				return conn.ProviderName;
 		}
 
-		[MonoTODO ("Not implemented")]
 		public override	object ParseExpression (string expression, Type propertyType, ExpressionBuilderContext context)
 		{
-			throw new NotImplementedException ();
+			return new Pair (expression, GetConnectionString (expression));
 		}
 
 		public override bool SupportsEvaluate {
