@@ -37,6 +37,7 @@ using System.Text;
 using System.Web;
 using System.Web.Configuration;
 using System.Web.Util;
+using System.Globalization;
 
 namespace System.Web.Security
 {
@@ -476,9 +477,9 @@ namespace System.Web.Security
 				throw new ArgumentNullException ("passwordFormat");
 
 			byte [] bytes;
-			if (String.Compare (passwordFormat, "MD5", true) == 0) {
+			if (String.Compare (passwordFormat, "MD5", true, CultureInfo.InvariantCulture) == 0) {
 				bytes = MD5.Create ().ComputeHash (Encoding.UTF8.GetBytes (password));
-			} else if (String.Compare (passwordFormat, "SHA1", true) == 0) {
+			} else if (String.Compare (passwordFormat, "SHA1", true, CultureInfo.InvariantCulture) == 0) {
 				bytes = SHA1.Create ().ComputeHash (Encoding.UTF8.GetBytes (password));
 			} else {
 				throw new ArgumentException ("The format must be either MD5 or SHA1", "passwordFormat");
@@ -626,6 +627,10 @@ namespace System.Web.Security
 			expiration_cookie.Expires = new DateTime (1999, 10, 12);
 			expiration_cookie.Path = cookiePath;
 			cc.Add (expiration_cookie);
+
+#if NET_2_0
+			Roles.DeleteCookie ();
+#endif
 		}
 
 		public static string FormsCookieName
