@@ -123,8 +123,11 @@ namespace System.Web.UI.HtmlControls {
 		{
 			base.OnPreRender (e);
 
-			if (Page != null) {
+			if (Page != null && !Disabled) {
 				Page.RegisterRequiresPostBack (this);
+#if NET_2_0
+				Page.RegisterEnabledControl (this);
+#endif
 			}
 		}
 
