@@ -298,9 +298,13 @@ namespace System.Web.UI.WebControls {
 		{
 			base.OnPreRender (e);
 
-			if (Page != null) {
+			if (Page != null && Enabled) {
 				Page.RegisterRequiresPostBack (this);
 			}
+#if NET_2_0
+			if (Page != null && Enabled)
+				Page.RegisterEnabledControl (this);
+#endif
 		}
 
 		static bool IsInputOrCommonAttr (string attname)
@@ -339,7 +343,7 @@ namespace System.Web.UI.WebControls {
 			}
 		}
 
-		void AddAttributesForSpan (HtmlTextWriter writer)
+		bool AddAttributesForSpan (HtmlTextWriter writer)
 		{
 			ICollection k = Attributes.Keys;
 			string [] keys = new string [k.Count];
@@ -352,7 +356,11 @@ namespace System.Web.UI.WebControls {
 				common_attrs [key] = Attributes [key];
 				Attributes.Remove (key);
 			}
-			Attributes.AddAttributes (writer);
+			if (Attributes.Count > 0) {
+				Attributes.AddAttributes (writer);
+				return true;
+			}
+			return false;
 		}
 #if NET_2_0
 		protected internal
@@ -365,8 +373,12 @@ namespace System.Web.UI.WebControls {
 				Page.VerifyRenderingInServerForm (this);
 			
 			bool need_span = ControlStyleCreated && !ControlStyle.IsEmpty;
-			if (need_span)
+			if (need_span) {
+#if NET_2_0
+				AddDisplayStyleAttribute (w);
+#endif
 				ControlStyle.AddAttributesToRender (w, this);
+			}
 
 			if (!Enabled) {
 				w.AddAttribute (HtmlTextWriterAttribute.Disabled, "disabled");
@@ -379,10 +391,8 @@ namespace System.Web.UI.WebControls {
 				need_span = true;
 			}
 
-			if (Attributes.Count > 0){
-				AddAttributesForSpan (w);
+			if (Attributes.Count > 0 && AddAttributesForSpan (w))
 				need_span = true;
-			}
 
 			if (need_span)
 				w.RenderBeginTag (HtmlTextWriterTag.Span);
