@@ -402,7 +402,7 @@ namespace System.Web.UI.WebControls
 		[BrowsableAttribute (false)]
 		public virtual DetailsViewRow BottomPagerRow {
 			get {
-				EnsureDataBound ();
+				EnsureChildControls ();
 				return bottomPagerRow;
 			}
 		}
@@ -804,6 +804,9 @@ namespace System.Web.UI.WebControls
 			get {
 				return pageCount;
 			}
+			private set {
+				pageCount = value;
+			}
 		}
 
 		[WebCategoryAttribute ("Paging")]
@@ -811,12 +814,14 @@ namespace System.Web.UI.WebControls
 		[DefaultValueAttribute (0)]
 		public virtual int PageIndex {
 			get {
+				if (CurrentMode == DetailsViewMode.Insert)
+					return -1;
 				return pageIndex;
 			}
 			set {
-				if (value < 0)
+				if (value < -1)
 					throw new ArgumentOutOfRangeException ("PageIndex must be non-negative");
-				if (value == pageIndex)
+				if (pageIndex == value || value == -1)
 					return;
 				pageIndex = value;
 				RequireBinding ();
@@ -898,7 +903,7 @@ namespace System.Web.UI.WebControls
 		[BrowsableAttribute (false)]
 		public virtual DetailsViewRow TopPagerRow {
 			get {
-				EnsureDataBound ();
+				EnsureChildControls ();
 				return topPagerRow;
 			}
 		}
@@ -907,7 +912,6 @@ namespace System.Web.UI.WebControls
 		[BrowsableAttribute (false)]
 		public virtual object DataItem {
 			get {
-				EnsureDataBound ();
 				return dataItem;
 			}
 		}
@@ -1045,7 +1049,6 @@ namespace System.Web.UI.WebControls
 				row = new DetailsViewPagerRow (rowIndex, rowType, rowState);
 			else
 				row = new DetailsViewRow (rowIndex, rowType, rowState);
-			OnItemCreated (EventArgs.Empty);
 			return row;
 		}
 		
@@ -1071,141 +1074,156 @@ namespace System.Web.UI.WebControls
 		
 		protected override int CreateChildControls (IEnumerable data, bool dataBinding)
 		{
-			PagedDataSource dataSource;
+			PagedDataSource dataSource = new PagedDataSource ();
+			dataSource.DataSource = CurrentMode != DetailsViewMode.Insert ? data : null;
+			dataSource.AllowPaging = AllowPaging;
+			dataSource.PageSize = 1;
+			dataSource.CurrentPageIndex = PageIndex;
 
-			if (dataBinding) {
+			if (dataBinding && CurrentMode != DetailsViewMode.Insert) {
 				DataSourceView view = GetData ();
-				dataSource = new PagedDataSource ();
-				dataSource.DataSource = data;
-				
-				if (AllowPaging) {
-					dataSource.AllowPaging = true;
-					dataSource.PageSize = 1;
-					dataSource.CurrentPageIndex = PageIndex;
-					if (view.CanPage) {
-						dataSource.AllowServerPaging = true;
-						if (SelectArguments.RetrieveTotalRowCount)
-							dataSource.VirtualCount = SelectArguments.TotalRowCount;
+				if (view != null && view.CanPage) {
+					dataSource.AllowServerPaging = true;
+					if (SelectArguments.RetrieveTotalRowCount)
+						dataSource.VirtualCount = SelectArguments.TotalRowCount;
+				}
+			}
+
+			bool showPager = AllowPaging && (dataSource.PageCount > 1);
+
+			Controls.Clear ();
+			table = CreateTable ();
+			Controls.Add (table);
+			headerRow = null;
+			footerRow = null;
+			topPagerRow = null;
+			bottomPagerRow = null;
+			ArrayList list = new ArrayList ();
+
+			// Gets the current data item
+
+			if (AllowPaging) {
+				PageCount = dataSource.DataSourceCount;
+				if (PageIndex >= PageCount && PageCount > 0) {
+					pageIndex = dataSource.CurrentPageIndex = PageCount - 1;
+				}
+				if (dataSource.DataSource != null) {
+					IEnumerator e = dataSource.GetEnumerator ();
+					if (e.MoveNext ())
+						dataItem = e.Current;
+				}
+			}
+			else {
+				int page = 0;
+				object lastItem = null;
+				if (dataSource.DataSource != null) {
+					IEnumerator e = dataSource.GetEnumerator ();
+					for (; e.MoveNext (); page++) {
+						lastItem = e.Current;
+						if (page == PageIndex)
+							dataItem = e.Current;
 					}
 				}
-				
-				pageCount = dataSource.DataSourceCount;
-			}
-			else
-			{
-				dataSource = new PagedDataSource ();
-				dataSource.DataSource = data;
-				if (AllowPaging) {
-					dataSource.AllowPaging = true;
-					dataSource.PageSize = 1;
-					dataSource.CurrentPageIndex = PageIndex;
+				PageCount = page;
+				if (PageIndex >= PageCount && PageCount > 0) {
+					pageIndex = PageCount - 1;
+					dataItem = lastItem;
 				}
 			}
 
-			if (AllowPaging && PageIndex >= PageCount) {
-				PageIndex = (PageCount > 0) ? PageCount - 1 : 0;
-				dataSource.CurrentPageIndex = PageIndex;
+			if (PageCount == 0 && CurrentMode != DetailsViewMode.Insert) {
+				DetailsViewRow row = CreateEmptyRow ();
+				if (row != null) {
+					table.Rows.Add (row);
+					list.Add (row);
+				}
 			}
+			else {
 
-			bool showPager = AllowPaging && (PageCount > 1);
-			
-			Controls.Clear ();
-			table = null;
-				
-			ArrayList list = new ArrayList ();
-			
-			// Gets the current data item
-			
-			IEnumerator e = dataSource.GetEnumerator (); 
-			if (e.MoveNext ())
-				dataItem = e.Current;
-			else
-				dataItem = null;
-			
-			// Creates the set of fields to show
+				// Creates the set of fields to show
 
-			ICollection fieldCollection = CreateFieldSet (dataItem, dataBinding && dataItem != null);
-			DataControlField[] fields = new DataControlField [fieldCollection.Count];
-			fieldCollection.CopyTo (fields, 0);
+				ICollection fieldCollection = CreateFieldSet (dataItem, dataBinding && dataItem != null);
+				DataControlField [] fields = new DataControlField [fieldCollection.Count];
+				fieldCollection.CopyTo (fields, 0);
 
-			foreach (DataControlField field in fields) {
-				field.Initialize (false, this);
-				if (EnablePagingCallbacks)
-					field.ValidateSupportsCallback ();
+				foreach (DataControlField field in fields) {
+					field.Initialize (false, this);
+					if (EnablePagingCallbacks)
+						field.ValidateSupportsCallback ();
+				}
+
+				// Main table creation
+
+				headerRow = CreateRow (-1, DataControlRowType.Header, DataControlRowState.Normal);
+				DataControlFieldCell headerCell = new DataControlFieldCell (null);
+				headerCell.ColumnSpan = 2;
+				if (headerTemplate != null)
+					headerTemplate.InstantiateIn (headerCell);
+				else if (!String.IsNullOrEmpty (HeaderText))
+					headerCell.Text = HeaderText;
+				else
+					headerRow.Visible = false;
+				headerRow.Cells.Add (headerCell);
+				table.Rows.Add (headerRow);
+
+				if (showPager && PagerSettings.Position == PagerPosition.Top ||
+						PagerSettings.Position == PagerPosition.TopAndBottom) {
+					topPagerRow = CreateRow (-1, DataControlRowType.Pager, DataControlRowState.Normal);
+					InitializePager (topPagerRow, dataSource);
+					table.Rows.Add (topPagerRow);
+				}
+
+				foreach (DataControlField field in fields) {
+					DataControlRowState rstate = GetRowState (list.Count);
+					DetailsViewRow row = CreateRow (PageIndex, DataControlRowType.DataRow, rstate);
+					InitializeRow (row, field);
+					table.Rows.Add (row);
+					list.Add (row);
+				}
+
+				footerRow = CreateRow (-1, DataControlRowType.Footer, DataControlRowState.Normal);
+				DataControlFieldCell footerCell = new DataControlFieldCell (null);
+				footerCell.ColumnSpan = 2;
+				if (footerTemplate != null)
+					footerTemplate.InstantiateIn (footerCell);
+				else if (!String.IsNullOrEmpty (FooterText))
+					footerCell.Text = FooterText;
+				else
+					footerRow.Visible = false;
+				footerRow.Cells.Add (footerCell);
+				table.Rows.Add (footerRow);
+
+				if (showPager && PagerSettings.Position == PagerPosition.Bottom ||
+						PagerSettings.Position == PagerPosition.TopAndBottom) {
+					bottomPagerRow = CreateRow (-1, DataControlRowType.Pager, DataControlRowState.Normal);
+					InitializePager (bottomPagerRow, dataSource);
+					table.Rows.Add (bottomPagerRow);
+				}
 			}
+			
+			rows = new DetailsViewRowCollection (list);
 
-			// Main table creation
+			if (dataBinding)
+				DataBind (false);
+			
+			OnItemCreated (EventArgs.Empty);
 
-            bool hasEmptyText = EmptyDataText != null && EmptyDataText != String.Empty;
-
-            if (fields.Length > 0)
-            {
-                headerRow = CreateRow(-1, DataControlRowType.Header, DataControlRowState.Normal);
-                DataControlFieldCell headerCell = new DataControlFieldCell(null);
-                headerCell.ColumnSpan = 2;
-                if (headerTemplate != null)
-                    headerTemplate.InstantiateIn(headerCell);
-                else
-                    headerCell.Text = HeaderText;
-                headerRow.Cells.Add(headerCell);
-                ContainedTable.Rows.Add(headerRow);
-
-                if (showPager && PagerSettings.Position == PagerPosition.Top ||
-                        PagerSettings.Position == PagerPosition.TopAndBottom)
-                {
-                    topPagerRow = CreateRow(-1, DataControlRowType.Pager, DataControlRowState.Normal);
-                    InitializePager(topPagerRow, dataSource);
-                    ContainedTable.Rows.Add(topPagerRow);
-                }
-            }
-
-            foreach (DataControlField field in fields)
-            {
-                DataControlRowState rstate = GetRowState(list.Count);
-                DetailsViewRow row = CreateRow(PageIndex, DataControlRowType.DataRow, rstate);
-                InitializeRow(row, field);
-                ContainedTable.Rows.Add(row);
-                list.Add(row);
-            }
-
-            rows = new DetailsViewRowCollection(list);
-
-            if (list.Count == 0 && hasEmptyText)
-            {
-                ContainedTable.Rows.Add(CreateEmptyRow());
-            }
-            
-            if (list.Count > 0)
-            {
-                footerRow = CreateRow(-1, DataControlRowType.Footer, DataControlRowState.Normal);
-                DataControlFieldCell footerCell = new DataControlFieldCell(null);
-                footerCell.ColumnSpan = 2;
-                if (footerTemplate != null)
-                    footerTemplate.InstantiateIn(footerCell);
-                else
-                    footerCell.Text = FooterText;
-
-                footerRow.Cells.Add(footerCell);
-                ContainedTable.Rows.Add(footerRow);
-
-                if (showPager && PagerSettings.Position == PagerPosition.Bottom ||
-                        PagerSettings.Position == PagerPosition.TopAndBottom)
-                {
-                    bottomPagerRow = CreateRow(-1, DataControlRowType.Pager, DataControlRowState.Normal);
-                    InitializePager(bottomPagerRow, dataSource);
-                    ContainedTable.Rows.Add(bottomPagerRow);
-                }
-            }
-
-            if (dataBinding)
-                DataBind(false);
-
-            return dataSource.DataSourceCount;
+			return PageCount;
 		}
 
 		protected override void EnsureDataBound ()
 		{
-			base.EnsureDataBound ();
+			if (CurrentMode == DetailsViewMode.Insert) {
+				if (RequiresDataBinding) {
+					OnDataBinding (EventArgs.Empty);
+					RequiresDataBinding = false;
+					PerformDataBinding (null);
+					MarkAsDataBound ();
+					OnDataBound (EventArgs.Empty);
+				}
+			}
+			else
+				base.EnsureDataBound ();
 		}
 		
 		DataControlRowState GetRowState (int index)
@@ -1231,20 +1249,25 @@ namespace System.Web.UI.WebControls
 		
 		DetailsViewRow CreateEmptyRow ()
 		{
-			DetailsViewRow row = CreateRow (-1, DataControlRowType.EmptyDataRow, DataControlRowState.Normal);
 			TableCell cell = new TableCell ();
-			
+
 			if (emptyDataTemplate != null)
 				emptyDataTemplate.InstantiateIn (cell);
-			else
+			else if (!String.IsNullOrEmpty (EmptyDataText))
 				cell.Text = EmptyDataText;
+			else
+				return null;
 			
+			DetailsViewRow row = CreateRow (-1, DataControlRowType.EmptyDataRow, DataControlRowState.Normal);
 			row.Cells.Add (cell);
 			return row;
 		}
 		
 		protected virtual void InitializeRow (DetailsViewRow row, DataControlField field)
 		{
+			if (!field.Visible)
+				return;
+			
 			row.ContainingField = field;
 			DataControlFieldCell cell;
 			
@@ -1314,13 +1337,6 @@ namespace System.Web.UI.WebControls
 		
 		public sealed override void DataBind ()
 		{
-			if (CurrentMode == DetailsViewMode.Insert) {
-				RequiresDataBinding = false;
-				PerformDataBinding (new object [] { null });
-				MarkAsDataBound ();
-				return;
-			}
-			
 			cachedKeyProperties = null;
 			base.DataBind ();
 			
@@ -1339,23 +1355,13 @@ namespace System.Web.UI.WebControls
 
 		protected internal virtual void PrepareControlHierarchy ()
 		{
-            ContainedTable.Caption = Caption;
-            ContainedTable.CaptionAlign = CaptionAlign;
+			if (table == null)
+				return;
 
-			// set visible for header and footer
-			if (HeaderRow != null) 
-			{
-				TableCell headerCell = (TableCell) HeaderRow.Controls [0];
-				HeaderRow.Visible = headerCell.Text.Length > 0 || headerCell.Controls.Count > 0;
-			}
-			if (FooterRow != null)
-			{
-				TableCell footerCell = (TableCell) FooterRow.Controls [0];
-				FooterRow.Visible = footerCell.Text.Length > 0 || footerCell.Controls.Count > 0;
-			}
+			table.Caption = Caption;
+			table.CaptionAlign = CaptionAlign;
 
-            foreach (DetailsViewRow row in ContainedTable.Rows)
-            {
+			foreach (DetailsViewRow row in table.Rows) {
 				switch (row.RowType) {
 				case DataControlRowType.Header:
 					if (headerStyle != null && !headerStyle.IsEmpty)
@@ -1444,15 +1450,21 @@ namespace System.Web.UI.WebControls
 		{
 			DetailsViewCommandEventArgs args = e as DetailsViewCommandEventArgs;
 			if (args != null) {
-				ProcessCommand (args);
+				bool causesValidation = false;
+				IButtonControl button = args.CommandSource as IButtonControl;
+				if (button != null && button.CausesValidation) {
+					Page.Validate (button.ValidationGroup);
+					causesValidation = true;
+				}
+				ProcessCommand (args, causesValidation);
 				return true;
 			}
 			return base.OnBubbleEvent (source, e);
 		}
 
-		void ProcessCommand (DetailsViewCommandEventArgs args) {
+		void ProcessCommand (DetailsViewCommandEventArgs args, bool causesValidation) {
 			OnItemCommand (args);
-			ProcessEvent (args.CommandName, args.CommandArgument as string);
+			ProcessEvent (args.CommandName, args.CommandArgument as string, causesValidation);
 		}
 
 		void IPostBackEventHandler.RaisePostBackEvent (string eventArgument)
@@ -1469,10 +1481,10 @@ namespace System.Web.UI.WebControls
 				arg = new CommandEventArgs (eventArgument.Substring (0, i), eventArgument.Substring (i + 1));
 			else
 				arg = new CommandEventArgs (eventArgument, null);
-			ProcessCommand (new DetailsViewCommandEventArgs (this, arg));
+			ProcessCommand (new DetailsViewCommandEventArgs (this, arg), false);
 		}
-		
-		void ProcessEvent (string eventName, string param)
+
+		void ProcessEvent (string eventName, string param, bool causesValidation)
 		{
 			switch (eventName)
 			{
@@ -1527,7 +1539,7 @@ namespace System.Web.UI.WebControls
 				break;
 					
 			case DataControlCommands.UpdateCommandName:
-				UpdateItem (param, true);
+				UpdateItem (param, causesValidation);
 				break;
 					
 			case DataControlCommands.CancelCommandName:
@@ -1539,7 +1551,7 @@ namespace System.Web.UI.WebControls
 				break;
 					
 			case DataControlCommands.InsertCommandName:
-				InsertItem (true);
+				InsertItem (causesValidation);
 				break;
 			}
 		}
@@ -1596,8 +1608,8 @@ namespace System.Web.UI.WebControls
 		
 		void UpdateItem (string param, bool causesValidation)
 		{
-			if (causesValidation && Page != null)
-				Page.Validate ();
+			if (causesValidation && Page != null && !Page.IsValid)
+				return;
 			
 			if (CurrentMode != DetailsViewMode.Edit) throw new HttpException ();
 
@@ -1636,8 +1648,8 @@ namespace System.Web.UI.WebControls
 		
 		void InsertItem (string param, bool causesValidation)
 		{
-			if (causesValidation && Page != null)
-				Page.Validate ();
+			if (causesValidation && Page != null && !Page.IsValid)
+				return;
 			
 			if (CurrentMode != DetailsViewMode.Insert) throw new HttpException ();
 			
@@ -1683,7 +1695,7 @@ namespace System.Web.UI.WebControls
 				DetailsViewDeletedEventArgs dargs = new DetailsViewDeletedEventArgs (0, null, currentEditRowKeys, currentEditNewValues);
 				OnItemDeleted (dargs);
 			}
-			if (PageIndex == PageCount - 1)
+			if (PageIndex > 0 && PageIndex == PageCount - 1)
 				PageIndex --;
 			RequireBinding ();
 		}
@@ -1905,6 +1917,7 @@ namespace System.Web.UI.WebControls
 				string script = string.Format ("var {0} = new Object ();\n", cgrid);
 				script += string.Format ("{0}.pageIndex = {1};\n", cgrid, ClientScriptManager.GetScriptLiteral (PageIndex));
 				script += string.Format ("{0}.uid = {1};\n", cgrid, ClientScriptManager.GetScriptLiteral (UniqueID));
+				script += string.Format ("{0}.form = {1};\n", cgrid, Page.theForm);
 				Page.ClientScript.RegisterStartupScript (typeof(TreeView), this.UniqueID, script, true);
 				
 				// Make sure the basic script infrastructure is rendered
@@ -1934,19 +1947,6 @@ namespace System.Web.UI.WebControls
 			table.Render (writer);
 		}
 
-        private Table ContainedTable
-        {
-            get
-            {
-                if (table == null)
-                {
-                    table = CreateTable();
-                    Controls.Add(table);
-                }
-                return table;
-            }
-        }
-
         PostBackOptions IPostBackContainer.GetPostBackOptions(IButtonControl control)
         {
 			if (control == null)
