@@ -382,6 +382,10 @@ namespace System.Web.UI.WebControls {
 		override void OnPreRender (EventArgs e)
 		{
 			base.OnPreRender (e);
+#if NET_2_0
+			if (Page != null && Enabled)
+				Page.RegisterEnabledControl (this);
+#endif
 		}
 
 #if NET_2_0
@@ -486,6 +490,9 @@ namespace System.Web.UI.WebControls {
 				}
 				writer.WriteAttribute ("value", item.Value, true);
 
+				if (item.HasAttributes)
+					item.Attributes.Render (writer);
+
 				writer.Write (">");
 				string encoded = HttpUtility.HtmlEncode (item.Text);
 				writer.Write (encoded);
@@ -639,3 +646,4 @@ namespace System.Web.UI.WebControls {
 
 
 
+
