@@ -114,11 +114,18 @@ namespace System.Web.UI.WebControls {
 
 		bool IsAuthenticated {
 			get {
-				if (Page != null)
-					isAuthenticated = Page.Request.IsAuthenticated;
-
 				return isAuthenticated;
 			}
+			set {
+				if (value == isAuthenticated)
+					return;
+				
+				isAuthenticated = value;
+
+				OnViewChanging (EventArgs.Empty);
+				ChildControlsCreated = false;
+				OnViewChanged (EventArgs.Empty);
+			}
 		}
 
 		[MonoTODO ("Handle RoleGroups")]
@@ -126,11 +133,13 @@ namespace System.Web.UI.WebControls {
 		{
 			Controls.Clear ();
 			Control c = new Control ();
-			if (IsAuthenticated) {
+			if (Page != null && Page.Request.IsAuthenticated) {
+				isAuthenticated = true;
 				if (LoggedInTemplate != null)
 					LoggedInTemplate.InstantiateIn (c);
 			}
 			else {
+				isAuthenticated = false;
 				if (AnonymousTemplate != null)
 					AnonymousTemplate.InstantiateIn (c);
 			}
@@ -155,14 +164,10 @@ namespace System.Web.UI.WebControls {
 
 		protected internal override void LoadControlState (object savedState)
 		{
-			if (savedState == null) {
-				base.LoadControlState (savedState);
+			if (savedState == null)
 				return;
-			}
 
-			Pair pair = (Pair)savedState;
-			base.LoadControlState (pair.First);
-			isAuthenticated = (bool)pair.Second;
+			isAuthenticated = (bool) savedState;
 		}
 
 		protected internal override void OnInit (EventArgs e)
@@ -175,8 +180,8 @@ namespace System.Web.UI.WebControls {
 		protected internal override void OnPreRender (EventArgs e)
 		{
 			base.OnPreRender (e);
-			isAuthenticated = IsAuthenticated;
-			EnsureChildControls ();
+			if (Page != null)
+				IsAuthenticated = Page.Request.IsAuthenticated;
 		}
 
 		protected virtual void OnViewChanged (EventArgs e)
@@ -200,11 +205,10 @@ namespace System.Web.UI.WebControls {
 
 		protected internal override object SaveControlState ()
 		{
-			object baseState = base.SaveControlState ();
 			if (isAuthenticated)
-				return new Pair (baseState, isAuthenticated);
+				return isAuthenticated;
 
-			return baseState;
+			return null;
 		}
 
 		[MonoTODO ("for design-time usage - no more details available")]
