@@ -72,7 +72,7 @@ namespace System.Web.UI.WebControls
 
 		Hashtable _menuItemControls;
 		bool _requiresChildControlsDataBinding;
-
+		SiteMapNode _currSiteMapNode;
 		int registeredStylesCounter = -1;
 		List<Style> levelSelectedLinkStyles;
 		List<Style> levelMenuItemLinkStyles;
@@ -856,12 +856,24 @@ namespace System.Web.UI.WebControls
 			EnsureChildControlsDataBound ();
 		}
 
-		private void FillBoundChildrenRecursive (IHierarchicalEnumerable hEnumerable, MenuItemCollection itemCollection) {
+		private void FillBoundChildrenRecursive (IHierarchicalEnumerable hEnumerable, MenuItemCollection itemCollection)
+		{
+			if (hEnumerable == null)
+				return;
 			foreach (object obj in hEnumerable) {
 				IHierarchyData hdata = hEnumerable.GetHierarchyData (obj);
 				MenuItem item = new MenuItem ();
 				itemCollection.Add (item);
 				item.Bind (hdata);
+
+				SiteMapNode siteMapNode = hdata as SiteMapNode;
+				if (siteMapNode != null) {
+					if (_currSiteMapNode == null)
+						_currSiteMapNode = siteMapNode.Provider.CurrentNode;
+					if (siteMapNode == _currSiteMapNode)
+						item.Selected = true;
+				}
+				
 				OnMenuItemDataBound (new MenuEventArgs (item));
 
 				if (hdata == null || !hdata.HasChildren)
@@ -1218,7 +1230,8 @@ namespace System.Web.UI.WebControls
 				script += string.Format ("{0}.dynamicHover = {1};\n", cmenu, ClientScriptManager.GetScriptLiteral (dynamicHoverStyle.RegisteredCssClass));
 				script += string.Format ("{0}.dynamicLinkHover = {1};\n", cmenu, ClientScriptManager.GetScriptLiteral (DynamicHoverLinkStyle.RegisteredCssClass));
 			}
-			
+
+			Page.ClientScript.RegisterWebFormClientScript ();
 			Page.ClientScript.RegisterStartupScript (typeof(Menu), ClientID, script, true);
 
 		}
@@ -1514,6 +1527,8 @@ namespace System.Web.UI.WebControls
 			if (itemSpacing != Unit.Empty && (item.Depth > 0 || !isFirst))
 				RenderMenuItemSpacing (writer, itemSpacing, vertical);
 
+			if(!String.IsNullOrEmpty(item.ToolTip))
+				writer.AddAttribute (HtmlTextWriterAttribute.Title, item.ToolTip);
 			if (vertical)
 				writer.RenderBeginTag (HtmlTextWriterTag.Tr);
 
@@ -1612,18 +1627,6 @@ namespace System.Web.UI.WebControls
 			writer.RenderBeginTag (HtmlTextWriterTag.Table);
 			writer.RenderBeginTag (HtmlTextWriterTag.Tr);
 
-			if (item.ImageUrl != "") {
-				writer.RenderBeginTag (HtmlTextWriterTag.Td);
-				RenderItemHref (writer, item);
-				writer.RenderBeginTag (HtmlTextWriterTag.A);
-				writer.AddAttribute ("src", ResolveClientUrl (item.ImageUrl));
-				writer.AddAttribute ("border", "0");
-				writer.RenderBeginTag (HtmlTextWriterTag.Img);
-				writer.RenderEndTag ();	// IMG
-				writer.RenderEndTag ();	// A
-				writer.RenderEndTag ();	// TD
-			}
-
 			// Menu item text
 
 			if (vertical)
@@ -1774,14 +1777,26 @@ namespace System.Web.UI.WebControls
 			if (_menuItemControls!=null && _menuItemControls [item] != null) {
 				((Control) _menuItemControls [item]).Render (writer);
 			}
-			else if (isDynamicItem && DynamicItemFormatString.Length > 0) {
-				writer.Write (string.Format (DynamicItemFormatString, item.Text));
-			}
-			else if (!isDynamicItem && StaticItemFormatString.Length > 0) {
-				writer.Write (string.Format (StaticItemFormatString, item.Text));
-			}
 			else {
-				writer.Write (item.Text);
+
+				if (!String.IsNullOrEmpty (item.ImageUrl)) {
+					writer.AddAttribute (HtmlTextWriterAttribute.Src, ResolveClientUrl (item.ImageUrl));
+					writer.AddAttribute (HtmlTextWriterAttribute.Alt, item.ToolTip);
+					writer.AddStyleAttribute (HtmlTextWriterStyle.BorderStyle, "none");
+					writer.AddStyleAttribute (HtmlTextWriterStyle.VerticalAlign, "middle");
+					writer.RenderBeginTag (HtmlTextWriterTag.Img);
+					writer.RenderEndTag ();	// IMG
+				}
+
+				if (isDynamicItem && DynamicItemFormatString.Length > 0) {
+					writer.Write (string.Format (DynamicItemFormatString, item.Text));
+				}
+				else if (!isDynamicItem && StaticItemFormatString.Length > 0) {
+					writer.Write (string.Format (StaticItemFormatString, item.Text));
+				}
+				else {
+					writer.Write (item.Text);
+				}
 			}
 		}
 			
@@ -1845,11 +1860,15 @@ namespace System.Web.UI.WebControls
 				writer.AddStyleAttribute ("cursor", "text");
 			}
 			else if (item.NavigateUrl != "") {
-				writer.AddAttribute ("href", ResolveClientUrl (item.NavigateUrl));
-				if (item.Target != "")
-					writer.AddAttribute ("target", item.Target);
-				else if (Target != "")
-					writer.AddAttribute ("target", Target);
+				string target = item.Target != "" ? item.Target : Target;
+#if TARGET_J2EE
+				string navUrl = ResolveClientUrl (item.NavigateUrl, String.Compare (target, "_blank", StringComparison.InvariantCultureIgnoreCase) != 0);
+#else
+				string navUrl = ResolveClientUrl (item.NavigateUrl);
+#endif
+				writer.AddAttribute ("href", navUrl);
+				if (target != "")
+					writer.AddAttribute ("target", target);
 			}
 			else {
 				writer.AddAttribute ("href", GetClientEvent (item));
