@@ -63,8 +63,8 @@ namespace System.Web.UI.WebControls
 		string typeName = null;
 		string updateMethod = null;
 		private static string privateBinPath = null;
-		
-		StateBag viewState = new StateBag ();
+
+		bool isTrackingViewState = false;
 		ParameterCollection selectParameters;
 		ParameterCollection updateParameters;
 		ParameterCollection deleteParameters;
@@ -261,10 +261,6 @@ namespace System.Web.UI.WebControls
 			}
 		}
 		
-		StateBag ViewState {
-			get { return viewState; }
-		}
-		
 		public override bool CanDelete {
 			get { return DeleteMethod.Length > 0; }
 		}
@@ -293,15 +289,12 @@ namespace System.Web.UI.WebControls
 		public override bool CanUpdate {
 			get { return UpdateMethod.Length > 0; }
 		}
-		
+
+		// LAME SPEC: MSDN says value should be stored in ViewState but tests show otherwise.
+		private ConflictOptions conflictDetection = ConflictOptions.OverwriteChanges;
 		public ConflictOptions ConflictDetection {
-			get {
-				object ret = ViewState ["ConflictDetection"];
-				return ret != null ? (ConflictOptions)ret : ConflictOptions.OverwriteChanges;
-			}
-			set {
-				ViewState ["ConflictDetection"] = value;
-			}
+			get { return conflictDetection; }
+			set { conflictDetection = value; }
 		}
 
 		public bool ConvertNullToDBNull	{
@@ -315,6 +308,7 @@ namespace System.Web.UI.WebControls
 			}
 			set {
 				dataObjectTypeName = value;
+				dataObjectType = null;
 			}
 		}
 
@@ -331,9 +325,6 @@ namespace System.Web.UI.WebControls
 			get {
 				if (deleteParameters == null) {
 					deleteParameters = new ParameterCollection ();
-					deleteParameters.ParametersChanged += new EventHandler (OnParametersChanged); 
-					if (IsTrackingViewState)
-						((IStateManager)deleteParameters).TrackViewState ();
 				}
 				return deleteParameters;
 			}
@@ -382,9 +373,6 @@ namespace System.Web.UI.WebControls
 			get {
 				if (insertParameters == null) {
 					insertParameters = new ParameterCollection ();
-					insertParameters.ParametersChanged += new EventHandler (OnParametersChanged); 
-					if (IsTrackingViewState)
-						((IStateManager)insertParameters).TrackViewState ();
 				}
 				return insertParameters;
 			}
@@ -479,9 +467,6 @@ namespace System.Web.UI.WebControls
 			get {
 				if (updateParameters == null) {
 					updateParameters = new ParameterCollection ();
-					updateParameters.ParametersChanged += new EventHandler (OnParametersChanged); 
-					if (IsTrackingViewState)
-						((IStateManager)updateParameters).TrackViewState ();
 				}
 				return updateParameters;
 			}
@@ -505,7 +490,6 @@ namespace System.Web.UI.WebControls
 			if (type != null)
 				return type;
 
-#if !TARGET_JVM
 			IList tla;
 			if ((tla = BuildManager.TopLevelAssemblies) != null) {
 				foreach (Assembly asm in tla) {
@@ -516,7 +500,7 @@ namespace System.Web.UI.WebControls
 						return type;
 				}
 			}
-#endif
+
 			if (!Directory.Exists (PrivateBinPath))
 				return null;
 
@@ -548,7 +532,7 @@ namespace System.Web.UI.WebControls
 			get {
 				if (dataObjectType == null) {
 					dataObjectType = LoadType (DataObjectTypeName);
-					if (objectType == null)
+					if (dataObjectType == null)
 						throw new InvalidOperationException ("Type not found: " + DataObjectTypeName);
 				}
 				return dataObjectType;
@@ -584,7 +568,7 @@ namespace System.Web.UI.WebControls
 			MethodInfo method;
 			
 			if (DataObjectTypeName.Length == 0) {
-				paramValues = MergeParameterValues (InsertParameters, values, null, true);
+				paramValues = MergeParameterValues (InsertParameters, values, null);
 				method = GetObjectMethod (InsertMethod, paramValues, DataObjectMethodType.Insert);
 			} else {
 				method = ResolveDataObjectMethod (InsertMethod, values, null, out paramValues);
@@ -618,20 +602,13 @@ namespace System.Web.UI.WebControls
 				throw new InvalidOperationException ("ConflictDetection is set to CompareAllValues and oldValues collection is null or empty.");
 
 			IDictionary oldDataValues;
-			if (ConflictDetection == ConflictOptions.CompareAllValues) {
-				oldDataValues = new Hashtable ();
-				foreach (DictionaryEntry de in keys)
-					oldDataValues [de.Key] = de.Value;
-				foreach (DictionaryEntry de in oldValues)
-					oldDataValues [de.Key] = de.Value;
-			} else
-				oldDataValues = keys;
+			oldDataValues = BuildOldValuesList (keys, oldValues, false);
 					
 			IOrderedDictionary paramValues;
 			MethodInfo method;
 			
 			if (DataObjectTypeName.Length == 0) {
-				paramValues = MergeParameterValues (DeleteParameters, null, oldDataValues, true);
+				paramValues = MergeParameterValues (DeleteParameters, null, oldDataValues);
 				method = GetObjectMethod (DeleteMethod, paramValues, DataObjectMethodType.Delete);
 			} else {
 				method = ResolveDataObjectMethod (DeleteMethod, oldDataValues, null, out paramValues);
@@ -656,51 +633,37 @@ namespace System.Web.UI.WebControls
 
 			return -1;
 		}
-		
+
 		protected override int ExecuteUpdate (IDictionary keys, IDictionary values, IDictionary oldValues)
 		{
 			IOrderedDictionary paramValues;
 			MethodInfo method;
-			
+
+			IDictionary oldDataValues;
+			oldDataValues = BuildOldValuesList (keys, oldValues, true);
+
 			if (DataObjectTypeName.Length == 0)
 			{
 				IDictionary dataValues;
-				IDictionary oldDataValues;
-				if (ConflictDetection == ConflictOptions.CompareAllValues) {
-					oldDataValues = new Hashtable ();
-					dataValues = values;
-					foreach (DictionaryEntry de in keys)
-						oldDataValues [de.Key] = de.Value;
-					foreach (DictionaryEntry de in oldValues)
-						oldDataValues [de.Key] = de.Value;
-				} else {
-					oldDataValues = keys;
-					dataValues = values;
-				}
-				paramValues = MergeParameterValues (UpdateParameters, dataValues, oldDataValues, false);
+				dataValues = values;
+				paramValues = MergeParameterValues (UpdateParameters, dataValues, oldDataValues);
 				method = GetObjectMethod (UpdateMethod, paramValues, DataObjectMethodType.Update);
 			}
 			else
 			{
-				IDictionary dataValues = new Hashtable ();
-				IDictionary oldDataValues;
-				
-				foreach (DictionaryEntry de in values)
-					dataValues [de.Key] = de.Value;
-					
-				if (ConflictDetection == ConflictOptions.CompareAllValues) {
-					oldDataValues = new Hashtable ();
-					foreach (DictionaryEntry de in keys) {
-						oldDataValues [de.Key] = de.Value;
-						dataValues [de.Key] = de.Value;
-					}
-					foreach (DictionaryEntry de in oldValues)
-						oldDataValues [de.Key] = de.Value;
-				} else {
+				if (ConflictDetection != ConflictOptions.CompareAllValues) {
 					oldDataValues = null;
+				}
+				IDictionary dataValues = new Hashtable ();
+				if (keys != null) {
 					foreach (DictionaryEntry de in keys)
 						dataValues [de.Key] = de.Value;
 				}
+				if (values != null) {
+					foreach (DictionaryEntry de in values)
+						dataValues [de.Key] = de.Value;
+				}
+
 				method = ResolveDataObjectMethod (UpdateMethod, dataValues, oldDataValues, out paramValues);
 			}			
 
@@ -722,12 +685,37 @@ namespace System.Web.UI.WebControls
 
 			return -1;
 		}
-		
+
+		private IDictionary BuildOldValuesList (IDictionary keys, IDictionary oldValues, bool keysWin) 
+		{
+			IDictionary oldDataValues;
+			if (ConflictDetection == ConflictOptions.CompareAllValues) {
+				oldDataValues = new Hashtable ();
+				if (keys != null && !keysWin) {
+					foreach (DictionaryEntry de in keys)
+						oldDataValues [de.Key] = de.Value;
+				}
+				if (oldValues != null) {
+					foreach (DictionaryEntry de in oldValues)
+						oldDataValues [de.Key] = de.Value;
+				}
+				if (keys != null && keysWin) {
+					foreach (DictionaryEntry de in keys)
+						oldDataValues [de.Key] = de.Value;
+				}
+			}
+			else {
+				oldDataValues = keys;
+			}
+
+			return oldDataValues;
+		}
+
 		protected internal override IEnumerable ExecuteSelect (DataSourceSelectArguments arguments)
 		{
 			arguments.RaiseUnsupportedCapabilitiesError (this);
 
-			IOrderedDictionary paramValues = MergeParameterValues (SelectParameters, null, null, true);
+			IOrderedDictionary paramValues = MergeParameterValues (SelectParameters, null, null);
 			ObjectDataSourceSelectingEventArgs args = new ObjectDataSourceSelectingEventArgs (paramValues, arguments, false);
 
 			object result = null;
@@ -865,6 +853,7 @@ namespace System.Web.UI.WebControls
 			MemberInfo[] methods = ObjectType.GetMember (methodName, MemberTypes.Method, BindingFlags.Instance | 
 										 BindingFlags.Static | 
 										 BindingFlags.Public | 
+										 BindingFlags.IgnoreCase |
 										 BindingFlags.FlattenHierarchy);
 			if (methods.Length > 1) {
 				// MSDN: The ObjectDataSource resolves method overloads by method name and number
@@ -1016,13 +1005,7 @@ namespace System.Web.UI.WebControls
 
 			foreach (DictionaryEntry de in values) {
 				string valueName = format == true ? FormatOldParameter (de.Key.ToString ()) : de.Key.ToString ();
-				if (name == valueName)
-					return values [de.Key];
-			}
-			foreach (DictionaryEntry de in values) {
-				string valueName = format == true ? FormatOldParameter (de.Key.ToString ()) : de.Key.ToString ();
-				valueName = valueName.ToLower ();
-				if (name.ToLower () == valueName)
+				if (String.Compare (name, valueName, StringComparison.InvariantCultureIgnoreCase) == 0)
 					return values [de.Key];
 			}
 			return null;
@@ -1037,33 +1020,27 @@ namespace System.Web.UI.WebControls
 		/// <param name="allwaysAddNewValues">true for insert, as current item is
 		/// irrelevant for insert</param>
 		/// <returns>merged values</returns>
-		IOrderedDictionary MergeParameterValues (ParameterCollection viewParams, IDictionary values, IDictionary oldValues, bool allwaysAddNewValues)
+		IOrderedDictionary MergeParameterValues (ParameterCollection viewParams, IDictionary values, IDictionary oldValues)
 		{
+			IOrderedDictionary parametersValues = viewParams.GetValues (context, owner);
 			OrderedDictionary mergedValues = new OrderedDictionary (StringComparer.InvariantCultureIgnoreCase);
-			foreach (Parameter p in viewParams) {
-				bool oldAdded = false;
+			foreach (string parameterName in parametersValues.Keys) {
+				mergedValues [parameterName] = parametersValues [parameterName];
 				if (oldValues != null) {
-					object value = FindValueByName (p.Name, oldValues, true);
+					object value = FindValueByName (parameterName, oldValues, true);
 					if (value != null) {
-						object dataValue = p.ConvertValue (value);
-						mergedValues [FormatOldParameter (p.Name)] = dataValue;
-						oldAdded = true;
+						object dataValue = viewParams [parameterName].ConvertValue (value);
+						mergedValues [parameterName] = dataValue;
 					}
 				}
 
-				bool newAdded = false;
 				if (values != null) {
-					object value = FindValueByName (p.Name, values, false);
+					object value = FindValueByName (parameterName, values, false);
 					if (value != null) {
-						object dataValue = p.ConvertValue (value);
-						mergedValues [p.Name] = dataValue;
-						newAdded = true;
+						object dataValue = viewParams [parameterName].ConvertValue (value);
+						mergedValues [parameterName] = dataValue;
 					}
 				}
-				if ((!newAdded && !oldAdded) || allwaysAddNewValues) {
-					object val = p.GetValue (context, owner);
-					mergedValues [p.Name] = val;
-				}
 			}
 
 			if (values != null) {
@@ -1073,9 +1050,11 @@ namespace System.Web.UI.WebControls
 			}
 
 			if (oldValues != null) {
-				foreach (DictionaryEntry de in oldValues)
-					if (FindValueByName ((string) de.Key, mergedValues, true) == null)
-						mergedValues [FormatOldParameter ((string) de.Key)] = de.Value;
+				foreach (DictionaryEntry de in oldValues) {
+					string oldValueKey = FormatOldParameter ((string) de.Key);
+					if (FindValueByName (oldValueKey, mergedValues, false) == null)
+						mergedValues [oldValueKey] = de.Value;
+				}
 			}
 
 			return mergedValues;
@@ -1136,30 +1115,28 @@ namespace System.Web.UI.WebControls
 		
 		protected virtual void LoadViewState (object savedState)
 		{
-			object[] state = (savedState == null) ? new object [6] : (object[]) savedState;
-			viewState.LoadViewState (state[0]);
-			((IStateManager)SelectParameters).LoadViewState (state[1]); 
-			((IStateManager)UpdateParameters).LoadViewState (state[2]); 
-			((IStateManager)DeleteParameters).LoadViewState (state[3]); 
-			((IStateManager)InsertParameters).LoadViewState (state[4]); 
-			((IStateManager)FilterParameters).LoadViewState (state[5]); 
+			object[] state = (savedState == null) ? new object [5] : (object[]) savedState;
+			((IStateManager)SelectParameters).LoadViewState (state[0]); 
+			((IStateManager)UpdateParameters).LoadViewState (state[1]); 
+			((IStateManager)DeleteParameters).LoadViewState (state[2]); 
+			((IStateManager)InsertParameters).LoadViewState (state[3]); 
+			((IStateManager)FilterParameters).LoadViewState (state[4]); 
 		}
 
 		protected virtual object SaveViewState()
 		{
-			object[] state = new object [6];
-			state [0] = viewState.SaveViewState ();
+			object[] state = new object [5];
 			
 			if (selectParameters != null)
-				state [1] = ((IStateManager)selectParameters).SaveViewState ();
+				state [0] = ((IStateManager)selectParameters).SaveViewState ();
 			if (updateParameters != null)
-				state [2] = ((IStateManager)updateParameters).SaveViewState ();
+				state [1] = ((IStateManager)updateParameters).SaveViewState ();
 			if (deleteParameters != null)
-				state [3] = ((IStateManager)deleteParameters).SaveViewState ();
+				state [2] = ((IStateManager)deleteParameters).SaveViewState ();
 			if (insertParameters != null)
-				state [4] = ((IStateManager)insertParameters).SaveViewState ();
+				state [3] = ((IStateManager)insertParameters).SaveViewState ();
 			if (filterParameters != null)
-				state [5] = ((IStateManager)filterParameters).SaveViewState ();
+				state [4] = ((IStateManager)filterParameters).SaveViewState ();
 			
 			foreach (object ob in state)
 				if (ob != null) return state;
@@ -1169,17 +1146,15 @@ namespace System.Web.UI.WebControls
 		
 		protected virtual void TrackViewState()
 		{
-			viewState.TrackViewState ();
+			isTrackingViewState = true;
+
 			if (selectParameters != null) ((IStateManager)selectParameters).TrackViewState ();
-			if (updateParameters != null) ((IStateManager)updateParameters).TrackViewState ();
-			if (deleteParameters != null) ((IStateManager)deleteParameters).TrackViewState ();
-			if (insertParameters != null) ((IStateManager)insertParameters).TrackViewState ();
 			if (filterParameters != null) ((IStateManager)filterParameters).TrackViewState ();
 		}
 		
 		protected bool IsTrackingViewState
 		{
-			get { return viewState.IsTrackingViewState; }
+			get { return isTrackingViewState; }
 		}
 		
 		
@@ -1207,3 +1182,6 @@ namespace System.Web.UI.WebControls
 #endif
 
 
+
+
+
