@@ -979,7 +979,10 @@ namespace System.Web.UI.WebControls
 			FillBoundChildrenRecursive (e, Nodes);
 		}
 		
-		private void FillBoundChildrenRecursive (IHierarchicalEnumerable hEnumerable, TreeNodeCollection nodeCollection) {
+		private void FillBoundChildrenRecursive (IHierarchicalEnumerable hEnumerable, TreeNodeCollection nodeCollection)
+		{
+			if (hEnumerable == null)
+				return;
 			foreach (object obj in hEnumerable) {
 				IHierarchyData hdata = hEnumerable.GetHierarchyData (obj);
 				TreeNode child = new TreeNode ();
@@ -1016,11 +1019,8 @@ namespace System.Web.UI.WebControls
 				res = true;
 			}
 
-			if (ShowCheckBoxes != TreeNodeTypes.None) {
-				UnsetCheckStates (Nodes, postCollection);
-				SetCheckStates (postCollection);
-				res = true;
-			}
+			UnsetCheckStates (Nodes, postCollection);
+			SetCheckStates (postCollection);
 			
 			if (EnableClientScript) {
 				string states = postCollection [ClientID + "_ExpandStates"];
@@ -1052,6 +1052,7 @@ namespace System.Web.UI.WebControls
 			
 			string ctree = ClientID + "_data";
 			string script = string.Format ("var {0} = new Object ();\n", ctree);
+			script += string.Format ("{0}.uid = {1};\n", ctree, ClientScriptManager.GetScriptLiteral (UniqueID));
 			script += string.Format ("{0}.showImage = {1};\n", ctree, ClientScriptManager.GetScriptLiteral (ShowExpandCollapse));
 			
 			if (ShowExpandCollapse) {
@@ -1067,6 +1068,7 @@ namespace System.Web.UI.WebControls
 			}
 
 			if (Page != null) {
+				script += string.Format ("{0}.form = {1};\n", ctree, Page.theForm);
 				script += string.Format ("{0}.populateFromClient = {1};\n", ctree, ClientScriptManager.GetScriptLiteral (PopulateNodesFromClient));
 				script += string.Format ("{0}.expandAlt = {1};\n", ctree, ClientScriptManager.GetScriptLiteral (GetNodeImageToolTip (true, null)));
 				script += string.Format ("{0}.collapseAlt = {1};\n", ctree, ClientScriptManager.GetScriptLiteral (GetNodeImageToolTip (false, null)));
@@ -1077,10 +1079,9 @@ namespace System.Web.UI.WebControls
 
 				if (EnableClientScript) {
 					Page.ClientScript.RegisterHiddenField (ClientID + "_ExpandStates", GetExpandStates ());
-				
+
 					// Make sure the basic script infrastructure is rendered
-					Page.ClientScript.GetCallbackEventReference (this, "null", "", "null");
-					Page.ClientScript.GetPostBackClientHyperlink (this, "");
+					Page.ClientScript.RegisterWebFormClientScript ();
 				}
 
 				if (EnableClientScript && PopulateNodesFromClient) {
@@ -1408,16 +1409,7 @@ namespace System.Web.UI.WebControls
 			
 			// Checkbox
 			
-			bool showChecks;
-			if (node.ShowCheckBox.HasValue)
-				showChecks = node.ShowCheckBox.Value;
-			else
-				showChecks = (ShowCheckBoxes == TreeNodeTypes.All) ||
-							 ((ShowCheckBoxes & TreeNodeTypes.Leaf) > 0 && node.IsLeafNode) ||
-							 ((ShowCheckBoxes & TreeNodeTypes.Parent) > 0 && node.IsParentNode && node.Parent != null) ||
-							 ((ShowCheckBoxes & TreeNodeTypes.Root) > 0 && node.Parent == null && node.ChildNodes.Count > 0);
-
-			if (showChecks) {
+			if (node.ShowCheckBoxInternal) {
 				writer.AddAttribute ("name", ClientID + "_cs_" + node.Path);
 				writer.AddAttribute ("type", "checkbox");
 				writer.AddAttribute ("title", node.Text);
@@ -1658,11 +1650,15 @@ namespace System.Web.UI.WebControls
 				writer.AddAttribute ("title", node.ToolTip);
 
 			if (node.NavigateUrl != "") {
-				writer.AddAttribute ("href", ResolveClientUrl (node.NavigateUrl));
-				if (node.Target.Length > 0)
-					writer.AddAttribute ("target", node.Target);
-				else if (Target.Length > 0)
-					writer.AddAttribute ("target", Target);
+				string target = node.Target.Length > 0 ? node.Target : Target;
+#if TARGET_J2EE
+				string navUrl = ResolveClientUrl (node.NavigateUrl, String.Compare (target, "_blank", StringComparison.InvariantCultureIgnoreCase) != 0);
+#else
+				string navUrl = ResolveClientUrl (node.NavigateUrl);
+#endif
+				writer.AddAttribute ("href", navUrl);
+				if (target.Length > 0)
+					writer.AddAttribute ("target", target);
 				writer.RenderBeginTag (HtmlTextWriterTag.A);
 			}
 			else if (node.SelectAction != TreeNodeSelectAction.None) {
@@ -1775,9 +1771,9 @@ namespace System.Web.UI.WebControls
 		void UnsetCheckStates (TreeNodeCollection col, NameValueCollection states)
 		{
 			foreach (TreeNode node in col) {
-				if (node.Checked) {
-					string val = states [ClientID + "_cs_" + node.Path];
-					if (val != "on") node.Checked = false;
+				if (node.ShowCheckBoxInternal && node.Checked) {
+					if (states == null || states [ClientID + "_cs_" + node.Path] == null)
+						node.Checked = false;
 				}
 				if (node.HasChildData)
 					UnsetCheckStates (node.ChildNodes, states);
@@ -1786,13 +1782,16 @@ namespace System.Web.UI.WebControls
 		
 		void SetCheckStates (NameValueCollection states)
 		{
+			if (states == null)
+				return;
+
 			string keyPrefix = ClientID + "_cs_";
 			foreach (string key in states) {
-				if (key.StartsWith (keyPrefix)) {
+				if (key.StartsWith (keyPrefix, StringComparison.Ordinal)) {
 					string id = key.Substring (keyPrefix.Length);
 					TreeNode node = FindNodeByPos (id);
-					if (node != null)
-						node.Checked = (Context.Request.Form [key] == "on");
+					if (node != null && !node.Checked)
+						node.Checked = true;
 				}
 			}
 		}
