@@ -40,6 +40,7 @@ using System.Collections.Generic;
 using System.Text;
 using System.Collections.Specialized;
 using System.Web.Util;
+using System.Globalization;
 
 namespace System.Web.UI
 {
@@ -53,13 +54,14 @@ namespace System.Web.UI
 		Hashtable registeredArrayDeclares;
 		ScriptEntry clientScriptBlocks;
 		ScriptEntry startupScriptBlocks;
-		Hashtable hiddenFields;
+		internal Hashtable hiddenFields;
 		ScriptEntry submitStatements;
 		ScriptEntry scriptIncludes;
 		Page page;
 #if NET_2_0
 		List <int> eventValidationValues;
 		Hashtable expandoAttributes;
+		bool _hasRegisteredForEventValidationOnCallback;
 #endif
 		
 		internal ClientScriptManager (Page page)
@@ -99,7 +101,11 @@ namespace System.Web.UI
 				throw new ArgumentNullException ("control");
 			
 			page.RequiresPostBackScript ();
-			return String.Format ("{0}('{1}','{2}')", page.PostBackFunctionName, control.UniqueID, argument);
+#if NET_2_0
+			return String.Format ("{0}.__doPostBack('{1}','{2}')", page.theForm, control.UniqueID, argument);
+#else
+			return String.Format ("__doPostBack('{0}','{1}')", control.UniqueID, argument);
+#endif
 		}
 
 #if NET_2_0
@@ -126,7 +132,7 @@ namespace System.Web.UI
 		{
 			if (options == null)
 				throw new ArgumentNullException ("options");
-			
+
 			if (options.ActionUrl == null && options.ValidationGroup == null && !options.TrackFocus && 
 				!options.AutoPostBack && !options.PerformValidation)
 			{
@@ -138,33 +144,46 @@ namespace System.Web.UI
 				else
 					return GetPostBackEventReference (options.TargetControl, options.Argument);
 			}
-			
-			if (options.ActionUrl != null)
+
+			RegisterWebFormClientScript ();
+
+			string actionUrl = options.ActionUrl;
+			if (actionUrl != null)
 				RegisterHiddenField (Page.PreviousPageID, page.Request.FilePath);
-			
-			if (options.ClientSubmit || options.ActionUrl != null)
-				page.RequiresPostBackScript ();
-			
-			return String.Format ("{0}WebForm_DoPostback({1},{2},{3},{4},{5},{6},{7},{8})", 
-					options.RequiresJavaScriptProtocol ? "javascript:" : "",
+
+			if(options.TrackFocus)
+				RegisterHiddenField (Page.LastFocusID, String.Empty);
+
+			string prefix = options.RequiresJavaScriptProtocol ? "javascript:" : "";
+#if TARGET_J2EE
+			// Allow the page to transform ActionUrl to a portlet action url
+			if (actionUrl != null && page.PortletNamespace != null) {
+				actionUrl = page.CreateActionUrl(actionUrl);
+				prefix += "Portal";
+			}
+#endif
+
+			return String.Format ("{0}WebForm_DoPostback({1},{2},{3},{4},{5},{6},{7},{8},{9})", 
+					prefix,
 					ClientScriptManager.GetScriptLiteral (options.TargetControl.UniqueID), 
 					ClientScriptManager.GetScriptLiteral (options.Argument),
-					ClientScriptManager.GetScriptLiteral (options.ActionUrl),
+					ClientScriptManager.GetScriptLiteral (actionUrl),
 					ClientScriptManager.GetScriptLiteral (options.AutoPostBack),
 					ClientScriptManager.GetScriptLiteral (options.PerformValidation),
 					ClientScriptManager.GetScriptLiteral (options.TrackFocus),
 					ClientScriptManager.GetScriptLiteral (options.ClientSubmit),
-					ClientScriptManager.GetScriptLiteral (options.ValidationGroup)
+					ClientScriptManager.GetScriptLiteral (options.ValidationGroup),
+					page.theForm
 				);
 		}
 
 		internal void RegisterWebFormClientScript ()
 		{
-			if (!IsClientScriptIncludeRegistered (typeof (Page), "webform"))
-				RegisterClientScriptInclude (typeof (Page), "webform", GetWebResourceUrl (typeof (Page), "webform.js"));
+			if (IsClientScriptIncludeRegistered (typeof (Page), "webform"))
+				return;
 
-			if (!IsClientScriptIncludeRegistered (typeof (Page), "callback"))
-				RegisterClientScriptInclude (typeof (Page), "callback", GetWebResourceUrl (typeof (Page), "callback.js"));
+			RegisterClientScriptInclude (typeof (Page), "webform", GetWebResourceUrl (typeof (Page), "webform.js"));
+			page.RequiresPostBackScript ();
 		}
 		
 		public string GetCallbackEventReference (Control control, string argument, string clientCallback, string context)
@@ -189,9 +208,9 @@ namespace System.Web.UI
 
 		public string GetCallbackEventReference (string target, string argument, string clientCallback, string context, string clientErrorCallback, bool useAsync)
 		{
-			page.RequiresPostBackScript ();
+			RegisterWebFormClientScript ();
 			
-			return string.Format ("WebForm_DoCallback('{0}',{1},{2},{3},{4},{5})", target, argument, clientCallback, context, ((clientErrorCallback == null) ? "null" : clientErrorCallback), (useAsync ? "true" : "false"));
+			return string.Format ("WebForm_DoCallback('{0}',{1},{2},{3},{4},{5},{6})", target, argument, clientCallback, context, ((clientErrorCallback == null) ? "null" : clientErrorCallback), (useAsync ? "true" : "false"), page.theForm);
 		}
 #endif
 		
@@ -285,8 +304,13 @@ namespace System.Web.UI
 				entry = entry.Next;
 			}
 			
-			if (addScriptTags)
-				script = "<script language=javascript>\n<!--\n" + script + "\n// -->\n</script>";
+			if (addScriptTags) {
+				script = "<script type=\"text/javascript\"" +
+#if !NET_2_0
+					"language=\"javascript\"" +
+#endif
+					">\n<!--\n" + script + "\n// -->\n</script>";
+			}
 
 			entry = new ScriptEntry (type, key, script);
 			
@@ -423,7 +447,9 @@ namespace System.Web.UI
 				return;
 			if (uniqueId == null || uniqueId.Length == 0)
 				return;
-			if (page.LifeCycle < PageLifeCycle.Render)
+			if (page.IsCallback)
+				_hasRegisteredForEventValidationOnCallback = true;
+			else if (page.LifeCycle < PageLifeCycle.Render)
 				throw new InvalidOperationException ("RegisterForEventValidation may only be called from the Render method");
 			if (eventValidationValues == null)
 				eventValidationValues = new List <int> ();
@@ -471,16 +497,45 @@ namespace System.Web.UI
 			if (!page.EnableEventValidation || fieldValue == null || fieldValue.Length == 0)
 				return;
 			IStateFormatter fmt = page.GetFormatter ();
-			eventValidationValues = (List <int>)fmt.Deserialize (fieldValue);
+			int [] eventValues = (int []) fmt.Deserialize (fieldValue);
+#if TARGET_JVM // FIXME: No support yet for passing 'int[]' as 'T[]'
+			eventValidationValues = new List<int> (eventValues.Length);
+			for (int i = 0; i < eventValues.Length; i++)
+				eventValidationValues.Add(eventValues[i]);
+#else
+			eventValidationValues = new List<int> (eventValues);
+#endif
+			eventValidationValues.Sort ();
 		}
 		
 		internal void SaveEventValidationState ()
 		{
-			if (!page.EnableEventValidation || eventValidationValues == null || eventValidationValues.Count == 0)
+			if (!page.EnableEventValidation)
 				return;
-			eventValidationValues.Sort ();
+
+			string eventValidation = GetEventValidationStateFormatted ();
+			if (eventValidation == null)
+				return;
+
+			RegisterHiddenField (EventStateFieldName, eventValidation);
+		}
+
+		internal string GetEventValidationStateFormatted ()
+		{
+			if (eventValidationValues == null || eventValidationValues.Count == 0)
+				return null;
+
+			if(page.IsCallback && !_hasRegisteredForEventValidationOnCallback)
+				return null;
+
 			IStateFormatter fmt = page.GetFormatter ();
-			RegisterHiddenField (EventStateFieldName, fmt.Serialize (eventValidationValues));
+			int [] array = new int [eventValidationValues.Count];
+#if TARGET_JVM // FIXME: No support yet for passing 'int[]' as 'T[]'
+			((ICollection)eventValidationValues).CopyTo (array, 0);
+#else
+			eventValidationValues.CopyTo (array);
+#endif
+			return fmt.Serialize (array);
 		}
 
 		internal string EventStateFieldName
@@ -494,8 +549,7 @@ namespace System.Web.UI
 				return;
 
 			writer.WriteLine ();
-			writer.WriteLine ("<script type=\"text/javascript\">");
-			writer.WriteLine ("<!--");
+			WriteBeginScriptBlock (writer);
 
 			foreach (string controlId in expandoAttributes.Keys) {
 				writer.WriteLine ("var {0} = document.all ? document.all [\"{0}\"] : document.getElementById (\"{0}\");", controlId);
@@ -504,11 +558,26 @@ namespace System.Web.UI
 					writer.WriteLine ("{0}.{1} = \"{2}\";", controlId, attributeName, attrs [attributeName]);
 				}
 			}
-			writer.WriteLine ("// -->");
-			writer.WriteLine ("</script>");
+			WriteEndScriptBlock (writer);
 			writer.WriteLine ();
 		}
+
 #endif
+		internal void WriteBeginScriptBlock (HtmlTextWriter writer)
+		{
+			writer.WriteLine ("<script"+
+#if !NET_2_0
+				" language=\"javascript\""+
+#endif
+				" type=\"text/javascript\">");
+			writer.WriteLine ("<!--");
+		}
+
+		internal void WriteEndScriptBlock (HtmlTextWriter writer)
+		{
+			writer.WriteLine ("// -->");
+			writer.WriteLine ("</script>");
+		}
 		
 		internal void WriteHiddenFields (HtmlTextWriter writer)
 		{
@@ -517,7 +586,7 @@ namespace System.Web.UI
 	
 			foreach (string key in hiddenFields.Keys) {
 				string value = hiddenFields [key] as string;
-				writer.WriteLine ("\n<input type=\"hidden\" name=\"{0}\" id=\"{0}\" value=\"{1}\" />", key, value);
+				writer.WriteLine ("<input type=\"hidden\" name=\"{0}\" id=\"{0}\" value=\"{1}\" />", key, value);
 			}
 	
 			hiddenFields = null;
@@ -527,7 +596,25 @@ namespace System.Web.UI
 		{
 			ScriptEntry entry = scriptIncludes;
 			while (entry != null) {
-				writer.WriteLine ("\n<script src=\"{0}\" type=\"text/javascript\"></script>", entry.Script);
+				if (!entry.Rendered) {
+#if TARGET_J2EE
+					if (!page.IsPortletRender)
+#endif
+						writer.WriteLine ("\n<script src=\"{0}\" type=\"text/javascript\"></script>", entry.Script);
+#if TARGET_J2EE
+					else {
+						string scriptKey = "inc_" + entry.Key.GetHashCode ().ToString ("X");
+						writer.WriteLine ("\n<script type=\"text/javascript\">");
+						writer.WriteLine ("<!--");
+						writer.WriteLine ("if (document.{0} == null) {{", scriptKey);
+						writer.WriteLine ("\tdocument.{0} = true", scriptKey);
+						writer.WriteLine ("\tdocument.write('<script src=\"{0}\" type=\"text/javascript\"><\\/script>'); }}", entry.Script);
+						writer.WriteLine ("// -->");
+						writer.WriteLine ("</script>");
+					}
+#endif
+					entry.Rendered = true;
+				}
 				entry = entry.Next;
 			}
 		}
@@ -546,8 +633,7 @@ namespace System.Web.UI
 		{
 			if (registeredArrayDeclares != null) {
 				writer.WriteLine();
-				writer.WriteLine("<script language=\"javascript\">");
-				writer.WriteLine("<!--");
+				WriteBeginScriptBlock (writer);
 				IDictionaryEnumerator arrayEnum = registeredArrayDeclares.GetEnumerator();
 				while (arrayEnum.MoveNext()) {
 					writer.Write("\tvar ");
@@ -563,22 +649,41 @@ namespace System.Web.UI
 						writer.Write(arrayListEnum.Current);
 					}
 					writer.WriteLine(");");
+#if TARGET_J2EE
+					// in addition, add a form array declaration
+					if (page.IsPortletRender) {
+						writer.Write ("\t" + page.theForm + ".");
+						writer.Write (arrayEnum.Key);
+						writer.Write (" = ");
+						writer.Write (arrayEnum.Key);
+						writer.WriteLine (";");
+					}
+#endif
 				}
-				writer.WriteLine("// -->");
-				writer.WriteLine("</script>");
-				writer.WriteLine();
+				WriteEndScriptBlock (writer);
+				writer.WriteLine ();
 			}
 		}
 
 #if NET_2_0
 		internal string GetClientValidationEvent (string validationGroup) {
-			return "if (typeof(Page_ClientValidate) == 'function') Page_ClientValidate('" + validationGroup + "');";
+			string eventScript = "if (typeof(Page_ClientValidate) == 'function') Page_ClientValidate('" + validationGroup + "');";
+#if TARGET_J2EE
+			if (page.IsPortletRender)
+				return "if (typeof(SetValidatorContext) == 'function') SetValidatorContext ('" + page.theForm + "'); " + eventScript;
+#endif
+			return eventScript;
 		}
 #endif
 
 		internal string GetClientValidationEvent ()
 		{
-			return "if (typeof(Page_ClientValidate) == 'function') Page_ClientValidate();";
+			string eventScript = "if (typeof(Page_ClientValidate) == 'function') Page_ClientValidate();";
+#if TARGET_J2EE
+			if (page.IsPortletRender)
+				return "if (typeof(SetValidatorContext) == 'function') SetValidatorContext ('" + page.theForm + "'); " + eventScript;
+#endif
+			return eventScript;
 		}
 
 
@@ -589,22 +694,42 @@ namespace System.Web.UI
 			StringBuilder sb = new StringBuilder ();
 			ScriptEntry entry = submitStatements;
 			while (entry != null) {
+#if NET_2_0
+				sb.Append (EnsureEndsWithSemicolon (entry.Script));
+#else
 				sb.Append (entry.Script);
+#endif
 				entry = entry.Next;
 			}
+#if NET_2_0
+			RegisterClientScriptBlock ("HtmlForm-OnSubmitStatemen",
+@"<script type=""text/javascript"">
+<!--
+" + page.theForm + @".WebForm_OnSubmit = function () {
+" + sb.ToString () + @"
+return true;
+}
+// -->
+</script>");
+			return "javascript:return this.WebForm_OnSubmit();";
+
+#else
 			return sb.ToString ();
+#endif
 		}
 		
+		[MonoTODO ("optimize s.Replace")]
 		internal static string GetScriptLiteral (object ob)
 		{
 			if (ob == null)
 				return "null";
 			else if (ob is string) {
 				string s = (string)ob;
+				s = s.Replace ("\\", "\\\\");
 				s = s.Replace ("\"", "\\\"");
 				return "\"" + s + "\"";
 			} else if (ob is bool) {
-				return ob.ToString().ToLower();
+				return ob.ToString ().ToLower (CultureInfo.InvariantCulture);
 			} else {
 				return ob.ToString ();
 			}
@@ -616,6 +741,7 @@ namespace System.Web.UI
 			public string Key;
 			public string Script;
 			public ScriptEntry Next;
+			public bool Rendered;
 			 
 			public ScriptEntry (Type type, string key, string script)
 			{
@@ -628,7 +754,7 @@ namespace System.Web.UI
 #if NET_2_0
 		// helper method
 		internal static string EnsureEndsWithSemicolon (string value) {
-			if (value != null && value.Length > 0 && !value.EndsWith (";"))
+			if (value != null && value.Length > 0 && value [value.Length - 1] != ';')
 				return value += ";";
 			return value;
 		}
