@@ -157,7 +157,16 @@ namespace System.Web.UI {
 				throw new ArgumentNullException ("inputString");
 			if (page != null && EnableMac)
 				length = ValidateInput (GetAlgo (), buffer, 0, length);
-			return Deserialize (new MemoryStream (buffer, 0, length, false, false));
+#if NET_2_0
+			bool isEncrypted = ((int)buffer [--length] == 1)? true : false;
+#endif
+			Stream ms = new MemoryStream (buffer, 0, length, false, false);
+#if NET_2_0
+			if (isEncrypted) {
+				ms = new CryptoStream (ms, page.GetCryptoTransform (CryptoStreamMode.Read), CryptoStreamMode.Read);
+			}
+#endif
+			return Deserialize (ms);
 		}
 		
 		public string Serialize (object stateGraph)
@@ -166,7 +175,17 @@ namespace System.Web.UI {
 				return "";
 			
 			MemoryStream ms = new MemoryStream ();
-			Serialize (ms, stateGraph);
+			Stream output = ms;
+#if NET_2_0
+			bool needEncryption = page.NeedViewStateEncryption;
+			if (needEncryption){
+				output = new CryptoStream (output, page.GetCryptoTransform (CryptoStreamMode.Write), CryptoStreamMode.Write);
+			}
+#endif
+			Serialize (output, stateGraph);
+#if NET_2_0
+			ms.WriteByte((byte)(needEncryption? 1 : 0));
+#endif
 			
 			#if TRACE
 				ms.WriteTo (File.OpenWrite (Path.GetTempFileName ()));
@@ -854,9 +873,17 @@ namespace System.Web.UI {
 		}
 		
 		class SingleRankArrayFormatter : ObjectFormatter {
+
+			readonly BinaryFormatter _binaryFormatter = new BinaryFormatter ();
+
 			protected override void Write (BinaryWriter w, object o, WriterContext ctx)
 			{
 				Array val = (Array) o;
+				if (val.GetType ().GetElementType ().IsPrimitive) {
+					w.Write (SecondaryId);
+					_binaryFormatter.Serialize (w.BaseStream, o);
+					return;
+				}
 				
 				w.Write (PrimaryId);
 				WriteObject (w, val.GetType ().GetElementType (), ctx);
@@ -868,6 +895,9 @@ namespace System.Web.UI {
 			
 			protected override object Read (byte token, BinaryReader r, ReaderContext ctx)
 			{
+				if (token == SecondaryId) {
+					return _binaryFormatter.Deserialize (r.BaseStream);
+				}
 				Type t = (Type) ReadObject (r, ctx);
 				int len = Read7BitEncodedInt (r);
 				Array val = Array.CreateInstance (t, len);
@@ -881,6 +911,10 @@ namespace System.Web.UI {
 			protected override Type Type {
 				get { return typeof (Array); }
 			}
+
+			protected override int NumberOfIds {
+				get { return 2; }
+			}
 		}
 		
 		class FontUnitFormatter : StringFormatter {
