@@ -47,6 +47,7 @@ namespace System.Web.UI {
 #else
 	public abstract class TemplateControl : Control, INamingContainer {
 #endif
+		static readonly Assembly _System_Web_Assembly = typeof (TemplateControl).Assembly;
 		static object abortTransaction = new object ();
 		static object commitTransaction = new object ();
 		static object error = new object ();
@@ -72,6 +73,10 @@ namespace System.Web.UI {
 					    BindingFlags.NonPublic |
 					    BindingFlags.Instance;
 
+#if NET_2_0
+		string _appRelativeVirtualPath;
+#endif
+		
 		#region Constructor
 		protected TemplateControl ()
 		{
@@ -95,6 +100,13 @@ namespace System.Web.UI {
 			get { return true; }
 		}
 
+#if NET_2_0
+		public string AppRelativeVirtualPath {
+			get { return _appRelativeVirtualPath; }
+			set { _appRelativeVirtualPath = value; }
+		}
+#endif
+
 		#endregion
 
 		#region Methods
@@ -116,9 +128,14 @@ namespace System.Web.UI {
 			if (!SupportAutoEvents || !AutoEventWireup)
 				return;
 
-			Type type = GetType ();
 			foreach (string methodName in methodNames) {
-				MethodInfo method = type.GetMethod (methodName, bflags);
+				MethodInfo method = null;
+				Type type;
+				for (type = GetType (); type.Assembly != _System_Web_Assembly; type = type.BaseType) {
+					method = type.GetMethod (methodName, bflags);
+					if (method != null)
+						break;
+				}
 				if (method == null)
 					continue;
 
@@ -148,11 +165,15 @@ namespace System.Web.UI {
 				}
 
 				if (noParams) {
-					NoParamsInvoker npi = new NoParamsInvoker (this, methodName);
+					NoParamsInvoker npi = new NoParamsInvoker (this, method);
 					evt.AddEventHandler (this, npi.FakeDelegate);
 				} else {
 					evt.AddEventHandler (this, Delegate.CreateDelegate (
+#if NET_2_0
+							typeof (EventHandler), this, method));
+#else
 							typeof (EventHandler), this, methodName));
+#endif
 				}
 			}
 		}
@@ -267,7 +288,8 @@ namespace System.Web.UI {
 
 		protected object GetLocalResourceObject (string resourceKey)
 		{
-			return HttpContext.GetLocalResourceObject (Context.Request.CurrentExecutionFilePath, resourceKey);
+			return HttpContext.GetLocalResourceObject (VirtualPathUtility.ToAbsolute (this.AppRelativeVirtualPath),
+								   resourceKey);
 		}
 		
 		protected object GetLocalResourceObject (string resourceKey, Type objType, string propName)
