@@ -30,6 +30,7 @@
 
 using System.CodeDom.Compiler;
 using System.Collections;
+using System.ComponentModel;
 using System.Globalization;
 using System.IO;
 using System.Reflection;
@@ -38,6 +39,10 @@ using System.Web.Compilation;
 using System.Web.Configuration;
 using System.Web.Util;
 
+#if NET_2_0
+using System.Collections.Generic;
+#endif
+
 namespace System.Web.UI {
 
 	// CAS
@@ -64,6 +69,7 @@ namespace System.Web.UI {
 		string language;
 		bool strictOn = false;
 		bool explicitOn = false;
+		bool linePragmasOn = false;
 		bool output_cache;
 		int oc_duration;
 		string oc_header, oc_custom, oc_param, oc_controls;
@@ -73,6 +79,10 @@ namespace System.Web.UI {
 #if NET_2_0
 		string src;
 		string partialClassName;
+		string codeFileBaseClass;
+		string metaResourceKey;
+		Type codeFileBaseClassType;
+		List <UnknownAttributeDescriptor> unknownMainAttributes;
 #endif
 		Assembly srcAssembly;
 		int appAssemblyIndex = -1;
@@ -153,61 +163,43 @@ namespace System.Web.UI {
 			foreach (NamespaceInfo nsi in namespaces)
 				imports.Add (nsi.Namespace);
 		}
-		
-		internal void RegisterConfigControls ()
-		{
-			PagesSection pages = WebConfigurationManager.GetSection ("system.web/pages") as PagesSection;
-			if (pages == null)
-				return;
-
-			TagPrefixCollection controls = pages.Controls;
-			if (controls == null || controls.Count == 0)
-				return;
-
-			foreach (TagPrefixInfo tpi in controls) {
-				if (!String.IsNullOrEmpty (tpi.TagName))
-					RegisterCustomControl (tpi.TagPrefix, tpi.TagName, tpi.Source);
-				else if (!String.IsNullOrEmpty (tpi.Namespace))
-					RegisterNamespace (tpi.TagPrefix, tpi.Namespace, tpi.Assembly);
-			}
-		}
 #endif
 		
 		internal void RegisterCustomControl (string tagPrefix, string tagName, string src)
-		{
-			string realpath = MapPath (src);
-			if (String.Compare (realpath, inputFile, false, invariantCulture) == 0)
-				return;
-			
-			if (!File.Exists (realpath))
-				throw new ParseException (Location, "Could not find file \"" + realpath + "\".");
-			string vpath = UrlUtils.Combine (BaseVirtualDir, src);
-			Type type = null;
-			AddDependency (realpath);
-			try {
-				ArrayList other_deps = new ArrayList ();
-				type = UserControlParser.GetCompiledType (vpath, realpath, other_deps, Context);
-				foreach (string s in other_deps) {
-					AddDependency (s);
-				}
-			} catch (ParseException pe) {
-				if (this is UserControlParser)
-					throw new ParseException (Location, pe.Message, pe);
-				throw;
-			}
+                {
+                        string realpath = MapPath (src);
+                        if (String.Compare (realpath, inputFile, false, invariantCulture) == 0)
+                                return;
+                        
+                        if (!File.Exists (realpath))
+                                throw new ParseException (Location, "Could not find file \"" + realpath + "\".");
+                        string vpath = UrlUtils.Combine (BaseVirtualDir, src);
+                        Type type = null;
+                        AddDependency (realpath);
+                        try {
+                                ArrayList other_deps = new ArrayList ();
+                                type = UserControlParser.GetCompiledType (vpath, realpath, other_deps, Context);
+                                foreach (string s in other_deps) {
+                                        AddDependency (s);
+                                }
+                        } catch (ParseException pe) {
+                                if (this is UserControlParser)
+                                        throw new ParseException (Location, pe.Message, pe);
+                                throw;
+                        }
 
-			AddAssembly (type.Assembly, true);
-			RootBuilder.Foundry.RegisterFoundry (tagPrefix, tagName, type);
-		}
+                        AddAssembly (type.Assembly, true);
+                        RootBuilder.Foundry.RegisterFoundry (tagPrefix, tagName, type);
+                }
+
+                internal void RegisterNamespace (string tagPrefix, string ns, string assembly)
+                {
+                        AddImport (ns);
+                        Assembly ass = AddAssemblyByName (assembly);
+                        AddDependency (ass.Location);
+                        RootBuilder.Foundry.RegisterFoundry (tagPrefix, ass, ns);
+                }
 
-		internal void RegisterNamespace (string tagPrefix, string ns, string assembly)
-		{
-			AddImport (ns);
-			Assembly ass = AddAssemblyByName (assembly);
-			AddDependency (ass.Location);
-			RootBuilder.Foundry.RegisterFoundry (tagPrefix, ass, ns);
-		}
-		
 		internal virtual void HandleOptions (object obj)
 		{
 		}
@@ -528,7 +520,7 @@ namespace System.Web.UI {
 			AddAssembly (assembly, true);
 			return assembly;
 		}
-
+		
 		internal virtual void ProcessMainAttributes (Hashtable atts)
 		{
 			atts.Remove ("Description"); // ignored
@@ -536,13 +528,19 @@ namespace System.Web.UI {
 			atts.Remove ("CodeBehind");  // ignored
 #endif
 			atts.Remove ("AspCompat"); // ignored
-
+#if NET_2_0
+			// these two are ignored for the moment
+			atts.Remove ("Async");
+			atts.Remove ("AsyncTimeOut");
+#endif
+			
 			debug = GetBool (atts, "Debug", true);
 			compilerOptions = GetString (atts, "CompilerOptions", "");
 			language = GetString (atts, "Language", CompilationConfig.DefaultLanguage);
 			strictOn = GetBool (atts, "Strict", CompilationConfig.Strict);
 			explicitOn = GetBool (atts, "Explicit", CompilationConfig.Explicit);
-
+			linePragmasOn = GetBool (atts, "LinePragmas", false);
+			
 			string inherits = GetString (atts, "Inherits", null);
 #if NET_2_0
 			// In ASP 2, the source file is actually integrated with
@@ -550,7 +548,11 @@ namespace System.Web.UI {
 			// means that the code file has to be confirmed, but not
 			// used at this point.
 			src = GetString (atts, "CodeFile", null);
+			codeFileBaseClass = GetString (atts, "CodeFileBaseClass", null);
 
+			if (src == null && codeFileBaseClass != null)
+				ThrowParseException ("The 'CodeFileBaseClass' attribute cannot be used without a 'CodeFile' attribute");
+			
 			if (src != null && inherits != null) {
 				// Make sure the source exists
 				src = UrlUtils.Combine (BaseVirtualDir, src);
@@ -558,11 +560,6 @@ namespace System.Web.UI {
 				if (!File.Exists (realPath))
 					ThrowParseException ("File " + src + " not found");
 
-				// Verify that the inherits is a valid identify not a
-				// fully-qualified name.
-				if (!CodeGenerator.IsValidLanguageIndependentIdentifier (inherits))
-					ThrowParseException (String.Format ("'{0}' is not valid for 'inherits'", inherits));
-
 				// We are going to create a partial class that shares
 				// the same name as the inherits tag, so reset the
 				// name. The base type is changed because it is the
@@ -573,6 +570,16 @@ namespace System.Web.UI {
 				// Add the code file as an option to the
 				// compiler. This lets both files be compiled at once.
 				compilerOptions += " \"" + realPath + "\"";
+
+				if (codeFileBaseClass != null) {
+					try {
+						codeFileBaseClassType = LoadType (codeFileBaseClass);
+					} catch (Exception) {
+					}
+
+					if (codeFileBaseClassType == null)
+						ThrowParseException ("Could not load type '{0}'", codeFileBaseClass);
+				}
 			} else if (inherits != null) {
 				// We just set the inherits directly because this is a
 				// Single-Page model.
@@ -586,16 +593,120 @@ namespace System.Web.UI {
 
 			if (inherits != null)
 				SetBaseType (inherits);
-
-			className = GetString (atts, "ClassName", null);
-			if (className != null && !CodeGenerator.IsValidLanguageIndependentIdentifier (className))
-				ThrowParseException (String.Format ("'{0}' is not valid for 'className'", className));
 #endif
+			className = GetString (atts, "ClassName", null);
+			if (className != null) {
+#if NET_2_0
+				string [] identifiers = className.Split ('.');
+				for (int i = 0; i < identifiers.Length; i++)
+					if (!CodeGenerator.IsValidLanguageIndependentIdentifier (identifiers [i]))
+						ThrowParseException (String.Format ("'{0}' is not a valid "
+							+ "value for attribute 'classname'.", className));
+#else
+				if (!CodeGenerator.IsValidLanguageIndependentIdentifier (className))
+					ThrowParseException (String.Format ("'{0}' is not a valid "
+						+ "value for attribute 'classname'.", className));
+#endif
+			}
 
+#if NET_2_0
+			if (this is TemplateControlParser)
+				metaResourceKey = GetString (atts, "meta:resourcekey", null);
+			
+			if (inherits != null && (this is PageParser || this is UserControlParser) && atts.Count > 0) {
+				if (unknownMainAttributes == null)
+					unknownMainAttributes = new List <UnknownAttributeDescriptor> ();
+				string key, val;
+				
+				foreach (DictionaryEntry de in atts) {
+					key = de.Key as string;
+					val = de.Value as string;
+					
+					if (String.IsNullOrEmpty (key) || String.IsNullOrEmpty (val))
+						continue;
+					CheckUnknownAttribute (key, val, inherits);
+				}
+				return;
+			}
+#endif
 			if (atts.Count > 0)
 				ThrowParseException ("Unknown attribute: " + GetOneKey (atts));
 		}
 
+#if NET_2_0
+		void CheckUnknownAttribute (string name, string val, string inherits)
+		{
+			MemberInfo mi = null;
+			bool missing = false;
+			string memberName = name.Trim ().ToLower (CultureInfo.InvariantCulture);
+			Type parent = codeFileBaseClassType;
+
+			if (parent == null)
+				parent = baseType;
+			
+			try {
+				MemberInfo[] infos = parent.GetMember (memberName,
+								       MemberTypes.Field | MemberTypes.Property,
+								       BindingFlags.Public | BindingFlags.Instance |
+								       BindingFlags.IgnoreCase | BindingFlags.Static);
+				if (infos.Length != 0) {
+					// prefer public properties to public methods (it's what MS.NET does)
+					foreach (MemberInfo tmp in infos) {
+						if (tmp is PropertyInfo) {
+							mi = tmp;
+							break;
+						}
+					}
+					if (mi == null)
+						mi = infos [0];
+				} else
+					missing = true;
+			} catch (Exception) {
+				missing = true;
+			}
+			if (missing)
+				ThrowParseException (
+					"Error parsing attribute '{0}': Type '{1}' does not have a public property named '{0}'",
+					memberName, inherits);
+			
+			Type memberType = null;
+			if (mi is PropertyInfo) {
+				PropertyInfo pi = mi as PropertyInfo;
+				
+				if (!pi.CanWrite)
+					ThrowParseException (
+						"Error parsing attribute '{0}': The '{0}' property is read-only and cannot be set.",
+						memberName);
+				memberType = pi.PropertyType;
+			} else if (mi is FieldInfo) {
+				memberType = ((FieldInfo)mi).FieldType;
+			} else
+				ThrowParseException ("Could not determine member the kind of '{0}' in base type '{1}",
+						     memberName, inherits);
+			TypeConverter converter = TypeDescriptor.GetConverter (memberType);
+			bool convertible = true;
+			object value = null;
+			
+			if (converter == null || !converter.CanConvertFrom (typeof (string)))
+				convertible = false;
+
+			if (convertible) {
+				try {
+					value = converter.ConvertFromInvariantString (val);
+				} catch (Exception) {
+					convertible = false;
+				}
+			}
+
+			if (!convertible)
+				ThrowParseException ("Error parsing attribute '{0}': Cannot create an object of type '{1}' from its string representation '{2}' for the '{3}' property.",
+						     memberName, memberType, val, mi.Name);
+			
+			UnknownAttributeDescriptor desc = new UnknownAttributeDescriptor (mi, value);
+			unknownMainAttributes.Add (desc);
+		}
+#endif
+		
 		internal void SetBaseType (string type)
 		{
 			if (type == DefaultBaseTypeName)
@@ -649,15 +760,31 @@ namespace System.Web.UI {
 		}
 
 #if NET_2_0
-		internal bool IsPartial
-		{
+		internal bool IsPartial {
 			get { return src != null; }
 		}
 
-		internal string PartialClassName
-		{
+		internal string PartialClassName {
 			get { return partialClassName; }
 		}
+
+		internal string CodeFileBaseClass {
+			get { return codeFileBaseClass; }
+		}
+
+		internal string MetaResourceKey {
+			get { return metaResourceKey; }
+		}
+		
+		internal Type CodeFileBaseClassType
+		{
+			get { return codeFileBaseClassType; }
+		}
+		
+		internal List <UnknownAttributeDescriptor> UnknownMainAttributes
+		{
+			get { return unknownMainAttributes; }
+		}
 #endif
 
 		internal string Text
@@ -685,6 +812,15 @@ namespace System.Web.UI {
 				if (className != null)
 					return className;
 
+#if NET_2_0
+				string physPath = HttpContext.Current.Request.PhysicalApplicationPath;
+				
+				if (StrUtils.StartsWith (inputFile, physPath)) {
+					className = inputFile.Substring (physPath.Length).ToLower (CultureInfo.InvariantCulture);
+					className = className.Replace ('.', '_');
+					className = className.Replace ('/', '_').Replace ('\\', '_');
+				} else
+#endif
 				className = Path.GetFileName (inputFile).Replace ('.', '_');
 				className = className.Replace ('-', '_'); 
 				className = className.Replace (' ', '_');
