@@ -73,6 +73,10 @@ using System.Threading;
 using System.Web.Configuration;
 using System.Web.SessionState;
 using System.Web.UI;
+
+#if TARGET_J2EE
+using vmw.@internal.j2ee;
+#endif
 	
 namespace System.Web {
 
@@ -132,8 +136,19 @@ namespace System.Web {
 		CultureInfo prev_appui_culture;
 		IPrincipal prev_user;
 #if NET_2_0
+#if TARGET_J2EE
+		const string initialization_exception_key = "System.Web.HttpApplication.initialization_exception";
+		static Exception initialization_exception {
+			get { return (Exception) AppDomain.CurrentDomain.GetData (initialization_exception_key); }
+			set { AppDomain.CurrentDomain.SetData (initialization_exception_key, value); }
+		}
+#else
 		static Exception initialization_exception;
 #endif
+		HttpHandlersSection httpHandlersSection;
+#else
+		HandlerFactoryConfiguration factory_config;
+#endif
 
 		//
 		// These are used to detect the case where the EndXXX method is invoked
@@ -156,7 +171,7 @@ namespace System.Web {
 
 #if NET_2_0
 				HttpModulesSection modules;
-				modules = (HttpModulesSection) WebConfigurationManager.GetSection ("system.web/httpModules");
+				modules = (HttpModulesSection) WebConfigurationManager.GetSection ("system.web/httpModules", HttpRuntime.AppDomainAppVirtualPath);
 #else
 				ModulesConfiguration modules;
 
@@ -183,6 +198,12 @@ namespace System.Web {
 					appui_culture = cfg.UICulture;
 				}
 #endif
+
+#if NET_2_0
+				httpHandlersSection = (HttpHandlersSection) WebConfigurationManager.GetSection ("system.web/httpHandlers");
+#else
+				factory_config = (HandlerFactoryConfiguration) HttpContext.GetAppConfig ("system.web/httpHandlers");
+#endif
 			}
 		}
 
@@ -770,6 +791,17 @@ namespace System.Web {
 			if (error != null){
 				HttpResponse response = context.Response;
 
+#if TARGET_J2EE
+				IPortletActionRequest actionRequest = context.ServletRequest as IPortletActionRequest;
+				IPortletActionResponse actionResponse = context.ServletResponse as IPortletActionResponse;
+				if (actionRequest != null && actionResponse != null && actionRequest.processActionOnly ()) {
+					string exception = "Exception of type " + context.Error.GetType () + 
+						" during processAction: " + context.Error.Message + Console.Out.NewLine + 
+						context.Error.StackTrace;
+					actionResponse.setRenderParameter ("vmw.action.exception", exception);
+					return;
+				}
+#endif
 				if (!response.HeadersSent){
 					response.ClearHeaders ();
 					response.ClearContent ();
@@ -1095,6 +1127,7 @@ namespace System.Web {
 				initialization_exception = e;
 #endif
 				FinalErrorWrite (context.Response, new HttpException ("", e).GetHtmlErrorMessage ());
+				PipelineDone ();
 				return;
 			}
 
@@ -1113,10 +1146,8 @@ namespace System.Web {
 			
 			IHttpHandler handler = null;
 #if NET_2_0
-			HttpHandlersSection section = (HttpHandlersSection) WebConfigurationManager.GetSection ("system.web/httpHandlers");
-			object o = section.LocateHandler (verb, url);
+			object o = httpHandlersSection.LocateHandler (verb, url);
 #else
-			HandlerFactoryConfiguration factory_config = (HandlerFactoryConfiguration) HttpContext.GetAppConfig ("system.web/httpHandlers");
 			object o = factory_config.LocateHandler (verb, url);
 #endif
 
@@ -1162,6 +1193,19 @@ namespace System.Web {
 			done.Reset ();
 			
 			begin_iar = new AsyncRequestState (done, cb, extraData);
+
+#if TARGET_J2EE
+			IPortletRenderRequest renderRequest = context.ServletRequest as IPortletRenderRequest;
+			if (renderRequest != null) {
+				string actionException = context.ServletRequest.getParameter ("vmw.action.exception");
+				if (actionException != null && actionException.Length > 0) {
+					FinalErrorWrite (context.Response, actionException.Replace("\n", "<br>"));
+					begin_iar.Complete ();
+					return begin_iar;
+				}
+			}
+#endif
+
 #if TARGET_JVM
 			if (true)
 #else
