@@ -34,11 +34,13 @@ using System.IO;
 using System.Text;
 using System.Globalization;
 using System.Collections;
+using System.Reflection;
 using System.Security;
 using System.Security.Permissions;
 using System.Web.Caching;
 using System.Web.Configuration;
 using System.Web.UI;
+using System.Web.Util;
 using System.Threading;
 
 #if NET_2_0 && !TARGET_JVM
@@ -56,7 +58,7 @@ namespace System.Web {
 		static TraceManager trace_manager { get { return _runtime._trace_manager; } }
 		static Cache cache { get { return _runtime._cache; } }
 		static WaitCallback do_RealProcessRequest;
-
+		
 		QueueManager _queue_manager;
 		TraceManager _trace_manager;
 		Cache _cache;
@@ -74,20 +76,30 @@ namespace System.Web {
 			_cache = new Cache ();
 		}
 
-		static private HttpRuntime _runtime {
+		static private HttpRuntime _runtimeInstance {
 			get {
-				HttpRuntime runtime = (HttpRuntime)AppDomain.CurrentDomain.GetData("HttpRuntime");
+				HttpRuntime runtime = (HttpRuntime) AppDomain.CurrentDomain.GetData ("HttpRuntime");
 				if (runtime == null)
-					lock (typeof(HttpRuntime)) {
-						runtime = (HttpRuntime)AppDomain.CurrentDomain.GetData("HttpRuntime");
+					lock (typeof (HttpRuntime)) {
+						runtime = (HttpRuntime) AppDomain.CurrentDomain.GetData ("HttpRuntime");
 						if (runtime == null) {
-							runtime = new HttpRuntime();
-							AppDomain.CurrentDomain.SetData("HttpRuntime", runtime);
+							runtime = new HttpRuntime ();
+							AppDomain.CurrentDomain.SetData ("HttpRuntime", runtime);
 						}
 					}
 				return runtime;
 			}
 		}
+		static private HttpRuntime _runtime
+		{
+			get
+			{
+				if (HttpContext.Current != null)
+					return HttpContext.Current.HttpRuntimeInstance;
+				else
+					return _runtimeInstance;
+			}
+		}
 #else
 		static QueueManager queue_manager;
 		static TraceManager trace_manager;
@@ -95,6 +107,11 @@ namespace System.Web {
 		static Cache cache;
 		static WaitCallback do_RealProcessRequest;
 
+#if NET_2_0
+		static bool assemblyMappingEnabled;
+		static object assemblyMappingLock = new object ();
+#endif
+		
 		static HttpRuntime ()
 		{
 #if NET_2_0
@@ -170,7 +187,7 @@ namespace System.Web {
 		
 		public static string BinDirectory {
 			get {
-				string dirname = Path.Combine (AppDomainAppPath, "bin");
+				string dirname = AppDomain.CurrentDomain.SetupInformation.PrivateBinPath;
 				if (SecurityManager.SecurityEnabled) {
 					new FileIOPermission (FileIOPermissionAccess.PathDiscovery, dirname).Demand ();
 				}
@@ -214,11 +231,7 @@ namespace System.Web {
 
 		public static string MachineConfigurationDirectory {
 			get {
-#if NET_2_0
-				string dirname = Path.GetDirectoryName (WebConfigurationManager.OpenMachineConfiguration().FilePath);
-#else
-				string dirname = Path.GetDirectoryName (WebConfigurationSettings.MachineConfigPath);
-#endif
+				string dirname = Path.GetDirectoryName (ICalls.GetMachineConfigPath ());
 				if ((dirname != null) && (dirname.Length > 0) && SecurityManager.SecurityEnabled) {
 					new FileIOPermission (FileIOPermissionAccess.PathDiscovery, dirname).Demand ();
 				}
@@ -226,6 +239,7 @@ namespace System.Web {
 			}
 		}
 
+		
 		[SecurityPermission (SecurityAction.Demand, UnmanagedCode = true)]
 		public static void Close ()
 		{
@@ -263,14 +277,6 @@ namespace System.Web {
 				HttpContext.Current = null;
 			} else {
 				context.ApplicationInstance = app;
-			
-#if NET_2_0 && !TARGET_JVM
-				//
-				// Compile the local resources, if any
-				//
-				AppResourcesCompiler ac = new AppResourcesCompiler (context, false);
-				ac.Compile ();
-#endif
 				
 				//
 				// Ask application to service the request
@@ -396,6 +402,73 @@ namespace System.Web {
 			wr.CloseConnection ();
 		}
 
+#if NET_2_0 && !TARGET_J2EE
+		static internal void WritePreservationFile (Assembly asm, string genericNameBase)
+		{
+			if (asm == null)
+				throw new ArgumentNullException ("asm");
+			if (String.IsNullOrEmpty (genericNameBase))
+				throw new ArgumentNullException ("genericNameBase");
+
+			string compiled = Path.Combine (AppDomain.CurrentDomain.SetupInformation.DynamicBase,
+							genericNameBase + ".compiled");
+			PreservationFile pf = new PreservationFile ();
+			try {
+				pf.VirtualPath = String.Format ("/{0}/", genericNameBase);
+
+				AssemblyName an = asm.GetName ();
+				pf.Assembly = an.Name;
+				pf.ResultType = BuildResultTypeCode.TopLevelAssembly;
+				pf.Save (compiled);
+			} catch (Exception ex) {
+				throw new HttpException (
+					String.Format ("Failed to write preservation file {0}", genericNameBase + ".compiled"),
+					ex);
+			}
+		}
+		
+		static Assembly ResolveAssemblyHandler(object sender, ResolveEventArgs e)
+		{
+			AssemblyName an = new AssemblyName (e.Name);
+			string dynamic_base = AppDomain.CurrentDomain.SetupInformation.DynamicBase;
+			string compiled = Path.Combine (dynamic_base, an.Name + ".compiled");
+
+			if (!File.Exists (compiled))
+				return null;
+
+			PreservationFile pf;
+			try {
+				pf = new PreservationFile (compiled);
+			} catch (Exception ex) {
+				throw new HttpException (
+					String.Format ("Failed to read preservation file {0}", an.Name + ".compiled"),
+					ex);
+			}
+			
+			Assembly ret = null;
+			try {
+				string asmPath = Path.Combine (dynamic_base, pf.Assembly + ".dll");
+				ret = Assembly.LoadFrom (asmPath);
+			} catch (Exception) {
+				// ignore
+			}
+			
+			return ret;
+		}
+		
+		internal static void EnableAssemblyMapping (bool enable)
+		{
+			lock (assemblyMappingLock) {
+				if (assemblyMappingEnabled == enable)
+					return;
+				if (enable)
+					AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler (ResolveAssemblyHandler);
+				else
+					AppDomain.CurrentDomain.AssemblyResolve -= new ResolveEventHandler (ResolveAssemblyHandler);
+			}
+		}
+#endif
+		
 		internal static TraceManager TraceManager {
 			get {
 				return trace_manager;
