@@ -261,7 +261,11 @@ namespace MonoTests.System.Web.UI.WebControls {
 		[TestFixtureSetUp]
 		public void SetUp ()
 		{
+#if VISUAL_STUDIO
+			WebTest.CopyResource (GetType (), "MonoTests.System.Web.UI.WebControls.Resources.NoEventValidation.aspx", "NoEventValidation.aspx");
+#else
 			WebTest.CopyResource (GetType (), "NoEventValidation.aspx", "NoEventValidation.aspx");
+#endif
 		}
 #endif
 
@@ -790,14 +794,13 @@ namespace MonoTests.System.Web.UI.WebControls {
 
 		[Test]
 		[Category ("NunitWeb")]
-		[Category("NotWorking")]
 		public void OnBubbleEvent ()
 		{
 			WebTest t = new WebTest (PageInvoker.CreateOnPreInit (_OnBubbleEvent));
 			string html = t.Run ();
 		}
 
-		public static void _OnBubbleEvent(Page p)
+		public static void DoOnBubbleEvent(Page p, string cmdname)
 		{
 			TestLogin l = new TestLogin ();
 			l.Page = p;
@@ -805,7 +808,7 @@ namespace MonoTests.System.Web.UI.WebControls {
 			l.MembershipProvider = "FakeProvider";
 			Button b = (Button)l.FindControl ("LoginButton");
 			Assert.IsNotNull (b, "LoginButton");
-			CommandEventArgs cea = new CommandEventArgs ("Login", null);
+			CommandEventArgs cea = new CommandEventArgs (cmdname, null);
 			l.DoBubbleEvent (b, cea);
 			Assert.IsTrue (l.OnLoggingInCalled, "OnLoggingIn");
 			Assert.IsFalse (l.Cancel, "Cancel");
@@ -814,7 +817,25 @@ namespace MonoTests.System.Web.UI.WebControls {
 			Assert.IsTrue (l.OnLoginErrorCalled, "OnLoginError");
 			Assert.IsFalse (l.OnLoggedInCalled, "OnLoggedIn");
 		}
+		
+		public static void _OnBubbleEvent(Page p)
+		{
+			DoOnBubbleEvent(p, "Login");
+		}
 
+		[Test]
+		[Category ("NunitWeb")]
+		public void OnBubbleEventCaseSensitivity ()
+		{
+			WebTest t = new WebTest (PageInvoker.CreateOnPreInit (_OnBubbleEventCaseSensitivity));
+			string html = t.Run ();
+		}
+		
+		public static void _OnBubbleEventCaseSensitivity(Page p)
+		{
+			DoOnBubbleEvent(p, "login");
+		}
+		
 		[Test]
 		public void OnBubbleEvent_Cancel_OnLoggingIn ()
 		{
@@ -833,7 +854,6 @@ namespace MonoTests.System.Web.UI.WebControls {
 		}
 
 		[Test]
-		[Category("NotWorking")]
 		public void OnBubbleEvent_Authenticated_OnAuthenticate ()
 		{
 			WebTest t = new WebTest (PageInvoker.CreateOnPreInit (_OnBubbleEvent_Authenticated_OnAuthenticate));
@@ -960,9 +980,9 @@ namespace MonoTests.System.Web.UI.WebControls {
 			fr.Controls.Add (GetDecoratedId (html, "LoginButton"));
 			fr.Controls ["__EVENTTARGET"].Value = "";
 			fr.Controls ["__EVENTARGUMENT"].Value = "";
-			fr.Controls ["Login1:UserName"].Value = "yonik";
-			fr.Controls ["Login1:Password"].Value = "123456";
-			fr.Controls ["Login1:LoginButton"].Value = "Log In";
+			fr.Controls ["Login1$UserName"].Value = "yonik";
+			fr.Controls ["Login1$Password"].Value = "123456";
+			fr.Controls ["Login1$LoginButton"].Value = "Log In";
 			t.Request = fr;
 			t.Run ();
 
@@ -990,9 +1010,9 @@ namespace MonoTests.System.Web.UI.WebControls {
 			fr.Controls.Add (GetDecoratedId (html, "LoginButton"));
 			fr.Controls ["__EVENTTARGET"].Value = "";
 			fr.Controls ["__EVENTARGUMENT"].Value = "";
-			fr.Controls ["Login1:UserName"].Value = "yonik";
-			fr.Controls ["Login1:Password"].Value = "123456";
-			fr.Controls ["Login1:LoginButton"].Value = "Log In";
+			fr.Controls ["Login1$UserName"].Value = "yonik";
+			fr.Controls ["Login1$Password"].Value = "123456";
+			fr.Controls ["Login1$LoginButton"].Value = "Log In";
 			t.Request = fr;
 			t.Run ();
 
