@@ -61,7 +61,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			TrackViewState ();
 		}
 
-		public bool IsTrackingViewState ()
+		public bool GetIsTrackingViewState ()
 		{
 			return base.IsTrackingViewState;
 		}
@@ -153,6 +153,12 @@ namespace MonoTests.System.Web.UI.WebControls
 			WebTest.Unload ();
 		}
 
+		[SetUp()]
+		public void Setup () 
+		{
+			eventsCalled = null;
+		}
+
 		[Test]
 		public void Defaults ()
 		{
@@ -189,14 +195,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			Assert.AreEqual ("startRowIndex", sql.StartRowIndexParameterName, "StartRowIndexParameterName");
 			Assert.AreEqual ("", sql.TypeName, "TypeName");
 			Assert.AreEqual ("", sql.UpdateMethod, "UpdateMethod");
-			Assert.AreEqual (true, sql.IsTrackingViewState (), "IsTrackingViewState");
-		}
-
-		[Test]
-		public void DefaultsNotWorking ()
-		{
-			ObjectDataSource ds = new ObjectDataSource ();
-			ObjectViewPoker sql = new ObjectViewPoker (ds, "DefaultView", null);
+			Assert.AreEqual (true, sql.GetIsTrackingViewState (), "IsTrackingViewState");
 			Assert.IsTrue (sql.CanRetrieveTotalRowCount, "CanRetrieveTotalRowCount");
 		}
 
@@ -253,6 +252,58 @@ namespace MonoTests.System.Web.UI.WebControls
 
 		}
 
+		[Test]
+		public void ViewStateSupport () 
+		{
+			ObjectDataSourceView view;
+			MyDataSource ds = new MyDataSource ();
+
+			ds.ID = "ObjectDataSource2";
+			ds.TypeName = "MonoTests.System.Web.UI.WebControls.DataSourceObject";
+			ds.SelectMethod = "Select";
+			ds.SelectCountMethod = "SelectCount";
+
+			view = (ObjectDataSourceView) ds.DoGetView ("DefaultView");
+			((IStateManager) view).TrackViewState ();
+
+			Parameter p1 = new Parameter ("test", TypeCode.String);
+
+			Assert.IsTrue (((IStateManager) view).IsTrackingViewState, "IsTrackingViewState");
+			Assert.IsTrue (((IStateManager) view.FilterParameters).IsTrackingViewState, "FilterParameters.IsTrackingViewState");
+			Assert.IsTrue (((IStateManager) view.SelectParameters).IsTrackingViewState, "SelecteParameters.IsTrackingViewState");
+			Assert.IsFalse (((IStateManager) view.DeleteParameters).IsTrackingViewState, "DeleteParameters.IsTrackingViewState");
+			Assert.IsFalse (((IStateManager) view.InsertParameters).IsTrackingViewState, "InsertParameters.IsTrackingViewState");
+			Assert.IsFalse (((IStateManager) view.UpdateParameters).IsTrackingViewState, "UpdateParameters.IsTrackingViewState");
+
+			object state = ((IStateManager) view).SaveViewState ();
+			Assert.IsNull (state, "view ViewState not null");
+
+			view.DeleteParameters.Add (p1);
+			view.InsertParameters.Add (p1);
+			//view.UpdateParameters.Add (p1);
+
+			state = ((IStateManager) view).SaveViewState ();
+			Assert.IsNull (state, "view ViewState not null");
+
+			view.FilterParameters.Add (p1);
+			//view.SelectParameters.Add (p1);
+
+			state = ((IStateManager) view).SaveViewState ();
+			Assert.IsNotNull (state, "view ViewState not null");
+
+			state = ((IStateManager) view.FilterParameters).SaveViewState ();
+			Assert.IsNotNull (state, "FilterParamenters ViewState not null");
+			state = ((IStateManager) view.SelectParameters).SaveViewState ();
+			Assert.IsNull (state, "SelectParameters ViewState not null");
+
+			state = ((IStateManager) view.DeleteParameters).SaveViewState ();
+			Assert.IsNotNull (state, "DeleteParameters ViewState not null");
+			state = ((IStateManager) view.InsertParameters).SaveViewState ();
+			Assert.IsNotNull (state, "InsertParameters ViewState not null");
+			state = ((IStateManager) view.UpdateParameters).SaveViewState ();
+			Assert.IsNull (state, "UpdateParameters ViewState not null");
+		}
+
 		[Test]
 		public void ViewState ()
 		{
@@ -268,11 +319,9 @@ namespace MonoTests.System.Web.UI.WebControls
 			ds.SelectMethod = "Select";
 			ds.SelectCountMethod = "SelectCount";
 
-			Parameter p1 = new Parameter ("test", TypeCode.String);
-			ds.SelectParameters.Add (p1);
-			ds.FilterParameters.Add (p1);
-
 			view = (ObjectDataSourceView) ds.DoGetView ("DefaultView");
+			((IStateManager) view).TrackViewState ();
+
 			view.ConflictDetection = ConflictOptions.CompareAllValues;
 			view.ConvertNullToDBNull = true;
 			view.DataObjectTypeName = "test";
@@ -288,13 +337,12 @@ namespace MonoTests.System.Web.UI.WebControls
 			view.TypeName = "test";
 			view.UpdateMethod = "test";
 
-			((IStateManager) view).TrackViewState ();
 			object state = ((IStateManager) view).SaveViewState ();
+			Assert.IsNull (state, "ViewState#1");
 
 			ObjectDataSourceView copy = new ObjectDataSourceView (ds, "DefaultView", null);
 			((IStateManager) copy).LoadViewState (state);
 
-			Assert.AreEqual (null, state, "ViewState#1");
 		}
 
 		[Test]
@@ -346,7 +394,8 @@ namespace MonoTests.System.Web.UI.WebControls
 		}
 
 		[Test]
-		public void CanRetrieveTotalRowCount () {
+		public void CanRetrieveTotalRowCount () 
+		{
 			ObjectDataSource ds = new ObjectDataSource ();
 			ObjectDataSourceView view = new ObjectDataSourceView (ds, "DefaultView", null);
 
@@ -447,29 +496,8 @@ namespace MonoTests.System.Web.UI.WebControls
 
 		[Test]
 		[Category ("NunitWeb")]
-		[Category ("NotWorking")]
 		public void SelectMethod ()
 		{
-// 8) MonoTests.System.Web.UI.WebControls.ObjectDataSourceViewTest.SelectMethod : NUnit.Framework.AssertionException: SelectedRowsCount ^M
-//         expected:<1>^M
-//          but was:<0>
-//   at NUnit.Framework.Assert.Fail (System.String message, System.Object[] args) [0x00000]
-//   at NUnit.Framework.Assert.Fail (System.String message) [0x00000]
-//   at NUnit.Framework.Assert.FailNotEquals (System.Object expected, System.Object actual, System.String message, System.Object[] args) [0x00000]
-//   at NUnit.Framework.Assert.AreEqual (Int32 expected, Int32 actual, System.String message, System.Object[] args) [0x00000]
-//   at NUnit.Framework.Assert.AreEqual (Int32 expected, Int32 actual, System.String message) [0x00000]
-//   at MonoTests.System.Web.UI.WebControls.ObjectDataSourceViewTest.select (System.Web.UI.Page p) [0x00000]
-//   at (wrapper delegate-invoke) System.MulticastDelegate:invoke_void_Page (System.Web.UI.Page)
-//   at MonoTests.SystemWeb.Framework.PageInvoker.Invoke (MonoTests.SystemWeb.Framework.PageDelegate callback) [0x00000]
-//   ----> NUnit.Framework.AssertionException : SelectedRowsCount ^M
-//         expected:<1>^M
-//          but was:<0>^M
-//   at System.Runtime.Remoting.Proxies.RealProxy.PrivateInvoke (System.Runtime.Remoting.Proxies.RealProxy rp, IMessage msg, System.Exception& exc, System.Object[]& out_args) [0x00188] in /home/cvs/mcs/class/corlib/System.Runtime.Remoting.Proxies/RealProxy.cs:221
-// --AssertionException
-//   at MonoTests.System.Web.UI.WebControls.ObjectDataSourceViewTest.select (System.Web.UI.Page p) [0x00000]
-//   at (wrapper delegate-invoke) System.MulticastDelegate:invoke_void_Page (System.Web.UI.Page)
-//   at MonoTests.SystemWeb.Framework.PageInvoker.Invoke (MonoTests.SystemWeb.Framework.PageDelegate callback) [0x00000]
-// 
 			WebTest t = new WebTest (PageInvoker.CreateOnLoad (select));
 			string html = t.Run ();
 			string origin = @"<div>
@@ -517,23 +545,8 @@ namespace MonoTests.System.Web.UI.WebControls
 
 		[Test]
 		[Category ("NunitWeb")]
-		[Category ("NotWorking")]
 		public void SelectCountMethod ()
 		{
-// 9) MonoTests.System.Web.UI.WebControls.ObjectDataSourceViewTest.SelectCountMethod : ObjectDataSourceViewSelectCount ^M
-//         String lengths differ.  Expected length=389, but was length=11.^M
-//         Strings differ at index 6.^M
-//         ^M
-//         expected:<"<div><table cellspacing="0" rules="all" s...">^M
-//          but was:<"<div></div>">^M
-//         -----------------^^M
-//         ^M
-//   at MonoTests.stand_alone.WebHarness.HtmlDiff.AssertAreEqual (System.String origin, System.String derived, System.String msg) [0x0001b] in /home/cvs/mcs/class/System.Web/Test/mainsoft/MainsoftWebTest/NunitWebTest.cs:98
-//   at MonoTests.System.Web.UI.WebControls.ObjectDataSourceViewTest.SelectCountMethod () [0x00024] in /home/cvs/mcs/class/System.Web/Test/System.Web.UI.WebControls/ObjectDataSourceViewTest.cs:516
-//   at <0x00000> <unknown method>
-//   at (wrapper managed-to-native) System.Reflection.MonoMethod:InternalInvoke (object,object[])
-//   at System.Reflection.MonoMethod.Invoke (System.Object obj, BindingFlags invokeAttr, System.Reflection.Binder binder, System.Object[] parameters, System.Globalization.CultureInfo culture) [0x00056] in /home/cvs/mcs/class/corlib/System.Reflection/MonoMethod.cs:143
-// 
 			// This method will render grid view with paging 
 			// Note : ObjectDataSource will return page counter 5 hard coded
 			WebTest t = new WebTest (PageInvoker.CreateOnLoad (selectcount));
@@ -567,6 +580,7 @@ namespace MonoTests.System.Web.UI.WebControls
 			GridView grid = new GridView ();
 			MyDataSource ds = new MyDataSource ();
 			ds.ID = "ObjectDataSource2";
+			ds.EnablePaging = true;
 			ds.TypeName = "MonoTests.System.Web.UI.WebControls.DataSourceObject";
 			ds.SelectMethod = "Select";
 			ds.SelectCountMethod = "SelectCount";
@@ -655,23 +669,8 @@ namespace MonoTests.System.Web.UI.WebControls
 
 		[Test]
 		[Category ("NunitWeb")]
-		[Category("NotWorking")]
 		public void UpdateMethod ()
 		{
-// 10) MonoTests.System.Web.UI.WebControls.ObjectDataSourceViewTest.UpdateMethod : ObjectDataSourceViewUpdate ^M
-//         String lengths are both 4.^M
-//         Strings differ at index 3.^M
-//         ^M
-//         expected:<"1001">^M
-//          but was:<"1000">^M
-//         --------------^^M
-//         ^M
-//   at MonoTests.stand_alone.WebHarness.HtmlDiff.AssertAreEqual (System.String origin, System.String derived, System.String msg) [0x0001b] in /home/cvs/mcs/class/System.Web/Test/mainsoft/MainsoftWebTest/NunitWebTest.cs:98
-//   at MonoTests.System.Web.UI.WebControls.ObjectDataSourceViewTest.UpdateMethod () [0x00024] in /home/cvs/mcs/class/System.Web/Test/System.Web.UI.WebControls/ObjectDataSourceViewTest.cs:631
-//   at <0x00000> <unknown method>
-//   at (wrapper managed-to-native) System.Reflection.MonoMethod:InternalInvoke (object,object[])
-//   at System.Reflection.MonoMethod.Invoke (System.Object obj, BindingFlags invokeAttr, System.Reflection.Binder binder, System.Object[] parameters, System.Globalization.CultureInfo culture) [0x00056] in /home/cvs/mcs/class/corlib/System.Reflection/MonoMethod.cs:143
-// 
 			WebTest t = new WebTest (PageInvoker.CreateOnLoad (update));
 			string html = t.Run ();
 			string origin = @"<div>
@@ -815,6 +814,414 @@ namespace MonoTests.System.Web.UI.WebControls
 			Eventassert ("ObjectDisposing");
 		}
 
+		enum InitViewType
+		{
+			MatchParamsToValues,
+			MatchParamsToOldValues,
+			DontMatchParams,
+		}
+
+		public class DummyDataSourceObject
+		{
+			public static IEnumerable Select (string filter) 
+			{
+				if (eventsCalled == null) {
+					eventsCalled = new ArrayList ();
+				}
+				eventsCalled.Add (String.Format ("Select(filter = {0})", filter));
+				return new string [] { "one", "two", "three" };
+			}
+
+			public static int Update (string ID) 
+			{
+				if (eventsCalled == null) {
+					eventsCalled = new ArrayList ();
+				}
+				eventsCalled.Add (String.Format ("Update(ID = {0})", ID));
+				return 1;
+			}
+
+			public static int Update (string ID, string oldvalue_ID) 
+			{
+				if (eventsCalled == null) {
+					eventsCalled = new ArrayList ();
+				}
+				eventsCalled.Add (String.Format ("Update(ID = {0}, oldvalue_ID = {1})", ID, oldvalue_ID));
+				return 1;
+			}
+
+			public static int UpdateOther (string ID, string OtherValue, string oldvalue_ID) 
+			{
+				if (eventsCalled == null) {
+					eventsCalled = new ArrayList ();
+				}
+				eventsCalled.Add (String.Format ("UpdateOther(ID = {0}, OtherValue = {1}, oldvalue_ID = {2})", ID, OtherValue, oldvalue_ID));
+				return 1;
+			}
+
+			public static int Insert (string ID) 
+			{
+				if (eventsCalled == null) {
+					eventsCalled = new ArrayList ();
+				}
+				eventsCalled.Add (String.Format ("Insert(ID = {0})", ID));
+				return 1;
+			}
+
+			public static int Insert (string ID, string oldvalue_ID) 
+			{
+				if (eventsCalled == null) {
+					eventsCalled = new ArrayList ();
+				}
+				eventsCalled.Add (String.Format ("Insert(ID = {0}, oldvalue_ID = {1})", ID, oldvalue_ID));
+				return 1;
+			}
+
+			public static int InsertOther (string ID, string OtherValue) 
+			{
+				if (eventsCalled == null) {
+					eventsCalled = new ArrayList ();
+				}
+				eventsCalled.Add (String.Format ("InsertOther(ID = {0}, OtherValue = {1})", ID, OtherValue));
+				return 1;
+			}
+
+			public static int Delete (string ID, string oldvalue_ID) 
+			{
+				if (eventsCalled == null) {
+					eventsCalled = new ArrayList ();
+				}
+				eventsCalled.Add (String.Format ("Delete(ID = {0}, oldvalue_ID = {1})", ID, oldvalue_ID));
+				return 1;
+			}
+
+			public static int Delete (string oldvalue_ID) 
+			{
+				if (eventsCalled == null) {
+					eventsCalled = new ArrayList ();
+				}
+				eventsCalled.Add (String.Format ("Delete(oldvalue_ID = {0})", oldvalue_ID));
+				return 1;
+			}
+
+			public static int DeleteOther (string oldvalue_ID, string OtherValue) 
+			{
+				if (eventsCalled == null) {
+					eventsCalled = new ArrayList ();
+				}
+				eventsCalled.Add (String.Format ("DeleteOther(oldvalue_ID = {0}, OtherValue = {1})", oldvalue_ID, OtherValue));
+				return 1;
+			}
+		}
+
+		public class AlwaysChangingParameter : Parameter
+		{
+			int evaluateCount;
+
+			public AlwaysChangingParameter (string name, TypeCode type, string defaultValue)
+				: base (name, type, defaultValue) 
+			{
+				evaluateCount = 0;
+			}
+
+			protected override object Evaluate (HttpContext context, Control control) 
+			{
+				evaluateCount++;
+				return String.Format ("{0}{1}", DefaultValue, evaluateCount);
+			}
+		}
+
+		private static void InitializeView (ObjectViewPoker view, InitViewType initType, out Hashtable keys, out Hashtable old_value, out Hashtable new_value) 
+		{
+			view.TypeName = typeof (DummyDataSourceObject).AssemblyQualifiedName;
+			view.OldValuesParameterFormatString = "oldvalue_{0}";
+			view.SelectMethod = "Select";
+			if (initType == InitViewType.DontMatchParams) {
+				view.UpdateMethod = "UpdateOther";
+				view.InsertMethod = "InsertOther";
+				view.DeleteMethod = "DeleteOther";
+			}
+			else {
+				view.UpdateMethod = "Update";
+				view.InsertMethod = "Insert";
+				view.DeleteMethod = "Delete";
+			}
+
+			Parameter selectParameter = null;
+			Parameter insertParameter = null;
+			Parameter updateParameter = null;
+			Parameter deleteParameter = null;
+
+			selectParameter = new AlwaysChangingParameter ("filter", TypeCode.String, "p_ValueSelect");
+			view.SelectParameters.Add (selectParameter);
+
+			switch (initType) {
+			case InitViewType.MatchParamsToOldValues:
+				insertParameter = new AlwaysChangingParameter ("oldvalue_ID", TypeCode.String, "p_OldValueInsert");
+				view.InsertParameters.Add (insertParameter);
+				updateParameter = new AlwaysChangingParameter ("oldvalue_ID", TypeCode.String, "p_OldValueUpdate");
+				view.UpdateParameters.Add (updateParameter);
+				deleteParameter = new AlwaysChangingParameter ("oldvalue_ID", TypeCode.String, "p_OldValueDelete");
+				view.DeleteParameters.Add (deleteParameter);
+				break;
+
+			case InitViewType.MatchParamsToValues:
+				insertParameter = new AlwaysChangingParameter ("ID", TypeCode.String, "p_ValueInsert");
+				view.InsertParameters.Add (insertParameter);
+				updateParameter = new AlwaysChangingParameter ("ID", TypeCode.String, "p_ValueUpdate");
+				view.UpdateParameters.Add (updateParameter);
+				deleteParameter = new AlwaysChangingParameter ("ID", TypeCode.String, "p_ValueDelete");
+				view.DeleteParameters.Add (deleteParameter);
+				break;
+
+			case InitViewType.DontMatchParams:
+				insertParameter = new AlwaysChangingParameter ("OtherValue", TypeCode.String, "p_OtherValueInsert");
+				view.InsertParameters.Add (insertParameter);
+				updateParameter = new AlwaysChangingParameter ("OtherValue", TypeCode.String, "p_OtherValueUpdate");
+				view.UpdateParameters.Add (updateParameter);
+				deleteParameter = new AlwaysChangingParameter ("OtherValue", TypeCode.String, "p_OtherValueDelete");
+				view.DeleteParameters.Add (deleteParameter);
+				break;
+			}
+
+			view.SelectParameters.ParametersChanged += new EventHandler (SelectParameters_ParametersChanged);
+			view.InsertParameters.ParametersChanged += new EventHandler (InsertParameters_ParametersChanged);
+			view.UpdateParameters.ParametersChanged += new EventHandler (UpdateParameters_ParametersChanged);
+			view.DeleteParameters.ParametersChanged += new EventHandler (DeleteParameters_ParametersChanged);
+
+			keys = new Hashtable ();
+			keys.Add ("ID", "k_1001");
+
+			old_value = new Hashtable ();
+			old_value.Add ("ID", "ov_1001");
+
+			new_value = new Hashtable ();
+			new_value.Add ("ID", "n_1001");
+
+			view.DataSourceViewChanged += new EventHandler (view_DataSourceViewChanged);
+		}
+
+		private static IList eventsCalled;
+
+		static void view_DataSourceViewChanged (object sender, EventArgs e) 
+		{
+			if (eventsCalled == null) {
+				eventsCalled = new ArrayList ();
+			}
+			eventsCalled.Add ("view_DataSourceViewChanged");
+		}
+
+		static void SelectParameters_ParametersChanged (object sender, EventArgs e) 
+		{
+			if (eventsCalled == null) {
+				eventsCalled = new ArrayList ();
+			}
+			eventsCalled.Add ("SelectParameters_ParametersChanged");
+		}
+
+		static void InsertParameters_ParametersChanged (object sender, EventArgs e) 
+		{
+			if (eventsCalled == null) {
+				eventsCalled = new ArrayList ();
+			}
+			eventsCalled.Add ("InsertParameters_ParametersChanged");
+		}
+
+		static void UpdateParameters_ParametersChanged (object sender, EventArgs e) 
+		{
+			if (eventsCalled == null) {
+				eventsCalled = new ArrayList ();
+			}
+			eventsCalled.Add ("UpdateParameters_ParametersChanged");
+		}
+
+		static void DeleteParameters_ParametersChanged (object sender, EventArgs e) 
+		{
+			if (eventsCalled == null) {
+				eventsCalled = new ArrayList ();
+			}
+			eventsCalled.Add ("DeleteParameters_ParametersChanged");
+		}
+
+		[Test]
+		public void ParametersAndViewChangedEvent_Select () 
+		{
+			ObjectViewPoker view = new ObjectViewPoker (new ObjectDataSource (), "", null);
+			Hashtable keys = null;
+			Hashtable old_values = null;
+			Hashtable new_values = null;
+			InitializeView (view, InitViewType.MatchParamsToValues, out keys, out old_values, out new_values);
+
+			view.Select (DataSourceSelectArguments.Empty);
+
+			Assert.IsNotNull (eventsCalled, "Events not raized");
+			Assert.AreEqual (3, eventsCalled.Count, "Events Count");
+			Assert.AreEqual ("view_DataSourceViewChanged", eventsCalled [0], "view_DataSourceViewChanged");
+			Assert.AreEqual ("SelectParameters_ParametersChanged", eventsCalled [1], "SelectParameters_ParametersChanged");
+			Assert.AreEqual ("Select(filter = p_ValueSelect1)", eventsCalled [2], "DataSource Method params");
+		}
+
+		[Test]
+		public void ParametersAndViewChangedEvent_MatchInsert () 
+		{
+			ObjectViewPoker view = new ObjectViewPoker (new ObjectDataSource (), "", null);
+			Hashtable keys = null;
+			Hashtable old_values = null;
+			Hashtable new_values = null;
+			InitializeView (view, InitViewType.MatchParamsToValues, out keys, out old_values, out new_values);
+
+			view.Insert (new_values);
+
+			Assert.IsNotNull (eventsCalled, "Events not raized");
+			Assert.AreEqual (3, eventsCalled.Count, "Events Count");
+			Assert.AreEqual ("InsertParameters_ParametersChanged", eventsCalled [0], "InsertParameters_ParametersChanged");
+			Assert.AreEqual ("Insert(ID = n_1001)", eventsCalled [1], "DataSource Method params");
+			Assert.AreEqual ("view_DataSourceViewChanged", eventsCalled [2], "view_DataSourceViewChanged");
+		}
+
+		[Test]
+		public void ParametersAndViewChangedEvent_MatchOldInsert () 
+		{
+			ObjectViewPoker view = new ObjectViewPoker (new ObjectDataSource (), "", null);
+			Hashtable keys = null;
+			Hashtable old_values = null;
+			Hashtable new_values = null;
+			InitializeView (view, InitViewType.MatchParamsToOldValues, out keys, out old_values, out new_values);
+
+			view.Insert (new_values);
+
+			Assert.IsNotNull (eventsCalled, "Events not raized");
+			Assert.AreEqual (3, eventsCalled.Count, "Events Count");
+			Assert.AreEqual ("InsertParameters_ParametersChanged", eventsCalled [0], "InsertParameters_ParametersChanged");
+			Assert.AreEqual ("Insert(ID = n_1001, oldvalue_ID = p_OldValueInsert1)", eventsCalled [1], "DataSource Method params");
+			Assert.AreEqual ("view_DataSourceViewChanged", eventsCalled [2], "view_DataSourceViewChanged");
+		}
+
+		[Test]
+		public void ParametersAndViewChangedEvent_DontMatchInsert () 
+		{
+			ObjectViewPoker view = new ObjectViewPoker (new ObjectDataSource (), "", null);
+			Hashtable keys = null;
+			Hashtable old_values = null;
+			Hashtable new_values = null;
+			InitializeView (view, InitViewType.DontMatchParams, out keys, out old_values, out new_values);
+
+			view.Insert (new_values);
+
+			Assert.IsNotNull (eventsCalled, "Events not raized");
+			Assert.AreEqual (3, eventsCalled.Count, "Events Count");
+			Assert.AreEqual ("InsertParameters_ParametersChanged", eventsCalled [0], "InsertParameters_ParametersChanged");
+			Assert.AreEqual ("InsertOther(ID = n_1001, OtherValue = p_OtherValueInsert1)", eventsCalled [1], "DataSource Method params");
+			Assert.AreEqual ("view_DataSourceViewChanged", eventsCalled [2], "view_DataSourceViewChanged");
+		}
+
+		[Test]
+		public void ParametersAndViewChangedEvent_MatchUpdate () 
+		{
+			ObjectViewPoker view = new ObjectViewPoker (new ObjectDataSource (), "", null);
+			Hashtable keys = null;
+			Hashtable old_values = null;
+			Hashtable new_values = null;
+			InitializeView (view, InitViewType.MatchParamsToValues, out keys, out old_values, out new_values);
+
+			view.Update (keys, new_values, old_values);
+
+			Assert.IsNotNull (eventsCalled, "Events not raized");
+			Assert.AreEqual (3, eventsCalled.Count, "Events Count");
+			Assert.AreEqual ("UpdateParameters_ParametersChanged", eventsCalled [0], "UpdateParameters_ParametersChanged");
+			Assert.AreEqual ("Update(ID = n_1001, oldvalue_ID = k_1001)", eventsCalled [1], "DataSource Method params");
+			Assert.AreEqual ("view_DataSourceViewChanged", eventsCalled [2], "view_DataSourceViewChanged");
+		}
+
+		[Test]
+		public void ParametersAndViewChangedEvent_MatchOldUpdate () 
+		{
+			ObjectViewPoker view = new ObjectViewPoker (new ObjectDataSource (), "", null);
+			Hashtable keys = null;
+			Hashtable old_values = null;
+			Hashtable new_values = null;
+			InitializeView (view, InitViewType.MatchParamsToOldValues, out keys, out old_values, out new_values);
+
+			view.Update (keys, new_values, old_values);
+
+			Assert.IsNotNull (eventsCalled, "Events not raized");
+			Assert.AreEqual (3, eventsCalled.Count, "Events Count");
+			Assert.AreEqual ("UpdateParameters_ParametersChanged", eventsCalled [0], "UpdateParameters_ParametersChanged");
+			Assert.AreEqual ("Update(ID = n_1001, oldvalue_ID = k_1001)", eventsCalled [1], "DataSource Method params");
+			Assert.AreEqual ("view_DataSourceViewChanged", eventsCalled [2], "view_DataSourceViewChanged");
+		}
+
+		[Test]
+		public void ParametersAndViewChangedEvent_DontMatchUpdate () 
+		{
+			ObjectViewPoker view = new ObjectViewPoker (new ObjectDataSource (), "", null);
+			Hashtable keys = null;
+			Hashtable old_values = null;
+			Hashtable new_values = null;
+			InitializeView (view, InitViewType.DontMatchParams, out keys, out old_values, out new_values);
+
+			view.Update (keys, new_values, old_values);
+
+			Assert.IsNotNull (eventsCalled, "Events not raized");
+			Assert.AreEqual (3, eventsCalled.Count, "Events Count");
+			Assert.AreEqual ("UpdateParameters_ParametersChanged", eventsCalled [0], "UpdateParameters_ParametersChanged");
+			Assert.AreEqual ("UpdateOther(ID = n_1001, OtherValue = p_OtherValueUpdate1, oldvalue_ID = k_1001)", eventsCalled [1], "DataSource Method params");
+			Assert.AreEqual ("view_DataSourceViewChanged", eventsCalled [2], "view_DataSourceViewChanged");
+		}
+
+		[Test]
+		public void ParametersAndViewChangedEvent_MatchDelete () 
+		{
+			ObjectViewPoker view = new ObjectViewPoker (new ObjectDataSource (), "", null);
+			Hashtable keys = null;
+			Hashtable old_values = null;
+			Hashtable new_values = null;
+			InitializeView (view, InitViewType.MatchParamsToValues, out keys, out old_values, out new_values);
+
+			view.Delete (keys, old_values);
+
+			Assert.IsNotNull (eventsCalled, "Events not raized");
+			Assert.AreEqual (3, eventsCalled.Count, "Events Count");
+			Assert.AreEqual ("DeleteParameters_ParametersChanged", eventsCalled [0], "DeleteParameters_ParametersChanged");
+			Assert.AreEqual ("Delete(ID = p_ValueDelete1, oldvalue_ID = k_1001)", eventsCalled [1], "DataSource Method params");
+			Assert.AreEqual ("view_DataSourceViewChanged", eventsCalled [2], "view_DataSourceViewChanged");
+		}
+
+		[Test]
+		public void ParametersAndViewChangedEvent_MatchOldDelete () 
+		{
+			ObjectViewPoker view = new ObjectViewPoker (new ObjectDataSource (), "", null);
+			Hashtable keys = null;
+			Hashtable old_values = null;
+			Hashtable new_values = null;
+			InitializeView (view, InitViewType.MatchParamsToOldValues, out keys, out old_values, out new_values);
+
+			view.Delete (keys, old_values);
+
+			Assert.IsNotNull (eventsCalled, "Events not raized");
+			Assert.AreEqual (3, eventsCalled.Count, "Events Count");
+			Assert.AreEqual ("DeleteParameters_ParametersChanged", eventsCalled [0], "DeleteParameters_ParametersChanged");
+			Assert.AreEqual ("Delete(oldvalue_ID = k_1001)", eventsCalled [1], "DataSource Method params");
+			Assert.AreEqual ("view_DataSourceViewChanged", eventsCalled [2], "view_DataSourceViewChanged");
+		}
+
+		[Test]
+		public void ParametersAndViewChangedEvent_DontMatchDelete () 
+		{
+			ObjectViewPoker view = new ObjectViewPoker (new ObjectDataSource (), "", null);
+			Hashtable keys = null;
+			Hashtable old_values = null;
+			Hashtable new_values = null;
+			InitializeView (view, InitViewType.DontMatchParams, out keys, out old_values, out new_values);
+
+			view.Delete (keys, old_values);
+
+			Assert.IsNotNull (eventsCalled, "Events not raized");
+			Assert.AreEqual (3, eventsCalled.Count, "Events Count");
+			Assert.AreEqual ("DeleteParameters_ParametersChanged", eventsCalled [0], "DeleteParameters_ParametersChanged");
+			Assert.AreEqual ("DeleteOther(oldvalue_ID = k_1001, OtherValue = p_OtherValueDelete1)", eventsCalled [1], "DataSource Method params");
+			Assert.AreEqual ("view_DataSourceViewChanged", eventsCalled [2], "view_DataSourceViewChanged");
+		}
 
 		/// <summary>
 		/// Helper methods
