@@ -37,6 +37,7 @@ using System.Globalization;
 using System.Web;
 using System.Web.UI;
 using System.Web.UI.WebControls;
+using MonoTests.stand_alone.WebHarness;
 
 namespace MonoTests.System.Web.UI.WebControls
 {
@@ -77,6 +78,43 @@ namespace MonoTests.System.Web.UI.WebControls
 			return matches == a1.Length;
 		}
 
+		public class CustomControl : WebControl
+		{
+			public virtual string CustomProperty {
+				get {
+					return (string) ViewState ["CustomProperty"];
+				}
+				set {
+					ViewState ["CustomProperty"] = value;
+				}
+			}
+
+			protected override Style CreateControlStyle () {
+				return new Style ();
+			}
+
+			public void DoTrackViewState () {
+				TrackViewState ();
+			}
+
+			public object DoSaveViewState () {
+				return SaveViewState ();
+			}
+
+			public void DoLoadViewState (object state) {
+				LoadViewState (state);
+			}
+		}
+		
+		public class CustomControl2 : CustomControl
+		{
+			protected override Style CreateControlStyle () {
+				Style style = new Style (ViewState);
+				style.BackColor = Color.Blue;
+				return style;
+			}
+		}
+
 		public class NamingContainer : WebControl, INamingContainer {
 
 		}
@@ -451,7 +489,43 @@ namespace MonoTests.System.Web.UI.WebControls
 				style="color:BlueViolet;background-color:Red;border-color:Green;border-width:12px;border-style:Dotted;height:6px;width:6.5cm;background-color: #ff00ff">
 				</xml>
 			*/
-			Assert.AreEqual(w.Render(), w_copy.Render(), "VS1");
+			HtmlDiff.AssertAreEqual (w.Render(), w_copy.Render(), "VS1");
+		}
+
+		[Test]
+		public void ViewState2 () {
+			CustomControl c = new CustomControl ();
+			CustomControl copy = new CustomControl ();
+			object state;
+
+			c.DoTrackViewState ();
+			c.CustomProperty = "CustomProperty";
+			c.ControlStyle.BackColor = Color.Red;
+			c.ControlStyle.BorderColor = Color.Green;
+			c.ControlStyle.BorderStyle = BorderStyle.Dotted;
+
+			state = c.DoSaveViewState ();
+
+			copy.DoLoadViewState (state);
+
+			Assert.IsFalse (copy.ControlStyleCreated, "copy.ControlStyleCreated");
+		}
+		
+		[Test]
+		public void ViewState3 () {
+			CustomControl2 c = new CustomControl2 ();
+			CustomControl2 copy = new CustomControl2 ();
+			object state;
+
+			c.DoTrackViewState ();
+			c.ControlStyle.BackColor = Color.Red;
+
+			state = c.DoSaveViewState ();
+
+			copy.DoLoadViewState (state);
+
+			Assert.AreEqual (Color.Blue, copy.ControlStyle.BackColor, "copy.BackColor");
+
 		}
 
 		[Test]
