@@ -103,7 +103,7 @@ namespace System.Xml.Schema
 		internal override int Compile(ValidationEventHandler h, XmlSchema schema)
 		{
 			// If this is already compiled this time, simply skip.
-			if (this.IsComplied (schema.CompilationId))
+			if (CompilationId == schema.CompilationId)
 				return 0;
 #if NET_2_0
 			if (BaseType != null)
@@ -184,7 +184,7 @@ namespace System.Xml.Schema
 				actualBaseSchemaType = baseType;
 			}
 			else if (baseTypeName != XmlQualifiedName.Empty) {
-				XmlSchemaType st = schema.SchemaTypes [baseTypeName] as XmlSchemaType;
+				XmlSchemaType st = schema.FindSchemaType (baseTypeName);
 				if (st != null) {
 					st.Validate (h, schema);
 					actualBaseSchemaType = st;
