@@ -609,7 +609,7 @@ namespace System.Xml.Serialization
 
 			CheckReferenceQueue ();
 
-			if (callbacks.ContainsKey (o.GetType ()))
+			if (callbacks != null && callbacks.ContainsKey (o.GetType ()))
 			{
 				WriteCallbackInfo info = (WriteCallbackInfo) callbacks[o.GetType()];
 				if (o.GetType ().IsEnum) {
@@ -840,29 +840,31 @@ namespace System.Xml.Serialization
 		{
 			string value;
 			TypeData td = TypeTranslator.GetTypeData (o.GetType ());
+			if (td.SchemaType != SchemaTypes.Primitive)
+				throw new InvalidOperationException ("The type of the argument object is not primitive.");
 
-			name = XmlCustomFormatter.FromXmlName (name);
+			if (name == null) {
+				ns = td.IsXsdType ? XmlSchema.Namespace : XmlSerializer.WsdlTypesNamespace;
+				name = td.XmlType;
+			}
+			else
+				name = XmlCustomFormatter.FromXmlName (name);
 			Writer.WriteStartElement (name, ns);
 
-			if (o is XmlNode[]) {
-				foreach (XmlNode node in (XmlNode[])o)
-					node.WriteTo (Writer);
-			}
-			else {
-				if (o is XmlQualifiedName)
-					value = FromXmlQualifiedName ((XmlQualifiedName) o);
-				else
-					value = XmlCustomFormatter.ToXmlString (td, o);
+			if (o is XmlQualifiedName)
+				value = FromXmlQualifiedName ((XmlQualifiedName) o);
+			else
+				value = XmlCustomFormatter.ToXmlString (td, o);
 
-				if (xsiType)
-				{
-					if (td.SchemaType != SchemaTypes.Primitive)
-						throw new InvalidOperationException (string.Format (unexpectedTypeError, o.GetType().FullName));
-					WriteXsiType (td.XmlType, XmlSchema.Namespace);
-				}
-
-				WriteValue (value);
+			if (xsiType)
+			{
+				if (td.SchemaType != SchemaTypes.Primitive)
+					throw new InvalidOperationException (string.Format (unexpectedTypeError, o.GetType().FullName));
+				WriteXsiType (td.XmlType, td.IsXsdType ? XmlSchema.Namespace : XmlSerializer.WsdlTypesNamespace);
 			}
+
+			WriteValue (value);
+
 			Writer.WriteEndElement ();
 		}
 
