@@ -140,7 +140,9 @@ namespace System.Xml
 				if (FirstChild == null)
 					return String.Empty;
 				if (FirstChild == LastChild)
-					return FirstChild.InnerText;
+					return FirstChild.NodeType != XmlNodeType.Comment ?
+						FirstChild.InnerText :
+						String.Empty;
 
 				StringBuilder builder = null;
 				AppendChildValues (ref builder);
@@ -415,7 +417,8 @@ namespace System.Xml
 			while (node != null) {
 				if (node.Prefix == prefix)
 					return node.NamespaceURI;
-				if (node.Attributes != null) {
+				if (node.NodeType == XmlNodeType.Element &&
+				    ((XmlElement) node).HasAttributes) {
 					int count = node.Attributes.Count;
 					for (int i = 0; i < count; i++) {
 						XmlAttribute attr = node.Attributes [i];
@@ -453,13 +456,16 @@ namespace System.Xml
 				break;
 			}
 
-			while (node != null && node.Attributes != null) {
-				for (int i = 0; i < Attributes.Count; i++) {
-					XmlAttribute attr = Attributes [i];
-					if (attr.Prefix == "xmlns" && attr.Value == namespaceURI)
-						return attr.LocalName;
-					else if (attr.Name == "xmlns" && attr.Value == namespaceURI)
-						return String.Empty;
+			while (node != null) {
+				if (node.NodeType == XmlNodeType.Element &&
+				    ((XmlElement) node).HasAttributes) {
+					for (int i = 0; i < node.Attributes.Count; i++) {
+						XmlAttribute attr = node.Attributes [i];
+						if (attr.Prefix == "xmlns" && attr.Value == namespaceURI)
+							return attr.LocalName;
+						else if (attr.Name == "xmlns" && attr.Value == namespaceURI)
+							return String.Empty;
+					}
 				}
 				node = node.ParentNode;
 			}
@@ -515,6 +521,9 @@ namespace System.Xml
 			if (checkNodeType)
 				CheckNodeInsertion (newChild, refChild);
 
+			if (newChild == refChild)
+				return newChild;
+
 			IHasXmlChildNode l = (IHasXmlChildNode) this;
 
 			XmlDocument ownerDoc = (NodeType == XmlNodeType.Document) ? (XmlDocument) this : OwnerDocument;
