@@ -337,6 +337,7 @@ namespace System.Xml
 			reader.XmlResolver = settings.XmlResolver;
 			// Normalization is set true by default.
 			reader.Normalization = true;
+			reader.EntityHandling = EntityHandling.ExpandEntities;
 
 			if (settings.ProhibitDtd)
 				reader.ProhibitDtd = true;
@@ -1124,8 +1125,9 @@ namespace System.Xml
 
 		public virtual string ReadElementContentAsString (string localName, string namespaceURI)
 		{
+			bool isEmpty = IsEmptyElement;
 			ReadStartElement (localName, namespaceURI);
-			if (IsEmptyElement)
+			if (isEmpty)
 				return String.Empty;
 			string s = ReadContentString (false);
 			ReadEndElement ();
