@@ -996,12 +996,18 @@ namespace MonoTests.System.XmlSerialization
 
 		[Test]
 		[ExpectedException (typeof (InvalidOperationException))]
+#if TARGET_JVM
+		[Ignore ("TD #7461")]
+#endif
 		public void TypeMapping_IEnumerable_Object_NoMatchingAddMethod ()
 		{
 			Map (typeof (ObjectEnumerableNoMatchingAddMethod));
 		}
 
 		[Test]
+#if TARGET_JVM
+		[Ignore ("TD #7461")]
+#endif
 		[ExpectedException (typeof (InvalidOperationException))]
 		public void TypeMapping_IEnumerable_Object_NoMatchingAddMethod_Array ()
 		{
@@ -1539,12 +1545,45 @@ namespace MonoTests.System.XmlSerialization
 		}
 
 #if NET_2_0
+
+		[Test]
+		public void ImportNullableInt ()
+		{
+			XmlReflectionImporter imp = new XmlReflectionImporter ();
+			XmlTypeMapping map = imp.ImportTypeMapping (typeof (int?));
+			XmlSchemas schemas = new XmlSchemas ();
+			XmlSchemaExporter exp = new XmlSchemaExporter (schemas);
+			exp.ExportTypeMapping (map);
+			XmlSchema schema = schemas [0];
+			XmlSchemaElement el = schema.Items [0] as XmlSchemaElement;
+			Assert.AreEqual ("int", el.Name, "#1");
+			Assert.AreEqual (new XmlQualifiedName ("int", XmlSchema.Namespace), el.SchemaTypeName, "#2");
+			Assert.AreEqual (true, el.IsNillable, "#3");
+		}
+
 		[Test]
 		public void ImportNullableContainer ()
 		{
 			new XmlSerializer (typeof (NullableContainer));
 		}
 
+		[Test]
+		public void ImportNullableContainer2 ()
+		{
+			XmlReflectionImporter imp = new XmlReflectionImporter ();
+			XmlTypeMapping map = imp.ImportTypeMapping (typeof (NullableContainer2));
+			XmlSchemas schemas = new XmlSchemas ();
+			XmlSchemaExporter exp = new XmlSchemaExporter (schemas);
+			exp.ExportTypeMapping (map);
+
+			XmlSchema schema = schemas [0];
+			XmlSchemaComplexType el = schema.Items [1] as XmlSchemaComplexType;
+
+			XmlSchemaSequence s = el.Particle as XmlSchemaSequence;
+			XmlSchemaElement el2 = s.Items [0] as XmlSchemaElement;
+			Assert.IsTrue (el2.IsNillable);
+		}
+
 		[Test]
 		[ExpectedException (typeof (InvalidOperationException))]
 		public void ImportGenericTypeDefinition ()
@@ -2025,6 +2064,16 @@ namespace MonoTests.System.XmlSerialization
 			[XmlElement (IsNullable = true)]
 			public int? NilInt;
 		}
+
+		public class NullableContainer2
+		{
+			int? value;
+
+			public int? NullableInt {
+				get { return value; }
+				set { this.value = value; }
+			}
+		}
 #endif
 	}
 }
