@@ -546,7 +546,6 @@ namespace MonoTests.System.XmlSerialization
 		}
 
 		[Test]
-		[Category ("NotWorking")] // #7 fails
 		public void TestWriteTypedPrimitive_Base64Binary ()
 		{
 			byte[] byteArray = new byte[] { 255, 20, 10, 5, 0, 7 };
@@ -625,7 +624,6 @@ namespace MonoTests.System.XmlSerialization
 		}
 
 		[Test]
-		[Category ("NotWorking")] // #7 fails
 		public void TestWriteTypedPrimitive_Boolean ()
 		{
 			XmlSerializarionWriterTester xsw = new XmlSerializarionWriterTester ();
@@ -694,7 +692,6 @@ namespace MonoTests.System.XmlSerialization
 		}
 
 		[Test]
-		[Category ("NotWorking")] // #7 fails
 		public void TestWriteTypedPrimitive_Char ()
 		{
 			XmlSerializarionWriterTester xsw = new XmlSerializarionWriterTester ();
@@ -742,7 +739,6 @@ namespace MonoTests.System.XmlSerialization
 		}
 
 		[Test]
-		[Category ("NotWorking")] // namespace should be wsdl types ns
 		public void TestWriteTypedPrimitive_Char_XsiType ()
 		{
 			XmlSerializarionWriterTester xsw = new XmlSerializarionWriterTester ();
@@ -764,7 +760,6 @@ namespace MonoTests.System.XmlSerialization
 		}
 
 		[Test]
-		[Category ("NotWorking")] // #7 fails
 		public void TestWriteTypedPrimitive_DateTime ()
 		{
 			DateTime dateTime = new DateTime (1973, 08, 13);
@@ -912,7 +907,6 @@ namespace MonoTests.System.XmlSerialization
 		}
 
 		[Test]
-		[Category ("NotWorking")] // #7 fails
 		public void TestWriteTypedPrimitive_Guid ()
 		{
 			Guid guid = new Guid ("CA761232-ED42-11CE-BACD-00AA0057B223");
@@ -968,7 +962,6 @@ namespace MonoTests.System.XmlSerialization
 		}
 
 		[Test]
-		[Category ("NotWorking")] // namespace should be wsdl types ns
 		public void TestWriteTypedPrimitive_Guid_XsiType ()
 		{
 			Guid guid = new Guid ("CA761232-ED42-11CE-BACD-00AA0057B223");
@@ -993,7 +986,6 @@ namespace MonoTests.System.XmlSerialization
 		}
 
 		[Test]
-		[Category ("NotWorking")] // #7 fails
 		public void TestWriteTypedPrimitive_Int ()
 		{
 			XmlSerializarionWriterTester xsw = new XmlSerializarionWriterTester ();
@@ -1062,7 +1054,6 @@ namespace MonoTests.System.XmlSerialization
 		}
 
 		[Test]
-		[Category ("NotWorking")] // #8 fails
 		public void TestWriteTypedPrimitive_String ()
 		{
 			XmlSerializarionWriterTester xsw = new XmlSerializarionWriterTester ();
@@ -1139,7 +1130,6 @@ namespace MonoTests.System.XmlSerialization
 		}
 
 		[Test]
-		[Category ("NotWorking")] // order of namespace declaration differs from that of MSFT
 		public void TestWriteTypedPrimitive_String_XsiType_Namespace ()
 		{
 			XmlSerializarionWriterTester xsw = new XmlSerializarionWriterTester ();
@@ -1192,7 +1182,6 @@ namespace MonoTests.System.XmlSerialization
 		}
 
 		[Test]
-		[Category ("NotWorking")] // #7 fails
 		public void TestWriteTypedPrimitive_UnsignedByte ()
 		{
 			XmlSerializarionWriterTester xsw = new XmlSerializarionWriterTester ();
@@ -1261,7 +1250,6 @@ namespace MonoTests.System.XmlSerialization
 		}
 
 		[Test]
-		[Category ("NotWorking")] // #A7 fails
 		public void TestWriteTypedPrimitive_XmlQualifiedName ()
 		{
 			XmlQualifiedName qname = new XmlQualifiedName ("something", AnotherNamespace);
@@ -1369,7 +1357,6 @@ namespace MonoTests.System.XmlSerialization
 		}
 
 		[Test]
-		[Category ("NotWorking")] // InvalidOperatinException is not thrown
 		[ExpectedException (typeof (InvalidOperationException))]
 		public void TestWriteTypedPrimitive_NonPrimitive ()
 		{
@@ -1379,6 +1366,13 @@ namespace MonoTests.System.XmlSerialization
 			WriteTypedPrimitive ("x", ANamespace, new Version (), false);
 		}
 
+		[Test]
+		[ExpectedException (typeof (InvalidOperationException))]
+		public void TestWriteTypedPrimitive_XmlNode ()
+		{
+			WriteTypedPrimitive ("x", ANamespace, new XmlDocument ().CreateElement ("foo"), false);
+		}
+
 		[Test]
 		public void TestWriteValue()
 		{
@@ -1470,7 +1464,6 @@ namespace MonoTests.System.XmlSerialization
 		}
 
 		[Test]
-		[Category ("NotWorking")] // order of namespace declaration differs from that of MSFT
 		public void TestWriteXsiType_Namespace ()
 		{
 			WriteStartElement ("x", ANamespace);
