@@ -1496,6 +1496,20 @@ namespace MonoTests.System.Xml
 			AssertEquals ("#3", "urn:foo", reader.LookupNamespace ("x"));
 		}
 
+		[Test]
+		public void ReadClosedReader ()
+		{
+			string xml = "<fin>aaa</fin>";
+			RunTest (xml, new TestMethod (ReadClosedReader));
+		}
+
+		void ReadClosedReader (XmlReader reader)
+		{
+			reader.Read ();
+			reader.Close();
+			reader.Read (); // silently returns false
+		}
+
 #if NET_2_0
 		[Test]
 		public void CreateSimple ()
@@ -1951,6 +1965,21 @@ namespace MonoTests.System.Xml
 			Assert (reader.MoveToAttribute ("id"));
 			AssertEquals ("myId", reader.ReadContentAsString ());
 		}
+
+		[Test]
+		public void ReadElementContentAsStringEmpty ()
+		{
+			string xml = "<root><sample/></root>";
+			RunTest (xml, new TestMethod (ReadElementContentAsStringEmpty));
+		}
+
+		void ReadElementContentAsStringEmpty (XmlReader reader)
+		{
+			reader.MoveToContent ();
+			reader.Read ();
+			AssertEquals (String.Empty, reader.ReadElementContentAsString ("sample", ""));
+			AssertEquals (XmlNodeType.EndElement, reader.NodeType);
+		}
 #endif
 	}
 }
