@@ -30,7 +30,9 @@
 
 namespace System.ComponentModel
 {
+#if !NET_2_0
 	[Serializable]
+#endif
         public enum ToolboxItemFilterType
 	{
 		Allow,
