@@ -28,12 +28,11 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-using System;
-using System.Diagnostics;
+using System.Security.Permissions;
 
 namespace System.Diagnostics 
 {
-
+	[PermissionSet (SecurityAction.LinkDemand, Unrestricted = true)]
 	public sealed class PerformanceCounterCategory 
 	{
 		private string categoryName;
