@@ -65,11 +65,10 @@ namespace System.IO.Ports
 		{
 		}
 
-		/*
-		  IContainer is in 2.0?
-		  public SerialPort (IContainer container) {
-		  }
-		*/
+		public SerialPort (IContainer container) : this ()
+		{
+			// TODO: What to do here?
+		}
 
 		public SerialPort (string portName) :
 			this (portName, DefaultBaudRate, DefaultParity, DefaultDataBits, DefaultStopBits)
@@ -452,6 +451,11 @@ namespace System.IO.Ports
 			stream.DiscardOutBuffer ();
 		}
 
+		static Exception GetNotImplemented ()
+		{
+			return new NotImplementedException ("Detection of ports is not implemented for this platform yet.");
+		}
+
 		public static string [] GetPortNames ()
 		{
 			int p = (int) Environment.OSVersion.Platform;
@@ -468,7 +472,7 @@ namespace System.IO.Ports
 				}
 				return serial_ports.ToArray ();
 			}
-			throw new NotImplementedException ("Detection of ports is not implemented for this platform yet.");
+			throw GetNotImplemented ();
 		}
 
 		static bool IsWindows {
