@@ -171,7 +171,8 @@ namespace System.Net
 		internal bool InternalAllowBuffering {
 			get {
 				return (allowBuffering && (method != "HEAD" && method != "GET" &&
-							method != "MKCOL" && method != "CONNECT"));
+							method != "MKCOL" && method != "CONNECT" &&
+							method != "DELETE" && method != "TRACE"));
 			}
 		}
 		
@@ -400,8 +401,10 @@ namespace System.Net
 			get { return proxy; }
 			set { 
 				CheckRequestStarted ();
+#if ONLY_1_1
 				if (value == null)
 					throw new ArgumentNullException ("value");
+#endif
 
 				proxy = value;
 				servicePoint = null; // we may need a new one
@@ -579,7 +582,8 @@ namespace System.Net
 			if (aborted)
 				throw new WebException ("The request was previosly aborted.");
 
-			bool send = !(method == "GET" || method == "CONNECT" || method == "HEAD");
+			bool send = !(method == "GET" || method == "CONNECT" || method == "HEAD" ||
+					method == "TRACE" || method == "DELETE");
 			if (method == null || !send)
 				throw new ProtocolViolationException ("Cannot send data when method is: " + method);
 
@@ -1000,7 +1004,7 @@ namespace System.Net
 				writeStream.Write (bodyBuffer, 0, bodyBufferLength);
 				bodyBuffer = null;
 				writeStream.Close ();
-			} else if (method == "PUT" || method == "POST") {
+			} else if (method == "PUT" || method == "POST" || method == "OPTIONS") {
 				if (getResponseCalled && !writeStream.RequestWritten)
 					writeStream.WriteRequest ();
 			}
