@@ -58,29 +58,27 @@ namespace System.Net {
 
 		private static short SwapShort (short number)
 		{
-			return (short) ( ((number >> 8) & 0xFF) + ((number << 8) & 0xFF00) );
+			return (short) ( ((number >> 8) & 0xFF) | ((number << 8) & 0xFF00) );
 		}
 
 		private static int SwapInt (int number)
 		{
-			byte b0 = (byte) ((number >> 24) & 0xFF);
-			byte b1 = (byte) ((number >> 16) & 0xFF);
-			byte b2 = (byte) ((number >> 8) & 0xFF);
-			byte b3 = (byte) (number & 0xFF);
-			return b0 + (b1 << 8) + (b2 << 16) + (b3 << 24);
+			return (((number >> 24) & 0xFF)
+				  | ((number >> 08) & 0xFF00)
+				  | ((number << 08) & 0xFF0000)
+				  | ((number << 24)));
 		}
 
-		private static long SwapLong (long number)
+		private static long SwapLong(long number)
 		{
-			byte b0 = (byte) ((number >> 56) & 0xFF);
-			byte b1 = (byte) ((number >> 48) & 0xFF);
-			byte b2 = (byte) ((number >> 40) & 0xFF);
-			byte b3 = (byte) ((number >> 32) & 0xFF);
-			byte b4 = (byte) ((number >> 24) & 0xFF);
-			byte b5 = (byte) ((number >> 16) & 0xFF);
-			byte b6 = (byte) ((number >> 8) & 0xFF);
-			byte b7 = (byte) (number & 0xFF);
-			return (long) b0 + ((long) b1 << 8) + ((long) b2 << 16) + ((long) b3 << 24) + ((long) b4 << 32) + ((long) b5 << 40) + ((long) b6 << 48) + ((long) b7 << 56);
+			return (((number >> 56) & 0xFF)
+				  | ((number >> 40) & 0xFF00)
+				  | ((number >> 24) & 0xFF0000)
+				  | ((number >> 08) & 0xFF000000)
+				  | ((number << 08) & 0xFF00000000)
+				  | ((number << 24) & 0xFF0000000000)
+				  | ((number << 40) & 0xFF000000000000)
+				  | ((number << 56)));
 		}
 
 		public static short HostToNetworkOrder(short host) {
@@ -143,7 +141,8 @@ namespace System.Net {
 
 #if NET_2_0
 			if (len != 16 && len != 4)
-				throw new ArgumentException ("address");
+				throw new ArgumentException ("An invalid IP address was specified.",
+					"address");
 #else
 			if (len != 16)
 				throw new ArgumentException ("address");
@@ -165,7 +164,12 @@ namespace System.Net {
 				throw new ArgumentNullException ("address");
 
 			if (address.Length != 16)
+#if NET_2_0
+				throw new ArgumentException ("An invalid IP address was specified.",
+					"address");
+#else
 				throw new ArgumentException("address");
+#endif
 
 			Buffer.BlockCopy(address, 0, m_Numbers, 0, 16);
 			m_Family = AddressFamily.InterNetworkV6;
