@@ -56,8 +56,8 @@ namespace System.Net
 		{
 			// the list of restricted header names as defined 
 			// by the ms.net spec
-			restricted = new Hashtable (CaseInsensitiveHashCodeProvider.Default,
-						    CaseInsensitiveComparer.Default);
+			restricted = new Hashtable (CaseInsensitiveHashCodeProvider.DefaultInvariant,
+						    CaseInsensitiveComparer.DefaultInvariant);
 
 			restricted.Add ("accept", true);
 			restricted.Add ("connection", true);
@@ -74,8 +74,8 @@ namespace System.Net
 			
 			// see par 14 of RFC 2068 to see which header names
 			// accept multiple values each separated by a comma
-			multiValue = new Hashtable (CaseInsensitiveHashCodeProvider.Default,
-						    CaseInsensitiveComparer.Default);
+			multiValue = new Hashtable (CaseInsensitiveHashCodeProvider.DefaultInvariant,
+						    CaseInsensitiveComparer.DefaultInvariant);
 
 			multiValue.Add ("accept", true);
 			multiValue.Add ("accept-charset", true);
@@ -99,6 +99,7 @@ namespace System.Net
 			multiValue.Add ("vary", true);
 			multiValue.Add ("via", true);
 			multiValue.Add ("warning", true);
+			multiValue.Add ("www-authenticate", true);
 
 			// Extra
 			multiValue.Add ("set-cookie", true);
@@ -112,11 +113,20 @@ namespace System.Net
 		protected WebHeaderCollection (SerializationInfo serializationInfo, 
 					       StreamingContext streamingContext)
 		{
-			// TODO: test for compatibility with ms.net
-			int count = serializationInfo.GetInt32("count");
-			for (int i = 0; i < count; i++) 
-				this.Add (serializationInfo.GetString ("k" + i),
-					  serializationInfo.GetString ("v" + i));
+			int count;
+
+			try {
+				count = serializationInfo.GetInt32("Count");
+				for (int i = 0; i < count; i++) 
+					this.Add (serializationInfo.GetString (i.ToString ()),
+						  serializationInfo.GetString ((count + i).ToString ()));
+			} catch (SerializationException){
+				count = serializationInfo.GetInt32("count");
+				for (int i = 0; i < count; i++) 
+					this.Add (serializationInfo.GetString ("k" + i),
+						  serializationInfo.GetString ("v" + i));
+			}
+			
 		}
 		
 		internal WebHeaderCollection (bool internallyCreated)
@@ -279,10 +289,10 @@ namespace System.Net
 						  StreamingContext streamingContext)
 		{
 			int count = base.Count;
-			serializationInfo.AddValue ("count", count);
+			serializationInfo.AddValue ("Count", count);
 			for (int i = 0; i < count; i++) {
-				serializationInfo.AddValue ("k" + i, GetKey (i));
-				serializationInfo.AddValue ("v" + i, Get (i));
+				serializationInfo.AddValue (i.ToString (), GetKey (i));
+				serializationInfo.AddValue ((count + i).ToString (), Get (i));
 			}
 		}
 
