@@ -71,6 +71,16 @@ namespace MonoTests.System.Collections.Generic {
 			s.Clear ();
 			
 			AssertEquals (s.Count, 0);
+
+			IEnumerator enumerator = s.GetEnumerator();
+			s.Clear();
+
+			try {
+				enumerator.Reset();
+				NUnit.Framework.Assert.Fail("Version should've been incremented");
+			}
+			catch(InvalidOperationException) {
+			}
 		}
 		
 		[Test]
@@ -85,7 +95,7 @@ namespace MonoTests.System.Collections.Generic {
 			AssertEquals (s.Contains (1), true);
 			AssertEquals (s.Contains (0), false);
 		}
-#if !TARGET_JVM
+
 		[Test]
 		public void TestCopyTo ()
 		{
@@ -100,7 +110,7 @@ namespace MonoTests.System.Collections.Generic {
 			AssertEquals (x [1], 1);
 			AssertEquals (x [2], 2);
 		}
-#endif
+
 		[Test]
 		public void TestPeek ()
 		{
