@@ -86,7 +86,7 @@ namespace MonoTests.System.Collections.Generic {
 			AssertEquals (s.Contains (1), true);
 			AssertEquals (s.Contains (0), false);
 		}
-#if !TARGET_JVM
+
 		[Test]
 		public void TestCopyTo ()
 		{
@@ -101,7 +101,7 @@ namespace MonoTests.System.Collections.Generic {
 			AssertEquals (x [1], 2);
 			AssertEquals (x [2], 1);
 		}
-#endif
+
 		[Test]
 		public void TestPeek ()
 		{
@@ -110,6 +110,10 @@ namespace MonoTests.System.Collections.Generic {
 			
 			AssertEquals (s.Peek (), 1);
 			AssertEquals (s.Count, 1);
+
+			IEnumerator enumerator = s.GetEnumerator();
+			s.Peek();
+			enumerator.Reset();
 		}
 		
 		[Test]
