@@ -247,13 +247,18 @@ namespace Mono.Security.Cryptography {
 			if (!encrypt && (0 > len) && ((algo.Padding == PaddingMode.None) || (algo.Padding == PaddingMode.Zeros))) {
 				throw new CryptographicException ("outputBuffer", Locale.GetText ("Overflow"));
 			} else 	if (KeepLastBlock) {
-				if (0 > len + BlockSizeByte)
+				if (0 > len + BlockSizeByte) {
+#if NET_2_0
 					throw new CryptographicException ("outputBuffer", Locale.GetText ("Overflow"));
+#else
+					throw new IndexOutOfRangeException (Locale.GetText ("Overflow"));
+#endif
+				}
 			} else {
 				if (0 > len) {
 					// there's a special case if this is the end of the decryption process
 					if (inputBuffer.Length - inputOffset - outputBuffer.Length == BlockSizeByte)
-						inputCount = inputCount = outputBuffer.Length - outputOffset;
+						inputCount = outputBuffer.Length - outputOffset;
 					else
 						throw new CryptographicException ("outputBuffer", Locale.GetText ("Overflow"));
 				}
@@ -263,7 +268,7 @@ namespace Mono.Security.Cryptography {
 
 		private bool KeepLastBlock {
 			get {
-				return ((!encrypt) && (algo.Mode != CipherMode.ECB) && (algo.Padding != PaddingMode.None));
+				return ((!encrypt) && (algo.Padding != PaddingMode.None) && (algo.Padding != PaddingMode.Zeros));
 			}
 		}
 
