@@ -97,8 +97,8 @@ namespace System.IO
 		ERROR_REQ_NOT_ACCEP = 71,
 		ERROR_REDIR_PAUSED = 72,
 	*/	ERROR_FILE_EXISTS = 80,
-	/*	ERROR_CANNOT_MAKE = 82,
-		ERROR_FAIL_I24 = 83,
+		ERROR_CANNOT_MAKE = 82,
+	/*	ERROR_FAIL_I24 = 83,
 		ERROR_OUT_OF_STRUCTURES = 84,
 		ERROR_ALREADY_ASSIGNED = 85,
 		ERROR_INVALID_PASSWORD = 86,
