@@ -348,7 +348,7 @@ namespace System
 
 		// InternalCall Methods
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		protected extern int GetRank ();
+		extern int GetRank ();
 
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		public extern int GetLength (int dimension);
@@ -606,7 +606,7 @@ namespace System
 			return CreateInstance (elementType, lengths);
 		}
 
-		public static Array CreateInstance (Type elementType, int[] lengths)
+		public static Array CreateInstance (Type elementType, params int[] lengths)
 		{
 			if (elementType == null)
 				throw new ArgumentNullException ("elementType");
