@@ -39,14 +39,19 @@ using System.Runtime.ConstrainedExecution;
 
 namespace System
 {
+#if NET_2_0
+	public static class Math
+	{
+#else
 	public sealed class Math
 	{
-		public const double E = 2.7182818284590452354;
-		public const double PI = 3.14159265358979323846;
-
 		private Math ()
 		{
 		}
+#endif
+
+		public const double E = 2.7182818284590452354;
+		public const double PI = 3.14159265358979323846;
 
 		public static decimal Abs (decimal value)
 		{
@@ -145,7 +150,7 @@ namespace System
 			r = x - (y * Math.Round(x/y));
 			if (r != 0)
 				return r;
-			return (x > 0) ? 0: (BitConverter.Int64BitsToDouble (Int64.MinValue));
+			return (x > 0) ? 0: (BitConverter.InternalInt64BitsToDouble (Int64.MinValue));
 		}
 
 		public static double Log (double a, double newBase)
