@@ -32,7 +32,11 @@
 
 namespace System
 {
+#if NET_2_0
+	[AttributeUsage (AttributeTargets.Field, Inherited=false)]
+#else
 	[AttributeUsage (AttributeTargets.Field)]
+#endif
 	public sealed class NonSerializedAttribute : Attribute
 	{
 		public NonSerializedAttribute ()
