@@ -543,6 +543,7 @@ namespace MonoTests.System.Collections.Generic {
 		}
 
 		[Test]
+		[Category ("TargetJvmNotWorking")] // BUGBUG Very very slow on TARGET_JVM.
 		public void SerializationTest()
 		{
 			for (int i = 0; i < 50; i++)
@@ -681,6 +682,15 @@ namespace MonoTests.System.Collections.Generic {
 			IDictionary d = new Dictionary<int, int> ();
 			d.Remove (null);
 		}
+		
+		[Test]
+		public void IDictionary_IndexerGetNonExistingTest ()
+		{
+			IDictionary d = new Dictionary<int, int> ();
+			d.Add(1, 2);
+			Assert.IsNull(d[2]);
+			Assert.IsNull(d["foo"]);
+		}
 	}
 }
 
