@@ -98,7 +98,7 @@ public class DirectoryTest
 		string path = TempFolder + DSC + "DirectoryTest.Test";
 		DeleteDirectory (path);
 		try {
-			path += Path.InvalidPathChars [0];
+			path += '\x00';
 			path += ".2";
 			DirectoryInfo info = Directory.CreateDirectory (path);
 		} finally {
@@ -250,6 +250,7 @@ public class DirectoryTest
 		Assert.IsFalse (Directory.Exists (null as string));
 	}
 
+#if !TARGET_JVM // We don't support yet the Process class.
 	[Test]
 	[Category("NotDotNet")]
 	public void ExistsAccessDenied ()
@@ -270,16 +271,19 @@ public class DirectoryTest
 			Directory.Delete (path);
 		}
 	}
+#endif
 	
 	[Test]
+	[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 	[ExpectedException(typeof(ArgumentNullException))]
-	public void GetCreationTimeException1 ()
+    public void GetCreationTimeException1 ()
 	{
 		Directory.GetCreationTime (null as string);
 	}
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 	public void GetCreationTimeException2 ()
 	{
 		Directory.GetCreationTime ("");
@@ -289,6 +293,7 @@ public class DirectoryTest
 #if !NET_2_0
 	[ExpectedException(typeof(IOException))]
 #endif
+	[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 	public void GetCreationTimeException_NonExistingPath ()
 	{
 		string path = TempFolder + DSC + "DirectoryTest.GetCreationTime.1";
@@ -312,6 +317,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 	public void GetCreationTimeException4 ()
 	{
 		Directory.GetCreationTime ("    ");
@@ -319,6 +325,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 	public void GetCreationTimeException5 ()
 	{
 		Directory.GetCreationTime (Path.InvalidPathChars [0].ToString ());
@@ -326,6 +333,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentNullException))]
+	[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 	public void GetCreationTimeUtcException1 ()
 	{
 		Directory.GetCreationTimeUtc (null as string);
@@ -333,6 +341,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 	public void GetCreationTimeUtcException2 ()
 	{
 		Directory.GetCreationTimeUtc ("");
@@ -342,6 +351,7 @@ public class DirectoryTest
 #if !NET_2_0
 	[ExpectedException (typeof (IOException))]
 #endif
+	[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 	public void GetCreationTimeUtc_NonExistingPath ()
 	{
 		string path = TempFolder + DSC + "DirectoryTest.GetCreationTimeUtc.1";
@@ -365,6 +375,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 	public void GetCreationTimeUtcException4 ()
 	{
 		Directory.GetCreationTimeUtc ("    ");
@@ -372,6 +383,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 	public void GetCreationTimeUtcException5 ()
 	{
 		Directory.GetCreationTime (Path.InvalidPathChars [0].ToString ());
@@ -379,6 +391,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentNullException))]
+	[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 	public void GetLastAccessTime_Null ()
 	{
 		Directory.GetLastAccessTime (null as string);
@@ -386,6 +399,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 	public void GetLastAccessTimeException2 ()
 	{
 		Directory.GetLastAccessTime ("");
@@ -395,6 +409,7 @@ public class DirectoryTest
 #if !NET_2_0
 	[ExpectedException (typeof (IOException))]
 #endif
+	[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 	public void GetLastAccessTime_NonExistingPath ()
 	{
 		string path = TempFolder + DSC + "DirectoryTest.GetLastAccessTime.1";
@@ -419,6 +434,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 	public void GetLastAccessTimeException4 ()
 	{
 		Directory.GetLastAccessTime ("    ");
@@ -426,6 +442,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 	public void GetLastAccessTimeException5 ()
 	{
 		Directory.GetLastAccessTime (Path.InvalidPathChars [0].ToString ());
@@ -433,6 +450,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentNullException))]
+	[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 	public void GetLastAccessTimeUtc_Null ()
 	{
 		Directory.GetLastAccessTimeUtc (null as string);
@@ -440,6 +458,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 	public void GetLastAccessTimeUtcException2 ()
 	{
 		Directory.GetLastAccessTimeUtc ("");
@@ -449,6 +468,7 @@ public class DirectoryTest
 #if !NET_2_0
 	[ExpectedException (typeof (IOException))]
 #endif
+	[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 	public void GetLastAccessTimeUtc_NonExistingPath ()
 	{
 		string path = TempFolder + DSC + "DirectoryTest.GetLastAccessTimeUtc.1";
@@ -471,6 +491,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 	public void GetLastAccessTimeUtcException4 ()
 	{
 		Directory.GetLastAccessTimeUtc ("    ");
@@ -478,6 +499,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 	public void GetLastAccessTimeUtcException5 ()
 	{
 		Directory.GetLastAccessTimeUtc (Path.InvalidPathChars [0].ToString ());
@@ -719,6 +741,7 @@ public class DirectoryTest
 	}
 	
 	[Test]
+	[Category("TargetJvmNotSupported")] // CreationTime not supported for TARGET_JVM
 	public void CreationTime ()
 	{
 		// check for Unix platforms - see FAQ for more details
@@ -772,6 +795,7 @@ public class DirectoryTest
 	}
 
 	[Test]
+	[Category("TargetJvmNotSupported")] // LastAccessTime not supported for TARGET_JVM
 	public void LastAccessTime ()
 	{
 		string path = TempFolder + DSC + "DirectoryTest.AccessTime.1";
@@ -993,6 +1017,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentNullException))]
+	[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 	public void SetLastAccessTimeException1 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
@@ -1001,6 +1026,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 	public void SetLastAccessTimeException2 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
@@ -1009,6 +1035,7 @@ public class DirectoryTest
 	
 	[Test]
 	[ExpectedException(typeof(FileNotFoundException))]
+	[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 	public void SetLastAccessTimeException3 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
@@ -1023,6 +1050,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 	public void SetLastAccessTimeException4 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
@@ -1031,6 +1059,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 	public void SetLastAccessTimeException5 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
@@ -1056,6 +1085,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentNullException))]
+	[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 	public void SetLastAccessTimeUtcException1 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
@@ -1064,6 +1094,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 	public void SetLastAccessTimeUtcException2 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
@@ -1072,6 +1103,7 @@ public class DirectoryTest
 	
 	[Test]
 	[ExpectedException(typeof(FileNotFoundException))]
+	[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 	public void SetLastAccessTimeUtcException3 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
@@ -1086,6 +1118,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 	public void SetLastAccessTimeUtcException4 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
@@ -1094,6 +1127,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 	public void SetLastAccessTimeUtcException5 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
@@ -1118,6 +1152,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentNullException))]
+	[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 	public void SetCreationTimeException1 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
@@ -1126,6 +1161,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 	public void SetCreationTimeException2 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
@@ -1134,6 +1170,7 @@ public class DirectoryTest
 	
 	[Test]
 	[ExpectedException(typeof(FileNotFoundException))]
+	[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 	public void SetCreationTimeException3 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
@@ -1149,6 +1186,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 	public void SetCreationTimeException4 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
@@ -1157,6 +1195,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 	public void SetCreationTimeException5 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
@@ -1183,6 +1222,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentNullException))]
+	[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 	public void SetCreationTimeUtcException1 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
@@ -1191,6 +1231,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 	public void SetCreationTimeUtcException2 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
@@ -1199,6 +1240,7 @@ public class DirectoryTest
 	
 	[Test]
 	[ExpectedException(typeof(FileNotFoundException))]
+	[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 	public void SetCreationTimeUtcException3 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
@@ -1215,6 +1257,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 	public void SetCreationTimeUtcException4 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
@@ -1223,6 +1266,7 @@ public class DirectoryTest
 
 	[Test]
 	[ExpectedException(typeof(ArgumentException))]
+	[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 	public void SetCreationTimeUtcException5 ()
 	{
 		DateTime time = new DateTime (2003, 4, 6, 6, 4, 2);
