@@ -587,6 +587,7 @@ namespace MonoTests.System.IO
                 }
 
 		[Test]
+		[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 		public void TestGetCreationTime ()
 		{
                         string path = TempFolder + Path.DirectorySeparatorChar + "baz";
@@ -603,6 +604,7 @@ namespace MonoTests.System.IO
 
 		// Setting the creation time on Unix is not possible
                 [Test]
+		[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
                 public void CreationTime ()
                 {
 			int platform = (int) Environment.OSVersion.Platform;
@@ -647,6 +649,7 @@ namespace MonoTests.System.IO
                 }
 
                 [Test]
+		[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
                 public void LastAccessTime ()
                 {
                         string path = TempFolder + Path.DirectorySeparatorChar + "lastAccessTime";                	
@@ -742,6 +745,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof(ArgumentNullException))]	
+		[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 		public void GetCreationTimeException1 ()
 		{
 			File.GetCreationTime (null as string);
@@ -749,6 +753,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof(ArgumentException))]	
+		[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 		public void GetCreationTimeException2 ()
 		{
 			File.GetCreationTime ("");
@@ -758,6 +763,7 @@ namespace MonoTests.System.IO
 #if !NET_2_0
 		[ExpectedException(typeof(IOException))]
 #endif
+		[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 		public void GetCreationTime_NonExistingPath ()
 		{
 			string path = TempFolder + Path.DirectorySeparatorChar + "GetCreationTimeException3";
@@ -777,6 +783,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof(ArgumentException))]
+		[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 		public void GetCreationTimeException4 ()
 		{
 			File.GetCreationTime ("    ");
@@ -784,6 +791,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof(ArgumentException))]
+		[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 		public void GetCreationTimeException5 ()
 		{
 			File.GetCreationTime (Path.InvalidPathChars [0].ToString ());
@@ -791,6 +799,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof(ArgumentNullException))]
+		[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 		public void GetCreationTimeUtcException1 ()
 		{
 			File.GetCreationTimeUtc (null as string);
@@ -798,6 +807,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof(ArgumentException))]
+		[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 		public void GetCreationTimeUtcException2 ()
 		{
 			File.GetCreationTimeUtc ("");
@@ -807,6 +817,7 @@ namespace MonoTests.System.IO
 #if !NET_2_0
 		[ExpectedException (typeof (IOException))]
 #endif
+		[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 		public void GetCreationTimeUtc_NonExistingPath ()
 		{
 			string path = TempFolder + Path.DirectorySeparatorChar + "GetCreationTimeUtcException3";
@@ -825,6 +836,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof(ArgumentException))]
+		[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 		public void GetCreationTimeUtcException4 ()
 		{
 			File.GetCreationTimeUtc ("    ");
@@ -832,6 +844,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof(ArgumentException))]
+		[Category("TargetJvmNotSupported")] // GetCreationTime not supported for TARGET_JVM
 		public void GetCreationTimeUtcException5 ()
 		{
 			File.GetCreationTime (Path.InvalidPathChars [0].ToString ());
@@ -839,6 +852,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof(ArgumentNullException))]
+		[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 		public void GetLastAccessTimeException1 ()
 		{
 			File.GetLastAccessTime (null as string);
@@ -846,6 +860,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof(ArgumentException))]
+		[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 		public void GetLastAccessTimeException2 ()
 		{
 			File.GetLastAccessTime ("");
@@ -855,6 +870,7 @@ namespace MonoTests.System.IO
 #if !NET_2_0
 		[ExpectedException (typeof (IOException))]
 #endif
+		[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 		public void GetLastAccessTime_NonExistingPath ()
 		{
 			string path = TempFolder + Path.DirectorySeparatorChar + "GetLastAccessTimeException3";
@@ -874,6 +890,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof(ArgumentException))]
+		[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 		public void GetLastAccessTimeException4 ()
 		{
 			File.GetLastAccessTime ("    ");
@@ -881,6 +898,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof(ArgumentException))]
+		[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 		public void GetLastAccessTimeException5 ()
 		{
 			File.GetLastAccessTime (Path.InvalidPathChars [0].ToString ());
@@ -888,6 +906,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof(ArgumentNullException))]
+		[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 		public void GetLastAccessTimeUtcException1 ()
 		{
 			File.GetLastAccessTimeUtc (null as string);
@@ -895,6 +914,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof(ArgumentException))]
+		[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 		public void GetLastAccessTimeUtcException2 ()
 		{
 			File.GetLastAccessTimeUtc ("");
@@ -904,6 +924,7 @@ namespace MonoTests.System.IO
 #if !NET_2_0
 		[ExpectedException (typeof (IOException))]
 #endif
+		[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 		public void GetLastAccessTimeUtc_NonExistingPath ()
 		{
 			string path = TempFolder + Path.DirectorySeparatorChar + "GetLastAccessTimeUtcException3";
@@ -922,6 +943,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof(ArgumentException))]
+		[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 		public void GetLastAccessTimeUtcException4 ()
 		{
 			File.GetLastAccessTimeUtc ("    ");
@@ -929,6 +951,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof(ArgumentException))]
+		[Category("TargetJvmNotSupported")] // GetLastAccessTime not supported for TARGET_JVM
 		public void GetLastAccessTimeUtcException5 ()
 		{
 			File.GetLastAccessTimeUtc (Path.InvalidPathChars [0].ToString ());
@@ -1051,6 +1074,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (ArgumentNullException))]
+		[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 		public void SetCreationTimeArgumentNullException1 ()
 		{
 			File.SetCreationTime (null as string, new DateTime (2000, 12, 12, 11, 59, 59));
@@ -1058,6 +1082,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (ArgumentException))]
+		[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 		public void SetCreationTimeArgumenException1 ()
 		{
 			File.SetCreationTime ("", new DateTime (2000, 12, 12, 11, 59, 59));
@@ -1065,6 +1090,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (ArgumentException))]
+		[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 		public void SetCreationTimeArgumenException2 ()
 		{
 			File.SetCreationTime ("     ", new DateTime (2000, 12, 12, 11, 59, 59));
@@ -1072,6 +1098,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		// On Unix there are no invalid path chars.
+		[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 		public void SetCreationTimeArgumenException3 ()
 		{
 			if (Path.InvalidPathChars.Length > 1) {
@@ -1088,6 +1115,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (FileNotFoundException))]
+		[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 		public void SetCreationTimeFileNotFoundException1 ()
 		{
 			string path = TempFolder + Path.DirectorySeparatorChar + "SetCreationTimeFileNotFoundException1";
@@ -1116,6 +1144,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (IOException))]
+		[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 		public void SetCreationTimeIOException1 ()
 		{
 			string path = TempFolder + Path.DirectorySeparatorChar + "CreationTimeIOException1";
@@ -1133,6 +1162,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (ArgumentNullException))]
+		[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 		public void SetCreationTimeUtcArgumentNullException1 ()
 		{ 
 			File.SetCreationTimeUtc (null as string, new DateTime (2000, 12, 12, 11, 59, 59));
@@ -1140,6 +1170,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (ArgumentException))]
+		[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 		public void SetCreationTimeUtcArgumenException1 ()
 		{
 			File.SetCreationTimeUtc ("", new DateTime (2000, 12, 12, 11, 59, 59));
@@ -1147,6 +1178,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (ArgumentException))]
+		[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 		public void SetCreationTimeUtcArgumenException2 ()
 		{
 			File.SetCreationTimeUtc ("     ", new DateTime (2000, 12, 12, 11, 59, 59));
@@ -1154,6 +1186,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		// On Unix there are no invalid path chars.
+		[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 		public void SetCreationTimeUtcArgumentException3 ()
 		{
 			if (Path.InvalidPathChars.Length > 1) {
@@ -1170,6 +1203,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (FileNotFoundException))]
+		[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 		public void SetCreationTimeUtcFileNotFoundException1 ()
 		{
 			string path = TempFolder + Path.DirectorySeparatorChar + "SetCreationTimeUtcFileNotFoundException1";
@@ -1198,6 +1232,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (IOException))]
+		[Category("TargetJvmNotSupported")] // SetCreationTime not supported for TARGET_JVM
 		public void SetCreationTimeUtcIOException1 ()
 		{
 			string path = TempFolder + Path.DirectorySeparatorChar + "SetCreationTimeUtcIOException1";
@@ -1217,6 +1252,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (ArgumentNullException))]
+		[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 		public void SetLastAccessTimeArgumentNullException1 ()
 		{
 			File.SetLastAccessTime (null as string, new DateTime (2000, 12, 12, 11, 59, 59));
@@ -1224,6 +1260,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (ArgumentException))]
+		[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 		public void SetLastAccessTimeArgumenException1 ()
 		{
 			File.SetLastAccessTime ("", new DateTime (2000, 12, 12, 11, 59, 59));
@@ -1231,6 +1268,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (ArgumentException))]
+		[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 		public void SetLastAccessTimeArgumenException2 ()
 		{
 			File.SetLastAccessTime ("     ", new DateTime (2000, 12, 12, 11, 59, 59));
@@ -1238,6 +1276,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		// On Unix there are no invalid path chars.
+		[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 		public void SetLastAccessTimeArgumenException3 ()
 		{
 			if (Path.InvalidPathChars.Length > 1) {
@@ -1254,6 +1293,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (FileNotFoundException))]
+		[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 		public void SetLastAccessTimeFileNotFoundException1 ()
 		{
 			string path = TempFolder + Path.DirectorySeparatorChar + "SetLastAccessTimeFileNotFoundException1";
@@ -1282,6 +1322,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (IOException))]
+		[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 		public void SetLastAccessTimeIOException1 ()
 		{
 			string path = TempFolder + Path.DirectorySeparatorChar + "LastAccessIOException1";
@@ -1299,6 +1340,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (ArgumentNullException))]
+		[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 		public void SetLastAccessTimeUtcArgumentNullException1 ()
 		{
 			File.SetLastAccessTimeUtc (null as string, new DateTime (2000, 12, 12, 11, 59, 59));
@@ -1306,6 +1348,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (ArgumentException))]
+		[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 		public void SetCLastAccessTimeUtcArgumenException1 ()
 		{
 			File.SetLastAccessTimeUtc ("", new DateTime (2000, 12, 12, 11, 59, 59));
@@ -1313,6 +1356,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (ArgumentException))]
+		[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 		public void SetLastAccessTimeUtcArgumenException2 ()
 		{
 			File.SetLastAccessTimeUtc ("     ", new DateTime (2000, 12, 12, 11, 59, 59));
@@ -1320,6 +1364,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		// On Unix there are no invalid path chars.
+		[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 		public void SetLastAccessTimeUtcArgumenException3 ()
 		{
 			if (Path.InvalidPathChars.Length > 1) {
@@ -1336,6 +1381,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (FileNotFoundException))]
+		[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 		public void SetLastAccessTimeUtcFileNotFoundException1 ()
 		{
 			string path = TempFolder + Path.DirectorySeparatorChar + "SetLastAccessTimeUtcFileNotFoundException1";
@@ -1364,6 +1410,7 @@ namespace MonoTests.System.IO
 
 		[Test]
 		[ExpectedException(typeof (IOException))]
+		[Category("TargetJvmNotSupported")] // SetLastAccessTime not supported for TARGET_JVM
 		public void SetLastAccessTimeUtcIOException1 ()
 		{
 			string path = TempFolder + Path.DirectorySeparatorChar + "SetLastAccessTimeUtcIOException1";
@@ -1507,7 +1554,7 @@ namespace MonoTests.System.IO
 			string path = TempFolder + Path.DirectorySeparatorChar + "SetLastWriteTimeUtcFileNotFoundException1";
 			DeleteFile (path);
 			
-			File.SetLastAccessTimeUtc (path, new DateTime (2000, 12, 12, 11, 59, 59));
+			File.SetLastWriteTimeUtc (path, new DateTime (2000, 12, 12, 11, 59, 59));
 		}
 
 //		[Test]
@@ -1537,7 +1584,7 @@ namespace MonoTests.System.IO
 			FileStream stream = null;
 			try {
 				stream = File.Create (path);
-				File.SetLastAccessTimeUtc (path, new DateTime (1000, 12, 12, 11, 59, 59));
+				File.SetLastWriteTimeUtc (path, new DateTime (1000, 12, 12, 11, 59, 59));
 			} finally {
 				if (stream != null)
 					stream.Close ();
