@@ -470,7 +470,9 @@ namespace MonoTests.System.IO
 		{
 			if (Windows) {
 				AssertEquals ("GetFullPath w#01", @"Z:\", Path.GetFullPath ("Z:"));
+#if !TARGET_JVM // Java full (canonical) path always starts with caps drive letter
 				AssertEquals ("GetFullPath w#02", @"c:\abc\def", Path.GetFullPath (@"c:\abc\def"));
+#endif
 				Assert ("GetFullPath w#03", Path.GetFullPath (@"\").EndsWith (@"\"));
 				// "\\\\" is not allowed
 				Assert ("GetFullPath w#05", Path.GetFullPath ("/").EndsWith (@"\"));
@@ -656,7 +658,11 @@ namespace MonoTests.System.IO
 
 			string curdir = Directory.GetCurrentDirectory ();
 			try {
+#if TARGET_JVM
+                string system = "C:\\WINDOWS\\system32\\";
+#else
 				string system = Environment.SystemDirectory;
+#endif
 				Directory.SetCurrentDirectory (system);
 				string drive = system.Substring (0, 2);
 				AssertEquals ("current dir", system, Path.GetFullPath (drive));
@@ -677,7 +683,11 @@ namespace MonoTests.System.IO
 
 			string curdir = Directory.GetCurrentDirectory ();
 			try {
+#if TARGET_JVM
+                string system = "C:\\WINDOWS\\system32\\";
+#else
 				string system = Environment.SystemDirectory;
+#endif
 				Directory.SetCurrentDirectory (system);
 				// e.g. C:dir (no backslash) will return CurrentDirectory + dir
 				string dir = system.Substring (0, 2) + "dir";
@@ -689,6 +699,9 @@ namespace MonoTests.System.IO
 		}
 
 		[Test]
+#if TARGET_JVM
+        [Ignore("Java full (canonical) path always returns windows dir in caps")]
+#endif
 		public void WindowsDriveC14N_77058 ()
 		{
 			// check for Unix platforms - see FAQ for more details
@@ -817,6 +830,22 @@ namespace MonoTests.System.IO
 			}
 		}
 
+		[Test]
+		public void GetInvalidPathChars_Order()
+		{
+			if (Windows) {
+				char [] invalid = Path.GetInvalidPathChars ();
+				char [] expected = new char [36] { '\x22', '\x3C', '\x3E', '\x7C', '\x00', '\x01', '\x02',
+					'\x03', '\x04', '\x05', '\x06', '\x07', '\x08', '\x09', '\x0A', '\x0B', '\x0C', '\x0D',
+					'\x0E', '\x0F', '\x10', '\x11', '\x12', '\x13', '\x14', '\x15', '\x16', '\x17', '\x18',
+					'\x19', '\x1A', '\x1B', '\x1C', '\x1D', '\x1E', '\x1F' };
+				AssertEquals (expected.Length, invalid.Length);
+				for (int i = 0; i < expected.Length; i++ ) {
+					AssertEquals( "Character at position " + i,expected [i], invalid [i]);
+				}
+			}
+		}
+
 		[Test]
 		public void GetInvalidPathChars_Modify ()
 		{
