@@ -10,7 +10,9 @@
 using System;
 using System.Threading;
 using System.Reflection;
+#if !TARGET_JVM
 using System.Reflection.Emit;
+#endif // TARGET_JVM
 using System.Runtime.InteropServices;
 
 using NUnit.Framework;
@@ -61,13 +63,13 @@ public class ParameterInfoTest : Assertion
 
 	public static void paramMethod (int i, [In] int j, [Out] int k, [Optional] int l, [In,Out] int m, [DefaultParameterValue (ParamEnum.Foo)] ParamEnum n) {
 	}
-
+#if !TARGET_JVM // No support for extern methods in TARGET_JVM
 	[DllImport ("foo")]
 	public extern static void marshalAsMethod (
 		[MarshalAs(UnmanagedType.Bool)]int p0, 
 		[MarshalAs(UnmanagedType.LPArray, ArraySubType=UnmanagedType.LPStr)] string [] p1,
 		[MarshalAs( UnmanagedType.CustomMarshaler, MarshalTypeRef = typeof (Marshal1), MarshalCookie = "5")] object p2);
-
+#endif
 	[Test]
 	public void DefaultValueEnum () {
 		ParameterInfo[] info = typeof (ParameterInfoTest).GetMethod ("paramMethod").GetParameters ();
@@ -85,6 +87,7 @@ public class ParameterInfoTest : Assertion
 		AssertEquals (1, info[3].GetCustomAttributes (typeof (OptionalAttribute), true).Length);
 		AssertEquals (2, info[4].GetCustomAttributes (true).Length);
 
+#if !TARGET_JVM // No support for extern methods in TARGET_JVM
 		ParameterInfo[] pi = typeof (ParameterInfoTest).GetMethod ("marshalAsMethod").GetParameters ();
 		MarshalAsAttribute attr;
 
@@ -99,6 +102,7 @@ public class ParameterInfoTest : Assertion
 		AssertEquals (UnmanagedType.CustomMarshaler, attr.Value);
 		AssertEquals ("5", attr.MarshalCookie);
 		AssertEquals (typeof (Marshal1), Type.GetType (attr.MarshalType));
+#endif
 	}
 #endif
 }		
