@@ -144,6 +144,19 @@ namespace MonoTests.System.Text {
                 	AssertEquals ("es", str);
         	}
 
+		// Test invalid byte handling
+		public void TestGetString3 () 
+		{
+			Encoding encoding = Encoding.ASCII;
+			byte [] bytes = new byte [] {0x61, 0xE1, 0xE2};
+			string s = encoding.GetString (bytes, 0, 3);
+#if NET_2_0
+			AssertEquals ("a??", s);
+#else
+			AssertEquals ("aab", s);
+#endif
+		}
+
 		// Test Decoder
 		public void TestDecoder ()
 		{
