@@ -25,11 +25,11 @@ namespace MonoTests.System.Threading
 				this.id = id;
 				this.mut = mut;
 			}
-			public void wait()
+			public void Wait()
 			{
 				mut.WaitOne();
 			}
-			public void signal()
+			public void Signal()
 			{
 				mut.ReleaseMutex();
 			}
@@ -47,7 +47,7 @@ namespace MonoTests.System.Threading
 			public void WithoutWait()
 			{
 				this.marker = this.id;
-				this.signal();
+				this.Signal();
 			}
 
 
@@ -55,15 +55,15 @@ namespace MonoTests.System.Threading
 			{
 				while (this.marker<100)
 				{
-					this.wait();
+					this.Wait();
 					this.marker++;
-					this.signal();
+					this.Signal();
 				}
 			}
 
 			public void WaitAndForget()
 			{
-				this.wait();
+				this.Wait();
 				this.marker = id;
 			}
 			public void WaitAndWait()
@@ -161,6 +161,7 @@ namespace MonoTests.System.Threading
 		}
 
 		[Test]
+		[Category("TargetJvmNotSupported")] // IntPtr native handles are not supported for TARGET_JVM.
 		public void TestHandle()
 		{
 			Mutex Sem = new Mutex();
@@ -168,7 +169,6 @@ namespace MonoTests.System.Threading
 		}
 
 		[Test] // bug #79358
-		[Category ("NotWorking")]
 		public void DoubleRelease ()
 		{
 			Mutex mutex = new Mutex ();
