@@ -600,6 +600,7 @@ namespace Mono.CSharp {
 		Constraints constraints;
 		Location loc;
 		GenericTypeParameterBuilder type;
+		MemberCache member_cache;
 
 		public TypeParameter (DeclSpace parent, DeclSpace decl, string name,
 				      Constraints constraints, Attributes attrs, Location loc)
@@ -619,10 +620,6 @@ namespace Mono.CSharp {
 			get { return constraints; }
 		}
 
-		public bool HasConstructorConstraint {
-			get { return constraints != null && constraints.HasConstructorConstraint; }
-		}
-
 		public DeclSpace DeclSpace {
 			get { return decl; }
 		}
@@ -882,11 +879,19 @@ namespace Mono.CSharp {
 		}
 
 		MemberCache IMemberContainer.BaseCache {
-			get { return null; }
+			get {
+				if (gc == null)
+					return null;
+
+				if (gc.EffectiveBaseClass.BaseType == null)
+					return null;
+
+				return TypeManager.LookupMemberCache (gc.EffectiveBaseClass.BaseType);
+			}
 		}
 
 		bool IMemberContainer.IsInterface {
-			get { return true; }
+			get { return false; }
 		}
 
 		MemberList IMemberContainer.GetMembers (MemberTypes mt, BindingFlags bf)
@@ -894,8 +899,19 @@ namespace Mono.CSharp {
 			return FindMembers (mt, bf, null, null);
 		}
 
-		MemberCache IMemberContainer.MemberCache {
-			get { return null; }
+		public MemberCache MemberCache {
+			get {
+				if (member_cache != null)
+					return member_cache;
+
+				if (gc == null)
+					return null;
+
+				Type[] ifaces = TypeManager.ExpandInterfaces (gc.InterfaceConstraints);
+				member_cache = new MemberCache (this, gc.EffectiveBaseClass, ifaces);
+
+				return member_cache;
+			}
 		}
 
 		public MemberList FindMembers (MemberTypes mt, BindingFlags bf,
@@ -987,7 +1003,7 @@ namespace Mono.CSharp {
 				bool has_class_constr = false;
 				if (list.Count > 0) {
 					Type first = (Type) list [0];
-					has_class_constr = !first.IsInterface && !first.IsGenericParameter;
+					has_class_constr = !first.IsGenericParameter && !first.IsInterface;
 				}
 
 				if ((list.Count > 0) && has_class_constr) {
@@ -1246,16 +1262,20 @@ namespace Mono.CSharp {
 					ok = false;
 					continue;
 				}
-				if (te is TypeParameterExpr)
-					has_type_args = true;
 
-#if !MS_COMPATIBLE
+				atypes[i] = te.Type;
+				if (te.Type.IsGenericParameter) {
+					if (te is TypeParameterExpr)
+						has_type_args = true;
+					continue;
+				}
+
 				if (te.Type.IsSealed && te.Type.IsAbstract) {
 					Report.Error (718, Location, "`{0}': static classes cannot be used as generic arguments",
 						te.GetSignatureForError ());
 					return false;
 				}
-#endif
+
 				if (te.Type.IsPointer) {
 					Report.Error (306, Location, "The type `{0}' may not be used " +
 							  "as a type argument", TypeManager.CSharpName (te.Type));
@@ -1266,11 +1286,18 @@ namespace Mono.CSharp {
 					Expression.Error_VoidInvalidInTheContext (Location);
 					return false;
 				}
-
-				atypes [i] = te.Type;
 			}
 			return ok;
 		}
+
+		public TypeArguments Clone ()
+		{
+			TypeArguments copy = new TypeArguments (Location);
+			foreach (Expression ta in args)
+				copy.args.Add (ta);
+
+			return copy;
+		}
 	}
 
 	public class TypeParameterName : SimpleName
@@ -1621,7 +1648,7 @@ namespace Mono.CSharp {
 			if (TypeManager.IsBuiltinType (atype) || atype.IsValueType)
 				return true;
 
-			if (HasDefaultConstructor (ec.DeclContainer.TypeBuilder, atype))
+			if (HasDefaultConstructor (atype))
 				return true;
 
 			Report_SymbolRelatedToPreviousError ();
@@ -1670,7 +1697,7 @@ namespace Mono.CSharp {
 			return false;
 		}
 
-		bool HasDefaultConstructor (Type containerType, Type atype)
+		bool HasDefaultConstructor (Type atype)
 		{
 			if (atype.IsAbstract)
 				return false;
@@ -1698,8 +1725,12 @@ namespace Mono.CSharp {
 			}
 
 			TypeParameter tparam = TypeManager.LookupTypeParameter (atype);
-			if (tparam != null)
-				return tparam.HasConstructorConstraint;
+			if (tparam != null) {
+				if (tparam.GenericConstraints == null)
+					return false;
+				else
+					return tparam.GenericConstraints.HasConstructorConstraint;
+			}
 
 			MemberList list = TypeManager.FindMembers (
 				atype, MemberTypes.Constructor,
@@ -2063,9 +2094,8 @@ namespace Mono.CSharp {
 		static void InitGenericCodeHelpers ()
 		{
 			// Activator
-			Type [] type_arg = { type_type };
 			activator_create_instance = GetMethod (
-				activator_type, "CreateInstance", type_arg);
+				activator_type, "CreateInstance", Type.EmptyTypes);
 		}
 
 		static Type CoreLookupType (string ns, string name, int arity)
@@ -2217,11 +2247,7 @@ namespace Mono.CSharp {
 			return true;
 		}
 
-		//
-		// Type inference.
-		//
-
-		static bool InferType (Type pt, Type at, Type[] inferred)
+		static bool UnifyType (Type pt, Type at, Type[] inferred)
 		{
 			if (pt.IsGenericParameter) {
 				if (pt.DeclaringMethod == null)
@@ -2229,20 +2255,15 @@ namespace Mono.CSharp {
 
 				int pos = pt.GenericParameterPosition;
 
-				if (inferred [pos] == null) {
+				if (inferred [pos] == null)
 					inferred [pos] = at;
-					return true;
-				}
 
-				if (inferred [pos] != at)
-					return false;
-
-				return true;
+				return inferred [pos] == at;
 			}
 
 			if (!pt.ContainsGenericParameters) {
 				if (at.ContainsGenericParameters)
-					return InferType (at, pt, inferred);
+					return UnifyType (at, pt, inferred);
 				else
 					return true;
 			}
@@ -2252,7 +2273,7 @@ namespace Mono.CSharp {
 					if (at.GetArrayRank () != pt.GetArrayRank ())
 						return false;
 
-					return InferType (pt.GetElementType (), at.GetElementType (), inferred);
+					return UnifyType (pt.GetElementType (), at.GetElementType (), inferred);
 				}
 
 				if (!pt.IsGenericType)
@@ -2264,7 +2285,7 @@ namespace Mono.CSharp {
 					return false;
 
 				Type[] args = GetTypeArguments (pt);
-				return InferType (args [0], at.GetElementType (), inferred);
+				return UnifyType (args [0], at.GetElementType (), inferred);
 			}
 
 			if (pt.IsArray) {
@@ -2272,11 +2293,11 @@ namespace Mono.CSharp {
 				    (pt.GetArrayRank () != at.GetArrayRank ()))
 					return false;
 
-				return InferType (pt.GetElementType (), at.GetElementType (), inferred);
+				return UnifyType (pt.GetElementType (), at.GetElementType (), inferred);
 			}
 
 			if (pt.IsByRef && at.IsByRef)
-				return InferType (pt.GetElementType (), at.GetElementType (), inferred);
+				return UnifyType (pt.GetElementType (), at.GetElementType (), inferred);
 			ArrayList list = new ArrayList ();
 			if (at.IsGenericType)
 				list.Add (at);
@@ -2285,51 +2306,26 @@ namespace Mono.CSharp {
 
 			list.AddRange (TypeManager.GetInterfaces (at));
 
-			bool found_one = false;
-
 			foreach (Type type in list) {
 				if (!type.IsGenericType)
 					continue;
 
-				Type[] inferred_types = new Type [inferred.Length];
-
-				if (!InferGenericInstance (pt, type, inferred_types))
+				if (DropGenericTypeArguments (pt) != DropGenericTypeArguments (type))
 					continue;
 
-				for (int i = 0; i < inferred_types.Length; i++) {
-					if (inferred [i] == null) {
-						inferred [i] = inferred_types [i];
-						continue;
-					}
-
-					if (inferred [i] != inferred_types [i])
-						return false;
-				}
-
-				found_one = true;
+				if (!UnifyTypes (pt.GetGenericArguments (), type.GetGenericArguments (), inferred))
+					return false;
 			}
 
-			return found_one;
+			return true;
 		}
 
-		static bool InferGenericInstance (Type pt, Type at, Type[] inferred_types)
+		static bool UnifyTypes (Type[] pts, Type [] ats, Type [] inferred)
 		{
-			Type[] at_args = at.GetGenericArguments ();
-			Type[] pt_args = pt.GetGenericArguments ();
-
-			if (at_args.Length != pt_args.Length)
-				return false;
-
-			for (int i = 0; i < at_args.Length; i++) {
-				if (!InferType (pt_args [i], at_args [i], inferred_types))
+			for (int i = 0; i < ats.Length; i++) {
+				if (!UnifyType (pts [i], ats [i], inferred))
 					return false;
 			}
-
-			for (int i = 0; i < inferred_types.Length; i++) {
-				if (inferred_types [i] == null)
-					return false;
-			}
-
 			return true;
 		}
 
@@ -2379,7 +2375,7 @@ namespace Mono.CSharp {
 				Type pt = pd.ParameterType (i);
 				Type at = a.Type;
 
-				if (!InferType (pt, at, inferred_types))
+				if (!UnifyType (pt, at, inferred_types))
 					return false;
 			}
 
@@ -2391,7 +2387,7 @@ namespace Mono.CSharp {
 				if ((a.Expr is NullLiteral) || (a.Expr is MethodGroupExpr))
 					continue;
 
-				if (!InferType (element_type, a.Type, inferred_types))
+				if (!UnifyType (element_type, a.Type, inferred_types))
 					return false;
 			}
 
@@ -2406,31 +2402,139 @@ namespace Mono.CSharp {
 		static bool InferTypeArguments (Type[] param_types, Type[] arg_types,
 						Type[] inferred_types)
 		{
-			if (inferred_types == null)
-				return false;
-
 			for (int i = 0; i < arg_types.Length; i++) {
 				if (arg_types [i] == null)
 					continue;
 
-				if (!InferType (param_types [i], arg_types [i], inferred_types))
+				if (!UnifyType (param_types [i], arg_types [i], inferred_types))
 					return false;
 			}
 
-			for (int i = 0; i < inferred_types.Length; i++)
+			for (int i = 0; i < inferred_types.Length; ++i)
 				if (inferred_types [i] == null)
 					return false;
 
 			return true;
 		}
 
+		//
+		// Infers the remaining inferred_types from lambda expressions contained in the 
+		// invocation call.
+		//
+		static bool LambdaInfer (EmitContext ec, ArrayList arguments,
+					 Type[] param_types, Type[] arg_types, Type[] inferred_types)
+		{
+			int arg_count = arg_types.Length;
+			
+			for (int i = 0; i < arg_count; i++){
+				Argument a = (Argument) arguments [i];
+				
+				LambdaExpression le = a.Expr as LambdaExpression;
+				
+				if (a == null)
+					continue;
+				
+				//
+				// TODO: "The argument is a lambda expression, in
+				// the following called L, from which no inferences
+				// have yet been made."
+				//
+				
+				//
+				// "The corresponding parameter’s type, in the
+				// following called P, is a delegate type with a
+				// return type that involves one or more method type
+				// parameters."
+				//
+				// 
+				if (!TypeManager.IsDelegateType (param_types [i]))
+					continue;
+				
+				Type p_type = param_types [i];
+				MethodGroupExpr method_group = Expression.MemberLookup (
+					ec.ContainerType, p_type, "Invoke", MemberTypes.Method,
+					Expression.AllBindingFlags, Location.Null) as MethodGroupExpr;
+				
+				if (method_group == null){
+					// This we report elsewhere as -200, but here we can ignore
+					continue;
+				}
+				MethodInfo delegate_method = method_group.Methods [0] as MethodInfo;
+				if (delegate_method == null){
+					// This should not happen.
+					continue;
+				}
+				
+				Type return_type = delegate_method.ReturnType;
+				if (!return_type.IsGenericParameter)
+					continue;
+				
+				//
+				// P and L have the same number of parameters, and
+				// each parameter in P has the same modifiers as the
+				// corresponding parameter in L, or no modifiers if
+				// L has an implicitly typed parameter list.
+				//
+				ParameterData delegate_pd = TypeManager.GetParameterData (delegate_method);
+				int delegate_pc = delegate_pd.Count;
+				if (delegate_pc != le.Parameters.Count)
+					continue;
+
+#if false
+				//FIXME
+				if (le.HasExplicitParameters){
+					for (int j = 0; j < delegate_pc; j++){
+						if (delegate_pd.ParameterModifier [j] !=
+						    le.Parameters.ParameterModifier[j])
+							goto do_continue;
+					}
+				} else {
+					for (int j = 0; j < delegate_pc; j++)
+						if (le.Parameters.ParameterModifier [j] != Parameter.Modifier.NONE)
+							goto do_continue;
+				}
+#endif
+				
+				//
+				// TODO: P’s parameter types involve no method type
+				// parameters or involve only method type parameters
+				// for which a consistent set of inferences have
+				// already been made.
+				//
+
+				if (le.HasExplicitParameters){
+					//
+					// TODO: If L has an explicitly typed parameter
+					// list, when inferred types are substituted for
+					// method type parameters in P, each parameter in P
+					// has the same type as the corresponding parameter
+					// in L.
+					//
+				} else {
+					//
+					// TODO: If L has an implicitly typed parameter
+					// list, when inferred types are substituted for
+					// method type parameters in P and the resulting
+					// parameter types are given to the parameters of L,
+					// the body of L is a valid expression or statement
+					// block.
+				}
+				
+			do_continue:
+				;
+			}
+
+			return true;
+		}
+	
 		/// <summary>
 		///   Type inference.  Try to infer the type arguments from `method',
 		///   which is invoked with the arguments `arguments'.  This is used
 		///   when resolving an Invocation or a DelegateInvocation and the user
 		///   did not explicitly specify type arguments.
 		/// </summary>
-		public static bool InferTypeArguments (ArrayList arguments,
+		public static bool InferTypeArguments (EmitContext ec,
+						       ArrayList arguments,
 						       ref MethodBase method)
 		{
 			if (!TypeManager.IsGenericMethod (method))
@@ -2465,21 +2569,56 @@ namespace Mono.CSharp {
 			Type[] param_types = new Type [pd.Count];
 			Type[] arg_types = new Type [pd.Count];
 
+			int lambdas = 0;
 			for (int i = 0; i < arg_count; i++) {
 				param_types [i] = pd.ParameterType (i);
 
 				Argument a = (Argument) arguments [i];
-				if ((a.Expr is NullLiteral) || (a.Expr is MethodGroupExpr) ||
-				    (a.Expr is AnonymousMethodExpression))
+				if (a.Expr is NullLiteral || a.Expr is MethodGroupExpr)
 					continue;
+								
+				if (a.Expr is AnonymousMethodExpression) {
+					if (RootContext.Version != LanguageVersion.LINQ)
+						continue;
 
+					Type dtype = param_types[i];
+					if (!TypeManager.IsDelegateType (dtype))
+						continue;
+
+					AnonymousMethodExpression am = (AnonymousMethodExpression)a.Expr;
+
+					Expression e = am.Compatible (ec, dtype);
+					if (e == null)
+						return false;
+
+					arg_types[i] = e.Type;
+					continue;
+				}
+
+				if (a.Expr is LambdaExpression)
+					lambdas++;
+				
 				arg_types [i] = a.Type;
 			}
 
-			if (!InferTypeArguments (param_types, arg_types, inferred_types))
-				return false;
+			if (!InferTypeArguments (param_types, arg_types, inferred_types)){
+				Type it;
+				if (lambdas == 0)
+					return false;
+				
+				if (!LambdaInfer (ec, arguments, param_types, arg_types, inferred_types))
+					return false;
+			}
 
 			method = ((MethodInfo)method).MakeGenericMethod (inferred_types);
+
+#if MS_COMPATIBLE
+			// MS implementation throws NotSupportedException for GetParameters
+			// on unbaked generic method
+			ParameterData p = TypeManager.GetParameterData (method);
+			p.InflateTypes (param_types, inferred_types);
+#endif
+
 			return true;
 		}
 
