@@ -39,7 +39,7 @@ namespace Mono.CSharp {
 
 		public static string MakeName (string prefix)
 		{
-			return String.Format ("<>c__{0}{1}", prefix, ++next_index);
+			return "<>c__" + prefix + next_index++;
 		}
 
 		protected CompilerGeneratedClass (DeclSpace parent, GenericMethod generic,
@@ -100,6 +100,27 @@ namespace Mono.CSharp {
 			get { return generic_method; }
 		}
 
+		public Parameters InflateParameters (Parameters ps)
+		{
+			if (generic_method == null)
+				return ps;
+
+			int n = ps.Count;
+			if (n == 0)
+				return ps;
+
+			Parameter[] inflated_params = new Parameter [n];
+			Type[] inflated_types = new Type [n];
+
+			for (int i = 0; i < n; ++i) {
+				Parameter p = ps [i];
+				Type it = InflateType (p.ExternalType ()).ResolveAsTypeTerminal (this, false).Type;
+				inflated_types [i] = it;
+				inflated_params [i] = new Parameter (it, p.Name, p.ModFlags, p.OptAttributes, p.Location);
+			}
+			return new Parameters (inflated_params, inflated_types);
+		}
+
 		public TypeExpr InflateType (Type it)
 		{
 #if GMCS_SOURCE
@@ -201,8 +222,13 @@ namespace Mono.CSharp {
 			return new_scope;
 		}
 
+		private static int default_modflags (DeclSpace parent)
+		{
+			return parent is CompilerGeneratedClass ? Modifiers.PUBLIC : Modifiers.PRIVATE;
+		}
+
 		protected ScopeInfo (Block block, DeclSpace parent, GenericMethod generic)
-			: base (parent, generic, 0, block.StartLocation)
+			: base (parent, generic, default_modflags (parent), block.StartLocation)
 		{
 			Parent = parent;
 			RootScope = block.Toplevel.RootScope;
@@ -216,7 +242,7 @@ namespace Mono.CSharp {
 
 		protected ScopeInfo (ToplevelBlock toplevel, DeclSpace parent,
 				     GenericMethod generic, Location loc)
-			: base (parent, generic, 0, loc)
+			: base (parent, generic, default_modflags (parent), loc)
 		{
 			Parent = parent;
 			RootScope = (RootScopeInfo) this;
@@ -244,7 +270,7 @@ namespace Mono.CSharp {
 			return (CapturedVariable) captured_scopes [scope];
 		}
 
-		public void EmitScopeInstance (EmitContext ec)
+		protected void EmitScopeInstance (EmitContext ec)
 		{
 			if (scope_initializer == null) {
 				//
@@ -444,6 +470,12 @@ namespace Mono.CSharp {
 
 			public override void EmitInstance (EmitContext ec)
 			{
+				if ((ec.CurrentAnonymousMethod != null) &&
+				    (ec.CurrentAnonymousMethod.Scope == Scope)) {
+					ec.ig.Emit (OpCodes.Ldarg_0);
+					return;
+				}
+
 				Scope.EmitScopeInstance (ec);
 			}
 
@@ -1037,12 +1069,12 @@ namespace Mono.CSharp {
 	{
 		public readonly AnonymousMethodExpression Parent;
 		public readonly TypeContainer Host;
-		public readonly Parameters Parameters;
+		public Parameters Parameters;
 
 		public ToplevelBlock Block;
 		protected AnonymousMethod anonymous;
 
-		protected readonly Block container;
+		protected Block container;
 		protected readonly GenericMethod generic;
 
 		public Block Container {
@@ -1128,7 +1160,7 @@ namespace Mono.CSharp {
 				      TypeManager.CSharpName (t));
 		}
 
-		public bool ImplicitStandardConversionExists (Type delegate_type)
+		public virtual bool ImplicitStandardConversionExists (Type delegate_type)
 		{
 			if (Parameters == null)
 				return true;
@@ -1148,22 +1180,15 @@ namespace Mono.CSharp {
 			return true;
 		}
 
-		//
-		// Returns true if this anonymous method can be implicitly
-		// converted to the delegate type `delegate_type'
-		//
-		public Expression Compatible (EmitContext ec, Type delegate_type)
+		protected Expression CompatibleChecks (EmitContext ec, Type delegate_type)
 		{
-			if (anonymous != null)
-				return anonymous.AnonymousDelegate;
-
 			if (!ec.IsAnonymousMethodAllowed) {
 				Report.Error (1706, loc,
 					      "Anonymous methods are not allowed in the " +
 					      "attribute declaration");
 				return null;
 			}
-
+			
 			if (!TypeManager.IsDelegateType (delegate_type)){
 				Report.Error (1660, loc,
 					      "Cannot convert anonymous method block to type " +
@@ -1171,7 +1196,61 @@ namespace Mono.CSharp {
 					      TypeManager.CSharpName (delegate_type));
 				return null;
 			}
+			return this;
+		}
 
+		protected bool VerifyExplicitParameterCompatibility (Type delegate_type, ParameterData invoke_pd)
+		{
+			if (Parameters.Count != invoke_pd.Count) {
+				Report.SymbolRelatedToPreviousError (delegate_type);
+				Report.Error (1593, loc, "Delegate `{0}' does not take `{1}' arguments",
+					      TypeManager.CSharpName (delegate_type), Parameters.Count.ToString ());
+				Error_ParameterMismatch (delegate_type);
+				return false;
+			}
+			
+			for (int i = 0; i < Parameters.Count; ++i) {
+				Parameter.Modifier p_mod = invoke_pd.ParameterModifier (i);
+				if (Parameters.ParameterModifier (i) != p_mod && p_mod != Parameter.Modifier.PARAMS) {
+					if (p_mod == Parameter.Modifier.NONE)
+						Report.Error (1677, loc, "Parameter `{0}' should not be declared with the `{1}' keyword",
+							      (i + 1).ToString (), Parameter.GetModifierSignature (Parameters.ParameterModifier (i)));
+					else
+						Report.Error (1676, loc, "Parameter `{0}' must be declared with the `{1}' keyword",
+							      (i+1).ToString (), Parameter.GetModifierSignature (p_mod));
+					Error_ParameterMismatch (delegate_type);
+					return false;
+				}
+
+				// We assume that generic parameters are always inflated
+				if (TypeManager.IsGenericParameter (invoke_pd.Types[i]))
+					continue;
+				
+				if (invoke_pd.ParameterType (i) != Parameters.ParameterType (i)) {
+					Report.Error (1678, loc, "Parameter `{0}' is declared as type `{1}' but should be `{2}'",
+						      (i+1).ToString (),
+						      TypeManager.CSharpName (Parameters.ParameterType (i)),
+						      TypeManager.CSharpName (invoke_pd.ParameterType (i)));
+					Error_ParameterMismatch (delegate_type);
+					return false;
+				}
+			}
+			return true;
+		}
+		
+		//
+		// Returns true if this anonymous method can be implicitly
+		// converted to the delegate type `delegate_type'
+		//
+		public virtual Expression Compatible (EmitContext ec, Type delegate_type)
+		{
+			if (anonymous != null)
+				return anonymous.AnonymousDelegate;
+
+			if (CompatibleChecks (ec, delegate_type) == null)
+				return null;
+
+			
 			//
 			// At this point its the first time we know the return type that is 
 			// needed for the anonymous method.  We create the method here.
@@ -1182,6 +1261,31 @@ namespace Mono.CSharp {
 			MethodInfo invoke_mb = (MethodInfo) invoke_mg.Methods [0];
 			ParameterData invoke_pd = TypeManager.GetParameterData (invoke_mb);
 
+#if GMCS_SOURCE
+			Type[] infered_arguments = null;
+			if (TypeManager.IsGenericType (delegate_type)) {
+
+				Type[] g_arguments = delegate_type.GetGenericArguments ();
+				infered_arguments = new Type[g_arguments.Length];
+				for (int i = 0; i < g_arguments.Length; ++i) {
+					infered_arguments [i] = g_arguments[i];
+				}
+
+				for (int i = 0; i < invoke_pd.Count; ++i) {
+					if (!invoke_pd.Types[i].IsGenericParameter)
+						continue;
+
+					infered_arguments [invoke_pd.Types[i].GenericParameterPosition] = Parameters.Types[i];
+				}
+			}
+
+			int return_type_pos = -1;
+			if (TypeManager.IsGenericParameter (invoke_mb.ReturnType)) {
+				ec.InferReturnType = true;
+				return_type_pos = invoke_mb.ReturnType.GenericParameterPosition;
+			}
+#endif
+
 			Parameters parameters;
 			if (Parameters == null) {
 				//
@@ -1208,36 +1312,8 @@ namespace Mono.CSharp {
 				if (!parameters.Resolve (ec))
 					return null;
 			} else {
-				if (Parameters.Count != invoke_pd.Count) {
-					Report.SymbolRelatedToPreviousError (delegate_type);
-					Report.Error (1593, loc, "Delegate `{0}' does not take `{1}' arguments",
-						TypeManager.CSharpName (delegate_type), Parameters.Count.ToString ());
-					Error_ParameterMismatch (delegate_type);
+				if (!VerifyExplicitParameterCompatibility (delegate_type, invoke_pd))
 					return null;
-				}
-
-				for (int i = 0; i < Parameters.Count; ++i) {
-					Parameter.Modifier p_mod = invoke_pd.ParameterModifier (i);
-					if (Parameters.ParameterModifier (i) != p_mod && p_mod != Parameter.Modifier.PARAMS) {
-						if (p_mod == Parameter.Modifier.NONE)
-							Report.Error (1677, loc, "Parameter `{0}' should not be declared with the `{1}' keyword",
-								(i + 1).ToString (), Parameter.GetModifierSignature (Parameters.ParameterModifier (i)));
-						else
-							Report.Error (1676, loc, "Parameter `{0}' must be declared with the `{1}' keyword",
-								(i+1).ToString (), Parameter.GetModifierSignature (p_mod));
-						Error_ParameterMismatch (delegate_type);
-						return null;
-					}
-
-					if (invoke_pd.ParameterType (i) != Parameters.ParameterType (i)) {
-						Report.Error (1678, loc, "Parameter `{0}' is declared as type `{1}' but should be `{2}'",
-							(i+1).ToString (),
-							TypeManager.CSharpName (Parameters.ParameterType (i)),
-							TypeManager.CSharpName (invoke_pd.ParameterType (i)));
-						Error_ParameterMismatch (delegate_type);
-						return null;
-					}
-				}
 
 				parameters = Parameters;
 			}
@@ -1264,6 +1340,19 @@ namespace Mono.CSharp {
 			if (!anonymous.Resolve (ec))
 				return null;
 
+#if GMCS_SOURCE
+			if (return_type_pos != -1) {
+				if (infered_arguments == null)
+					infered_arguments = new Type [delegate_type.GetGenericArguments ().Length];
+
+				infered_arguments [return_type_pos] = anonymous.ReturnType;
+			}
+
+			if (infered_arguments != null & TypeManager.IsGenericType (delegate_type)) {
+				anonymous.AnonymousDelegate.Type = delegate_type.GetGenericTypeDefinition ().MakeGenericType (infered_arguments);
+				anonymous.DelegateType = anonymous.AnonymousDelegate.Type;
+			}
+#endif
 			return anonymous.AnonymousDelegate;
 		}
 
@@ -1296,14 +1385,22 @@ namespace Mono.CSharp {
 		public bool IsIterator {
 			get { return false; }
 		}
+
+		protected override void CloneTo (CloneContext clonectx, Expression t)
+		{
+			AnonymousMethodExpression target = (AnonymousMethodExpression) t;
+
+			target.Block = (ToplevelBlock) clonectx.LookupBlock (Block);
+			target.container = clonectx.LookupBlock (Block);
+			target.Parameters = Parameters.Clone ();
+		}
 	}
 
 	public abstract class AnonymousContainer : IAnonymousContainer
 	{
 		public readonly Location Location;
 
-		// An array list of AnonymousMethodParameter or null
-		public readonly Parameters Parameters;
+		public Parameters Parameters;
 
 		//
 		// The block that makes up the body for the anonymous mehtod
@@ -1380,6 +1477,9 @@ namespace Mono.CSharp {
 				ReturnType = return_type_expr.Type;
 			}
 
+			if (RootScope != null)
+				Parameters = RootScope.InflateParameters (Parameters);
+
 			aec = new EmitContext (
 				ec.ResolveContext, ec.TypeContainer,
 				RootScope != null ? RootScope : Host, Location, null, ReturnType,
@@ -1389,6 +1489,7 @@ namespace Mono.CSharp {
 			aec.CurrentAnonymousMethod = this;
 			aec.IsFieldInitializer = ec.IsFieldInitializer;
 			aec.IsStatic = ec.IsStatic;
+			aec.InferReturnType = ec.InferReturnType;
 
 			Report.Debug (64, "RESOLVE ANONYMOUS METHOD #1", this, Location, ec, aec,
 				      RootScope, Parameters, Block);
@@ -1399,6 +1500,9 @@ namespace Mono.CSharp {
 
 			Report.Debug (64, "RESOLVE ANONYMOUS METHOD #3", this, ec, aec, Block);
 
+			if (aec.InferReturnType)
+				ReturnType = aec.ReturnType;
+
 			method = DoCreateMethodHost (ec);
 
 			if (Scope != null)
@@ -1454,12 +1558,20 @@ namespace Mono.CSharp {
 				aec.MethodIsStatic = Scope == null;
 				return aec;
 			}
+
+#if GMCS_SOURCES
+			public override void Emit ()
+			{
+				MethodBuilder.SetCustomAttribute (TypeManager.compiler_generated_attr);
+				base.Emit ();
+			}
+#endif
 		}
 	}
 
 	public class AnonymousMethod : AnonymousContainer
 	{
-		public readonly Type DelegateType;
+		public Type DelegateType;
 
 		//
 		// The value return by the Compatible call, this ensure that
@@ -1670,6 +1782,11 @@ namespace Mono.CSharp {
 				loc);
 
 			constructor_method = ((MethodGroupExpr) ml).Methods [0];
+#if MS_COMPATIBLE
+			if (type.IsGenericType)
+				constructor_method = TypeBuilder.GetConstructor (type, (ConstructorInfo)constructor_method);
+#endif
+			
 			delegate_method = am.GetMethodBuilder (ec);
 			base.Emit (ec);
 
