@@ -81,7 +81,34 @@ namespace Mono.CSharp {
 				MyEmptyExpr = new EmptyExpression ();
 			MyEmptyExpr.SetType (TypeManager.GetElementType (array));
 
-			return ImplicitReferenceConversionExists (MyEmptyExpr, arg_type);
+			return ImplicitReferenceConversionCore (MyEmptyExpr, arg_type);
+#else
+			return false;
+#endif
+		}
+		
+		static bool IList_To_Array(Type list, Type array)
+		{
+# if GMCS_SOURCE
+			if (!list.IsGenericType || !array.IsArray || array.GetArrayRank() != 1)
+				return false;
+			
+			Type gt = list.GetGenericTypeDefinition();
+			if (gt != TypeManager.generic_ilist_type &&
+				gt != TypeManager.generic_icollection_type &&
+				gt != TypeManager.generic_ienumerable_type)
+				return false;
+			
+			Type arg_type = TypeManager.GetTypeArguments(list)[0];
+			Type element_type = TypeManager.GetElementType(array);
+			
+			if (element_type == arg_type)
+				return true;
+			
+			if (MyEmptyExpr == null)
+				MyEmptyExpr = new EmptyExpression();
+			MyEmptyExpr.SetType(element_type);
+			return ImplicitReferenceConversionExists(MyEmptyExpr, arg_type) || ExplicitReferenceConversionExists(element_type, arg_type);
 #else
 			return false;
 #endif
@@ -132,6 +159,54 @@ namespace Mono.CSharp {
 			return null;
 		}
 
+		static bool ImplicitTypeParameterBoxingConversion (Type expr_type, Type target_type,
+								   out bool use_class_cast)
+		{
+#if GMCS_SOURCE
+			GenericConstraints gc = TypeManager.GetTypeParameterConstraints (expr_type);
+
+			if (gc == null) {
+				use_class_cast = false;
+				return target_type == TypeManager.object_type;
+			}
+
+			use_class_cast = true;
+
+			if (!gc.HasReferenceTypeConstraint)
+				return false;
+
+			// We're converting from a type parameter which is known to be a reference type.
+			Type base_type = TypeParam_EffectiveBaseType (gc);
+
+			if (TypeManager.IsSubclassOf (base_type, target_type))
+				return true;
+
+			if (target_type.IsInterface) {
+				if (TypeManager.ImplementsInterface (base_type, target_type))
+					return true;
+
+				foreach (Type t in gc.InterfaceConstraints) {
+					if (TypeManager.IsSubclassOf (t, target_type))
+						return true;
+					if (TypeManager.ImplementsInterface (t, target_type))
+						return true;
+				}
+			}
+
+			foreach (Type t in gc.InterfaceConstraints) {
+				if (!TypeManager.IsGenericParameter (t))
+					continue;
+				if (TypeManager.IsSubclassOf (t, target_type))
+					return true;
+				if (TypeManager.ImplementsInterface (t, target_type))
+					return true;
+			}
+#endif
+
+			use_class_cast = false;
+			return false;
+		}
+
 		static bool ExplicitTypeParameterConversionExists (Type source_type, Type target_type)
 		{
 #if GMCS_SOURCE
@@ -197,6 +272,30 @@ namespace Mono.CSharp {
 			if (TypeManager.IsGenericParameter (expr_type))
 				return ImplicitTypeParameterConversion (expr, target_type);
 
+			// from the null type to any reference-type.
+			if (expr_type == TypeManager.null_type) {
+				NullConstant nc = (NullConstant)expr;
+				return nc.ConvertImplicitly(target_type);
+			}
+
+			if (ImplicitReferenceConversionCore (expr, target_type))
+				return new EmptyCast (expr, target_type);
+
+			bool use_class_cast;
+			if (ImplicitBoxingConversionExists (expr, target_type, out use_class_cast)) {
+				if (use_class_cast)
+					return new ClassCast (expr, target_type);
+				else
+					return new BoxedCast (expr, target_type);
+			}
+
+			return null;
+		}
+
+		static public bool ImplicitReferenceConversionCore (Expression expr, Type target_type)
+		{
+			Type expr_type = expr.Type;
+
 			//
 			// notice that it is possible to write "ValueType v = 1", the ValueType here
 			// is an abstract class, and not really a value type, so we apply the same rules.
@@ -206,24 +305,17 @@ namespace Mono.CSharp {
 				// A pointer type cannot be converted to object
 				//
 				if (expr_type.IsPointer)
-					return null;
+					return false;
 
 				if (TypeManager.IsValueType (expr_type))
-					return new BoxedCast (expr, target_type);
+					return false;
 				if (expr_type.IsClass || expr_type.IsInterface || expr_type == TypeManager.enum_type){
-					if (expr_type == TypeManager.anonymous_method_type)
-						return null;
-					return new EmptyCast (expr, target_type);
+					return expr_type != TypeManager.anonymous_method_type;
 				}
 
-				return null;
+				return false;
 			} else if (target_type == TypeManager.value_type) {
-				if (TypeManager.IsValueType (expr_type))
-					return new BoxedCast (expr, target_type);
-				if (expr_type == TypeManager.null_type)
-					return new EmptyConstantCast ((Constant)expr, target_type);
-
-				return null;
+				return expr_type == TypeManager.enum_type;
 			} else if (TypeManager.IsSubclassOf (expr_type, target_type)) {
 				//
 				// Special case: enumeration to System.Enum.
@@ -231,9 +323,9 @@ namespace Mono.CSharp {
 				// a boxing conversion
 				//
 				if (expr_type.IsEnum || TypeManager.IsGenericParameter (expr_type))
-					return new BoxedCast (expr, target_type);
+					return false;
 
-				return new EmptyCast (expr, target_type);
+				return true;
 			}
 
 			// This code is kind of mirrored inside ImplicitStandardConversionExists
@@ -241,12 +333,6 @@ namespace Mono.CSharp {
 			//
 			// Always ensure that the code here and there is in sync
 
-			// from the null type to any reference-type.
-			if (expr_type == TypeManager.null_type) {
-				NullConstant nc = (NullConstant)expr;
-				return nc.ConvertImplicitly(target_type);
-			}
-
 			// from any class-type S to any interface-type T.
 			if (target_type.IsInterface) {
 				if (target_type != TypeManager.iconvertible_type &&
@@ -256,23 +342,18 @@ namespace Mono.CSharp {
 				      expr is LongLiteral || expr is CharLiteral ||
 				      expr is StringLiteral || expr is DecimalLiteral ||
 				      expr is UIntLiteral || expr is ULongLiteral)) {
-					return null;
+					return false;
 				}
 
 				if (TypeManager.ImplementsInterface (expr_type, target_type)){
-					if (TypeManager.IsGenericParameter (expr_type) || TypeManager.IsValueType (expr_type))
-						return new BoxedCast (expr, target_type);
-					else
-						return new EmptyCast (expr, target_type);
+					return !TypeManager.IsGenericParameter (expr_type) &&
+						!TypeManager.IsValueType (expr_type);
 				}
 			}
 
 			// from any interface type S to interface-type T.
 			if (expr_type.IsInterface && target_type.IsInterface) {
-				if (TypeManager.ImplementsInterface (expr_type, target_type))
-					return new EmptyCast (expr, target_type);
-				else
-					return null;
+				return TypeManager.ImplementsInterface (expr_type, target_type);
 			}
 
 			// from an array-type S to an array-type of type T
@@ -288,67 +369,74 @@ namespace Mono.CSharp {
 					Type target_element_type = TypeManager.GetElementType (target_type);
 
 					if (!expr_element_type.IsValueType && !target_element_type.IsValueType)
-						if (ImplicitStandardConversionExists (MyEmptyExpr,
-										      target_element_type))
-							return new EmptyCast (expr, target_type);
+						return ImplicitStandardConversionExists (
+							MyEmptyExpr, target_element_type);
 				}
 			}
 
 			// from an array-type to System.Array
 			if (expr_type.IsArray && target_type == TypeManager.array_type)
-				return new EmptyCast (expr, target_type);
+				return true;
 
 			// from an array-type of type T to IList<T>
 			if (expr_type.IsArray && Array_To_IList (expr_type, target_type))
-				return new EmptyCast (expr, target_type);
+				return true;
 
 			// from any delegate type to System.Delegate
 			if ((expr_type == TypeManager.delegate_type || TypeManager.IsDelegateType (expr_type)) &&
 			    target_type == TypeManager.delegate_type)
-				return new EmptyCast (expr, target_type);
+				return true;
 
 			// from any array-type or delegate type into System.ICloneable.
 			if (expr_type.IsArray ||
 			    expr_type == TypeManager.delegate_type || TypeManager.IsDelegateType (expr_type))
 				if (target_type == TypeManager.icloneable_type)
-					return new EmptyCast (expr, target_type);
+					return true;
 
 			// from a generic type definition to a generic instance.
 			if (TypeManager.IsEqual (expr_type, target_type))
-				return new EmptyCast (expr, target_type);
+				return true;
 
-			return null;
+			return false;
 		}
 
-		//
-		// Tests whether an implicit reference conversion exists between expr_type
-		// and target_type
-		//
-		public static bool ImplicitReferenceConversionExists (Expression expr, Type target_type)
+		static public bool ImplicitBoxingConversionExists (Expression expr, Type target_type,
+								   out bool use_class_cast)
 		{
-			if (target_type.IsValueType)
-				return false;
-
 			Type expr_type = expr.Type;
 
-			if (TypeManager.IsGenericParameter (expr_type))
-				return ImplicitTypeParameterConversion (expr, target_type) != null;
+			use_class_cast = false;
 
 			//
-			// This is the boxed case.
+			// notice that it is possible to write "ValueType v = 1", the ValueType here
+			// is an abstract class, and not really a value type, so we apply the same rules.
 			//
 			if (target_type == TypeManager.object_type) {
-				if (expr_type.IsClass || TypeManager.IsValueType (expr_type) ||
-				    expr_type.IsInterface || expr_type == TypeManager.enum_type)
-					if (target_type != TypeManager.anonymous_method_type)
+				//
+				// A pointer type cannot be converted to object
+				//
+				if (expr_type.IsPointer)
+					return false;
+
+				return TypeManager.IsValueType (expr_type);
+			} else if (target_type == TypeManager.value_type) {
+				return TypeManager.IsValueType (expr_type);
+			} else if (TypeManager.IsSubclassOf (expr_type, target_type)) {
+				//
+				// Special case: enumeration to System.Enum.
+				// System.Enum is not a value type, it is a class, so we need
+				// a boxing conversion
+				//
+				if (expr_type.IsEnum || TypeManager.IsGenericParameter (expr_type))
 					return true;
 
 				return false;
-			} else if (TypeManager.IsSubclassOf (expr_type, target_type))
-				return true;
+			}
 
-			// Please remember that all code below actually comes
-			// from ImplicitReferenceConversion so make sure code remains in sync
+			// This code is kind of mirrored inside ImplicitStandardConversionExists
+			// with the small distinction that we only probe there
+			//
+			// Always ensure that the code here and there is in sync
 
 			// from any class-type S to any interface-type T.
 			if (target_type.IsInterface) {
@@ -363,52 +451,27 @@ namespace Mono.CSharp {
 				}
 
 				if (TypeManager.ImplementsInterface (expr_type, target_type))
-					return true;
+					return TypeManager.IsGenericParameter (expr_type) ||
+						TypeManager.IsValueType (expr_type);
 			}
 
-			// from any interface type S to interface-type T.
-			if (expr_type.IsInterface && target_type.IsInterface)
-				if (TypeManager.ImplementsInterface (expr_type, target_type))
-					return true;
+			if (TypeManager.IsGenericParameter (expr_type))
+				return ImplicitTypeParameterBoxingConversion (
+					expr_type, target_type, out use_class_cast);
 
-			// from an array-type S to an array-type of type T
-			if (expr_type.IsArray && target_type.IsArray) {
-				if (expr_type.GetArrayRank () == target_type.GetArrayRank ()) {
+			return false;
+		}
 
-					Type expr_element_type = expr_type.GetElementType ();
+		//
+		// Tests whether an implicit reference conversion exists between expr_type
+		// and target_type
+		//
+		public static bool ImplicitReferenceConversionExists (Expression expr, Type target_type)
+		{
+			if (target_type.IsValueType)
+				return false;
 
-					if (MyEmptyExpr == null)
-						MyEmptyExpr = new EmptyExpression ();
-
-					MyEmptyExpr.SetType (expr_element_type);
-					Type target_element_type = TypeManager.GetElementType (target_type);
-
-					if (!expr_element_type.IsValueType && !target_element_type.IsValueType)
-						if (ImplicitStandardConversionExists (MyEmptyExpr,
-										      target_element_type))
-							return true;
-				}
-			}
-
-			// from an array-type to System.Array
-			if (expr_type.IsArray && (target_type == TypeManager.array_type))
-				return true;
-
-			// from an array-type of type T to IList<T>
-			if (expr_type.IsArray && Array_To_IList (expr_type, target_type))
-				return true;
-
-			// from any delegate type to System.Delegate
-			if ((expr_type == TypeManager.delegate_type || TypeManager.IsDelegateType (expr_type)) &&
-			    target_type == TypeManager.delegate_type)
-				if (target_type.IsAssignableFrom (expr_type))
-					return true;
-
-			// from any array-type or delegate type into System.ICloneable.
-			if (expr_type.IsArray ||
-			    expr_type == TypeManager.delegate_type || TypeManager.IsDelegateType (expr_type))
-				if (target_type == TypeManager.icloneable_type)
-					return true;
+			Type expr_type = expr.Type;
 
 			// from the null type to any reference-type.
 			if (expr_type == TypeManager.null_type){
@@ -419,8 +482,12 @@ namespace Mono.CSharp {
 					return true;
 			}
 
-			// from a generic type definition to a generic instance.
-			if (TypeManager.IsEqual (expr_type, target_type))
+			if (TypeManager.IsGenericParameter (expr_type))
+				return ImplicitTypeParameterConversion (expr, target_type) != null;
+
+			bool use_class_cast;
+			if (ImplicitReferenceConversionCore (expr, target_type) ||
+			    ImplicitBoxingConversionExists (expr, target_type, out use_class_cast))
 				return true;
 
 			return false;
@@ -811,15 +878,6 @@ namespace Mono.CSharp {
 					return true;
 			}
 
-			if ((target_type == TypeManager.enum_type ||
-			     target_type.IsSubclassOf (TypeManager.enum_type)) &&
-			     expr is IntLiteral){
-				IntLiteral i = (IntLiteral) expr;
-
-				if (i.Value == 0)
-					return true;
-			}
-
 			//
 			// If `expr_type' implements `target_type' (which is an iface)
 			// see TryImplicitIntConversion
@@ -1701,6 +1759,9 @@ namespace Mono.CSharp {
 				return true;
 
 
+			if (source_type.IsInterface && IList_To_Array (source_type, target_type))
+				return true;
+
 			// From an array type S with an element type Se to an array type T with an
 			// element type Te provided all the following are true:
 			//     * S and T differe only in element type, in other words, S and T
@@ -1823,6 +1884,13 @@ namespace Mono.CSharp {
 						return new UnboxCast (source, target_type);
 				}
 
+				//
+				// From System.Collecitons.Generic.IList<T> and its base interfaces to a one-dimensional
+				// array type S[], provided there is an implicit or explicit reference conversion from S to T.
+				//
+				if (IList_To_Array (source_type, target_type))
+					return new ClassCast (source, target_type);
+
 				return null;
 			}
 
