@@ -144,7 +144,7 @@ namespace Mono.CSharp {
 				if (TypeArguments != null)
 					return new SimpleName (Basename, TypeArguments, Location);
 				else
-				return new SimpleName (Name, Location);
+					return new SimpleName (Name, Location);
 			}
 
 			if (is_double_colon) {
@@ -766,6 +766,10 @@ namespace Mono.CSharp {
 		{
 			return (MemberCore)defined_names [name];
 		}
+
+		public bool IsStaticClass {
+			get { return (ModFlags & Modifiers.STATIC) != 0; }
+		}
 		
 		// 
 		// root_types contains all the types.  All TopLevel types
@@ -805,7 +809,7 @@ namespace Mono.CSharp {
 		///  Should be overriten by the appropriate declaration space
 		/// </remarks>
 		public abstract TypeBuilder DefineType ();
-		
+
 		/// <summary>
 		///   Define all members, but don't apply any attributes or do anything which may
 		///   access not-yet-defined classes.  This method also creates the MemberCache.
@@ -1014,9 +1018,17 @@ namespace Mono.CSharp {
 			while (p.IsArray || p.IsPointer || p.IsByRef)
 				p = TypeManager.GetElementType (p);
 
-			if (TypeManager.IsGenericParameter(p))
-				return true; // FIXME
+#if GMCS_SOURCE
+			if (p.IsGenericParameter)
+				return true;
 
+			if (TypeManager.IsGenericType (p)) {
+				foreach (Type t in p.GetGenericArguments ()) {
+					if (!AsAccessible (t, flags))
+						return false;
+				}
+			}
+#endif
 			AccessLevel pAccess = TypeEffectiveAccessLevel (p);
 			AccessLevel mAccess = this.EffectiveAccessLevel &
 				GetAccessLevelFromModifiers (flags);
@@ -1041,24 +1053,43 @@ namespace Mono.CSharp {
 			for (Type current_type = TypeBuilder;
 			     current_type != null && current_type != TypeManager.object_type;
 			     current_type = current_type.BaseType) {
-				current_type = TypeManager.DropGenericTypeArguments (current_type);
-				if (current_type is TypeBuilder) {
-					TypeContainer tc = current_type == TypeBuilder
-						? PartialContainer
-						: TypeManager.LookupTypeContainer (current_type);
+
+				Type ct = TypeManager.DropGenericTypeArguments (current_type);
+				if (ct is TypeBuilder) {
+					TypeContainer tc = ct == TypeBuilder
+						? PartialContainer : TypeManager.LookupTypeContainer (ct);
 					if (tc != null)
 						t = tc.FindNestedType (name);
 				} else {
-					t = TypeManager.GetNestedType (current_type, name);
+					t = TypeManager.GetNestedType (ct, name);
 				}
 
-				if (t != null && CheckAccessLevel (t))
+				if ((t == null) || !CheckAccessLevel (t))
+					continue;
+
+#if GMCS_SOURCE
+				if (!TypeManager.IsGenericType (current_type))
 					return t;
+
+				Type[] args = TypeManager.GetTypeArguments (current_type);
+				Type[] targs = TypeManager.GetTypeArguments (t);
+				for (int i = 0; i < args.Length; i++)
+					targs [i] = args [i];
+
+				t = t.MakeGenericType (targs);
+#endif
+
+				return t;
 			}
 
 			return null;
 		}
 
+		public virtual ExtensionMethodGroupExpr LookupExtensionMethod (Type extensionType, string name)
+		{
+			return null;
+		}
+
 		//
 		// Public function used to locate types.
 		//
@@ -1066,7 +1097,7 @@ namespace Mono.CSharp {
 		//
 		// Returns: Type or null if they type can not be found.
 		//
-		public FullNamedExpression LookupType (string name, Location loc, bool ignore_cs0104)
+		public FullNamedExpression LookupNamespaceOrType (string name, Location loc, bool ignore_cs0104)
 		{
 			if (Cache.Contains (name))
 				return (FullNamedExpression) Cache [name];
@@ -1076,7 +1107,7 @@ namespace Mono.CSharp {
 			if (t != null)
 				e = new TypeExpression (t, Location.Null);
 			else if (Parent != null)
-				e = Parent.LookupType (name, loc, ignore_cs0104);
+				e = Parent.LookupNamespaceOrType (name, loc, ignore_cs0104);
 			else
 				e = NamespaceEntry.LookupNamespaceOrType (this, name, loc, ignore_cs0104);
 
@@ -1237,6 +1268,8 @@ namespace Mono.CSharp {
 			get {
 				if (!IsGeneric)
 					throw new InvalidOperationException ();
+				if ((PartialContainer != null) && (PartialContainer != this))
+					return PartialContainer.TypeParameters;
 				if (type_param_list == null)
 					initialize_type_params ();
 
@@ -1248,6 +1281,8 @@ namespace Mono.CSharp {
 			get {
 				if (!IsGeneric)
 					throw new InvalidOperationException ();
+				if ((PartialContainer != null) && (PartialContainer != this))
+					return PartialContainer.CurrentTypeParameters;
 				if (type_params != null)
 					return type_params;
 				else
@@ -1289,6 +1324,12 @@ namespace Mono.CSharp {
 			return null;
 		}
 
+		// Used for error reporting only
+		public virtual Type LookupAnyGeneric (string typeName)
+		{
+			return NamespaceEntry.NS.LookForAnyGenericType (typeName);
+		}
+
 		public override string[] ValidAttributeTargets {
 			get { return attribute_targets; }
 		}
@@ -1586,7 +1627,7 @@ namespace Mono.CSharp {
 			Type type = container.Type;
 			if (!(type is TypeBuilder) && !type.IsInterface &&
 			    // !(type.IsGenericType && (type.GetGenericTypeDefinition () is TypeBuilder)) &&
-			    !TypeManager.IsGenericType (type) &&
+			    !TypeManager.IsGenericType (type) && !TypeManager.IsGenericParameter (type) &&
 			    (Container.BaseCache == null || Container.BaseCache.method_hash != null)) {
 				method_hash = new Hashtable ();
 				AddMethods (type);
@@ -1614,6 +1655,28 @@ namespace Mono.CSharp {
 				AddCacheContents (TypeManager.LookupMemberCache (itype));
 		}
 
+		public MemberCache (IMemberContainer container, Type base_class, Type[] ifaces)
+		{
+			this.Container = container;
+
+			// If we have a base class (we have a base class unless we're
+			// TypeManager.object_type), we deep-copy its MemberCache here.
+			if (Container.BaseCache != null)
+				member_hash = SetupCache (Container.BaseCache);
+			else
+				member_hash = new Hashtable ();
+
+			if (base_class != null)
+				AddCacheContents (TypeManager.LookupMemberCache (base_class));
+			if (ifaces != null) {
+				foreach (Type itype in ifaces) {
+					MemberCache cache = TypeManager.LookupMemberCache (itype);
+					if (cache != null)
+						AddCacheContents (cache);
+				}
+			}
+		}
+
 		/// <summary>
 		///   Bootstrap this member cache by doing a deep-copy of our base.
 		/// </summary>
@@ -1683,6 +1746,24 @@ namespace Mono.CSharp {
 			AddMembers (mt, BindingFlags.Instance | BindingFlags.NonPublic, container);
 		}
 
+		void AddMember (MemberTypes mt, BindingFlags bf, IMemberContainer container,
+				string name, MemberInfo member)
+		{
+			// We use a name-based hash table of ArrayList's.
+			ArrayList list = (ArrayList) member_hash [name];
+			if (list == null) {
+				list = new ArrayList ();
+				member_hash.Add (name, list);
+			}
+
+			// When this method is called for the current class, the list will
+			// already contain all inherited members from our base classes.
+			// We cannot add new members in front of the list since this'd be an
+			// expensive operation, that's why the list is sorted in reverse order
+			// (ie. members from the current class are coming last).
+			list.Add (new CacheEntry (container, member, mt, bf));
+		}
+
 		/// <summary>
 		///   Add all members from class `container' with the requested MemberTypes and
 		///   BindingFlags to the cache.  This method is called multiple times with different
@@ -1695,23 +1776,13 @@ namespace Mono.CSharp {
 			foreach (MemberInfo member in members) {
 				string name = member.Name;
 
-				int pos = name.IndexOf ('<');
-				if (pos > 0)
-					name = name.Substring (0, pos);
+				AddMember (mt, bf, container, name, member);
 
-				// We use a name-based hash table of ArrayList's.
-				ArrayList list = (ArrayList) member_hash [name];
-				if (list == null) {
-					list = new ArrayList ();
-					member_hash.Add (name, list);
+				if (member is MethodInfo) {
+					string gname = TypeManager.GetMethodName ((MethodInfo) member);
+					if (gname != name)
+						AddMember (mt, bf, container, gname, member);
 				}
-
-				// When this method is called for the current class, the list will
-				// already contain all inherited members from our base classes.
-				// We cannot add new members in front of the list since this'd be an
-				// expensive operation, that's why the list is sorted in reverse order
-				// (ie. members from the current class are coming last).
-				list.Add (new CacheEntry (container, member, mt, bf));
 			}
 		}
 
@@ -1857,12 +1928,14 @@ namespace Mono.CSharp {
 			Property	= 0x200,
 			NestedType	= 0x400,
 
+			NotExtensionMethod	= 0x800,
+
 			MaskType	= Constructor|Event|Field|Method|Property|NestedType
 		}
 
 		protected class CacheEntry {
 			public readonly IMemberContainer Container;
-			public readonly EntryType EntryType;
+			public EntryType EntryType;
 			public readonly MemberInfo Member;
 
 			public CacheEntry (IMemberContainer container, MemberInfo member,
@@ -2082,12 +2155,7 @@ namespace Mono.CSharp {
 
 		public MemberInfo FindBaseEvent (Type invocationType, string name)
 		{
-			ArrayList applicable;
-			if (method_hash != null)
-				applicable = (ArrayList) method_hash [name];
-			else
-				applicable = (ArrayList) member_hash [name];
-			
+			ArrayList applicable = (ArrayList) member_hash [name];
 			if (applicable == null)
 				return null;
 
@@ -2106,6 +2174,52 @@ namespace Mono.CSharp {
 
 			return null;
 		}
+
+		//
+		// Looks for extension methods with defined name and extension type
+		//
+		public ArrayList FindExtensionMethods (Type extensionType, string name, bool publicOnly)
+		{
+			ArrayList entries;
+			if (method_hash != null)
+				entries = (ArrayList)method_hash [name];
+			else
+				entries = (ArrayList)member_hash [name];
+
+			if (entries == null)
+				return null;
+
+			EntryType entry_type = EntryType.Static | EntryType.Method | EntryType.NotExtensionMethod;
+			if (publicOnly) {
+				entry_type |= EntryType.Public;
+			}
+			EntryType found_entry_type = entry_type & ~EntryType.NotExtensionMethod;
+
+			ArrayList candidates = null;
+			foreach (CacheEntry entry in entries) {
+				if ((entry.EntryType & entry_type) == found_entry_type) {
+					MethodBase mb = (MethodBase)entry.Member;
+
+					IMethodData md = TypeManager.GetMethod (mb);
+					ParameterData pd = md == null ?
+						TypeManager.GetParameterData (mb) : md.ParameterInfo;
+
+					Type ex_type = pd.ExtensionMethodType;
+					if (ex_type == null) {
+						entry.EntryType |= EntryType.NotExtensionMethod;
+						continue;
+					}
+
+					//if (implicit conversion between ex_type and extensionType exist) {
+						if (candidates == null)
+							candidates = new ArrayList (2);
+						candidates.Add (mb);
+					//}
+				}
+			}
+
+			return candidates;
+		}
 		
 		//
 		// This finds the method or property for us to override. invocationType is the type where
