@@ -65,9 +65,6 @@ namespace Mono.CSharp {
 			if (t.Delegates != null)
 				foreach (Delegate de in t.Delegates)
 					de.GenerateDocComment (t);
-			if (t.Enums != null)
-				foreach (Enum en in t.Enums)
-					en.GenerateDocComment (t);
 
 			if (t.Constants != null)
 				foreach (Const c in t.Constants)
@@ -327,7 +324,7 @@ namespace Mono.CSharp {
 			case "void":
 				return typeof (void);
 			}
-			FullNamedExpression e = ds.LookupType (identifier, mc.Location, false);
+			FullNamedExpression e = ds.LookupNamespaceOrType (identifier, mc.Location, false);
 			if (e != null) {
 				if (!(e is TypeExpr))
 					return null;
@@ -366,7 +363,7 @@ namespace Mono.CSharp {
 
 		static bool IsOverride (PropertyInfo deriv_prop, PropertyInfo base_prop)
 		{
-			if (!Invocation.IsAncestralType (base_prop.DeclaringType, deriv_prop.DeclaringType))
+			if (!MethodGroupExpr.IsAncestralType (base_prop.DeclaringType, deriv_prop.DeclaringType))
 				return false;
 
 			Type [] deriv_pd = TypeManager.GetArgumentTypes (deriv_prop);
@@ -405,7 +402,7 @@ namespace Mono.CSharp {
 							continue;
 						MethodBase my = ml [j] as MethodBase;
 						if (mx != null && my != null &&
-							Invocation.IsOverride (my, mx)) {
+							MethodGroupExpr.IsOverride (my, mx)) {
 							overriden = true;
 							break;
 						}
@@ -887,9 +884,11 @@ namespace Mono.CSharp {
 
 		static string GetSignatureForDoc (Type type)
 		{
-			return TypeManager.IsGenericParameter (type) ?
-				"`" + TypeManager.GenericParameterPosition (type) :
-				type.FullName.Replace ("+", ".").Replace ('&', '@');
+			if (TypeManager.IsGenericParameter (type))
+				return "`" + TypeManager.GenericParameterPosition (type);
+
+			string name = type.FullName != null ? type.FullName : type.Name;
+			return name.Replace ("+", ".").Replace ('&', '@');
 		}
 
 		//
@@ -1029,9 +1028,6 @@ namespace Mono.CSharp {
 		public void GenerateDocComment ()
 		{
 			TypeContainer root = RootContext.ToplevelTypes;
-			if (root.Interfaces != null)
-				foreach (Interface i in root.Interfaces) 
-					DocUtil.GenerateTypeDocComment (i, null);
 
 			if (root.Types != null)
 				foreach (TypeContainer tc in root.Types)
@@ -1040,11 +1036,6 @@ namespace Mono.CSharp {
 			if (root.Delegates != null)
 				foreach (Delegate d in root.Delegates) 
 					DocUtil.GenerateDocComment (d, null);
-
-			if (root.Enums != null)
-				foreach (Enum e in root.Enums)
-					e.GenerateDocComment (null);
-
 		}
 	}
 }
