@@ -100,6 +100,13 @@ namespace Mono.CSharp {
 		{
 			ec.CurrentIterator.MarkYield (ec, expr, finally_blocks);
 		}
+
+		protected override void CloneTo (CloneContext clonectx, Statement t)
+		{
+			Yield target = (Yield) t;
+
+			target.expr = expr.Clone (clonectx);
+		}
 	}
 
 	public class YieldBreak : Statement {
@@ -443,7 +450,7 @@ namespace Mono.CSharp {
 
 					ce = TypeManager.int_interlocked_compare_exchange;
 
-					ec.CurrentBranching.CurrentUsageVector.Return ();
+					ec.CurrentBranching.CurrentUsageVector.Goto ();
 					return true;
 				}
 
@@ -577,7 +584,7 @@ namespace Mono.CSharp {
 
 			public override bool Resolve (EmitContext ec)
 			{
-				ec.CurrentBranching.CurrentUsageVector.Return ();
+				ec.CurrentBranching.CurrentUsageVector.Goto ();
 				return true;
 			}
 
@@ -968,7 +975,7 @@ namespace Mono.CSharp {
 				if (Expr == null)
 					return false;
 
-				ec.CurrentBranching.CurrentUsageVector.Return ();
+				ec.CurrentBranching.CurrentUsageVector.Goto ();
 
 				return true;
 			}
@@ -986,12 +993,16 @@ namespace Mono.CSharp {
 			bool is_enumerable;
 			Type iterator_type;
 
-			if (!CheckType (method.ReturnType, out iterator_type, out is_enumerable)) {
+			Type ret = method.ReturnType;
+			if (ret == null)
+				return null;
+
+			if (!CheckType (ret, out iterator_type, out is_enumerable)) {
 				Report.Error (1624, method.Location,
 					      "The body of `{0}' cannot be an iterator block " +
 					      "because `{1}' is not an iterator interface type",
 					      method.GetSignatureForError (),
-					      TypeManager.CSharpName (method.ReturnType));
+					      TypeManager.CSharpName (ret));
 				return null;
 			}
 
