@@ -162,19 +162,24 @@ namespace Mono.CSharp {
 			base.ApplyAttributes (mb, cb, index);
 
 			CustomAttributeBuilder a = new CustomAttributeBuilder (
-				TypeManager.ConsParamArrayAttribute, new object [0]);
+				TypeManager.cons_param_array_attribute, new object[0]);
 				
 			builder.SetCustomAttribute (a);
 		}
 	}
 
 	public class ArglistParameter : Parameter {
-		// Doesn't have proper type because it's never choosed for better conversion
+		// Doesn't have proper type because it's never chosen for better conversion
 		public ArglistParameter () :
-			base (typeof (ArglistParameter), "", Parameter.Modifier.ARGLIST, null, Location.Null)
+			base (typeof (ArglistParameter), String.Empty, Parameter.Modifier.ARGLIST, null, Location.Null)
 		{
 		}
 
+		public override bool CheckAccessibility (InterfaceMemberBase member)
+		{
+			return true;
+		}
+
 		public override bool Resolve (IResolveContext ec)
 		{
 			return true;
@@ -200,13 +205,14 @@ namespace Mono.CSharp {
 			ISBYREF = 8,
 			ARGLIST = 16,
 			REFMASK	= 32,
-			OUTMASK = 64
+			OUTMASK = 64,
+			This	= 128
 		}
 
 		static string[] attribute_targets = new string [] { "param" };
 
 		public Expression TypeName;
-		public readonly Modifier ModFlags;
+		public Modifier modFlags;
 		public string Name;
 		public bool IsCaptured;
 		protected Type parameter_type;
@@ -220,14 +226,25 @@ namespace Mono.CSharp {
 		}
 
 #if GMCS_SOURCE
+		public bool IsTypeParameter;
 		GenericConstraints constraints;
+#else
+	    	public bool IsTypeParameter {
+			get {
+				return false;
+			}
+			set {
+				if (value)
+					throw new Exception ("You can not se TypeParameter in MCS");
+			}
+		}
 #endif
 		
 		public Parameter (Expression type, string name, Modifier mod, Attributes attrs, Location loc)
 			: base (attrs)
 		{
 			Name = name;
-			ModFlags = mod;
+			modFlags = mod;
 			TypeName = type;
 			Location = loc;
 		}
@@ -236,7 +253,7 @@ namespace Mono.CSharp {
 			: base (attrs)
 		{
 			Name = name;
-			ModFlags = mod;
+			modFlags = mod;
 			parameter_type = type;
 			Location = loc;
 		}
@@ -264,7 +281,7 @@ namespace Mono.CSharp {
 				Report.Warning (3022, 1, a.Location, "CLSCompliant attribute has no meaning when applied to parameters. Try putting it on the method instead");
 			}
 
-			// TypeManager.default_parameter_value_attribute_type is null if !NET_2_0
+			// TypeManager.default_parameter_value_attribute_type is null if !NET_2_0, or if System.dll is not referenced
 			if (a.Type == TypeManager.default_parameter_value_attribute_type) {
 				object val = a.GetParameterDefaultValue ();
 				if (val != null) {
@@ -296,6 +313,14 @@ namespace Mono.CSharp {
 			base.ApplyAttributeBuilder (a, cb);
 		}
 
+		public virtual bool CheckAccessibility (InterfaceMemberBase member)
+		{
+			if (IsTypeParameter)
+				return true;
+
+			return member.ds.AsAccessible (parameter_type, member.ModFlags);
+		}
+
 		public override IResolveContext ResolveContext {
 			get {
 				return resolve_context;
@@ -316,16 +341,20 @@ namespace Mono.CSharp {
 			if (texpr == null)
 				return false;
 
+			parameter_type = texpr.Type;
+
 #if GMCS_SOURCE
 			TypeParameterExpr tparam = texpr as TypeParameterExpr;
-			if (tparam != null)
+			if (tparam != null) {
+				IsTypeParameter = true;
 				constraints = tparam.TypeParameter.Constraints;
+				return true;
+			}
 #endif
 
-			parameter_type = texpr.Type;
-			
-			if (parameter_type.IsAbstract && parameter_type.IsSealed) {
-				Report.Error (721, Location, "`{0}': static types cannot be used as parameters", GetSignatureForError ());
+			if ((parameter_type.Attributes & Class.StaticClassAttribute) == Class.StaticClassAttribute) {
+				Report.Error (721, Location, "`{0}': static types cannot be used as parameters", 
+					texpr.GetSignatureForError ());
 				return false;
 			}
 
@@ -334,7 +363,7 @@ namespace Mono.CSharp {
 				return false;
 			}
 
-			if ((ModFlags & Parameter.Modifier.ISBYREF) != 0){
+			if ((modFlags & Parameter.Modifier.ISBYREF) != 0){
 				if (parameter_type == TypeManager.typed_reference_type ||
 				    parameter_type == TypeManager.arg_iterator_type){
 					Report.Error (1601, Location, "Method or delegate parameter cannot be of type `{0}'",
@@ -342,6 +371,12 @@ namespace Mono.CSharp {
 					return false;
 				}
 			}
+
+			if ((modFlags & Modifier.This) != 0 && parameter_type.IsPointer) {
+				Report.Error (1103, Location, "The type of extension method cannot be `{0}'",
+					TypeManager.CSharpName (parameter_type));
+				return false;
+			}
 			
 			return true;
 		}
@@ -356,16 +391,28 @@ namespace Mono.CSharp {
 
 		public Type ExternalType ()
 		{
-			if ((ModFlags & Parameter.Modifier.ISBYREF) != 0)
+			if ((modFlags & Parameter.Modifier.ISBYREF) != 0)
 				return TypeManager.GetReferenceType (parameter_type);
 			
 			return parameter_type;
 		}
 
+		public bool HasExtensionMethodModifier {
+			get { return (modFlags & Modifier.This) != 0; }
+		}
+
+		public Modifier ModFlags {
+			get { return modFlags & ~Modifier.This; }
+		}
+
 		public Type ParameterType {
 			get {
 				return parameter_type;
 			}
+			set {
+				parameter_type = value;
+				IsTypeParameter = false;
+			}
 		}
 
 #if GMCS_SOURCE
@@ -376,23 +423,14 @@ namespace Mono.CSharp {
 		}
 #endif
 
-		public ParameterAttributes Attributes {
+		ParameterAttributes Attributes {
 			get {
-				switch (ModFlags) {
-				case Modifier.NONE:
-					return ParameterAttributes.None;
-				case Modifier.REF:
-					return ParameterAttributes.None;
-				case Modifier.OUT:
-					return ParameterAttributes.Out;
-				case Modifier.PARAMS:
-					return 0;
-				}
-				
-				return ParameterAttributes.None;
+				return (modFlags & Modifier.OUT) == Modifier.OUT ?
+					ParameterAttributes.Out : ParameterAttributes.None;
 			}
 		}
 
+		// TODO: should be removed !!!!!!!
 		public static ParameterAttributes GetParameterAttributes (Modifier mod)
 		{
 			int flags = ((int) mod) & ~((int) Parameter.Modifier.ISBYREF);
@@ -424,7 +462,7 @@ namespace Mono.CSharp {
 			else
 				type_name = TypeName.GetSignatureForError ();
 
-			string mod = GetModifierSignature (ModFlags);
+			string mod = GetModifierSignature (modFlags);
 			if (mod.Length > 0)
 				return String.Concat (mod, ' ', type_name);
 
@@ -442,6 +480,8 @@ namespace Mono.CSharp {
 					return "ref";
 				case Modifier.ARGLIST:
 					return "__arglist";
+				case Modifier.This:
+					return "this";
 				default:
 					return "";
 			}
@@ -546,6 +586,14 @@ namespace Mono.CSharp {
 			}
 		}
 
+		public Parameter Clone ()
+		{
+			Parameter p = new Parameter (TypeName, Name, ModFlags, attributes, Location);
+			p.parameter_type = parameter_type;
+			p.IsTypeParameter = IsTypeParameter;
+
+			return p;
+		}
 	}
 
 	/// <summary>
@@ -562,7 +610,7 @@ namespace Mono.CSharp {
 		static readonly Parameter ArgList = new ArglistParameter ();
 
 #if GMCS_SOURCE
-		public readonly TypeParameter[] TypeParameters;
+//		public readonly TypeParameter[] TypeParameters;
 #endif
 
 		private Parameters ()
@@ -578,7 +626,7 @@ namespace Mono.CSharp {
 			count = types.Length;
 		}
 		
-		public Parameters (Parameter[] parameters)
+		public Parameters (params Parameter[] parameters)
 		{
 			if (parameters == null)
 				throw new ArgumentException ("Use EmptyReadOnlyPatameters");
@@ -630,6 +678,29 @@ namespace Mono.CSharp {
 			}
 		}
 
+		//
+		// The property can be used after parameter types were resolved.
+		//
+		public Type ExtensionMethodType {
+			get {
+				if (count == 0)
+					return null;
+
+				return FixedParameters [0].HasExtensionMethodModifier ?
+					types [0] : null;
+			}
+		}
+
+		public bool HasExtensionMethodType {
+			get {
+				if (count == 0)
+					return false;
+
+				return FixedParameters [0].HasExtensionMethodModifier;
+			}
+		}
+
+
 		bool VerifyArgs ()
 		{
 			if (count < 2)
@@ -736,6 +807,30 @@ namespace Mono.CSharp {
 			}
 		}
 
+#if MS_COMPATIBLE
+		public void InflateTypes (Type[] genArguments, Type[] argTypes)
+		{
+			for (int i = 0; i < count; ++i) {
+				if (FixedParameters[i].IsTypeParameter) {
+					for (int ii = 0; ii < genArguments.Length; ++ii) {
+						if (types[i] != genArguments[ii])
+							continue;
+
+						types[i] = argTypes[ii];
+						FixedParameters[i].ParameterType = types[i];
+						break;
+					}
+					continue;
+				}
+
+				if (types[i].IsGenericType) {
+					types[i] = types[i].GetGenericTypeDefinition().MakeGenericType (argTypes[0]); // FIXME: The order can be different
+					FixedParameters[i].ParameterType = types[i];
+				}
+			}
+		}
+#endif
+
 		public void VerifyClsCompliance ()
 		{
 			foreach (Parameter p in FixedParameters)
@@ -817,6 +912,18 @@ namespace Mono.CSharp {
 			return this [pos].ModFlags;
 		}
 
+		public Parameters Clone ()
+		{
+			Parameter [] parameters_copy = new Parameter [FixedParameters.Length];
+			int i = 0;
+			foreach (Parameter p in FixedParameters)
+				parameters_copy [i++] = p.Clone ();
+			Parameters ps = new Parameters (parameters_copy, HasArglist);
+			if (types != null)
+				ps.types = (Type[])types.Clone ();
+			return ps;
+		}
+		
 		#endregion
 	}
 }
