@@ -159,11 +159,8 @@ namespace Mono.CSharp {
 			// a set of interfaces, we need to be able to tell
 			// them appart by just using the TypeManager.
 			//
-			ArrayList ifaces = root.Interfaces;
-			if (ifaces != null){
-				foreach (TypeContainer i in ifaces) 
-					i.DefineType ();
-			}
+			foreach (TypeContainer tc in root.Types)
+				tc.CreateType ();
 
 			foreach (TypeContainer tc in root.Types)
 				tc.DefineType ();
@@ -171,10 +168,6 @@ namespace Mono.CSharp {
 			if (root.Delegates != null)
 				foreach (Delegate d in root.Delegates) 
 					d.DefineType ();
-
-			if (root.Enums != null)
-				foreach (Enum e in root.Enums)
-					e.DefineType ();
 		}
 
 		static void Error_TypeConflict (string name, Location loc)
@@ -449,10 +442,6 @@ namespace Mono.CSharp {
 		// </remarks>
 		static public void CloseTypes ()
 		{
-			if (root.Enums != null)
-				foreach (Enum en in root.Enums)
-					en.CloseType ();
-
 			//
 			// We do this in two passes, first we close the structs,
 			// then the classes, because it seems the code needs it this
@@ -531,10 +520,8 @@ namespace Mono.CSharp {
 		{
 
 			if (type_container_resolve_order != null){
-#if GMCS_SOURCE
 				foreach (TypeContainer tc in type_container_resolve_order)
 					tc.ResolveType ();
-#endif
 				foreach (TypeContainer tc in type_container_resolve_order)
 					tc.DefineMembers ();
 			}
@@ -545,12 +532,6 @@ namespace Mono.CSharp {
 					d.DefineMembers ();
 			}
 
-			ArrayList enums = root.Enums;
-			if (enums != null){
-				foreach (Enum en in enums)
-					en.DefineMembers ();
-			}
-
 			//
 			// Check for cycles in the struct layout
 			//
@@ -596,21 +577,10 @@ namespace Mono.CSharp {
 					tc.Define ();
 				}
 			}
-
-			ArrayList enums = root.Enums;
-			if (enums != null){
-				foreach (Enum en in enums)
-					en.Define ();
-			}
 		}
 
 		static public void EmitCode ()
 		{
-			if (root.Enums != null) {
-				foreach (Enum e in root.Enums)
-					e.Emit ();
-			}
-
 			if (type_container_resolve_order != null) {
 				foreach (TypeContainer tc in type_container_resolve_order)
 					tc.EmitType ();
