@@ -22,11 +22,17 @@ namespace Mono.CSharp {
 		Type ParameterType (int pos);
 		Type [] Types { get; }
 		int  Count { get; }
+		Type ExtensionMethodType { get; }
 		bool HasParams { get; }
 		string ParameterName (int pos);
 		string ParameterDesc (int pos);
+
 		Parameter.Modifier ParameterModifier (int pos);
 		string GetSignatureForError ();
+
+#if MS_COMPATIBLE
+		void InflateTypes (Type[] genArguments, Type[] argTypes);
+#endif
 	}
 
 	public class ReflectionParameters : ParameterData {
@@ -34,6 +40,7 @@ namespace Mono.CSharp {
 		Type [] types;
 		int params_idx = -1;
 		bool is_varargs;
+		bool is_extension;
 		ParameterData gpd;
 
 		public ReflectionParameters (MethodBase mb)
@@ -80,12 +87,16 @@ namespace Mono.CSharp {
 				if (!pi [i].ParameterType.IsArray)
 					continue;
 
-				object [] attrs = pi [i].GetCustomAttributes (TypeManager.param_array_type, true);
-				if (attrs.Length == 1) {
+				if (pi [i].IsDefined (TypeManager.param_array_type, false)) {
 					params_idx = i;
 					return;
 				}
 			}
+
+			if (TypeManager.extension_attribute_type != null && mb.IsStatic &&
+				(mb.DeclaringType.Attributes & Class.StaticClassAttribute) == Class.StaticClassAttribute &&
+				mb.IsDefined (TypeManager.extension_attribute_type, false))
+				is_extension = true;
 		}
 
 		public override bool Equals (object obj)
@@ -126,12 +137,39 @@ namespace Mono.CSharp {
 			return sb.ToString ();
 		}
 
+#if MS_COMPATIBLE
+		public void InflateTypes (Type[] genArguments, Type[] argTypes)
+		{
+			for (int i = 0; i < types.Length; ++i) {
+				if (types[i].IsGenericParameter) {
+					for (int ii = 0; ii < genArguments.Length; ++ii) {
+						if (types[i] != genArguments[ii])
+							continue;
+
+						types[i] = argTypes[ii];
+						break;
+					}
+					continue;
+				}
+
+				if (types[i].IsGenericType) {
+					Type[] gen_arguments = types[i].GetGenericTypeDefinition ().GetGenericArguments ();
+					for (int ii = 0; ii < gen_arguments.Length; ++ii) {
+						gen_arguments[ii] = argTypes[gen_arguments[ii].GenericParameterPosition];
+					}
+
+					types[i] = types[i].GetGenericTypeDefinition ().MakeGenericType (gen_arguments);
+				}
+			}
+		}
+#endif
+
 		public Type ParameterType (int pos)
 		{
 			if (is_varargs && pos >= pi.Length)
 				return TypeManager.runtime_argument_handle_type;
 
-			return pi [pos].ParameterType;
+			return types [pos];
 		}
 
 		public string ParameterName (int pos)
@@ -167,6 +205,9 @@ namespace Mono.CSharp {
 			if (params_idx == pos)
 				sb.Append ("params ");
 
+			if (ExtensionMethodType != null)
+				sb.Append ("this ");
+
 			sb.Append (TypeManager.CSharpName (partype).Replace ("&", ""));
 
 			return sb.ToString ();
@@ -182,7 +223,7 @@ namespace Mono.CSharp {
 			if (gpd != null)
 				return gpd.ParameterModifier (pos);
 
-			Type t = pi [pos].ParameterType;
+			Type t = types [pos];
 			if (t.IsByRef){
 				if ((pi [pos].Attributes & (ParameterAttributes.Out|ParameterAttributes.In)) == ParameterAttributes.Out)
 					return Parameter.Modifier.OUT;
@@ -197,6 +238,15 @@ namespace Mono.CSharp {
 			get { return is_varargs ? pi.Length + 1 : pi.Length; }
 		}
 
+		public Type ExtensionMethodType {
+			get {
+				if (!is_extension)
+					return null;
+
+				return types [0];
+			}
+		}
+
 		public bool HasParams {
 			get { return params_idx != -1; }
 		}
@@ -361,7 +411,7 @@ namespace Mono.CSharp {
 	/// </summary>
 	public class SeekableStreamReader
 	{
-		public SeekableStreamReader (StreamReader reader)
+		public SeekableStreamReader (TextReader reader)
 		{
 			this.reader = reader;
 			this.buffer = new char [AverageReadLength * 3];
@@ -374,7 +424,7 @@ namespace Mono.CSharp {
 			: this (new StreamReader (stream, encoding, true))
 		{ }
 
-		StreamReader reader;
+		TextReader reader;
 
 		private const int AverageReadLength = 1024;
 
