@@ -1,5 +1,5 @@
 // cilc -- a CIL-to-C binding generator
-// Copyright (C) 2003, 2004, 2005, 2006 Alp Toker <alp@atoker.com>
+// Copyright (C) 2003, 2004, 2005, 2006, 2007 Alp Toker <alp@atoker.com>
 // Licensed under the terms of the MIT License
 
 using System;
@@ -117,7 +117,7 @@ public class cilc
 
 	static bool verbose = false;
 
-	static string extpkgs = "";
+	static string extpkgs = String.Empty;
 	static string[] extsubpkgs = {};
 	static string[] extincludes = {};
 
@@ -136,7 +136,7 @@ public class cilc
 		//string cmd = "gcc";
 		//string args = "-E " + cflags + " " + includedir + "/" + hname;
 
-		string cppincludes = "";
+		string cppincludes = String.Empty;
 		foreach (string include in extincludes)
 			cppincludes += " -include " + include;
 
@@ -242,7 +242,7 @@ public class cilc
 		makefile_defs.WriteLine ("SONAME = " + soname);
 		makefile_defs.WriteLine (@"OBJS = $(shell ls *.c | sed -e 's/\.c/.o/')");
 
-		if (extpkgs != "") {
+		if (extpkgs != String.Empty) {
 			makefile_defs.WriteLine ("EXTRAINCLUDES = $(shell pkg-config --cflags" + extpkgs + ")");
 			makefile_defs.WriteLine ("EXTRALIBS = $(shell pkg-config --libs" + extpkgs + ")");
 		}
@@ -304,7 +304,7 @@ public class cilc
 			if (t.IsEnum)
 				RegisterByVal (t);
 
-			string tns = t.Namespace == null ? "" : t.Namespace;
+			string tns = t.Namespace == null ? String.Empty : t.Namespace;
 
 			if (!ns_types.Contains (tns))
 				ns_types[tns] = new ArrayList ();
@@ -1080,7 +1080,7 @@ public class cilc
 			case "get_type":
 				return "retrieve_type";
 			default:
-			return name;
+				return name;
 		}
 	}
 
@@ -1088,7 +1088,7 @@ public class cilc
 
 	static void FunctionGen (ParameterInfo[] parameters, MethodBase m, Type t, Type ret_type, bool ctor)
 	{
-		string myargs = "";
+		string myargs = String.Empty;
 		bool has_return = !ctor && ret_type != null && ret_type != typeof (void);
 		bool stat = m.IsStatic;
 
@@ -1155,7 +1155,7 @@ public class cilc
 		funcs_done.Add (myname);
 
 		//handle the parameters
-		string mycsargs = "";
+		string mycsargs = String.Empty;
 
 		for (int i = 0 ; i < parameters.Length ; i++) {
 			ParameterInfo p = parameters[i];
@@ -1167,7 +1167,7 @@ public class cilc
 			}
 		}
 
-		if (myargs == "")
+		if (myargs == String.Empty)
 			myargs = "void";
 
 		C.WriteLine ();
@@ -1300,7 +1300,7 @@ public class cilc
 	static string NsToC (string s)
 	{
 		if (s == null)
-			return "";
+			return String.Empty;
 
 		s = s.Replace ('.', '_');
 		return CamelToC (s);
@@ -1309,9 +1309,9 @@ public class cilc
 	static string NsToFlat (string s)
 	{
 		if (s == null)
-			return "";
+			return String.Empty;
 
-		s = s.Replace (".", "");
+		s = s.Replace (".", String.Empty);
 		return s;
 	}
 
@@ -1319,7 +1319,7 @@ public class cilc
 	{
 		//converts camel case to c-style
 
-		string o = "";
+		string o = String.Empty;
 
 		bool prev_is_cap = true;
 
@@ -1341,109 +1341,3 @@ public class cilc
 		return o;
 	}
 }
-
-class CodeWriter
-{
-	private StreamWriter w;
-
-	public CodeWriter (string fname)
-	{
-		Init (fname);
-	}
-
-	public bool IsDuplicate = false;
-
-	void Init (string fname)
-	{
-		if (File.Exists (fname)) {
-			string newfname = fname + ".x";
-			//Console.WriteLine ("Warning: File " + fname + " already exists, using " + newfname);
-			IsDuplicate = true;
-			Init (newfname);
-			return;
-		}
-
-		FileStream fs = new FileStream (fname, FileMode.OpenOrCreate, FileAccess.Write);
-		w = new StreamWriter (fs);
-	}
-
-	public string Indenter = "  ";
-	string cur_indent = "";
-	int level = 0;
-
-	public void Indent ()
-	{
-		level++;
-		cur_indent = "";
-		for (int i = 0; i != level ; i++) cur_indent += Indenter;
-	}
-
-	public void Outdent ()
-	{
-		level--;
-		cur_indent = "";
-		for (int i = 0; i != level ; i++) cur_indent += Indenter;
-	}
-
-	public void Write (string text)
-	{
-		w.Write (text);
-	}
-
-	public void WriteLine (string text)
-	{
-		WriteLine (text, true);
-	}
-
-	public void WriteLine (string text, bool autoindent)
-	{
-		char[] opentags = {'{', '('};
-		char[] closetags = {'}', ')'};
-
-		if (autoindent && text.TrimStart (closetags) != text)
-			Outdent ();
-
-		w.Write (cur_indent);
-		w.WriteLine (text);
-
-		if (autoindent && text.TrimEnd (opentags) != text)
-			Indent ();
-	}
-
-	public void WriteLine (string text, CodeWriter cc)
-	{
-		WriteLine (text, "", cc, "");
-	}
-
-	public void WriteLine (string text, CodeWriter cc, string suffix)
-	{
-		WriteLine (text, "", cc, suffix);
-	}
-
-	public void WriteLine (string text, string prefix, CodeWriter cc)
-	{
-		WriteLine (text, prefix, cc, "");
-	}
-
-	public void WriteLine (string text, string prefix, CodeWriter cc, string suffix)
-	{
-		WriteLine (text);
-		cc.WriteLine (prefix + text + suffix);
-	}
-
-	public void WriteComment (string text)
-	{
-		w.WriteLine ("/* " + text + " */");
-	}
-
-	public void WriteLine ()
-	{
-		w.WriteLine ("");
-	}
-
-	public void Close ()
-	{
-		w.Flush ();
-		w.Close ();
-	}
-}
