@@ -5,7 +5,7 @@ using System.Xml;
 using System.Xml.Schema;
 using System.Xml.Xsl;
 using System.Xml.XPath;
-#if !TARGET_JVM
+#if !TARGET_JVM && !MSNET
 using Commons.Xml.Nvdl;
 using Commons.Xml.Relaxng;
 using Commons.Xml.Relaxng.Rnc;
@@ -63,7 +63,7 @@ environment variable that affects behavior:
 			case "--help":
 				Usage ();
 				return;
-#if !TARGET_JVM
+#if !TARGET_JVM && !MSNET
 			case "--validate":
 				ValidateAuto (args);
 				return;
@@ -92,7 +92,7 @@ environment variable that affects behavior:
 			}
 		}
 
-#if !TARGET_JVM
+#if !TARGET_JVM && !MSNET
 		static void ValidateAuto (string [] args)
 		{
 			if (args.Length < 1) {
@@ -163,6 +163,7 @@ environment variable that affects behavior:
 			XmlTextReader schemaxml = new XmlTextReader (args [1]);
 			XSchema xsd = XSchema.Read (schemaxml, null);
 			schemaxml.Close ();
+			xsd.Compile (null);
 			for (int i = 2; i < args.Length; i++) {
 				XmlTextReader xtr = new XmlTextReader (args [i]);
 				XmlValidatingReader xvr = new XmlValidatingReader (xtr);
