@@ -29,6 +29,12 @@
 
 #undef DEBUG
 
+#ifdef DISABLE_SHARED_HANDLES
+gboolean _wapi_shm_disabled = TRUE;
+#else
+gboolean _wapi_shm_disabled = FALSE;
+#endif
+
 static gchar *_wapi_shm_file (_wapi_shm_t type)
 {
 	static gchar file[_POSIX_PATH_MAX];
@@ -214,6 +220,18 @@ try_again:
 	return(fd);
 }
 
+static gboolean check_disabled (void)
+{
+	if (_wapi_shm_disabled || g_getenv ("MONO_DISABLE_SHM")) {
+		const char* val = g_getenv ("MONO_DISABLE_SHM");
+		if (val == NULL || *val == '1' || *val == 'y' || *val == 'Y') {
+			_wapi_shm_disabled = TRUE;
+		}
+	}
+
+	return(_wapi_shm_disabled);
+}
+
 /*
  * _wapi_shm_attach:
  * @success: Was it a success
@@ -237,8 +255,15 @@ gpointer _wapi_shm_attach (_wapi_shm_t type)
 	case WAPI_SHM_FILESHARE:
 		size = sizeof(struct _WapiFileShareLayout);
 		break;
+	default:
+		g_error ("Invalid type in _wapi_shm_attach ()");
+		return NULL;
 	}
-	
+
+	if (check_disabled ()) {
+		return g_malloc0 (size);
+	}
+
 	fd = _wapi_shm_file_open (filename, size);
 	if (fd == -1) {
 		g_critical ("%s: shared file [%s] open error", __func__,
@@ -269,7 +294,7 @@ gpointer _wapi_shm_attach (_wapi_shm_t type)
 	return(shm_seg);
 }
 
-void _wapi_shm_semaphores_init ()
+static void shm_semaphores_init (void)
 {
 	key_t key;
 	key_t oldkey;
@@ -330,10 +355,10 @@ again:
 		while ((_wapi_sem_id = semget (key, _WAPI_SHARED_SEM_COUNT,
 					       IPC_CREAT | IPC_EXCL | 0600)) == -1) {
 			if (errno == ENOMEM) {
-				g_critical ("%s: semget error: %s", __func__,
+				g_error ("%s: semget error: %s", __func__,
 					    g_strerror (errno));
 			} else if (errno == ENOSPC) {
-				g_critical ("%s: semget error: %s.  Try deleting some semaphores with ipcs and ipcrm", __func__, g_strerror (errno));
+				g_error ("%s: semget error: %s.  Try deleting some semaphores with ipcs and ipcrm\nor increase the maximum number of semaphore in the system.", __func__, g_strerror (errno));
 			} else if (errno != EEXIST) {
 				if (retries > 3)
 					g_warning ("%s: semget error: %s key 0x%x - trying again", __func__,
@@ -419,10 +444,20 @@ again:
 	
 	_wapi_shm_sem_unlock (_WAPI_SHARED_SEM_PROCESS_COUNT_LOCK);
 
-	munmap (tmp_shared, sizeof(struct _WapiHandleSharedLayout));
+	if (_wapi_shm_disabled)
+		g_free (tmp_shared);
+	else
+		munmap (tmp_shared, sizeof(struct _WapiHandleSharedLayout));
 }
 
-void _wapi_shm_semaphores_remove (void)
+static mono_mutex_t noshm_sems[_WAPI_SHARED_SEM_COUNT] = {MONO_MUTEX_INITIALIZER};
+
+static void noshm_semaphores_init (void)
+{
+	/* No need to do anything */
+}
+
+static void shm_semaphores_remove (void)
 {
 	int thr_ret;
 	int proc_count;
@@ -459,7 +494,12 @@ void _wapi_shm_semaphores_remove (void)
 	}
 }
 
-int _wapi_shm_sem_lock (int sem)
+static void noshm_semaphores_remove (void)
+{
+	/* No need to do anything */
+}
+
+static int shm_sem_lock (int sem)
 {
 	struct sembuf ops;
 	int ret;
@@ -505,7 +545,20 @@ int _wapi_shm_sem_lock (int sem)
 	return(ret);
 }
 
-int _wapi_shm_sem_trylock (int sem)
+static int noshm_sem_lock (int sem)
+{
+	int ret;
+	
+#ifdef DEBUG
+	g_message ("%s: locking nosem %d", __func__, sem);
+#endif
+	
+	ret = mono_mutex_lock (&noshm_sems[sem]);
+	
+	return(ret);
+}
+
+static int shm_sem_trylock (int sem)
 {
 	struct sembuf ops;
 	int ret;
@@ -556,7 +609,20 @@ int _wapi_shm_sem_trylock (int sem)
 	return(ret);
 }
 
-int _wapi_shm_sem_unlock (int sem)
+static int noshm_sem_trylock (int sem)
+{
+	int ret;
+	
+#ifdef DEBUG
+	g_message ("%s: trying to lock nosem %d", __func__, sem);
+#endif
+	
+	ret = mono_mutex_trylock (&noshm_sems[sem]);
+	
+	return(ret);
+}
+
+static int shm_sem_unlock (int sem)
 {
 	struct sembuf ops;
 	int ret;
@@ -603,3 +669,60 @@ int _wapi_shm_sem_unlock (int sem)
 	return(ret);
 }
 
+static int noshm_sem_unlock (int sem)
+{
+	int ret;
+	
+#ifdef DEBUG
+	g_message ("%s: unlocking nosem %d", __func__, sem);
+#endif
+	
+	ret = mono_mutex_unlock (&noshm_sems[sem]);
+	
+	return(ret);
+}
+
+void _wapi_shm_semaphores_init (void)
+{
+	if (check_disabled ()) {
+		noshm_semaphores_init ();
+	} else {
+		shm_semaphores_init ();
+	}
+}
+
+void _wapi_shm_semaphores_remove (void)
+{
+	if (_wapi_shm_disabled) {
+		noshm_semaphores_remove ();
+	} else {
+		shm_semaphores_remove ();
+	}
+}
+
+int _wapi_shm_sem_lock (int sem)
+{
+	if (_wapi_shm_disabled) {
+		return(noshm_sem_lock (sem));
+	} else {
+		return(shm_sem_lock (sem));
+	}
+}
+
+int _wapi_shm_sem_trylock (int sem)
+{
+	if (_wapi_shm_disabled) {
+		return(noshm_sem_trylock (sem));
+	} else {
+		return(shm_sem_trylock (sem));
+	}
+}
+
+int _wapi_shm_sem_unlock (int sem)
+{
+	if (_wapi_shm_disabled) {
+		return(noshm_sem_unlock (sem));
+	} else {
+		return(shm_sem_unlock (sem));
+	}
+}
