@@ -177,7 +177,8 @@ typedef enum {
 	MONO_MARSHAL_CONV_FTN_DEL,
 	MONO_MARSHAL_FREE_ARRAY,
 	MONO_MARSHAL_CONV_BSTR_STR,
-	MONO_MARSHAL_CONV_SAFEHANDLE
+	MONO_MARSHAL_CONV_SAFEHANDLE,
+	MONO_MARSHAL_CONV_HANDLEREF
 } MonoMarshalConv;
 
 typedef struct {
